@@ -1,7 +1,7 @@
 /*
  * auxsys.c - Auxiliary system functions
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: auxsys.c,v 1.11 2007/09/29 09:43:12 shirok Exp $
  */
 
 #include <gauche.h>
@@ -184,11 +182,11 @@ clock_t times(struct tms *info)
 
 extern void Scm_Init_auxsyslib(ScmModule *mod);
 
-SCM_EXTENSION_ENTRY void Scm_Init_auxsys(void)
+SCM_EXTENSION_ENTRY void Scm_Init_gauche__auxsys(void)
 {
     ScmModule *mod;
 
-    SCM_INIT_EXTENSION(auxsys);
+    SCM_INIT_EXTENSION(gauche__auxsys);
     mod = SCM_FIND_MODULE("gauche.auxsys", SCM_FIND_MODULE_CREATE);
     Scm_Init_auxsyslib(mod);
 }
