@@ -1,7 +1,7 @@
 /*
  * binary.h - Binary I/O routines
  *
- *   Copyright (c) 2004-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2004-2010  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,15 +29,10 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: binary.h,v 1.7 2007/03/22 11:20:13 shirok Exp $
  */
 
 #include <gauche.h>
 #include <gauche/builtin-syms.h>
-#include <gauche/uvector.h>
-
-extern ScmObj Scm_DefaultEndian(void);
 
 extern ScmObj Scm_ReadBinaryU8(ScmPort *iport, ScmSymbol *e);
 extern ScmObj Scm_ReadBinaryU16(ScmPort *iport, ScmSymbol *e);
