@@ -15,14 +15,11 @@
 ;; [SK]: 
 ;;
 
-;; $Id: io.scm,v 1.5 2007/02/20 08:43:19 shirok Exp $
-
 (define-module binary.io
   (use gauche.uvector)
   (use srfi-1)  ;; list library
   (use srfi-13) ;; string library
-  (export default-endian
-          read-uint read-u8 read-u16 read-u32 read-u64
+  (export read-uint read-u8 read-u16 read-u32 read-u64
           read-sint read-s8 read-s16 read-s32 read-s64
           read-ber-integer read-f16 read-f32 read-f64
           write-uint write-u8 write-u16 write-u32 write-u64
@@ -61,7 +58,7 @@
           ))
 (select-module binary.io)
 
-(dynamic-load "binary")
+(dynamic-load "binary--io")
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; config
@@ -77,26 +74,25 @@
 
 ;; mind-numblingly slow, consider a uvector approach but it doesn't
 ;; handle endianess
-(define (read-uint size . args)
-  (let-optionals* args ((port (current-input-port))
-                        (endian (default-endian)))
-    (case size
-      ((1) (read-u8 port endian))
-      ((2) (read-u16 port endian))
-      ((4) (read-u32 port endian))
-      ((8) (read-u64 port endian))
-      (else
-       (let loop ((ls '())
-                  (cnt 0))
-         (if (= cnt size)
-           (fold (lambda (a b) (+ a (* b *byte-magnitude*)))
-                 0
-                 (if (eq? endian 'big-endian) (reverse ls) ls))
-           (let1 byte (read-byte port)
-             (if (eof-object? byte)
-               byte
-               (loop (cons byte ls) (+ cnt 1)))))))
-      )))
+(define (read-uint size :optional (port (current-input-port))
+                   (endian (default-endian)))
+  (case size
+    [(1) (read-u8 port endian)]
+    [(2) (read-u16 port endian)]
+    [(4) (read-u32 port endian)]
+    [(8) (read-u64 port endian)]
+    [else
+     (let loop ((ls '())
+                (cnt 0))
+       (if (= cnt size)
+         (fold (lambda (a b) (+ a (* b *byte-magnitude*)))
+               0
+               (if (eq? endian 'big-endian) (reverse ls) ls))
+         (let1 byte (read-byte port)
+           (if (eof-object? byte)
+             byte
+             (loop (cons byte ls) (+ cnt 1))))))]
+    ))
 
 (define (lognot-small int bytes)
   (logand (lognot int) (- (expt *bit-size* (* *byte-size* bytes)) 1)))
@@ -112,50 +108,48 @@
     (+ 1 (lognot-small (abs int) bytes))
     int))
 
-(define (read-sint size . args)
-  (let-optionals* args ((port (current-input-port))
-                        (endian (default-endian)))
-    (case size
-      ((1) (read-s8 port endian))
-      ((2) (read-s16 port endian))
-      ((4) (read-s32 port endian))
-      ((8) (read-s64 port endian))
-      (else
-       (uint->sint (read-uint size port endian) size)))))
+(define (read-sint size :optional
+                   (port (current-input-port))
+                   (endian (default-endian)))
+  (case size
+    [(1) (read-s8 port endian)]
+    [(2) (read-s16 port endian)]
+    [(4) (read-s32 port endian)]
+    [(8) (read-s64 port endian)]
+    [else (uint->sint (read-uint size port endian) size)]))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; basic writing
 
-(define (write-uint size int . args)
-  (let-optionals* args ((port (current-output-port))
-                        (endian (default-endian)))
-    (case size
-      ((1) (write-u8 int port endian))
-      ((2) (write-u16 int port endian))
-      ((4) (write-u32 int port endian))
-      ((8) (write-u64 int port endian))
-      (else
-       (let ((ls '()))
-         ;; build a list of bytes
-         (dotimes (i size)
-           (push! ls (logand int *byte-mask*))
-           (set! int (ash int *byte-right-shift*)))
-         ;; reverse if big-endian
-         (unless (eq? endian 'big-endian)
-           (set! ls (reverse ls)))
-         ;; write the list
-         (for-each (cut write-byte <> port) ls))))))
+(define (write-uint size int :optional
+                    (port (current-output-port))
+                    (endian (default-endian)))
+  (case size
+    [(1) (write-u8 int port endian)]
+    [(2) (write-u16 int port endian)]
+    [(4) (write-u32 int port endian)]
+    [(8) (write-u64 int port endian)]
+    [else
+     (let ((ls '()))
+       ;; build a list of bytes
+       (dotimes (i size)
+         (push! ls (logand int *byte-mask*))
+         (set! int (ash int *byte-right-shift*)))
+       ;; reverse if big-endian
+       (unless (eq? endian 'big-endian)
+         (set! ls (reverse ls)))
+       ;; write the list
+       (for-each (cut write-byte <> port) ls))]))
 
-(define (write-sint size int . args)
-  (let-optionals* args ((port (current-output-port))
-                        (endian (default-endian)))
-    (case size
-      ((1) (write-s8 int port endian))
-      ((2) (write-s16 int port endian))
-      ((4) (write-s32 int port endian))
-      ((8) (write-s64 int port endian))
-      (else
-       (write-uint size (sint->uint int size) port endian)))))
+(define (write-sint size int :optional
+                    (port (current-output-port))
+                    (endian (default-endian)))
+  (case size
+    [(1) (write-s8 int port endian)]
+    [(2) (write-s16 int port endian)]
+    [(4) (write-s32 int port endian)]
+    [(8) (write-s64 int port endian)]
+    [else (write-uint size (sint->uint int size) port endian)]))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; compatibility
@@ -211,32 +205,27 @@
 ;; encoded, but the encoding is efficient and small integers don't take
 ;; up any more space than they would in normal char/short/int encodings.
 
-(define (read-ber-integer . opt-port)
-  (let ((port (get-optional opt-port (current-input-port))))
-    (let ((first (read-byte port)))
-      (if (eof-object? first)
-        first ;; stop on eof
-        (if (< first 128)
-          first
-          (let loop ((res (ash (logand first #b01111111) 7))
-                     (byte (read-u8 port)))
-            (if (< byte 128)
-              (+ res byte) ;; final byte
-              (loop (ash (+ res (logand byte #b01111111)) 7)
-                    (read-u8 port)))))))))
+(define (read-ber-integer :optional (port (current-input-port)))
+  (let1 first (read-byte port)
+    (if (eof-object? first)
+      first ;; stop on eof
+      (if (< first 128)
+        first
+        (let loop ((res (ash (logand first #b01111111) 7))
+                   (byte (read-u8 port)))
+          (if (< byte 128)
+            (+ res byte) ;; final byte
+            (loop (ash (+ res (logand byte #b01111111)) 7)
+                  (read-u8 port))))))))
 
-(define (write-ber-integer number . opt-port)
-  (let ((port (get-optional opt-port (current-output-port))))
-    (let ((final (logand number #b01111111))
-          (start (ash number -7)))
-      (unless (zero? start)
-        (let loop ((n start))
-          (cond ((< n 128)
-                 (write-u8 (logior n #b10000000)))
-                (else
-                 (loop (ash n -7)) ;; write high bytes first
-                 (write-u8 (logior (logand n #b01111111) #b10000000))))))
-      (write-u8 final))))
-
-(provide "binary/io")
+(define (write-ber-integer number :optional (port (current-output-port)))
+  (let ((final (logand number #b01111111))
+        (start (ash number -7)))
+    (unless (zero? start)
+      (let loop ((n start))
+        (cond [(< n 128) (write-u8 (logior n #b10000000))]
+              [else
+               (loop (ash n -7)) ;; write high bytes first
+               (write-u8 (logior (logand n #b01111111) #b10000000))])))
+    (write-u8 final)))
 
