@@ -1,7 +1,7 @@
 /*
  * charconv.c - character code conversion library
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: charconv.c,v 1.60 2007/09/13 12:30:25 shirok Exp $
  */
 
 #include <string.h>
@@ -604,14 +602,13 @@ static int chartoucs(ScmChar ch)
 /*====================================================================
  * Initialization
  */
-extern void Scm_Init_convlib(ScmModule *module);
 extern void Scm_Init_convaux(void);
 extern void Scm_Init_convguess(void);
 
-SCM_EXTENSION_ENTRY void Scm_Init_libcharconv(void)
+SCM_EXTENSION_ENTRY void Scm_Init_gauche__charconv(void)
 {
     ScmModule *mod;
-    SCM_INIT_EXTENSION(charconv);
+    SCM_INIT_EXTENSION(gauche__charconv);
     mod = SCM_FIND_MODULE("gauche.charconv", SCM_FIND_MODULE_CREATE);
     guess.procs = NULL;
     (void)SCM_INTERNAL_MUTEX_INIT(guess.mutex);
@@ -628,7 +625,6 @@ SCM_EXTENSION_ENTRY void Scm_Init_libcharconv(void)
 #endif
     (void)SCM_INTERNAL_MUTEX_INIT(ucsconv.mutex);
     Scm_Init_convguess();
-    Scm_Init_convlib(mod);
     Scm_Init_convaux();
     Scm__InstallCharconvHooks(ucstochar, chartoucs);
     Scm__InstallCodingAwarePortHook(coding_aware_conv);
