@@ -1,7 +1,7 @@
 /*
  * fcntl.c - fcntl interface
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,15 +29,13 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: fcntl.c,v 1.22 2007/09/15 12:30:47 shirok Exp $
  */
 
 #define _GNU_SOURCE  /* for Linux, this enables additional features */
 
+#include <gauche.h>
 #include <string.h>
 #include <errno.h>
-#include <gauche.h>
 #include <gauche/class.h>
 #include <gauche/extend.h>
 
@@ -60,20 +58,20 @@ static ScmObj flock_allocate(ScmClass *klass, ScmObj initargs)
     return SCM_OBJ(f);
 }
 
-#define FLOCK_GET_N_SET(name, type)                                       \
+#define FLOCK_GET_N_SET(name, type, make, get)                            \
   static ScmObj SCM_CPP_CAT3(flock_, name, _get)(ScmSysFlock* t)          \
-  { return Scm_MakeInteger(t->lock.name); }                               \
+  { return make(t->lock.name); }                                          \
   static void SCM_CPP_CAT3(flock_, name, _set)(ScmSysFlock* t, ScmObj v)  \
   {                                                                       \
       if (!SCM_INTEGERP(v)) Scm_Error("integer required, but got %S", v); \
-      t->lock.name = (type)Scm_GetInteger(v);                             \
+      t->lock.name = (type)get(v);                                        \
   }
 
-FLOCK_GET_N_SET(l_type, short)
-FLOCK_GET_N_SET(l_whence, short)
-FLOCK_GET_N_SET(l_start, off_t)
-FLOCK_GET_N_SET(l_len, off_t)
-FLOCK_GET_N_SET(l_pid, pid_t)
+FLOCK_GET_N_SET(l_type, short, Scm_MakeInteger, Scm_GetInteger)
+FLOCK_GET_N_SET(l_whence, short, Scm_MakeInteger, Scm_GetInteger)
+FLOCK_GET_N_SET(l_start, off_t, Scm_OffsetToInteger, Scm_IntegerToOffset)
+FLOCK_GET_N_SET(l_len, off_t, Scm_OffsetToInteger, Scm_IntegerToOffset)
+FLOCK_GET_N_SET(l_pid, pid_t, Scm_MakeInteger, Scm_GetInteger)
 
 static ScmClassStaticSlotSpec flock_slots[] = {
     SCM_CLASS_SLOT_SPEC("type",   flock_l_type_get, flock_l_type_set),
@@ -203,11 +201,11 @@ ScmObj Scm_SysFcntl(ScmObj port_or_fd, int op, ScmObj arg)
 
 extern void Scm_Init_fcntlib(ScmModule *mod);
 
-SCM_EXTENSION_ENTRY void Scm_Init_fcntl(void)
+SCM_EXTENSION_ENTRY void Scm_Init_gauche__fcntl(void)
 {
     ScmModule *mod;
 
-    SCM_INIT_EXTENSION(fcntl);
+    SCM_INIT_EXTENSION(gauche__fcntl);
     mod = SCM_FIND_MODULE("gauche.fcntl", SCM_FIND_MODULE_CREATE);
     Scm_InitStaticClass(&Scm_SysFlockClass, "<sys-flock>",
                         mod, flock_slots, 0);
