@@ -10,7 +10,6 @@
  *     the allocated memory for random number generator object.
  *   - changed the names of the functions
  *   - added stuff to make it as a Gauche extension module.
- * $Id: mt-random.c,v 1.20 2008/02/03 13:07:43 shirok Exp $
  *
  * The original copyright notice follows.
  */
@@ -305,10 +304,10 @@ static ScmObj mt_allocate(ScmClass *klass, ScmObj initargs)
 
 extern void Scm_Init_mt_lib(ScmModule*);
 
-SCM_EXTENSION_ENTRY void Scm_Init_mt_random(void)
+SCM_EXTENSION_ENTRY void Scm_Init_math__mt_random(void)
 {
     ScmModule *mod = SCM_FIND_MODULE("math.mt-random", SCM_FIND_MODULE_CREATE);
-    SCM_INIT_EXTENSION(mtrandom);
+    SCM_INIT_EXTENSION(math__mtrandom);
     Scm_InitStaticClass(&Scm_MersenneTwisterClass, "<mersenne-twister>",
                         mod, NULL, 0);
     key_seed = SCM_MAKE_KEYWORD("seed");
