@@ -1,7 +1,7 @@
 ;;;
 ;;; mt-random - Mersenne Twister interface
 ;;;  
-;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,8 +30,6 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: mt-random.scm,v 1.11 2007/03/02 07:39:05 shirok Exp $
-;;;
 
 (define-module math.mt-random
   (use gauche.uvector)
@@ -48,7 +46,7 @@
   )
 (select-module math.mt-random)
 
-(dynamic-load "mt-random" :export-symbols #t)
+(dynamic-load "math--mt-random")
 
 (define (%get-nword-random-int mt n)
   (let loop ((i 0) (r (%mt-random-uint32 mt)))
@@ -69,4 +67,3 @@
               (quotient r q)
               (loop (%get-nword-random-int mt siz)))))))
 
-(provide "math/mt-random")
