@@ -2,8 +2,11 @@
 ;; testing net
 ;;
 
+#!no-fold-case
+
 (use gauche.test)
 (use gauche.uvector)
+(use gauche.experimental.app)
 (use srfi-1)
 (use srfi-13)
 (test-start "net")
@@ -181,20 +184,24 @@
          (and host
               (or (equal? (slot-ref host 'name) "localhost")
                   (member "localhost" (slot-ref host 'aliases))
-                  ;; cygwin usually doesn't define "localhost", and
-                  ;; returns hostname.  For now we skip this test.
-                  (string-suffix? "cygwin" (gauche-architecture)))
+                  ;; cygwin and mingw usually doesn't define "localhost", and
+                  ;; returns hostname.
+                  (cond-expand
+                   [(or gauche.os.cygwin gauche.os.windows) #t]
+                   [else #f]))
               (member "127.0.0.1" (slot-ref host 'addresses))
               #t)))
 
 (test* "gethostbyaddr" #t
-       (let ((host (sys-gethostbyaddr "127.0.0.1" |AF_INET|)))
+       (let ((host (sys-gethostbyaddr "127.0.0.1" AF_INET)))
          (and host
               (or (equal? (slot-ref host 'name) "localhost")
                   (member "localhost" (slot-ref host 'aliases))
-                  ;; cygwin usually doesn't define "localhost", and
+                  ;; cygwin and mingw usually doesn't define "localhost", and
                   ;; returns hostname.  For now we skip this test.
-                  (string-suffix? "cygwin" (gauche-architecture)))
+                  (cond-expand
+                   [(or gauche.os.cygwin gauche.os.windows) #t]
+                   [else #f]))
               (member "127.0.0.1" (slot-ref host 'addresses))
               #t)))
 
@@ -244,23 +251,45 @@
 ;;-----------------------------------------------------------------
 (test-section "socket")
 
-(define (simple-server socket)
-  (let loop ((clnt (socket-accept socket)))
-    (let ((in   (socket-input-port clnt))
-          (out  (socket-output-port clnt)))
-      (let loop2 ((line (read-line in)))
-        (cond ((eof-object? line)
-               (socket-close clnt)
-               (loop (socket-accept socket)))
-              ((string=? line "END")
-               (socket-close clnt)
-               (socket-close socket)
-               (sys-exit 33))
-              (else
-               (display (string-upcase line) out)
-               (newline out)
-               (flush out)
-               (loop2 (read-line in))))))))
+;; NB: we used to run the server loop with sys-fork.  However, some platforms
+;; (e.g. cygwin) doesn't go well with fork(), and Windows native platform
+;; doesn't even support fork() at all, we instead run a child process via
+;; gauche.process.  Note that we assume this script is run where ../../src/gosh
+;; is THE executable.
+
+(use gauche.process)
+
+(define (run-simple-server sockargs)
+  (with-output-to-file "testserv.o"
+    (lambda ()
+      (write '(use gauche.net))
+      (write '(use srfi-13))
+      (write '(define (simple-server socket)
+                (newline) (flush) ;; handshake
+                (let loop ((clnt (socket-accept socket)))
+                  (let ((in   (socket-input-port clnt))
+                        (out  (socket-output-port clnt)))
+                    (let loop2 ((line (read-line in)))
+                      (cond ((eof-object? line)
+                             (socket-close clnt)
+                             (loop (socket-accept socket)))
+                            ((string=? line "END")
+                             (socket-close clnt)
+                             (socket-close socket)
+                             (sys-exit 33))
+                            (else
+                             (display (string-upcase line) out)
+                             (newline out)
+                             (flush out)
+                             (loop2 (read-line in)))))))))
+      (write `(define (main args)
+                (simple-server ,sockargs)
+                0)))
+    :if-exists :supersede)
+  (let1 p (run-process `("../../src/gosh" "-ftest" "./testserv.o")
+                       :output :pipe)
+    (read-line (process-output p)) ;; handshake
+    #t))
 
 ;; max size of the packet.  increase this to test robustness for
 ;; buffer overrun attack.  right now, Gauche can bear fairly large
@@ -273,56 +302,52 @@
 
 (sys-unlink "sock.o")
 
-(test* "unix server socket" #f
-       (let ((pid (sys-fork)))
-         (if (= pid 0)
-           (simple-server (make-server-socket 'unix "sock.o"))
-           (begin
-             (sys-select #f #f #f 300000)
-             (let ((stat (sys-stat "sock.o")))
-               (not (memq (sys-stat->file-type stat) '(socket fifo))))))))
+(cond-expand
+ [gauche.os.windows #f]
+ [else
+  (test* "unix server socket" #f
+	 (begin
+	   (run-simple-server '(make-server-socket 'unix "sock.o"))
+	   (let1 stat (sys-stat "sock.o")
+		 (not (memq (sys-stat->file-type stat) '(socket fifo))))))
 
-(test* "unix client socket" '("ABC" "XYZ")
-       (call-with-client-socket (make-client-socket 'unix "sock.o")
-         (lambda (in out)
-           (display "abc\n" out) (flush out)
-           (let ((abc (read-line in)))
-             (display "xyz\n" out) (flush out)
-             (list abc (read-line in))))))
+  (test* "unix client socket" '("ABC" "XYZ")
+         (call-with-client-socket (make-client-socket 'unix "sock.o")
+           (lambda (in out)
+             (display "abc\n" out) (flush out)
+             (let ((abc (read-line in)))
+               (display "xyz\n" out) (flush out)
+               (list abc (read-line in))))))
 
-(test* "unix client socket" #t
-       (call-with-client-socket (make-client-socket 'unix "sock.o")
-         (lambda (in out)
-           (display (make-string *chunk-size* #\a) out)
-           (newline out)
-           (flush out)
-           (string=? (read-line in) (make-string *chunk-size* #\A)))))
+  (test* "unix client socket" #t
+         (call-with-client-socket (make-client-socket 'unix "sock.o")
+           (lambda (in out)
+             (display (make-string *chunk-size* #\a) out)
+             (newline out)
+             (flush out)
+             (string=? (read-line in) (make-string *chunk-size* #\A)))))
 
-(test* "unix client socket" #t
-       (call-with-client-socket (make-client-socket (make <sockaddr-un> :path "sock.o"))
-         (lambda (in out)
-           (display (make-string *chunk-size* #\a) out)
-           (newline out)
-           (flush out)
-           (string=? (read-line in) (make-string *chunk-size* #\A)))))
-
-(test* "unix client socket" 33
-       (call-with-client-socket (make-client-socket 'unix "sock.o")
-         (lambda (in out)
-           (display "END\n" out) (flush out)
-           (receive (pid code) (sys-wait)
-             (sys-wait-exit-status code)))))
+  (test* "unix client socket" #t
+         (call-with-client-socket
+          (make-client-socket (make <sockaddr-un> :path "sock.o"))
+          (lambda (in out)
+            (display (make-string *chunk-size* #\a) out)
+            (newline out)
+            (flush out)
+            (string=? (read-line in) (make-string *chunk-size* #\A)))))
 
+  (test* "unix client socket" 33
+         (call-with-client-socket (make-client-socket 'unix "sock.o")
+           (lambda (in out)
+             (display "END\n" out) (flush out)
+             (receive (pid code) (sys-wait)
+                      (sys-wait-exit-status code)))))
+  ])
 
 (sys-unlink "sock.o")
 
 (test* "inet server socket" #t
-       (let ((pid (sys-fork)))
-         (if (= pid 0)
-           (simple-server (make-server-socket 'inet *inet-port* :reuse-addr? #t))
-           (begin
-             (sys-select #f #f #f 300000)
-             #t))))
+       (run-simple-server `(make-server-socket 'inet ,*inet-port* :reuse-addr? #t)))
 
 (test* "inet client socket" '("ABC" "XYZ")
        (call-with-client-socket (make-client-socket 'inet "localhost" *inet-port*)
@@ -332,7 +357,7 @@
              (display "xyz\n" out) (flush out)
              (list abc (read-line in))))))
 
-(test* "inet client socket" #t
+(test* "inet client socket (host,port)" #t
        (call-with-client-socket (make-client-socket "localhost" *inet-port*)
          (lambda (in out)
            (display (make-string *chunk-size* #\a) out)
@@ -340,39 +365,66 @@
            (flush out)
            (string=? (read-line in) (make-string *chunk-size* #\A)))))
 
-;; On IPv6 system, the loopback may have different name than "localhost".
-;; We apply some heuristics here.
-(test* "inet client socket" #t
-       (and-let* ((sock (any (lambda (name)
-                               (guard (e (else #f))
-                                 (make-client-socket
-                                  (make (cond-expand
-                                         (gauche.net.ipv6 <sockaddr-in6>)
-                                         (else <sockaddr-in>))
-                                    :host name :port *inet-port*))
-                                 ))
-                             '("localhost" "ip6-localhost" "ipv6-localhost"
-                               "::1" "127.0.0.1"))))
-         (call-with-client-socket sock
-           (lambda (in out)
-             (display (make-string *chunk-size* #\a) out)
-             (newline out)
-             (flush out)
-             (string=? (read-line in) (make-string *chunk-size* #\A))))))
+(test* "inet client socket (sockaddr)" #t
+       (call-with-client-socket (make-client-socket
+                                 (make <sockaddr-in>
+                                   :host "localhost" :port *inet-port*))
+         (lambda (in out)
+           (display (make-string *chunk-size* #\a) out)
+           (newline out)
+           (flush out)
+           (string=? (read-line in) (make-string *chunk-size* #\A)))))
 
-(test* "inet client socket" 33
+(test* "inet client socket (termination)" 33
        (call-with-client-socket (make-client-socket 'inet "localhost" *inet-port*)
          (lambda (in out)
            (display "END\n" out) (flush out)
            (receive (pid code) (sys-wait)
              (sys-wait-exit-status code)))))
 
-(test* "socket w/ port error handling" *test-error*
+
+(cond-expand
+ [gauche.net.ipv6
+  (test* "inet server socket (ipv6)" #t
+         (run-simple-server `(make-server-socket
+                              (make <sockaddr-in6>
+                                :host :any :port ,*inet-port*)
+                              :reuse-addr? #t)))
+
+  ;; On IPv6 system, the loopback may have different name than "localhost".
+  ;; We apply some heuristics here.
+  (define (get-ipv6-sock)
+    (any (lambda (name)
+           (guard (e (else #f))
+             (make-client-socket
+              (make <sockaddr-in6> :host name :port *inet-port*))))
+         '("localhost" "ip6-localhost" "ipv6-localhost" "::1")))
+  
+  (test* "inet client socket (ipv6)" #t
+         (and-let* ((sock (get-ipv6-sock)))
+           (call-with-client-socket sock
+             (lambda (in out)
+               (display (make-string *chunk-size* #\a) out)
+               (newline out)
+               (flush out)
+               (string=? (read-line in) (make-string *chunk-size* #\A))))))
+
+  (test* "inet client socket (ipv6, termination)" 33
+         (and-let* ((sock (get-ipv6-sock)))
+           (call-with-client-socket sock
+             (lambda (in out)
+               (display "END\n" out) (flush out)
+               (receive (pid code) (sys-wait)
+                 (sys-wait-exit-status code))))))
+  ]
+ [else])
+
+(test* "socket w/ port error handling" (test-error)
        (let1 s (make-client-socket 'inet "localhost" *inet-port*)
          (close-socket s)
          (socket-input-port s)))
 
-(test* "socket w/ port error handling" *test-error*
+(test* "socket w/ port error handling" (test-error)
        (let1 s (make-client-socket 'inet "localhost" *inet-port*)
          (close-socket s)
          (socket-output-port s)))
@@ -390,48 +442,97 @@
                  (socket-close serv))))
 
 (test* "udp server socket" #t
-       (let ((pid (sys-fork)))
-         (if (= pid 0)
-           (let ((sock (make-socket |PF_INET| |SOCK_DGRAM|))
-                 (addr (make <sockaddr-in> :host :any :port *inet-port*)))
-             (socket-setsockopt sock |SOL_SOCKET| |SO_REUSEADDR| 1)
-             (socket-bind sock addr)
-             (receive (msg from) (socket-recvfrom sock 1024)
-               (socket-sendto sock (string-upcase msg) from))
-             (socket-close sock)
-             (sys-exit 33))
-           (begin
-             (sys-select #f #f #f 300000)
-             #t))))
+       (begin
+         (with-output-to-file "testserv.o"
+           (lambda ()
+             (write '(use gauche.net))
+             (write '(use srfi-13))
+             (write
+              `(define (main args)
+                 (let ((sock (make-socket PF_INET SOCK_DGRAM))
+                       (addr (make <sockaddr-in> :host :any :port ,*inet-port*)))
+                   (socket-setsockopt sock SOL_SOCKET SO_REUSEADDR 1)
+                   (socket-bind sock addr)
+                   (newline) (flush) ;; handshake
+                   (receive (msg from) (socket-recvfrom sock 1024)
+                     (socket-sendto sock (string-upcase msg) from))
+                   (socket-close sock)
+                   (sys-exit 33))))))
+         (let1 p (run-process '("../../src/gosh" "-ftest" "./testserv.o")
+                              :output :pipe)
+           (read-line (process-output p)) ; handshake
+           #t)))
 
 (test* "udp client socket" "ABC"
-       (let ((sock (make-socket |PF_INET| |SOCK_DGRAM|))
+       (let ((sock (make-socket PF_INET SOCK_DGRAM))
              (addr (make <sockaddr-in> :host :loopback :port *inet-port*)))
-         (socket-setsockopt sock |SOL_SOCKET| |SO_REUSEADDR| 1)
-         (socket-connect sock addr)
-         (socket-send sock "abc")
+         ;; NB: Cygwin weirdness... Somehow the first udp packet
+         ;; is always dropped on CYGWIN_NT-5.1 1.5.25(0.156/4/2) on XP SP3,
+         ;; so we send it twice.  Curiously such symptom does not appear
+         ;; on Vista.
+         (cond-expand
+          [gauche.os.cygwin
+           (socket-sendto sock "abc" addr)
+           (sys-sleep 1)
+           (socket-sendto sock "abc" addr)]
+          [else
+           (socket-connect sock addr)
+           (socket-send sock "abc")])
          (begin0 (string-incomplete->complete (socket-recv sock 1024))
-                 (socket-close sock)
-                 (sys-wait))))
+           (socket-close sock)
+           (sys-wait))))
 
-(test* "udp uvector API" '(#t #t)
-       (let ((s-sock (make-socket |PF_INET| |SOCK_DGRAM|))
-             (r-sock (make-socket |PF_INET| |SOCK_DGRAM|))
-             (s-addr (make <sockaddr-in> :host :loopback :port *inet-port*))
-             (r-addr (make <sockaddr-in> :host :any :port *inet-port*))
-             (from   (make <sockaddr-in>))
-             (data   (make-u8vector 1024 #x3c))
-             (buf    (make-u8vector 1024 #xa5)))
-         (socket-setsockopt r-sock |SOL_SOCKET| |SO_REUSEADDR| 1)
-         (socket-bind r-sock r-addr)
-         (socket-sendto s-sock data s-addr)
-         (unwind-protect
-             (receive (size f-addr) (socket-recvfrom! r-sock buf (list from))
-               (list (eq? f-addr from)
-                     (equal? buf data)))
-           (begin
-             (socket-close r-sock)
-             (socket-close s-sock)))
-         ))
+(define (with-sr-udp proc)
+  (let ((s-sock (make-socket PF_INET SOCK_DGRAM))
+        (r-sock (make-socket PF_INET SOCK_DGRAM))
+        (s-addr (make <sockaddr-in> :host :loopback :port *inet-port*))
+        (r-addr (make <sockaddr-in> :host :any :port *inet-port*)))
+    (socket-setsockopt r-sock SOL_SOCKET SO_REUSEADDR 1)
+    (socket-bind r-sock r-addr)
+    (unwind-protect
+        (proc s-sock s-addr r-sock r-addr)
+      (begin (socket-close r-sock)
+             (socket-close s-sock)))))
+
+(with-sr-udp
+ (lambda (s-sock s-addr r-sock r-addr)
+   (let ([from (make <sockaddr-in>)]
+         [data (make-u8vector 1024 #x3c)]
+         [buf  (make-u8vector 1024 #xa5)])
+     (test* "udp uvector API" '(#t #t)
+            (begin
+              (socket-sendto s-sock data s-addr)
+              (receive (size f-addr) (socket-recvfrom! r-sock buf (list from))
+                (list (eq? f-addr from)
+                      (equal? buf data))))))))
+
+(cond-expand
+ ;; NB: as of 0.9, sendmsg fails on cygwin.  We don't have time to track
+ ;; it down yet.  For now, we skip the tests.
+ [(and (not gauche.os.cygwin)
+       (not gauche.os.windows))
+  (with-sr-udp
+   (lambda (s-sock s-addr r-sock r-addr)
+     (let ([from   (make <sockaddr-in>)]
+	   [data   `#(,(make-string 255 #\a) ,(make-u8vector 255 48))]
+	   [sbuf   (make-u8vector 1024)]
+	   [rbuf   (make-u8vector 1024 0)])
+
+       (define (xtest sbuf)
+	 (socket-sendmsg s-sock (socket-buildmsg r-addr data '() 0 sbuf))
+	 (receive (size f-addr) (socket-recvfrom! r-sock rbuf (list from))
+		  (list (eq? f-addr from)
+			(equal? (uvector-alias <u8vector> rbuf 0 size)
+				($ string->u8vector
+				   $ string-concatenate
+				   $ map (lambda (z)
+					   (if (string? z)
+					       z
+					       (u8vector->string z)))
+				   $ vector->list data)))))
+       
+       (test* "udp sendmsg w/sendbuf" '(#t #t) (xtest sbuf))
+       (test* "udp sendmsg w/o sendbuf" '(#t #t) (xtest #f)))))]
+ [else #f])
 
 (test-end)
