@@ -76,6 +76,23 @@
           (thread-join! t1)
           (reverse l))))
 
+;; thread stop and cont
+(let1 t1 (make-thread (lambda () (while #t (sys-nanosleep #e5e8))))
+  (test* "thread-status" 'new (thread-state t1))
+  (thread-start! t1)
+  (test* "thread-status" 'runnable (thread-state t1))
+  (thread-stop! t1)
+  (test* "thread-status" 'stopped (thread-state t1))
+  (thread-stop! t1) ; duplicate stop test
+  (test* "thread-status" 'stopped (thread-state t1))
+  (thread-cont! t1)
+  (test* "thread-status" 'runnable (thread-state t1))
+  (thread-terminate! t1)
+  (test* "thread-status" 'terminated 
+         (guard (e [(<terminated-thread-exception> e)
+                    (thread-state t1)])
+           (thread-join! t1))))
+
 ;;---------------------------------------------------------------------
 (test-section "thread and error")
 
@@ -448,7 +465,7 @@
          (list (p) *thr1-val* *thr2-val*)))
 
 (test* "check parameter identification"
-       *test-error*
+       (test-error)
        (let* ((local #f))
          (thread-join!
           (thread-start!
@@ -460,6 +477,58 @@
            (make-thread
             (lambda ()
               (local)))))))
-         
+
+;;---------------------------------------------------------------------
+(test-section "synchrnization by queues")
+
+;; These are actually for testing mtqueue, but put here since they
+;; require threads to work.
+
+(use util.queue)
+
+(define (test-producer-consumer name qs ndata nthreads)
+  (define qr (make-mtqueue))
+  (define data (iota ndata))
+  (define (producer)
+    (dolist [n data] (enqueue/wait! qs n))
+    (dotimes [k nthreads] (enqueue/wait! qs #f)))
+  (define (consumer)
+    (let loop ([x (dequeue/wait! qs)])
+      (when x
+        (enqueue! qr x)
+        (sys-nanosleep #e1e7)
+        (loop (dequeue/wait! qs)))))
+
+  (test* #`"synchronized queue ,name" data
+         (let* ([cs (map (^_ (thread-start! (make-thread consumer)))
+                         (iota nthreads))]
+                [p1 (make-thread producer)])
+           (sys-nanosleep #e1e8)
+           (thread-start! p1)
+           (for-each thread-join! cs)
+           (thread-join! p1)
+           (queue->list qr))
+         (cut lset= eqv? <> <>)))
+
+(test-producer-consumer "(unbound queue length)"
+                        (make-mtqueue)
+                        100 3)
+
+(test-producer-consumer "(bound queue length)"
+                        (make-mtqueue :max-length 5)
+                        100 3)
+
+
+(test* "dequeue/wait! timeout" "timed out!"
+       (dequeue/wait! (make-mtqueue) 0.01 "timed out!"))
+(test* "enqueue/wait! timeout" "timed out!"
+       (let1 q (make-mtqueue :max-length 1)
+         (enqueue! q 'a)
+         (enqueue/wait! q 'b 0.01 "timed out!")))
+(test* "queue-push/wait! timeout" "timed out!"
+       (let1 q (make-mtqueue :max-length 1)
+         (enqueue! q 'a)
+         (queue-push/wait! q 'b 0.01 "timed out!")))
+
 (test-end)
 
