@@ -3,6 +3,7 @@
 ;;
 
 (use gauche.test)
+(use gauche.parameter)
 (use srfi-1)
 
 (test-start "uniform vector and array")
@@ -247,6 +248,30 @@
                       '#f64(0.0 -1.0 1.0)
                       '(0.0 -1.0 1.0)))
 
+;;-------------------------------------------------------------------
+(test-section "comparison")
+
+(define (uvcomp-tester list-> samples)
+  (dolist (x samples)
+    (dolist (y samples)
+      (test* #`"equal? ,(list-> x) ,(list-> y)"
+             (and (= (length x) (length y))
+                  (every = x y))
+             (equal? (list-> x) (list-> y))))))
+
+(for-each (cut uvcomp-tester <> '(() (0) (1) (0 0) (0 1)))
+          (list list->s8vector list->u8vector
+                list->s16vector list->u16vector
+                list->s32vector list->u32vector
+                list->s64vector list->u64vector))
+
+(for-each (cut uvcomp-tester <> '(() (0) (1) (0 0) (0 1)
+                                  (+inf.0) (-inf.0) (+nan.0)
+                                  (0 +inf.0) (0 -inf.0) (0 +nan.0)))
+          (list list->f16vector
+                list->f32vector
+                list->f64vector))
+
 ;;-------------------------------------------------------------------
 (test-section "copying and filling")
 
@@ -1214,6 +1239,87 @@
 (clamp-test-generate u64 #u64(127 0 4 200 255)
                      #u64(3 3 3 3 3) #u64(199 199 199 199 199))
 
+;;-------------------------------------------------------------------
+(test-section "block i/o")
+
+(sys-unlink "test.o")
+
+(let* ([data1 '#u8(0 1 2 3 255 254 253 252 4 5 6 7 251 250 249 248)]
+       [data2 '#u8(1 0 3 2 254 255 252 253 5 4 7 6 250 251 248 249)]
+       [data3 '#u8(3 2 1 0 252 253 254 255 7 6 5 4 248 249 250 251)]
+       [data4 '#u8(252 253 254 255 3 2 1 0 248 249 250 251 7 6 5 4)]
+
+       [data5 '#u8(3 0 1 3 3 1 0 3 131 0 1 131 131 1 0 131)]
+       )
+
+  ;; Invaliance:
+  ;; w = (uvector-alias T u8v)  <--> w = read-block! T + write-block u8v
+  (define (test-default-endian data)
+    (lambda (T maker size)
+      (test* (format "native endian ~a" (class-name T))
+             (uvector-alias T data)
+             (rlet1 buf (maker size)
+               (call-with-output-file "test.o" (cut write-block data <>))
+               (call-with-input-file "test.o" (cut read-block! buf <>))))))
+
+  (define (test-reverse-endian T maker size)
+    (let* ([rev-endian (case (native-endian)
+                         [(big-endian) 'little-endian]
+                         [(little-endian arm-little-endian) 'big-endian])]
+           [rev-vec (cond
+                     [(memq T (list <u8vector> <s8vector>)) data1]
+                     [(memq T (list <u16vector> <s16vector>)) data2]
+                     [(memq T (list <u32vector> <s32vector>)) data3]
+                     [(memq T (list <u64vector> <s64vector>)) data4])])
+      (test* (format "reading ~a ~a" rev-endian (class-name T))
+             (uvector-alias T rev-vec)
+             (rlet1 buf (maker size)
+               (call-with-output-file "test.o" (cut write-block data1 <>))
+               (call-with-input-file "test.o"
+                 (cut read-block! buf <> 0 -1 rev-endian))))
+      (test* (format "reading ~a ~a, parameter" rev-endian (class-name T))
+             (uvector-alias T rev-vec)
+             (rlet1 buf (maker size)
+               (call-with-output-file "test.o" (cut write-block data1 <>))
+               (parameterize ((default-endian rev-endian))
+                 (call-with-input-file "test.o" (cut read-block! buf <>)))))
+      (test* (format "writing ~a ~a" rev-endian (class-name T))
+             (uvector-alias T data1)
+             (rlet1 buf (maker size)
+               (call-with-output-file "test.o"
+                 (cut write-block (uvector-alias T rev-vec) <> 0 -1 rev-endian))
+               (call-with-input-file "test.o" (cut read-block! buf <>))))
+      (test* (format "writing ~a ~a, parameter" rev-endian (class-name T))
+             (uvector-alias T data1)
+             (rlet1 buf (maker size)
+               (parameterize ((default-endian rev-endian))
+                 (call-with-output-file "test.o"
+                   (cut write-block (uvector-alias T rev-vec) <>)))
+               (call-with-input-file "test.o" (cut read-block! buf <>))))))
+
+  (define (run-across fn)
+    (let1 s (u8vector-length data1)
+      (fn <u8vector> make-u8vector s)
+      (fn <s8vector> make-s8vector s)
+      (fn <u16vector> make-u16vector (/ s 2))
+      (fn <s16vector> make-s16vector (/ s 2))
+      (fn <u32vector> make-u32vector (/ s 4))
+      (fn <s32vector> make-s32vector (/ s 4))
+      (fn <u64vector> make-u64vector (/ s 8))
+      (fn <s64vector> make-s64vector (/ s 8))
+      ))
+  (define (run-across-f fn)
+    (let1 s (u8vector-length data5)
+      (fn <f16vector> make-f16vector (/ s 2))
+      (fn <f32vector> make-f32vector (/ s 4))
+      (fn <f64vector> make-f64vector (/ s 8))
+      ))
+  
+  (run-across (test-default-endian data1))
+  (run-across-f (test-default-endian data5))
+  (run-across test-reverse-endian)
+  )
+
 ;;-------------------------------------------------------------------
 (test-section "string <-> uvector")
 
@@ -1225,7 +1331,7 @@
        (string->u8vector "@ABCD" 1 4))
 (test* "string->u8vector (start, end)" '#u8(64 65 66 67 68)
        (string->u8vector "@ABCD" 0 5))
-(test* "string->u8vector (OOB)" *test-error*
+(test* "string->u8vector (OOB)" (test-error)
        (string->u8vector "abcde" 2 6))
 
 (test* "string->u8vector!" '#u8(64 65 66 67 68)
@@ -1258,7 +1364,7 @@
        (u8vector->string '#u8(64 65 66 67 68) 2 4))
 (test* "u8vector->string (start, end)" "@ABCD"
        (u8vector->string '#u8(64 65 66 67 68) 0 5))
-(test* "u8vector->string (OOB)" *test-error*
+(test* "u8vector->string (OOB)" (test-error)
        (u8vector->string '#u8(64 65 66 67 68) 0 8))
 
 (test* "string->s8vector" '#s8(64 65 66 67 68)
@@ -1269,7 +1375,7 @@
        (string->s8vector "@ABCD" 1 4))
 (test* "string->s8vector (start, end)" '#s8(64 65 66 67 68)
        (string->s8vector "@ABCD" 0 5))
-(test* "string->s8vector (OOB)" *test-error*
+(test* "string->s8vector (OOB)" (test-error)
        (string->s8vector "abcde" 2 6))
 
 (test* "string->s8vector!" '#s8(64 65 66 67 68)
@@ -1302,7 +1408,7 @@
        (s8vector->string '#s8(64 65 66 67 68) 2 4))
 (test* "s8vector->string (start, end)" "@ABCD"
        (s8vector->string '#s8(64 65 66 67 68) 0 5))
-(test* "s8vector->string (OOB)" *test-error*
+(test* "s8vector->string (OOB)" (test-error)
        (s8vector->string '#s8(64 65 66 67 68) 0 8))
 
 (test* "string->u32vector" '#u32(64 65 66 67 68)
@@ -1313,7 +1419,7 @@
        (string->u32vector "@ABCD" 1 4))
 (test* "string->u32vector (start, end)" '#u32(64 65 66 67 68)
        (string->u32vector "@ABCD" 0 5))
-(test* "string->u32vector (OOB)" *test-error*
+(test* "string->u32vector (OOB)" (test-error)
        (string->u32vector "abcde" 2 6))
 
 (test* "u32vector->string" "@ABCD"
@@ -1324,7 +1430,7 @@
        (u32vector->string '#u32(64 65 66 67 68) 2 4))
 (test* "u32vector->string (start, end)" "@ABCD"
        (u32vector->string '#u32(64 65 66 67 68) 0 5))
-(test* "u32vector->string (OOB)" *test-error*
+(test* "u32vector->string (OOB)" (test-error)
        (u32vector->string '#u32(64 65 66 67 68) 0 8))
 
 (test* "string->s32vector" '#s32(64 65 66 67 68)
@@ -1335,7 +1441,7 @@
        (string->s32vector "@ABCD" 1 4))
 (test* "string->s32vector (start, end)" '#s32(64 65 66 67 68)
        (string->s32vector "@ABCD" 0 5))
-(test* "string->s32vector (OOB)" *test-error*
+(test* "string->s32vector (OOB)" (test-error)
        (string->s32vector "abcde" 2 6))
 
 (test* "s32vector->string" "@ABCD"
@@ -1346,7 +1452,7 @@
        (s32vector->string '#s32(64 65 66 67 68) 2 4))
 (test* "s32vector->string (start, end)" "@ABCD"
        (s32vector->string '#s32(64 65 66 67 68) 0 5))
-(test* "s32vector->string (OOB)" *test-error*
+(test* "s32vector->string (OOB)" (test-error)
        (s32vector->string '#s32(64 65 66 67 68) 0 8))
 
 ;; test for multibyte chars
@@ -1406,21 +1512,21 @@
                       4 8))
 
 ;; test alignment check
-(test* "alias u32 u8 (alignment violation)" *test-error*
+(test* "alias u32 u8 (alignment violation)" (test-error)
        (let* ((src (make-u8vector 9))
               (dst (uvector-alias <u32vector>)))
          dst))
-(test* "alias u32 u8 (alignment violation)" *test-error*
+(test* "alias u32 u8 (alignment violation)" (test-error)
        (let* ((src (make-u8vector 32))
               (dst (uvector-alias <u32vector> 2)))
          dst))
-(test* "alias u32 u8 (alignment violation)" *test-error*
+(test* "alias u32 u8 (alignment violation)" (test-error)
        (let* ((src (make-u8vector 32))
               (dst (uvector-alias <u32vector> 4 5)))
          dst))
 
 ;; test if immutable property propagates
-(test* "immutability violation" *test-error*
+(test* "immutability violation" (test-error)
        (let* ((src '#u8(0 1 2 3))
               (dst (uvector-alias <u8vector> src)))
          (u8vector-set! dst 0 1)))
@@ -1444,9 +1550,9 @@
             (shape 1 2 3 4 5 6 7 8 1 2 3 4 5 6 7 8 1 2 3 4 5 6 7 8)
             #t))
 
-(test* "shape" *test-error* (shape 1))
-(test* "shape" *test-error* (shape 1 2 3))
-(test* "shape" *test-error* (shape 3 1))
+(test* "shape" (test-error) (shape 1))
+(test* "shape" (test-error) (shape 1 2 3))
+(test* "shape" (test-error) (shape 3 1))
 
 (test* "make-array" #t
        (and (make-array (shape))
