@@ -1,7 +1,7 @@
 /*
  * uvectorP.h - internal macros for the uniform vector module
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: uvectorP.h,v 1.26 2007/03/22 11:20:25 shirok Exp $
  */
 
 #ifndef GAUCHE_UVECTOR_P_H
@@ -201,4 +199,15 @@ static inline ScmUInt64 range_u64lo(ScmUInt64 val, int clamp)
     return val;
 }
 
+/*
+ * 'option' argument for Scm_UVectorSwapBytes.
+ */
+
+enum {
+    SWAPB_STD,                  /* simple swapping */
+    SWAPB_ARM_LE,               /* arm-little-endian <-> little-endian */
+    SWAPB_ARM_BE                /* arm-little-endian <-> big-endian */
+};
+
+
 #endif /* GAUCHE_UVECTOR_P_H */
