@@ -1,8 +1,6 @@
 ;;
 ;; Generates uvect.c from uvect.c.tmpl
 ;;
-;; $Id: uvgen.scm,v 1.10 2007/09/13 12:30:26 shirok Exp $
-;;
 
 (use srfi-1)
 (use srfi-13)
@@ -133,7 +131,9 @@
    `(NUNBOX . ,(lambda (dst src clamp)
                 #`",dst = Scm_GetInteger8Clamp(,src, ,clamp, NULL)"))
    `(BOX   . ,(lambda (dst src) #`",dst = SCM_MAKE_INT(,src)"))
+   `(VMBOX . ,(lambda (dst src) #`",dst = SCM_MAKE_INT(,src)"))
    `(NBOX  . ,(lambda (dst src) #`",dst = Scm_MakeInteger(,src)"))
+   `(VMNBOX  . ,(lambda (dst src) #`",dst = Scm_MakeInteger(,src)"))
    `(EQ    . ,(lambda (x y)     #`"(,x == ,y)"))
    `(PRINT . ,(lambda (out elt) #`"Scm_Printf(,out, \"%d\", ,elt)"))
    (common-rules 's8)))
@@ -149,7 +149,9 @@
    `(NUNBOX . ,(lambda (dst src clamp)
                 #`",dst = Scm_GetIntegerU8Clamp(,src, ,clamp, NULL)"))
    `(BOX   . ,(lambda (dst src) #`",dst = SCM_MAKE_INT(,src)"))
+   `(VMBOX . ,(lambda (dst src) #`",dst = SCM_MAKE_INT(,src)"))
    `(NBOX  . ,(lambda (dst src) #`",dst = Scm_MakeIntegerU(,src)"))
+   `(VMNBOX  . ,(lambda (dst src) #`",dst = Scm_MakeIntegerU(,src)"))
    `(EQ    . ,(lambda (x y)     #`"(,x == ,y)"))
    `(PRINT . ,(lambda (out elt) #`"Scm_Printf(,out, \"%d\", ,elt)"))
    (common-rules 'u8)))
@@ -165,7 +167,9 @@
    `(NUNBOX . ,(lambda (dst src clamp)
                 #`",dst = Scm_GetInteger16Clamp(,src, ,clamp, NULL)"))
    `(BOX   . ,(lambda (dst src) #`",dst = SCM_MAKE_INT(,src)"))
+   `(VMBOX . ,(lambda (dst src) #`",dst = SCM_MAKE_INT(,src)"))
    `(NBOX  . ,(lambda (dst src) #`",dst = Scm_MakeInteger(,src)"))
+   `(VMNBOX  . ,(lambda (dst src) #`",dst = Scm_MakeInteger(,src)"))
    `(EQ    . ,(lambda (x y)     #`"(,x == ,y)"))
    `(PRINT . ,(lambda (out elt) #`"Scm_Printf(,out, \"%d\", ,elt)"))
    (common-rules 's16)))
@@ -181,7 +185,9 @@
    `(NUNBOX . ,(lambda (dst src clamp)
                 #`",dst = Scm_GetIntegerU16Clamp(,src, ,clamp, NULL)"))
    `(BOX   . ,(lambda (dst src) #`",dst = SCM_MAKE_INT(,src)"))
+   `(VMBOX . ,(lambda (dst src) #`",dst = SCM_MAKE_INT(,src)"))
    `(NBOX  . ,(lambda (dst src) #`",dst = Scm_MakeIntegerU(,src)"))
+   `(VMNBOX  . ,(lambda (dst src) #`",dst = Scm_MakeIntegerU(,src)"))
    `(EQ    . ,(lambda (x y)     #`"(,x == ,y)"))
    `(PRINT . ,(lambda (out elt) #`"Scm_Printf(,out, \"%d\", ,elt)"))
    (common-rules 'u16)))
@@ -197,7 +203,9 @@
    `(NUNBOX . ,(lambda (dst src clamp)
                 #`",dst = Scm_GetInteger32Clamp(,src,, ,clamp,, NULL)"))
    `(BOX   . ,(lambda (dst src) #`",dst = Scm_MakeInteger(,src)"))
+   `(VMBOX . ,(lambda (dst src) #`",dst = Scm_MakeInteger(,src)"))
    `(NBOX  . ,(lambda (dst src) #`",dst = Scm_MakeInteger(,src)"))
+   `(VMNBOX  . ,(lambda (dst src) #`",dst = Scm_MakeInteger(,src)"))
    `(EQ    . ,(lambda (x y)     #`"(,x == ,y)"))
    `(PRINT . ,(lambda (out elt) #`"Scm_Printf(,out,, \"%d\",, ,elt)"))
    (common-rules 's32)))
@@ -213,7 +221,9 @@
    `(NUNBOX . ,(lambda (dst src clamp)
                 #`",dst = Scm_GetIntegerU32Clamp(,src,, ,clamp,, NULL)"))
    `(BOX   . ,(lambda (dst src) #`",dst = Scm_MakeIntegerU(,src)"))
+   `(VMBOX . ,(lambda (dst src) #`",dst = Scm_MakeIntegerU(,src)"))
    `(NBOX  . ,(lambda (dst src) #`",dst = Scm_MakeIntegerU(,src)"))
+   `(VMNBOX  . ,(lambda (dst src) #`",dst = Scm_MakeIntegerU(,src)"))
    `(EQ    . ,(lambda (x y)     #`"(,x == ,y)"))
    `(PRINT . ,(lambda (out elt) #`"Scm_Printf(,out,, \"%u\",, ,elt)"))
    (common-rules 'u32)))
@@ -229,7 +239,9 @@
    `(NUNBOX . ,(lambda (dst src clamp)
                 #`",dst = Scm_GetInteger64Clamp(,src,, ,clamp,, NULL)"))
    `(BOX   . ,(lambda (dst src) #`",dst = Scm_MakeInteger64(,src)"))
+   `(VMBOX . ,(lambda (dst src) #`",dst = Scm_MakeInteger64(,src)"))
    `(NBOX  . ,(lambda (dst src) #`",dst = Scm_MakeInteger64(,src)"))
+   `(VMNBOX  . ,(lambda (dst src) #`",dst = Scm_MakeInteger64(,src)"))
    `(EQ    . ,(lambda (x y)     #`"int64eqv(,x,, ,y)"))
    `(PRINT . ,(lambda (out elt) #`"int64print(,out,, ,elt)"))
    (common-rules 's64)))
@@ -245,7 +257,9 @@
    `(NUNBOX . ,(lambda (dst src clamp)
                 #`",dst = Scm_GetIntegerU64Clamp(,src,, ,clamp,, NULL)"))
    `(BOX   . ,(lambda (dst src) #`",dst = Scm_MakeIntegerU64(,src)"))
+   `(VMBOX . ,(lambda (dst src) #`",dst = Scm_MakeIntegerU64(,src)"))
    `(NBOX  . ,(lambda (dst src) #`",dst = Scm_MakeIntegerU64(,src)"))
+   `(VMNBOX  . ,(lambda (dst src) #`",dst = Scm_MakeIntegerU64(,src)"))
    `(EQ    . ,(lambda (x y)     #`"uint64eqv(,x,, ,y)"))
    `(PRINT . ,(lambda (out elt) #`"uint64print(,out,, ,elt)"))
    (common-rules 'u64)))
@@ -261,8 +275,10 @@
    `(NUNBOX . ,(lambda (dst src clamp)
                 #`",dst = Scm_GetDouble(,src)"))
    `(BOX   . ,(lambda (dst src) #`",dst = Scm_MakeFlonum(Scm_HalfToDouble(,src))"))
+   `(VMBOX . ,(lambda (dst src) #`",dst = Scm_VMReturnFlonum(Scm_HalfToDouble(,src))"))
    `(NBOX  . ,(lambda (dst src) #`",dst = Scm_MakeFlonum(,src)"))
-   `(EQ    . ,(lambda (x y)     #`"(,x == ,y)"))
+   `(VMNBOX  . ,(lambda (dst src) #`",dst = Scm_VMReturnFlonum(,src)"))
+   `(EQ    . ,(lambda (x y)     #`"SCM_HALF_FLOAT_CMP(==,, ,x,, ,y)"))
    `(PRINT . ,(lambda (out elt) #`"Scm_PrintDouble(,out,, Scm_HalfToDouble(,elt),, 0)"))
    (common-rules 'f16)))
 
@@ -277,7 +293,9 @@
    `(NUNBOX . ,(lambda (dst src clamp)
                 #`",dst = Scm_GetDouble(,src)"))
    `(BOX   . ,(lambda (dst src) #`",dst = Scm_MakeFlonum((double),src)"))
+   `(VMBOX . ,(lambda (dst src) #`",dst = Scm_VMReturnFlonum((double),src)"))
    `(NBOX  . ,(lambda (dst src) #`",dst = Scm_MakeFlonum(,src)"))
+   `(VMNBOX  . ,(lambda (dst src) #`",dst = Scm_VMReturnFlonum(,src)"))
    `(EQ    . ,(lambda (x y)     #`"(,x == ,y)"))
    `(PRINT . ,(lambda (out elt) #`"Scm_PrintDouble(,out,, (double),elt,, 0)"))
    (common-rules 'f32)))
@@ -291,7 +309,9 @@
    `(UNBOX . ,(lambda (dst src clamp) #`",dst = Scm_GetDouble(,src)"))
    `(NUNBOX . ,(lambda (dst src clamp) #`",dst = Scm_GetDouble(,src)"))
    `(BOX   . ,(lambda (dst src)       #`",dst = Scm_MakeFlonum(,src)"))
+   `(VMBOX . ,(lambda (dst src)       #`",dst = Scm_VMReturnFlonum(,src)"))
    `(NBOX   . ,(lambda (dst src)      #`",dst = Scm_MakeFlonum(,src)"))
+   `(VMNBOX   . ,(lambda (dst src)      #`",dst = Scm_VMReturnFlonum(,src)"))
    `(EQ    . ,(lambda (x y)           #`"(,x == ,y)"))
    `(PRINT . ,(lambda (out elt)  #`"Scm_PrintDouble(,out,, (double),elt,, 0)"))
    (common-rules 'f64)))
@@ -427,9 +447,9 @@
     (let1 tag (string->symbol (assq-ref rule 't))
       (define (SWAPB)
         (case tag
-          ((s16 u16 f16) "swapb16")
-          ((s32 u32 f32) "swapb32")
-          ((s64 u64 f64) "swapb64")))
+          ((s16 u16 f16) "SWAP_2")
+          ((s32 u32 f32) "SWAP_4")
+          ((s64 u64 f64) "SWAP_8")))
       (unless (memq tag '(s8 u8))
         (for-each (cute substitute <> (list* `(SWAPB . ,SWAPB)
                                              rule))
