@@ -1,7 +1,7 @@
 /*
  * uvinit.c - initialize routine for uvector extension
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: uvinit.c,v 1.18 2007/09/15 07:23:39 shirok Exp $
  */
 
 #include <gauche.h>
@@ -74,6 +72,7 @@ static ScmObj read_uvector(ScmPort *port, const char *tag,
  * Initialization
  */
 extern void Scm_Init_uvlib(ScmModule *);
+extern void Scm_Init_uvutil();
 extern void Scm_Init_uvseq(void);
  
 SCM_EXTENSION_ENTRY void Scm_Init_libgauche_uvector(void)
@@ -96,6 +95,6 @@ SCM_EXTENSION_ENTRY void Scm_Init_libgauche_uvector(void)
     Scm_InitStaticClassWithMeta(&Scm_F64VectorClass, "<f64vector>", m, NULL, SCM_NIL, NULL, 0);
 
     Scm_Init_uvlib(m);
-    Scm_Init_uvseq();
+    Scm_Init_uvutil();
     Scm__InstallReadUvectorHook(read_uvector);
 }
