@@ -7,6 +7,7 @@
 (test-start "vport")
 (use gauche.vport)
 (use gauche.uvector)
+(use gauche.sequence)
 (use srfi-1)
 (use srfi-13)
 (use file.util)
@@ -187,23 +188,46 @@
       (close-input-port p)
       (list (equal? a b) (port-closed? src))))
 
-  (test* "vport.c"      '(#t #t) (test-biport "vport.c" #f))
-  (test* "vport_head.c" '(#t #t) (test-biport "vport_head.c" #f))
+  (test* "vport.c" '(#t #t) (test-biport "vport.c" #f))
+  (test* "vport.h" '(#t #t) (test-biport "vport.h" #f))
 
   (test* "vport.c (bufsize=100)"
          '(#t #t) (test-biport "vport.c" 100))
-  (test* "vport_head.c (bufsize=100)"
-         '(#t #t) (test-biport "vport_head.c" 100))
+  (test* "vport.h (bufsize=100)"
+         '(#t #t) (test-biport "vport.h" 100))
   (test* "vport.c (bufsize=65536)"
          '(#t #t) (test-biport "vport.c" 65536))
-  (test* "vport_head.c (bufsize=65536)"
-         '(#t #t) (test-biport "vport_head.c" 65536))
+  (test* "vport.h (bufsize=65536)"
+         '(#t #t) (test-biport "vport.h" 65536))
   (test* "vport.c (bufsize=1)"
          '(#t #t) (test-biport "vport.c" 1))
   (test* "vport.c (bufsize=0)"
          '(#t #t) (test-biport "vport.c" 0))
   )
 
+;; test with no seeker; pointed in WiLiKi:Gauche:Bugs
+(let ()
+  (define in
+    (let1 index 0
+      (make <buffered-input-port>
+        :fill (lambda (buf)
+                (for-each-with-index
+                 (lambda (i _)
+                   (u8vector-set! buf i (logand (+ index i) #xFF)))
+                 buf)
+                (let ((size (u8vector-length buf)))
+                  (inc! index size)
+                  size))
+        )))
+  (test* "buffered-input-port w/o seeker"
+         '(0 1 #f 2)
+         (let* ([a (read-byte in)]
+                [b (read-byte in)]
+                [s (port-seek in 5 SEEK_SET)] ;this shouldn't move the point
+                [c (read-byte in)])
+           (list a b s c)))
+  )
+
 ;;-----------------------------------------------------------
 (test-section "buffered-output-port")
 
@@ -226,16 +250,16 @@
             closed?)))
   
   (test* "vport.c"      '(#t #t) (test-boport "vport.c" #f))
-  (test* "vport_head.c" '(#t #t) (test-boport "vport_head.c" #f))
+  (test* "vport.h" '(#t #t) (test-boport "vport.h" #f))
 
   (test* "vport.c (bufsize=100)"
          '(#t #t) (test-boport "vport.c" 100))
-  (test* "vport_head.c (bufsize=100)"
-         '(#t #t) (test-boport "vport_head.c" 100))
+  (test* "vport.h (bufsize=100)"
+         '(#t #t) (test-boport "vport.h" 100))
   (test* "vport.c (bufsize=65536)"
          '(#t #t) (test-boport "vport.c" 65536))
-  (test* "vport_head.c (bufsize=65536)"
-         '(#t #t) (test-boport "vport_head.c" 65536))
+  (test* "vport.h (bufsize=65536)"
+         '(#t #t) (test-boport "vport.h" 65536))
   (test* "vport.c (bufsize=1)"
          '(#t #t) (test-boport "vport.c" 1))
   (test* "vport.c (bufsize=0)"
