@@ -1,7 +1,7 @@
 /*
  * vport.c - 'virtual port'
  *
- *   Copyright (c) 2004-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2004-2010  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,16 +29,13 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: vport.c,v 1.19 2007/09/15 04:00:24 shirok Exp $
  */
 
 #include <gauche.h>
 #include <gauche/class.h>
 #include <gauche/extend.h>
-#include <gauche/uvector.h>
 
-#include "gauche/vport.h"
+#include "vport.h"
 
 #undef close    /* windows black magic */
 
@@ -709,10 +706,10 @@ static ScmClassStaticSlotSpec boport_slots[] = {
 
 void Scm_Init_vportlib(ScmModule*);
 
-SCM_EXTENSION_ENTRY void Scm_Init_vport(void)
+SCM_EXTENSION_ENTRY void Scm_Init_gauche__vport(void)
 {
     ScmModule *mod;
-    SCM_INIT_EXTENSION(vport);
+    SCM_INIT_EXTENSION(gauche__vport);
     mod = SCM_FIND_MODULE("gauche.vport", SCM_FIND_MODULE_CREATE);
     
     Scm_InitStaticClass(&Scm_VirtualInputPortClass,
