@@ -1,7 +1,7 @@
 ;;;
 ;;; port.stk - stk compatibility interface
 ;;;  
-;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -29,10 +29,10 @@
 ;;;   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-;;;  
-;;;  $Id: stk.scm,v 1.3 2007/03/02 07:39:07 shirok Exp $
 ;;;
 
+#!no-fold-case
+
 (define-module compat.stk
   (use srfi-1)
   (use srfi-13)
@@ -94,9 +94,8 @@
   (string-contains str sub))
 (define string-lower string-downcase)
 (define string-upper string-upcase)
-(define (split-string str . args)
-  (let-optionals* args ((delim #[\s]))
-    (string-tokenize str (char-set-complement delim))))
+(define (split-string str :optional (delim #[\s]))
+  (string-tokenize str (char-set-complement delim)))
 
 ; vector-copy
 ; vector-resize
@@ -172,9 +171,9 @@
 (define getenv sys-getenv)
 (define setenv! sys-putenv)
 
-(define (file-is-readable? file)   (sys-access file |R_OK|))
-(define (file-is-writable? file)   (sys-access file |W_OK|))
-(define (file-is-executable? file) (sys-access file |X_OK|))
+(define (file-is-readable? file)   (sys-access file R_OK))
+(define (file-is-writable? file)   (sys-access file W_OK))
+(define (file-is-executable? file) (sys-access file X_OK))
 
 (define glob sys-glob)
 (define remove-file sys-unlink)
@@ -242,5 +241,4 @@
 (define posix-host-name sys-gethostname)
 (define posix-domain-name sys-getdomainname)
 
-(provide "compat/stk")
 
