@@ -31,12 +31,6 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;
-;;;  $Id: dbi.scm,v 1.38 2007/01/21 14:21:50 rui314159 Exp $
-;;;
-
-;;; *EXPERIMENTAL*
-;;; This module provides an abstract interface to various database
-;;; management systems.  
 
 (define-module dbi
   (use text.sql)
@@ -129,8 +123,8 @@
 ;; The default method uses text.sql to parse the SQL statement, and calls
 ;; legacy dbd API.  This will go away once the drivers switched to the
 ;; new dbd API.
-(define-method dbi-prepare ((c <dbi-connection>) (sql <string>) . options)
-  (let-keywords options ((pass-through #f))
+(define-method dbi-prepare ((c <dbi-connection>) (sql <string>) . args)
+  (let-keywords args ((pass-through #f))
     (let1 prepared (if pass-through
                      (lambda args
                        (unless (null? args)
@@ -139,9 +133,7 @@
                                 sql))
                        sql)
                      (dbi-prepare-sql c sql))
-      (make <dbi-query>
-        :connection c
-        :prepared prepared))))
+      (make <dbi-query> :connection c :prepared prepared))))
 
 (define-method dbi-execute ((q <dbi-query>) . params)
   (dbi-execute-using-connection (ref q 'connection) q params))
@@ -180,13 +172,11 @@
 ;; Subclass SHOULD implement this.
 (define-method dbi-make-connection ((d <dbi-driver>)
                                     (options <string>)
-                                    (option-alist <list>)
-                                    . args)
+                                    (option-alist <list>) . args)
   ;; The default method here is just a temporary one to use
   ;; older dbd drivers.  Will go away once the drivers catch up
   ;; the new interface.
-  (let-keywords args ((username "")
-                      (password ""))
+  (let-keywords args ((username "") (password ""))
     ;; call deprecated dbi-make-connection API.
     (dbi-make-connection d username password (or options ""))))
 
@@ -338,4 +328,3 @@
 (define-method dbi-execute-query ((q <dbi-query>) (s <string>))
   (dbi-do (ref q 'connection) s '(:pass-through #t)))
 
-(provide "dbi")
