@@ -1,7 +1,7 @@
 ;;;
 ;;; fsdbm - dbm on filesystem
 ;;;  
-;;;   Copyright (c) 2003-2007  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2003-2010  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,8 +30,6 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: fsdbm.scm,v 1.8 2007/03/02 07:39:08 shirok Exp $
-;;;
 
 (define-module dbm.fsdbm
   (extend dbm)
@@ -99,14 +97,14 @@
         (rwmode  (ref self 'rw-mode))
         (fmode   (ref self 'file-mode)))
     (case rwmode
-      ((:read)
+      [(:read)
        (unless (fsdbm-directory? path)
-         (errorf "dbm-open: no fsdbm database ~a" path)))
-      ((:write)
+         (errorf "dbm-open: no fsdbm database ~a" path))]
+      [(:write)
        (unless (fsdbm-directory? path)
-         (fsdbm-create path fmode)))
-      ((:create)
-       (fsdbm-create path fmode))
+         (fsdbm-create path fmode))]
+      [(:create)
+       (fsdbm-create path fmode)]
       )
     self))
 
@@ -178,9 +176,21 @@
 
 (define-method dbm-db-remove ((class <fsdbm-meta>) name)
   (unless (fsdbm-directory? name)
-    (errorf "given path is not an fsdbm database: ~a" name))
+    (error "given path is not an fsdbm database:" name))
   (remove-directory* name))
 
+(define-method dbm-db-copy ((class <fsdbm-meta>) from to)
+  (unless (fsdbm-directory? from)
+    (error "source path is not an fsdbm database:" from))
+  (copy-directory* from to))
+
+(define-method dbm-db-move ((class <fsdbm-meta>) from to)
+  (unless (fsdbm-directory? from)
+    (error "source path is not an fsdbm database:" from))
+  ;; NB: move-directory*
+  (copy-directory* from to)
+  (remove-directory* from))
+
 ;;
 ;; Internal utilities
 ;;
@@ -216,40 +226,39 @@
       (write-char #\_) ;; always emit one char, so we can deal with null key
       (let loop ((c (read-byte))
                  (count 0))
-        (cond ((eof-object? c))
-              ((>= count *file-name-limit*)
-               (write-char #\/) (loop c 0))
-              ((and (< c 127)
+        (cond [(eof-object? c)]
+              [(>= count *file-name-limit*)
+               (write-char #\/) (loop c 0)]
+              [(and (< c 127)
                     (char-set-contains? #[0-9a-zA-Z] (integer->char c)))
-               (write-byte c) (loop (read-byte) (+ count 1)))
-              (else
+               (write-byte c) (loop (read-byte) (+ count 1))]
+              [else
                (format #t "_~2,'0x" c)
-               (loop (read-byte) (+ count 1))))))))
+               (loop (read-byte) (+ count 1))])))))
 
 ;; reverse fn of key->path.  returns #f is path is invalid.
 (define (path->key path)
-  (call/cc
-   (lambda (return)
-     (string-incomplete->complete
-      (with-string-io path
-        (lambda ()
-          ;; skip the first char
-          (unless (eqv? (read-char) #\_) (return #f))
-          (let loop ((c (read-char)))
-            (cond ((eof-object? c))
-                  ((char=? c #\/) (loop (read-char)))
-                  ((char=? c #\_)
-                   (let* ((c1 (read-char))
-                          (c2 (read-char)))
-                     (when (or (eof-object? c1) (eof-object? c2)) (return #f))
-                     (unless (and (char-set-contains? #[[:xdigit:]] c1)
-                                  (char-set-contains? #[[:xdigit:]] c2))
-                       (return #f))
-                     (write-byte (+ (* (digit->integer c1 16) 16)
-                                    (digit->integer c2 16)))
-                     (loop (read-char))))
-                  (else
-                   (write-char c) (loop (read-char)))))))))))
+  (let/cc return
+    (string-incomplete->complete
+     (with-string-io path
+       (lambda ()
+         ;; skip the first char
+         (unless (eqv? (read-char) #\_) (return #f))
+         (let loop ((c (read-char)))
+           (cond [(eof-object? c)]
+                 [(char=? c #\/) (loop (read-char))]
+                 [(char=? c #\_)
+                  (let* ((c1 (read-char))
+                         (c2 (read-char)))
+                    (when (or (eof-object? c1) (eof-object? c2)) (return #f))
+                    (unless (and (char-set-contains? #[[:xdigit:]] c1)
+                                 (char-set-contains? #[[:xdigit:]] c2))
+                      (return #f))
+                    (write-byte (+ (* (digit->integer c1 16) 16)
+                                   (digit->integer c2 16)))
+                    (loop (read-char)))]
+                 [else
+                  (write-char c) (loop (read-char))])))))))
 
 (define (path->hash path)
   ;; NB: we use our own hash fn to keep backward compatibility.
@@ -265,9 +274,8 @@
 
 (define (value-file-path key . maybe-dir)
   (let1 p (key->path key)
-    (cond ((get-optional maybe-dir #f)
-           => (cut build-path <> (path->hash p) p))
-          (else (build-path (path->hash p) p)))))
+    (cond [(get-optional maybe-dir #f)
+           => (cut build-path <> (path->hash p) p)]
+          [else (build-path (path->hash p) p)])))
 
-(provide "dbm/fsdbm")
 
