@@ -1,7 +1,7 @@
 ;;;
 ;;; file/filter.scm - utility to build filter programs
 ;;;  
-;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,8 +30,6 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: filter.scm,v 1.7 2007/03/02 07:39:08 shirok Exp $
-;;;
 
 ;;; This module provides utilities for a common pattern in
 ;;; filter-type commands, that is, to take an input, to process
@@ -52,61 +50,58 @@
   )
 (select-module file.filter)
 
-(define (file-filter proc . options)
-  (let-keywords options
-      ((input  (current-input-port))
-       (output (current-output-port))
-       (temporary-file #f)
-       (keep-output?   #f))
-
-    (define (process-with-output oport)
-      (cond
-       ((input-port? input) (proc input oport))
-       ((string? input)
-        (call-with-input-file input (lambda (iport) (proc iport oport))))
-       (else
-        (error "input must be either an input port or a file name, but got"
-               input))))
-
-    (define (process-with-tempfile ofile)
-      (let*-values (((tempfile) (cond ((string-prefix? "/" temporary-file)
-                                       temporary-file)
-                                      ((string-prefix? "./" temporary-file)
-                                       temporary-file)
-                                      ((string-prefix? "../" temporary-file)
-                                       temporary-file)
-                                      (else (string-append
-                                             (sys-dirname ofile)
-                                             "/"
-                                             temporary-file))))
-                    ((tport tfile) (sys-mkstemp tempfile)))
-        (guard (e
-                (else 
-                 (unless keep-output? (sys-unlink tfile))
-                 (raise e)))
-          (receive r (process-with-output tport)
-            (close-output-port tport)
-            (sys-rename tfile ofile)
-            (apply values r)))))
+(define (file-filter proc :key
+                     (input (current-input-port))
+                     (output (current-output-port))
+                     (temporary-file #f)
+                     (keep-output? #f))
 
+  (define (process-with-output oport)
     (cond
-     ((output-port? output) (process-with-output output))
-     ((string? output)
-      (if temporary-file
-          (process-with-tempfile output)
-          (with-error-handler
-           (lambda (e)
-             (unless keep-output? (sys-unlink output))
-             (raise e))
-           (lambda ()
-             (call-with-output-file output process-with-output)))))
+     ((input-port? input) (proc input oport))
+     ((string? input)
+      (call-with-input-file input (lambda (iport) (proc iport oport))))
      (else
-      (error "output must be either an output port or a file name, but got"
-             output)))
-    )
+      (error "input must be either an input port or a file name, but got"
+             input))))
+
+  (define (process-with-tempfile ofile)
+    (let*-values (((tempfile) (cond ((string-prefix? "/" temporary-file)
+                                     temporary-file)
+                                    ((string-prefix? "./" temporary-file)
+                                     temporary-file)
+                                    ((string-prefix? "../" temporary-file)
+                                     temporary-file)
+                                    (else (string-append
+                                           (sys-dirname ofile)
+                                           "/"
+                                           temporary-file))))
+                  ((tport tfile) (sys-mkstemp tempfile)))
+      (guard (e
+              (else 
+               (unless keep-output? (sys-unlink tfile))
+               (raise e)))
+        (receive r (process-with-output tport)
+          (close-output-port tport)
+          (sys-rename tfile ofile)
+          (apply values r)))))
+
+  (cond
+   ((output-port? output) (process-with-output output))
+   ((string? output)
+    (if temporary-file
+      (process-with-tempfile output)
+      (with-error-handler
+          (lambda (e)
+            (unless keep-output? (sys-unlink output))
+            (raise e))
+        (lambda ()
+          (call-with-output-file output process-with-output)))))
+   (else
+    (error "output must be either an output port or a file name, but got"
+           output)))
   )
 
-(provide "file/filter")
 
 
 
