@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.cgen - support for C code generation
 ;;;  
-;;;   Copyright (c) 2004-2007  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2010  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,11 +30,9 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: cgen.scm,v 1.20 2007/04/23 11:18:31 shirok Exp $
-;;;
 
 ;; *EXPERIMENTAL*
-;; gauche.cgen.* modules is intended to provide a common framework
+;; gauche.cgen.* modules are intended to provide a common framework
 ;; to generate C code from Scheme.  They will be used for stub
 ;; generators or any other kind of automatic code generation.
 
@@ -43,7 +41,7 @@
 ;; any extension modules.  Keep this in mind if you make modifications here.
 
 (define-module gauche.cgen
-  (extend gauche.cgen.unit gauche.cgen.literal)
+  (extend gauche.cgen.unit gauche.cgen.literal gauche.cgen.type)
   )
 (select-module gauche.cgen)
 
@@ -116,7 +114,7 @@
 ;;     - #include <gauche.h>
 ;;     - declarations of static array ;; scm__staticObjs etc.  not yet
 ;;                                ;; defined, but decls are put here
-;;                                ;; to allow foward reference.
+;;                                ;; to allow forward reference.
 ;;     - <decl part>
 ;;     - definitions of static array
 ;;     - <body part>
@@ -131,4 +129,3 @@
 ;;   and the opening curly brace) and the epilogue (including the closing
 ;;   curly brace).
 
-(provide "gauche/cgen")
