@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.cgen.literal - static literal data
 ;;;  
-;;;   Copyright (c) 2004-2007  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2010  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,8 +30,6 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: literal.scm,v 1.6 2007/09/13 12:30:27 shirok Exp $
-;;;
 
 (define-module gauche.cgen.literal
   (use srfi-1)
@@ -40,35 +38,18 @@
   (use gauche.parameter)
   (use gauche.sequence)
   (use gauche.cgen.unit)
+  (use gauche.experimental.ref)
+  (use gauche.experimental.lamb)
+  (use gauche.experimental.app)
+  (use util.match)
   (export <cgen-literal> cgen-c-name cgen-cexpr cgen-make-literal
           cgen-literal-static?
-
           define-cgen-literal cgen-literal
-          cgen-literal-emit-extern cgen-literal-emit-decl
-          cgen-literal-emit-body cgen-literal-emit-init
-
           cgen-allocate-static-datum
           )
   )
 (select-module gauche.cgen.literal)
 
-;; NB: a small experiment to see how I feel this...
-;;  [@ a b c d] => (ref (ref (ref a b) c) d)
-;; In string interpolations I have to use ,(@ ...) instead of ,[@ ...], for
-;; the previous versions of interpolation code doesn't like #`",[...]".
-;; Ideally this should be a compiler-macro (we can't make it a macro,
-;; for we want to say (set! [@ x'y] val).
-(define @
-  (getter-with-setter
-   (case-lambda
-     ((obj selector) (ref obj selector))
-     ((obj selector . more) (apply @ (ref obj selector) more)))
-   (case-lambda
-     ((obj selector val) ((setter ref) obj selector val))
-     ((obj selector selector2 . rest)
-      (apply (setter ref) (ref obj selector) selector2 rest)))))
-;; end experiment
-
 ;;=============================================================
 ;; Static objects
 ;;
@@ -118,45 +99,41 @@
 
 (define (static-data-c-struct-name category)
   (case category
-    ((constant) "scm__sc")
-    ((runtime)  "scm__rc")
-    (else (error "[cgen internal] invalid category:" category))))
+    [(constant) "scm__sc"]
+    [(runtime)  "scm__rc"]
+    [else (error "[cgen internal] invalid category:" category)]))
 
-(define (cgen-allocate-static-datum . opts)
+(define (cgen-allocate-static-datum :optional (category 'runtime)
+                                              (c-type   'ScmObj)
+                                              (init-thunk #f))
 
   (define (ensure-static-data-list category c-type)
-    (and-let* ((unit (cgen-current-unit)))
-      (let* ((cppc (cgen-cpp-condition))
-             (dl   (find (lambda (dl)
-                           (and (eq? [@ dl'c-type] c-type)
-                                (eq? [@ dl'category] category)
-                                (equal? [@ dl'cpp-condition] cppc)))
-                         [@ unit'static-data-list])))
+    (and-let* ([unit (cgen-current-unit)])
+      (let* ([cppc (cgen-cpp-condition)]
+             [dl   (find (^(dl) (and (eq? (~ dl'c-type) c-type)
+                                     (eq? (~ dl'category) category)
+                                     (equal? (~ dl'cpp-condition) cppc)))
+                         (~ unit'static-data-list))])
         (or dl
-            (let1 new (make <cgen-static-data-list>
-                        :category category :c-type c-type)
-              (push! [@ unit'static-data-list] new)
-              new)))))
+            (rlet1 new (make <cgen-static-data-list>
+                         :category category :c-type c-type)
+              (push! (~ unit'static-data-list) new))))))
   
-  (let-optionals* opts ((category 'runtime)
-                        (c-type   'ScmObj)
-                        (init-thunk #f))
-    (let ((dl (ensure-static-data-list category c-type))
-          (value-type? (not init-thunk))
-          (ithunk (or init-thunk
-                      (if (eq? c-type 'ScmObj) "SCM_UNBOUND" "NULL"))))
-      (let1 count [@ dl'count]
-        (slot-push! dl'init-thunks ithunk)
-        (inc! [@ dl'count])
-        (if value-type?
-          (format "~a.~a[~a]" ; no cast, for this'll be also used as lvalue.
-                  (static-data-c-struct-name category)
-                  [@ dl'c-member-name]
-                  count)
-          (format "SCM_OBJ(&~a.~a[~a])"
-                  (static-data-c-struct-name category)
-                  [@ dl'c-member-name]
-                  count))))))
+  (let ([dl (ensure-static-data-list category c-type)]
+        [value-type? (not init-thunk)]
+        [ithunk (or init-thunk (if (eq? c-type 'ScmObj) "SCM_UNBOUND" "NULL"))])
+    (let1 count (~ dl'count)
+      (slot-push! dl'init-thunks ithunk)
+      (inc! (~ dl'count))
+      (if value-type?
+        (format "~a.~a[~a]" ; no cast, for this'll be also used as lvalue.
+                (static-data-c-struct-name category)
+                (~ dl'c-member-name)
+                count)
+        (format "SCM_OBJ(&~a.~a[~a])"
+                (static-data-c-struct-name category)
+                (~ dl'c-member-name)
+                count)))))
 
 (define (cgen-allocate-static-array category c-type init-thunks)
   (fold (lambda (init-thunk seed)
@@ -167,7 +144,7 @@
 (define-method cgen-emit-static-data ((unit <cgen-unit>))
 
   (define (emit-one-category category dls)
-    (let1 dls (filter (lambda (dl) (eq? [@ dl'category] category)) dls)
+    (let1 dls (filter (^(dl) (eq? (~ dl'category) category)) dls)
       (unless (null? dls)
         (emit-struct-def category dls)
         (print "{")
@@ -179,24 +156,32 @@
       (format #t "static ~astruct ~aRec {\n"
               (if (eq? category 'constant) "SCM_CGEN_CONST " "")
               name)
-      (dolist (dl dls)
-        (cond ([@ dl'cpp-condition] => (cut print "#if "<>)))
-        (format #t "  ~a ~a[~a];\n" [@ dl'c-type] [@ dl'c-member-name]
-                [@ dl'count])
-        (cond ([@ dl'cpp-condition] => (cut print "#endif /*"<>"*/"))))
+      (dolist [dl dls]
+        (cond [(~ dl'cpp-condition) => (cut print "#if "<>)])
+        (format #t "  ~a ~a[~a];\n" (~ dl'c-type) (~ dl'c-member-name)
+                (~ dl'count))
+        (cond [(~ dl'cpp-condition) => (cut print "#endif /*"<>"*/")]))
       (format #t "} ~a = " name)))
 
   (define (emit-initializers dl)
-    (cond ([@ dl'cpp-condition] => (cut print "#if "<>)))
-    (print "  {   /* "[@ dl'c-type]" "[@ dl'c-member-name]" */")
-    (dolist (thunk (reverse [@ dl'init-thunks]))
+    (cond [(~ dl'cpp-condition) => (cut print "#if "<>)])
+    (print "  {   /* "(~ dl'c-type)" "(~ dl'c-member-name)" */")
+    (dolist [thunk (reverse (~ dl'init-thunks))]
       (if (string? thunk)
         (format #t "    ~a,\n" thunk)
         (begin (format #t "    ") (thunk) (print ","))))
     (print "  },")
-    (cond ([@ dl'cpp-condition] => (cut print "#endif /*"<>"*/"))))
+    (cond [(~ dl'cpp-condition) => (cut print "#endif /*"<>"*/")]))
 
-  (and-let* ((dls [@ unit'static-data-list]))
+  (and-let* ([dls (~ unit'static-data-list)])
+    (unless (null? dls)
+      ;; This piece of code is required, for Win32 DLL doesn't like
+      ;; structures to be const if it contains SCM_CLASS_PTR.  Doh!
+      (print "#if defined(__CYGWIN__) || defined(GAUCHE_WINDOWS)")
+      (print "#define SCM_CGEN_CONST /*empty*/")
+      (print "#else")
+      (print "#define SCM_CGEN_CONST const")
+      (print "#endif"))
     (emit-one-category 'constant dls)
     (emit-one-category 'runtime dls)
     ))
@@ -257,31 +242,34 @@
    ;;  It may have a thunk that computes the expression.
    (value  :init-keyword :value :init-value #f)
    ;; VALUE: the Scheme value this literal represents.
+
+   (inferred-literal-types :allocation :class :init-value '())
    ))
 
 (define-method initialize ((node <cgen-literal>) initargs)
   (next-method)
-  (when [@ node'c-name]
-    (and-let* ((unit (cgen-current-unit)))
+  (when (~ node'c-name)
+    (and-let* ([unit (cgen-current-unit)])
       (register-literal-value unit node)
       (slot-push! unit 'toplevels node))))
 
 ;; Fallback methods
 ;;
 (define-method cgen-c-name ((node <cgen-literal>))
-  (and-let* ((n [@ node'c-name]))
+  (and-let* ([n (~ node'c-name)])
     (if (string? n) n (n))))
 
 (define-method cgen-cexpr ((node <cgen-literal>))
   (cgen-c-name node))
 
 (define-method cgen-make-literal (value)
-  (error "cannot make a static C data for Scheme value:" value))
+  (or (infer-literal-handler value)
+      (error "cannot make a static C data for Scheme value:" value)))
 
 (define-method cgen-literal-static? (self) #t)
 
-(define-method cgen-emit-xtrn ((self <cgen-literal>))
-  (when (and [@ self'extern?] (cgen-c-name node))
+(define-method cgen-emit-xtrn ((node <cgen-literal>))
+  (when (and (~ node'extern?) (cgen-c-name node))
     (print "extern ScmObj " (cgen-c-name node) ";")))
 
 ;; define-cgen-literal macro
@@ -378,58 +366,53 @@
   (define mask #x0fffffff)
   (define (rec val)
     (cond
-     ((pair? val) (logand (+ (rec (car val)) (rec (cdr val))) mask))
-     ((vector? val)
-      (fold (lambda (v r) (logand (+ (rec v) r) mask)) 0 val))
-     ((string? val)
-      (logand (string-hash val) mask))
-     ((identifier? val)
-      (logand (+ (rec [@ val'name]) (rec [@ val'module])) mask))
-     (else (eqv-hash val))))
+     [(pair? val) (logand (+ (rec (car val)) (rec (cdr val))) mask)]
+     [(vector? val) (fold (^(v r) (logand (+ (rec v) r) mask)) 0 val)]
+     [(string? val) (logand (string-hash val) mask)]
+     [(identifier? val) (logand (+ (rec (~ val'name))(rec (~ val'module))) mask)]
+     [else (eqv-hash val)]))
   (modulo (rec literal) .literal-hash-size.))
 
 (define (literal-value=? x y)
   (define (rec x y)
     (cond
-     ((pair? x) (and (pair? y) (rec (car x) (car y)) (rec (cdr x) (cdr y))))
-     ((vector? x)
+     [(pair? x) (and (pair? y) (rec (car x) (car y)) (rec (cdr x) (cdr y)))]
+     [(vector? x)
       (and (vector? y)
            (let1 len (vector-length x)
              (and (= len (vector-length y))
                   (every?-ec (: i len)
-                             (rec (vector-ref x i) (vector-ref y i)))))))
-     ((string? x) (and (string? y) (string=? x y)))
-     ((identifier? x)
+                             (rec (vector-ref x i) (vector-ref y i))))))]
+     [(string? x) (and (string? y) (string=? x y))]
+     [(identifier? x)
       (and (identifier? y)
-           (eq? [@ x'name] [@ y'name])
-           (eq? [@ x'module] [@ y'module])))
-     (else (and (eq? (class-of x) (class-of y)) (eqv? x y)))))
+           (eq? (~ x'name) (~ y'name))
+           (eq? (~ x'module) (~ y'module)))]
+     [else (and (eq? (class-of x) (class-of y)) (eqv? x y))]))
   (rec x y))
 
 (define (ensure-literal-hash unit)
-  (or [@ unit'literals]
-      (let1 hash (make-vector .literal-hash-size. '())
-        (set! [@ unit'literals] hash)
-        hash)))
+  (or (~ unit'literals)
+      (rlet1 hash (make-vector .literal-hash-size. '())
+        (set! (~ unit'literals) hash))))
 
 (define (register-literal-value unit literal-obj)
-  (let ((lh   (ensure-literal-hash unit))
-        (cppc [@ literal-obj'cpp-condition])
-        (h    (literal-value-hash [@ literal-obj'value])))
-    (or (and-let* ((entry (find (lambda (e)
-                                  (and (equal? (caar e) cppc)
-                                       (literal-value=? [@ literal-obj'value]  (cdar e))))
-                                (vector-ref lh h))))
+  (let ([lh   (ensure-literal-hash unit)]
+        [cppc (~ literal-obj'cpp-condition)]
+        [h    (literal-value-hash (~ literal-obj'value))])
+    (or (and-let* ([entry (find (^(e) (and (equal? (caar e) cppc)
+                                           (literal-value=? (~ literal-obj'value)
+                                                            (cdar e))))
+                                (vector-ref lh h))])
           (set-cdr! entry literal-obj))
-        (push! (vector-ref lh h) (acons cppc [@ literal-obj'value] literal-obj)))))
+        (push! (vector-ref lh h) (acons cppc (~ literal-obj'value) literal-obj)))))
 
 (define (lookup-literal-value unit val)
-  (let ((lh (ensure-literal-hash unit))
-        (cppc (cgen-cpp-condition)))
-    (and-let* ((entry (find (lambda (e)
-                              (and (equal? (caar e) cppc)
-                                   (literal-value=? val (cdar e))))
-                            (vector-ref lh (literal-value-hash val)))))
+  (let ([lh (ensure-literal-hash unit)]
+        [cppc (cgen-cpp-condition)])
+    (and-let* ([entry (find (^(e) (and (equal? (caar e) cppc)
+                                       (literal-value=? val (cdar e))))
+                            (vector-ref lh (literal-value-hash val)))])
       (cdr entry))))
 
 ;; primitive values -------------------------------------------
@@ -440,7 +423,7 @@
   (make (value)
     (if value *cgen-scheme-true* *cgen-scheme-false*))
   (cexpr (self)
-    (if [@ self'value] "SCM_TRUE" "SCM_FALSE")))
+    (if (~ self'value) "SCM_TRUE" "SCM_FALSE")))
 
 (define *cgen-scheme-true*
   (make <cgen-scheme-boolean> :c-name #f :value #t))
@@ -453,7 +436,7 @@
   (make (value)
     (make <cgen-scheme-char> :c-name #f :value value))
   (cexpr (self)
-    (format "SCM_MAKE_CHAR(~a)" (char->integer [@ self'value]))))
+    (format "SCM_MAKE_CHAR(~a)" (char->integer (~ self'value)))))
 
 ;; ()
 (define-cgen-literal <cgen-scheme-null> <null>
@@ -485,11 +468,25 @@
     (make <cgen-scheme-string>
       :c-name (cgen-allocate-static-datum
                'constant 'ScmString
-               (format "  SCM_STRING_CONST_INITIALIZER(~s, ~a, ~a)"
-                       value (string-size value) (string-length value)))
+               (format "  SCM_STRING_CONST_INITIALIZER(~a, ~a, ~a)"
+                       (c-safe-string-literal value)
+                       (string-size value) (string-length value)))
       :value value))
   )
 
+(define (c-safe-string-literal value)
+  (with-string-io value
+    (lambda ()
+      (display "\"")
+      (port-for-each (lambda (b)
+                       (if (or (= #x20 b) (= #x21 b) ; #x22 = #\"
+                               (<= #x23 b #x5b)      ; #x5c = #\\
+                               (<= #x5d b #x7e))
+                         (write-byte b)
+                         (format #t "\\~3,'0o" b)))
+                     read-byte)
+      (display "\""))))
+
 ;; symbol ------------------------------------------------------
 
 (define-cgen-literal <cgen-scheme-symbol> <symbol>
@@ -501,8 +498,8 @@
   (init (self)
     (print "  " (cgen-c-name self)
            " = Scm_Intern(SCM_STRING("
-           (cgen-cexpr [@ self'symbol-name])
-           "));"))
+           (cgen-cexpr (~ self'symbol-name))
+           ")); /* "(cgen-safe-comment (~ self'value))" */"))
   (static (self) #f)
   )
 
@@ -517,8 +514,8 @@
   (init (self)
     (print "  " (cgen-c-name self)
            " = Scm_MakeKeyword(SCM_STRING("
-           (cgen-cexpr [@ self'keyword-name])
-           "));"))
+           (cgen-cexpr (~ self'keyword-name))
+           ")); /* "(cgen-safe-comment (~ self'value))" */"))
   (static (self) #f)
   )
 
@@ -531,34 +528,34 @@
    )
   (make (value)
     (cond
-     ((fixnum? value)
-      (make <cgen-scheme-integer> :value value :c-name #f))
-     ((< (- (expt 2 31)) value (- (expt 2 32)))
+     [(fixnum? value)
+      (make <cgen-scheme-integer> :value value :c-name #f)]
+     [(< (- (expt 2 31)) value (- (expt 2 32)))
       (make <cgen-scheme-integer> :value value
-            :c-name (cgen-allocate-static-datum)))
-     (else
+            :c-name (cgen-allocate-static-datum))]
+     [else
       (make <cgen-scheme-integer> :value value
             :c-name (cgen-allocate-static-datum)
-            :string-rep (cgen-literal (number->string value 16))))))
+            :string-rep (cgen-literal (number->string value 16)))]))
   (cexpr (self)
     (or (cgen-c-name self)
-        (if (positive? [@ self'value])
-          (format "SCM_MAKE_INT(~aU)" [@ self'value])
-          (format "SCM_MAKE_INT(~a)" [@ self'value]))))
+        (if (positive? (~ self'value))
+          (format "SCM_MAKE_INT(~aU)" (~ self'value))
+          (format "SCM_MAKE_INT(~a)" (~ self'value)))))
   (init (self)
     (when (cgen-c-name self)
       ;; Kludge: we just assume the machine's 'long' can hold at least
       ;; 32 bits.  The right thing may be to insert #ifdefs to check if
       ;; we can use 64bit literal, but we'll leave it for later revision.
-      (let ((val   [@ self'value])
-            (cname (cgen-c-name self)))
-        (cond ((< (- (expt 2 31)) val 0)
-               (print "  " cname " = Scm_MakeInteger("val");"))
-              ((<= 0 val (- (expt 2 32) 1))
-               (print "  " cname " = Scm_MakeIntegerU("val"U);"))
-              (else
+      (let ([val   (~ self'value)]
+            [cname (cgen-c-name self)])
+        (cond [(< (- (expt 2 31)) val 0)
+               (print "  " cname " = Scm_MakeInteger("val");")]
+              [(<= 0 val (- (expt 2 32) 1))
+               (print "  " cname " = Scm_MakeIntegerU("val"U);")]
+              [else
                (print "  " cname " = Scm_StringToNumber(SCM_STRING("
-                      (cgen-cexpr [@ self'string-rep])"), 16, TRUE);"))))))
+                      (cgen-cexpr (~ self'string-rep))"), 16, TRUE);")]))))
   (static (self)
     (if (cgen-c-name self) #f #t))
   )
@@ -573,10 +570,20 @@
           :denom (and (exact? value) (cgen-make-literal (denominator value)))))
   (cexpr (self) (cgen-c-name self))
   (init (self)
-    (let ((v [@ self'value]))
-      (if (exact? v)
-        (print "  "(cgen-c-name self)" = Scm_MakeRational("(cgen-cexpr [@ self'numer])","(cgen-cexpr [@ self'denom])");")
-        (print "  "(cgen-c-name self)" = Scm_MakeFlonum("v");"))))
+    (let1 v (~ self'value)
+      (print "  "(cgen-c-name self)" = ")
+      (cond
+       [(exact? v)
+        (print "Scm_MakeRational("(cgen-cexpr (~ self'numer))","(cgen-cexpr (~ self'denom))");")]
+       [(finite? v)
+        (print "Scm_MakeFlonum("v");")]
+       [(nan? v)
+        (print "SCM_NAN;")]
+       [(infinite? v)
+        (if (positive? v)
+          (print "SCM_POSITIVE_INFINITY;")
+          (print "SCM_NEGATIVE_INFINITY;"))]
+       [else (error "don't know how to emit real value literal for " v)])))
   (static (self) #f))
 
 (define-cgen-literal <cgen-scheme-complex> <complex>
@@ -586,8 +593,8 @@
           :c-name (cgen-allocate-static-datum)))
   (cexpr (self) (cgen-c-name self))
   (init (self)
-    (let ((real (real-part [@ self'value]))
-          (imag (imag-part [@ self'value])))
+    (let ([real (real-part (~ self'value))]
+          [imag (imag-part (~ self'value))])
       (print "  "(cgen-c-name self)" = Scm_MakeComplex("real", "imag");")))
   (static (self) #f))
 
@@ -597,22 +604,19 @@
   ((car :init-keyword :car)
    (cdr :init-keyword :cdr))
   (make (value)
-    (let* ((ca (cgen-literal (car value)))
-           (cd (cgen-literal (cdr value)))
-           (sobj (cgen-allocate-static-array
+    (let* ([ca (cgen-literal (car value))]
+           [cd (cgen-literal (cdr value))]
+           [sobj (cgen-allocate-static-array
                   'runtime 'ScmObj
                   (list (get-literal-initializer ca)
-                        (get-literal-initializer cd)))))
-      (make <cgen-scheme-pair> :value value :car ca :cdr cd
-            :c-name sobj)))
+                        (get-literal-initializer cd)))])
+      (make <cgen-scheme-pair> :value value :car ca :cdr cd :c-name sobj)))
   (init (self)
-    (let ((cname (cgen-cexpr self)))
-      (unless (cgen-literal-static? [@ self'car])
-        (format #t "  SCM_SET_CAR(~a, ~a);\n" cname
-                (cgen-cexpr [@ self'car])))
-      (unless (cgen-literal-static? [@ self'cdr])
-        (format #t "  SCM_SET_CDR(~a, ~a);\n"
-                cname (cgen-cexpr [@ self'cdr])))
+    (let1 cname (cgen-cexpr self)
+      (unless (cgen-literal-static? (~ self'car))
+        (format #t "  SCM_SET_CAR(~a, ~a);\n" cname (cgen-cexpr (~ self'car))))
+      (unless (cgen-literal-static? (~ self'cdr))
+        (format #t "  SCM_SET_CDR(~a, ~a);\n" cname (cgen-cexpr (~ self'cdr))))
       ))
   )
 
@@ -625,17 +629,16 @@
   ((literals :init-keyword :literals) ;; list of cgen-literals for elements.
    )
   (make (value)
-    (let* ((literals (map cgen-literal value))
-           (sobj (cgen-allocate-static-array
+    (let* ([literals (map cgen-literal value)]
+           [sobj (cgen-allocate-static-array
                   'runtime 'ScmObj
                   (list*
                    "SCM_OBJ(SCM_CLASS_STATIC_TAG(Scm_VectorClass)) /* <vector> */"
                    (format "SCM_OBJ(~a)" (length literals))
-                   (map (lambda (lit)
-                          (if (cgen-literal-static? lit)
-                            (cgen-cexpr lit)
-                            "SCM_UNDEFINED"))
-                        literals)))))
+                   (map (^(lit) (if (cgen-literal-static? lit)
+                                  (cgen-cexpr lit)
+                                  "SCM_UNDEFINED"))
+                        literals)))])
       (make <cgen-scheme-vector>
         :c-name sobj
         :value value
@@ -645,9 +648,98 @@
      (lambda (ind elt)
        (unless (cgen-literal-static? elt)
          (print "  ((ScmObj*)"(cgen-c-name self)")["(+ ind 2)"] = "(cgen-cexpr elt)";")))
-     [@ self'literals]))
+     (~ self'literals)))
   )
 
+;; NB: Because of building dependencies, we can't rely on gauche.uvector
+;; module (since this file is used to precompile gauche.uvector).
+;; The following code is carefully assembled to use only the built-in features.
+
+(define-cgen-literal <cgen-scheme-uvector> <uvector>
+  ((elements :init-keyword :elements))
+  [make (value)
+    (let1 elts (format "uvector__~a" (cgen-unique-name))
+      (make <cgen-scheme-uvector>
+        :value value :elements elts
+        :c-name (cgen-allocate-static-datum
+                 'runtime 'ScmUVector
+                 (format "  SCM_UVECTOR_INITIALIZER(~a, ~a, ~a, ~a, NULL)"
+                         (uvector-class->c-name (class-of value))
+                         (uvector-length value)
+                         elts
+                         (if (uvector-immutable? value) 1 0)))))]
+  [decl (self)
+    (let* ([value (~ self'value)]
+           [class (class-of value)])
+      (print (uvector-class->c-type-name class)" "(~ self'elements)"[] = {")
+      (dotimes [i (uvector-length value)]
+        ($ uvector-class-emit-elt class
+           $ %uvector-ref value (uvector-class->type-enum class) i))
+      (print  "};"))]
+  [static (self) #f]
+  )
+
+(define (uvector-class->tag-name class)
+  (rxmatch->string #/<(.\d+)vector>/ (symbol->string (class-name class)) 1))
+      
+(define (uvector-class->c-name class)
+  #`"Scm_,(string-upcase (uvector-class->tag-name class))VectorClass")
+
+(define (uvector-class->type-enum class)
+  (global-variable-ref
+   (find-module 'gauche)
+   (string->symbol
+    #`"SCM_UVECTOR_,(string-upcase (uvector-class->tag-name class))")))
+
+(define (uvector-class->c-type-name class)
+  (cond
+    [(eq? class <s8vector>) "signed char"]
+    [(eq? class <u8vector>) "unsigned char"]
+    [(eq? class <s16vector>) "short"]
+    [(eq? class <u16vector>) "unsigned short"]
+    [(eq? class <s32vector>) "ScmInt32"]
+    [(eq? class <u32vector>) "ScmUInt32"]
+    [(eq? class <s64vector>) "ScmInt64"]
+    [(eq? class <u64vector>) "ScmUInt64"]
+    [(eq? class <f16vector>) "ScmHalfFloat"]
+    [(eq? class <f32vector>) "float"]
+    [(eq? class <f64vector>) "double"]
+    ))
+
+(define (uvector-class-emit-elt class v)
+  (cond
+   [(memq class `(,<s8vector> ,<s16vector> ,<s32vector>))
+    (format #t "~d,\n" v)]
+   [(memq class `(,<u8vector> ,<u16vector> ,<u32vector>))
+    (format #t "~du,\n" v)]
+   [(eq? class <s64vector>)
+    (print "#if SIZEOF_LONG == 8")
+    (format #t "~dl,\n" v)
+    (print "#else if SCM_EMULATE_INT64")
+    (print "#ifdef WORDS_BIGENDIAN")
+    (print "#else  /*!WORDS_BIGENDIAN*/")
+    (print "#endif /*!WORDS_BIGENDIAN*/")
+    (print "#else  /*SIZEOF_LONG == 4 && !SCM_EMULATE_INT64*/")
+    (format #t " (((int64_t)~dl << 32)|~dl),\n"
+            (ash v -32) (logand v (- (expt 2 32) 1)))
+    (print "#endif /*SIZEOF_LONG == 4 && !SCM_EMULATE_INT64*/")]
+   [(eq? class <u64vector>)
+    (print "#if SIZEOF_LONG == 8")
+    (format #t "~dlu,\n" v)
+    (print "#else if SCM_EMULATE_INT64")
+    (print "#ifdef WORDS_BIGENDIAN")
+    (print "#else  /*!WORDS_BIGENDIAN*/")
+    (print "#endif /*!WORDS_BIGENDIAN*/")
+    (print "#else  /*SIZEOF_LONG == 4 && !SCM_EMULATE_INT64*/")
+    (format #t " (((int64_t)~dlu << 32)|~dlu),\n"
+            (ash v -32) (logand v (- (expt 2 32) 1)))
+    (print "#endif /*SIZEOF_LONG == 4 && !SCM_EMULATE_INT64*/")]
+   [(eq? class <f16vector>)
+    (error "gauche.cgen.literal: literal f16vector isn't supported yet")]
+   [(eq? class <f32vector>) (format #t "~sf,\n" v)]
+   [(eq? class <f64vector>) (format #t "~s,\n" v)]
+   ))
+
 ;; char-set -----------------------------------------------------
 
 (define-cgen-literal <cgen-scheme-char-set> <char-set>
@@ -658,7 +750,7 @@
   (init (self)
     (print "  {")
     (print "     ScmCharSet *cs = SCM_CHARSET(Scm_MakeEmptyCharSet());")
-    (dolist (range (%char-set-ranges [@ self'value]))
+    (dolist (range (%char-set-ranges (~ self'value)))
       (format #t "     Scm_CharSetAddRange(cs, SCM_CHAR(~a), SCM_CHAR(~a));\n"
               (car range) (cdr range)))
     (print "     "(cgen-c-name self)" = SCM_OBJ(cs);")
@@ -678,20 +770,62 @@
   (init (self)
     (format #t "  ~a = Scm_RegComp(SCM_STRING(~a), ~a);\n"
             (cgen-c-name self)
-            (cgen-c-name [@ self'source-string])
-            (if [@ self'case-fold?]
-              "SCM_REGEXP_CASE_FOLD"
-              "0")))
+            (cgen-c-name (~ self'source-string))
+            (if (~ self'case-fold?) "SCM_REGEXP_CASE_FOLD" "0")))
   (static (self) #f))
 
+;;---------------------------------------------------------------
+;; Inferring literal handlers.
+;;  The code generator does not know how to generate literals for
+;;  user-defined objects.  But we can do something if its read-time
+;;  constructor is defined.
+
+;;  NB: this routine writes value to a string and read it back (sans
+;;  srfi-10 prefix if any).   If the written out value contains *nested*
+;;  srfi-10 syntax, reader constructor of the inner srfi-10 is invoked
+;;  in this process.  If the reader constructor have side effects, this
+;;  routine may cause undesirable side effects as well.  There's no way
+;;  we can know if it is safe. 
+
+(define-class <cgen-user-defined-type> (<cgen-literal>)
+  ((ctor-name :init-keyword :ctor-name)
+   (ctor-args :init-keyword :ctor-args)))
+
+(define-method cgen-emit-init ((self <cgen-user-defined-type>))
+  (print "{ ScmObj ctor = Scm_GetReaderCtor("(cgen-cexpr (~ self'ctor-name))","
+         "SCM_FALSE);")
+  (print "ScmObj args = "(cgen-cexpr (~ self'ctor-args))";")
+  (print "if (SCM_FALSEP(ctor)) {")
+  (print "  Scm_Error(\"Unknown reader constructor name: ~S\", "
+         (cgen-cexpr (~ self'ctor-name))");")
+  (print "}")
+  (print "  "(cgen-c-name self)" = Scm_ApplyRec(SCM_CAR(ctor), args);")
+  (print "}"))
+
+(define-method cgen-literal-static? ((self <cgen-user-defined-type>)) #f)
+
+(define (infer-literal-handler value)
+  (and-let* ([s (guard (e [else #f]) (write-to-string value))]
+             [m (#/#,\(/ s)]  ; srfi-10 syntax
+             [xpr (guard (e [else #f]) (read-from-string (string-drop s 2)))])
+    (match xpr
+      [(tag . args)
+       (unless (%get-reader-ctor tag)
+         (errorf "Reader constructor name '~s' is unknown \
+                 to the code generator" tag))
+       (make <cgen-user-defined-type>
+         :value value
+         :c-name (cgen-allocate-static-datum)
+         :ctor-name (cgen-make-literal tag)
+         :ctor-args (cgen-make-literal args))]
+      [_ #f])))
+
 ;;=============================================================
 ;; Utilities
 ;;
 
 (define cgen-unique-name
-  (let ((counter 0))
-    (lambda ()
-      (format "~5,'0d" (inc! counter)))))
+  (let1 counter 0
+    (lambda () (format "~5,'0d" (inc! counter)))))
 
-(provide "gauche/cgen/literal")
 
