@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.cgen.unit - cgen-unit
 ;;;  
-;;;   Copyright (c) 2004-2007  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2010  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,8 +30,6 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: unit.scm,v 1.6 2007/08/24 23:56:30 shirok Exp $
-;;;
 
 (define-module gauche.cgen.unit
   (use srfi-1)
@@ -40,6 +38,8 @@
   (use util.match)
   (use gauche.parameter)
   (use gauche.sequence)
+  (use gauche.experimental.ref)
+  (use gauche.experimental.lamb)
   (export <cgen-unit> cgen-current-unit
           cgen-unit-c-file cgen-unit-init-name cgen-unit-h-file
           cgen-unit-toplevel-nodes cgen-add!
@@ -50,30 +50,13 @@
           cgen-extern cgen-decl cgen-body cgen-init
           cgen-include cgen-define
           
-          cgen-safe-name cgen-safe-name-friendly
+          cgen-safe-name cgen-safe-name-friendly cgen-safe-comment
 
           ;; semi-private routines
           cgen-emit-static-data)
   )
 (select-module gauche.cgen.unit)
 
-;; NB: a small experiment to see how I feel this...
-;;  [@ a b c d] => (ref (ref (ref a b) c) d)
-;; In string interpolations I have to use ,(@ ...) instead of ,[@ ...], for
-;; the previous versions of interpolation code doesn't like #`",[...]".
-;; Ideally this should be a compiler-macro (we can't make it a macro,
-;; for we want to say (set! [@ x'y] val).
-(define @
-  (getter-with-setter
-   (case-lambda
-     ((obj selector) (ref obj selector))
-     ((obj selector . more) (apply @ (ref obj selector) more)))
-   (case-lambda
-     ((obj selector val) ((setter ref) obj selector val))
-     ((obj selector selector2 . rest)
-      (apply (setter ref) (ref obj selector) selector2 rest)))))
-;; end experiment
-
 ;;=============================================================
 ;; Unit
 ;;
@@ -88,7 +71,7 @@
    (c-file   :init-keyword :c-file :init-value #f)
    (h-file   :init-keyword :h-file :init-value #f)
    (preamble :init-keyword :preamble
-             :init-value '("/* Generated by gauche.cgen $Revision: 1.6 $ */"))
+             :init-value '("/* Generated by gauche.cgen $Revision: 1.7 $ */"))
    (pre-decl :init-keyword :pre-decl :init-value '())
    (init-prologue :init-keyword :init-prologue :init-value #f)
    (init-epilogue :init-keyword :init-epilogue :init-value #f)
@@ -101,18 +84,18 @@
 (define cgen-current-unit (make-parameter #f))
 
 (define-method cgen-unit-c-file ((unit <cgen-unit>))
-  (or [@ unit'c-file]
-      #`",(@ unit 'name).c"))
+  (or (~ unit'c-file)
+      #`",(~ unit 'name).c"))
 
 (define-method cgen-unit-init-name ((unit <cgen-unit>))
   (format "Scm__Init_~a"
-          (or [@ unit'init-name] (cgen-safe-name [@ unit'name]))))
+          (or (~ unit'init-name) (cgen-safe-name (~ unit'name)))))
 
 (define-method cgen-unit-h-file ((unit <cgen-unit>))
-  [@ unit'h-file])
+  (~ unit'h-file))
 
 (define-method cgen-unit-toplevel-nodes ((unit <cgen-unit>))
-  [@ unit'toplevels])
+  (~ unit'toplevels))
 
 (define (cgen-add! node)
   (and-let* ((unit (cgen-current-unit)))
@@ -126,38 +109,31 @@
         (hash-table-put! context node #t)
         (cgen-node-traverse node walker)
         (cgen-emit node part)))
-    (for-each walker (reverse [@ unit'toplevels]))))
+    (for-each walker (reverse (~ unit'toplevels)))))
 
 (define-method cgen-emit-h ((unit <cgen-unit>))
-  (and-let* ((h-file (cgen-unit-h-file unit)))
+  (and-let* ([h-file (cgen-unit-h-file unit)])
     (cgen-with-output-file h-file
       (lambda ()
-        (cond ([@ unit'preamble] => emit-raw))
+        (cond [(~ unit'preamble) => emit-raw])
         (cgen-emit-part unit 'extern)))))
 
 (define-method cgen-emit-c ((unit <cgen-unit>))
   (cgen-with-output-file (cgen-unit-c-file unit)
     (lambda ()
-      (cond ([@ unit'preamble] => emit-raw))
-      (cond ([@ unit'pre-decl] => emit-raw))
+      (cond [(~ unit'preamble) => emit-raw])
+      (cond [(~ unit'pre-decl) => emit-raw])
       (print "#include <gauche.h>")
-      ;; This piece of code is required, for Win32 DLL doesn't like
-      ;; structures to be const if it contains SCM_CLASS_PTR.  Doh!
-      (print "#if defined(__CYGWIN__) || defined(GAUCHE_WINDOWS)")
-      (print "#define SCM_CGEN_CONST /*empty*/")
-      (print "#else")
-      (print "#define SCM_CGEN_CONST const")
-      (print "#endif")
       (cgen-emit-part unit 'decl)
       (cgen-emit-static-data unit)
       (cgen-emit-part unit 'body)
-      (cond ([@ unit'init-prologue] => emit-raw)
-            (else
-             (print "Scm__Init_"(cgen-safe-name [@ unit'name])"(void)")
-             (print "{")))
+      (cond [(~ unit'init-prologue) => emit-raw]
+            [else
+             (print "Scm__Init_"(cgen-safe-name (~ unit'name))"(void)")
+             (print "{")])
       (cgen-emit-part unit 'init)
-      (cond ([@ unit'init-epilogue] => emit-raw)
-            (else (print "}")))
+      (cond [(~ unit'init-epilogue) => emit-raw]
+            [else (print "}")])
       )))
 
 ;; NB: temporary solution for inter-module dependency.
@@ -184,7 +160,6 @@
 (define-method cgen-emit-init ((node <cgen-node>)) #f)
 
 ;; Should apply WALKER to the <cgen-node> instances the NODE
-
 ;; routine should visit.  The default method scans all slots
 ;; and picks up any <cgen-node>.  It fails to recognize, for example,
 ;; the node keeps a list of <cgen-nodes>; in which case the subclass
@@ -199,25 +174,24 @@
 (define-method cgen-emit ((node <cgen-node>) part)
   ;; A kludge for emitting cpp-condition only when necessary.
   (define (method-overridden? gf)
-    (and-let* ((meths (compute-applicable-methods gf (list node)))
-               ( (not (null? meths)) ))
-      (match [@ (car meths)'specializers]
-        (((? (cut eq? <> <cgen-node>))) #f)
-        (_ #t))))
+    (and-let* ([meths (compute-applicable-methods gf (list node))]
+               [ (not (null? meths)) ])
+      (match (~ (car meths)'specializers)
+        [((? (cut eq? <> <cgen-node>))) #f]
+        [_ #t])))
   (define (with-cpp-condition gf)
-    (cond ([@ node'cpp-condition]
-           => (lambda (cppc)
-                (cond ((method-overridden? gf)
-                       (print "#if "cppc)
-                       (gf node)
-                       (print "#endif /* "cppc" */"))
-                      (else (gf node)))))
-          (else (gf node))))
+    (cond [(~ node'cpp-condition)
+           => (^(cppc) (cond [(method-overridden? gf)
+                              (print "#if "cppc)
+                              (gf node)
+                              (print "#endif /* "cppc" */")]
+                             [else (gf node)]))]
+          [else (gf node)]))
   (case part
-    ((extern) (with-cpp-condition cgen-emit-xtrn))
-    ((decl)   (with-cpp-condition cgen-emit-decl))
-    ((body)   (with-cpp-condition cgen-emit-body))
-    ((init)   (with-cpp-condition cgen-emit-init))))
+    [(extern) (with-cpp-condition cgen-emit-xtrn)]
+    [(decl)   (with-cpp-condition cgen-emit-decl)]
+    [(body)   (with-cpp-condition cgen-emit-body)]
+    [(init)   (with-cpp-condition cgen-emit-init)]))
 
 ;;=============================================================
 ;; Raw nodes - can be used to insert a raw piece of code
@@ -226,22 +200,22 @@
 (define-class <cgen-raw-xtrn> (<cgen-node>)
   ((code  :init-keyword :code :init-value "")))
 (define-method cgen-emit-xtrn ((node <cgen-raw-xtrn>))
-  (emit-raw [@ node'code]))
+  (emit-raw (~ node'code)))
 
 (define-class <cgen-raw-decl> (<cgen-node>)
   ((code  :init-keyword :code :init-value "")))
 (define-method cgen-emit-decl ((node <cgen-raw-decl>))
-  (emit-raw [@ node'code]))
+  (emit-raw (~ node'code)))
 
 (define-class <cgen-raw-body> (<cgen-node>)
   ((code  :init-keyword :code :init-value "")))
 (define-method cgen-emit-body ((node <cgen-raw-body>))
-  (emit-raw [@ node'code]))
+  (emit-raw (~ node'code)))
 
 (define-class <cgen-raw-init> (<cgen-node>)
   ((code  :init-keyword :code :init-value "")))
 (define-method cgen-emit-init ((node <cgen-raw-init>))
-  (emit-raw [@ node'code]))
+  (emit-raw (~ node'code)))
 
 
 (define (cgen-extern . code)
@@ -266,9 +240,9 @@
 
 (define (include-common node)
   (print "#include "
-         (if (string-prefix? "<" [@ node'path])
-           [@ node'path]
-           #`"\",(@ node'path)\"")))
+         (if (string-prefix? "<" (~ node'path))
+           (~ node'path)
+           #`"\",(~ node'path)\"")))
 
 (define-method cgen-emit-xtrn ((node <cgen-include>))
   (include-common node))
@@ -285,7 +259,7 @@
    ))
 
 (define (cpp-define-common node)
-  (print "#define "[@ node'name]" "[@ node'value]))
+  (print "#define "(~ node'name)" "(~ node'value)))
 
 (define-method cgen-emit-xtrn ((node <cgen-cpp-define>))
   (cpp-define-common node))
@@ -353,6 +327,9 @@
             (else (display c) (loop (read-char)))
             ))))))
 
-(provide "gauche/cgen/unit")
+;; Escape  '*/' so that str can be inserted safely within a comment.
+(define (cgen-safe-comment str)
+  (regexp-replace-all* (x->string str) #/\/\*/ "/ *" #/\*\// "* /"))
+
 
 
