@@ -1,7 +1,7 @@
 ;;;
 ;;; interactive.scm - useful stuff in the interactive session
 ;;;  
-;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,8 +30,8 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: interactive.scm,v 1.15 2007/03/02 07:39:08 shirok Exp $
-;;;
+
+#!no-fold-case
 
 (define-module gauche.interactive
   (export apropos describe d
@@ -144,7 +144,6 @@
 
 ;; For convenience
 (let ((dotfile (sys-normalize-pathname "~/.gaucherc" :expand #t)))
-  (when (sys-access dotfile |F_OK|)
+  (when (sys-access dotfile F_OK)
     (load dotfile :environment (find-module 'user))))
 
-(provide "gauche/interactive")
