@@ -1,7 +1,7 @@
 ;;;
 ;;; logger.scm - simple use-level logging
 ;;;  
-;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,8 +30,8 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: logger.scm,v 1.11 2007/09/15 12:30:50 shirok Exp $
-;;;
+
+#!no-fold-case
 
 (define-module gauche.logger
   (use srfi-1)
@@ -155,10 +155,10 @@
         (cond-expand
          (gauche.sys.fcntl
           (guard (e ((<system-error> e) 'file))
-            (let ((lk (make <sys-flock> :type |F_WRLCK| :whence 0))
-                  (un (make <sys-flock> :type |F_UNLCK| :whence 0)))
-              (and (sys-fcntl port |F_SETLK| lk)
-                   (sys-fcntl port |F_SETLK| un))
+            (let ((lk (make <sys-flock> :type F_WRLCK :whence 0))
+                  (un (make <sys-flock> :type F_UNLCK :whence 0)))
+              (and (sys-fcntl port F_SETLK lk)
+                   (sys-fcntl port F_SETLK un))
               'fcntl)))
          (else
           'file)))
@@ -166,14 +166,14 @@
 
 (define (lock-data drain port)
   (case (slot-ref drain 'lock-policy)
-    ((fcntl) (make <sys-flock> :type |F_WRLCK| :whence 0))
+    ((fcntl) (make <sys-flock> :type F_WRLCK :whence 0))
     ((file)  (string-append (slot-ref drain 'path) ".lock"))
     ((tbd)   (lock-data (determine-lock-policy drain port) port))
     (else    #t)))
 
 (define (lock-file drain port data)
   (case (slot-ref drain 'lock-policy)
-    ((fcntl) (sys-fcntl port |F_SETLKW| data))
+    ((fcntl) (sys-fcntl port F_SETLKW data))
     ((file)
      (let loop ((retry 0)
                 (o (open-output-file data :if-exists #f)))
@@ -195,8 +195,8 @@
 (define (unlock-file drain port data)
   (case (slot-ref drain 'lock-policy)
     ((fcntl)
-     (slot-set! data 'type |F_UNLCK|)
-     (sys-fcntl port |F_SETLK| data))
+     (slot-set! data 'type F_UNLCK)
+     (sys-fcntl port F_SETLK data))
     ((file)
      (sys-unlink data))
     ((tdb)
@@ -248,4 +248,3 @@
 (define (log-open path . args)
   (log-default-drain (apply make <log-drain> :path path args)))
 
-(provide "gauche/logger")
