@@ -1,7 +1,7 @@
 ;;;
 ;;; instance-pool.scm - instance pool metaclass
 ;;;  
-;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,14 +30,11 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: instance-pool.scm,v 1.7 2007/03/02 07:39:09 shirok Exp $
-;;;
 
 ;; EXPERIMENTAL.   THE API MAY CHANGE.
 
 (define-module gauche.mop.instance-pool
   (use srfi-1)
-  (use util.queue)
   (export <instance-pool-meta> <instance-pool-mixin>
           <instance-table-meta> <instance-table-mixin>
           instance-pool-of instance-pools-of
@@ -184,5 +181,4 @@
 (define-method instance-pool-map ((self <instance-pool-mixin>) proc)
   (instance-pool-map (class-of self) proc))
 
-(provide "gauche/mop/instance-pool")
 
