@@ -1,7 +1,7 @@
 ;;;
 ;;; singleton.scm - implements singleton mixin
 ;;;  
-;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,10 +30,6 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: singleton.scm,v 1.4 2007/03/02 07:39:09 shirok Exp $
-;;;
-
-;; EXPERIMENTAL
 
 (define-module gauche.mop.singleton
   (export <singleton-meta> <singleton-mixin> instance-of))
@@ -60,4 +56,3 @@
   ()
   :metaclass <singleton-meta>)
 
-(provide "gauche/mop/singleton")
