@@ -1,7 +1,7 @@
 ;;;
 ;;; numerical.scm - auxiliary numerical functions - to be autoloaded
 ;;;  
-;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,8 +30,6 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: numerical.scm,v 1.22 2007/12/25 12:00:42 shirok Exp $
-;;;
 
 (select-module gauche)
 
@@ -101,16 +99,61 @@
         [(complex? z) (make-polar (%exp (real-part z)) (imag-part z))]
         [else (error "number required, but got" z)]))
 
-(define-in-module scheme (log z)
-  (cond [(real? z) (%log z)]
-        [(complex? z) (make-rectangular (%log (magnitude z)) (angle z))]
-        [else (error "number required, but got" z)]))
+(define-in-module scheme (log z . base)
+  (if (null? base)
+    (cond [(real? z) (%log z)]
+          [(complex? z) (make-rectangular (%log (magnitude z)) (angle z))]
+          [else (error "number required, but got" z)])
+    (/ (log z) (log (car base)))))  ; R6RS addition
 
 (define-in-module scheme (sqrt z)
-  (cond [(real? z) (%sqrt z)]
+  (cond [(and (exact? z) (>= z 0))
+         ;; Gauche doesn't have exact complex, so we have real z.
+         (if (integer? z)
+           (receive (s r) (%exact-integer-sqrt z)
+             (if (= r 0) s (%sqrt z)))
+           ;; we have ratnum.  take expensive path.
+           (let ([n (numerator z)]
+                 [d (denominator z)])
+             (receive (ns nr) (%exact-integer-sqrt n)
+               (if (= nr 0)
+                 (receive (ds dr) (%exact-integer-sqrt d)
+                   (if (= dr 0)
+                     (/ ns ds)
+                     (%sqrt z)))
+                 (%sqrt z)))))]
+        [(real? z) (%sqrt z)]
         [(complex? z) (make-polar (%sqrt (magnitude z)) (/ (angle z) 2.0))]
         [else (error "number required, but got" z)]))
 
+;; a la R6RS
+(define (exact-integer-sqrt k)
+  (unless (and (exact? k) (integer? k) (>= k 0))
+    (error "exact nonnegative integer required, but got" k))
+  (%exact-integer-sqrt k))
+
+(define (%exact-integer-sqrt k)  ; k >= 0
+  (if (< k 9007199254740992)            ;2^53
+    ;; k can be converted to a double without loss.
+    (let1 s (floor->exact (%sqrt k))
+      (values s (- k (* s s))))
+    ;; use Newton-Rhapson
+    ;; If k is representable with double, we use (%sqrt k) as the initial
+    ;; estimate, for calculating double sqrt is fast.  If k is too large,
+    ;; we use 2^floor((log2(k)+1)/2) as the initial value.
+    ;; TODO: integer-length can be a lot faster if we make it built-in.
+    (let loop ([s (let1 ik (%sqrt k)
+                    (if (finite? ik)
+                      (floor->exact (%sqrt k))
+                      (ash 1 (quotient (integer-length k) 2))))])
+      (let1 s2 (* s s)
+        (if (< k s2)
+          (loop (quotient (+ s2 k) (* 2 s)))
+          (let1 s2+ (+ s2 (* 2 s) 1)
+            (if (< k s2+)
+              (values s (- k s2))
+              (loop (quotient (+ s2 k) (* 2 s))))))))))
+
 (define-in-module scheme (expt x y)
   (cond [(real? x)
          (cond [(real? y) (%expt x y)]
@@ -227,6 +270,64 @@
   (/ (- (log (+ 1 z)) (log (- 1 z))) 2))
 
 ;;
+;; R6RS compatibiliby
+;;
+(define inexact exact->inexact)
+(define exact   inexact->exact)
+
+;; Since Gauche coerces complex number whose imag part is 0.0i into
+;; real, these procedures has no difference from real?, rational? and
+;; integer?
+(define real-valued?     real?)
+(define rational-valued? rational?)
+(define integer-valued?  integer?)
+
+;; An extension of quotient&remainder to the real domain.  Their results
+;; match with div/mod when divisor is positive.
+(define (%div&mod x y)
+  (define (edom what)
+      (error "real number required for integer division, but got" what))
+  (cond [(= y 0) (error "zero divisor is not allowed in integer division")]
+        [(and (integer? x) (integer? y)) (quotient&remainder x y)]
+        [(and (exact? x) (exact? y))    ;at least one of arg is rational
+         (let1 q (truncate (/ x y)) (values q (- x (* q y))))]
+        [(not (real? x)) (edom x)]
+        [(not (real? y)) (edom y)]
+        [else (receive (r q) (modf (/. x y)) (values q (- x (* q y))))]))
+
+(define (div-and-mod x y)
+  (if (>= x 0)
+    (%div&mod x y)
+    (receive (q r) (%div&mod x y)
+      (if (= r 0)
+        (values q r)
+        (if (> y 0)
+          (values (- q 1) (+ r y))
+          (values (+ q 1) (- r y)))))))
+
+(define (div0-and-mod0 x y)
+  (receive (q r) (%div&mod x y)
+    (if (>= x 0)
+      (if (> y 0)
+        (if (>= (* r 2) y)
+          (values (+ q 1) (- r y))
+          (values q r))
+        (if (>= (* r 2) (- y))
+          (values (- q 1) (+ r y))
+          (values q r)))
+      (if (> y 0)
+        (if (< (* r 2) (- y))
+          (values (- q 1) (+ r y))
+          (values q r))
+        (if (< (* r 2) y)
+          (values (+ q 1) (- r y))
+          (values q r))))))
+
+(define (div x y)  (values-ref (div-and-mod x y) 0))
+(define (mod x y)  (values-ref (div-and-mod x y) 1))
+(define (div0 x y) (values-ref (div0-and-mod0 x y) 0))
+(define (mod0 x y) (values-ref (div0-and-mod0 x y) 1))
+    
 ;; Nearly equal comparison
 ;;  (Unofficial yet; see how it works)
 
@@ -234,4 +335,3 @@
   (< (abs (- x y))
      (/ (max (abs x) (abs y)) tolerance)))
 
-(provide "gauche/numerical")
