@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.package - package management
 ;;;  
-;;;   Copyright (c) 2004-2007  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2010  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,8 +30,6 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: package.scm,v 1.9 2007/03/02 07:39:08 shirok Exp $
-;;;
 
 ;; *EXPERIMENTAL*
 ;; The spec might be changed in future.  Do not count on the current version.
@@ -140,11 +138,10 @@
   ((paths :init-form *load-path* :init-keyword :paths)))
 
 ;; Returns a singleton of <gauche-package-descriptions>.
-(define (gauche-package-description-paths . args)
-  (let-keywords args ((all-versions #f))
-    (if all-versions
-      (make <gauche-package-description-paths> :paths (get-all-version-paths))
-      (make <gauche-package-description-paths>))))
+(define (gauche-package-description-paths :key (all-versions #f))
+  (if all-versions
+    (make <gauche-package-description-paths> :paths (get-all-version-paths))
+    (make <gauche-package-description-paths>)))
 
 ;; scan the directory to find older verison of Gauche library directories.
 (define (get-all-version-paths)
@@ -197,11 +194,9 @@
       (proc end? next))))
 
 ;; utility
-(define (find-gauche-package-description name . args)
-  (let-keywords args ((all-versions #f))
-    (and-let* ((path (find (string->regexp #`"/,|name|\\.gpd$")
-                           (gauche-package-description-paths
-                            :all-versions all-versions))))
-      (path->gauche-package-description path))))
+(define (find-gauche-package-description name :key (all-versions #f))
+  (and-let* ((path (find (string->regexp #`"/,|name|\\.gpd$")
+                         (gauche-package-description-paths
+                          :all-versions all-versions))))
+    (path->gauche-package-description path)))
 
-(provide "gauche/package")
