@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.package.build - build a package
 ;;;  
-;;;   Copyright (c) 2004-2007  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2010  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,8 +30,6 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: build.scm,v 1.11 2007/03/02 07:39:09 shirok Exp $
-;;;
 
 ;; *EXPERIMENTAL*
 ;; gauche.package.build module is intended to automate standard building
@@ -86,11 +84,11 @@
 (define (configure config dir package-name configure-options)
   (let1 conf-cmd
       (cond
-       (configure-options #`"./configure ,configure-options")
-       ((and-let* ((package-name)
-                   (gpd (find-gauche-package-description package-name)))
-          (ref gpd 'configure)))
-       (else "./configure"))
+       [configure-options #`"./configure ,configure-options"]
+       [(and-let* ([ package-name ]
+                   [gpd (find-gauche-package-description package-name)])
+          (ref gpd 'configure))]
+       [else "./configure"])
     (run #`"cd \",dir\"; ,conf-cmd")))
 
 (define (make config dir)
@@ -129,36 +127,33 @@
 ;;; Driver
 ;;;
 
-(define (gauche-package-build uri . opts)
-  (let-keywords opts ((config  '())
-                      (configure-options #f)
-                      (install-only? :install-only #f)
-                      (dry?          :dry-run #f)
-                      (reconfigure?  :reconfigure #f)
-                      (check?        :check #t)
-                      (install?      :install #f)
-                      (clean?        :clean #f)
-                      (sudo-user     :sudo-install #f)
-                      (sudo-pass     :sudo-password #f))
-    (parameterize ((dry-run dry?))
-      (let* ((tarball   (gauche-package-ensure uri :config config))
-             (build-dir (assq-ref config 'build-dir "."))
-             (basename  (tarball->package-directory tarball))
-             (dir       (build-path build-dir basename))
-             (packname  (package-name basename)))
-        (when (and sudo-user (not sudo-pass))
-          (set! sudo-pass (get-password)))
-        (unless install-only?
-          (clean config dir)
-          (untar config tarball)
-          (configure config dir packname
-                     (or configure-options
-                         (and reconfigure?
-                              (get-reconf-options packname))))
-          (make config dir)
-          (when check?   (make-check config dir)))
-        (when (or install? install-only?)
-          (make-install config dir sudo-user sudo-pass))
-        (when clean?   (clean config dir))))))
+(define (gauche-package-build uri :key (config  '()) (configure-options #f)
+                              ((:install-only install-only?) #f)
+                              ((:dry-run dry?) #f)
+                              ((:reconfigure reconfigure?) #f)
+                              ((:check check?) #t)
+                              ((:install install?) #f)
+                              ((:clean clean?) #f)
+                              ((:sudo-install sudo-user) #f)
+                              ((:sudo-password sudo-pass) #f))
+  (parameterize ((dry-run dry?))
+    (let* ((tarball   (gauche-package-ensure uri :config config))
+           (build-dir (assq-ref config 'build-dir "."))
+           (basename  (tarball->package-directory tarball))
+           (dir       (build-path build-dir basename))
+           (packname  (package-name basename)))
+      (when (and sudo-user (not sudo-pass))
+        (set! sudo-pass (get-password)))
+      (unless install-only?
+        (clean config dir)
+        (untar config tarball)
+        (configure config dir packname
+                   (or configure-options
+                       (and reconfigure?
+                            (get-reconf-options packname))))
+        (make config dir)
+        (when check?   (make-check config dir)))
+      (when (or install? install-only?)
+        (make-install config dir sudo-user sudo-pass))
+      (when clean?   (clean config dir)))))
 
-(provide "gauche/package/build")
