@@ -1,7 +1,7 @@
 ;;;
 ;;; parseopt.scm - yet another command-line argument parser
 ;;;  
-;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,31 +30,36 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: parseopt.scm,v 1.13 2008/02/03 08:25:49 shirok Exp $
-;;;
 
 (define-module gauche.parseopt
   (use gauche.regexp)
+  (use gauche.experimental.ref) ; remove this after 0.9.1 release
   (use srfi-1)
   (use srfi-2)
   (use srfi-13)
-  (export make-option-parser parse-options let-args))
+  (export make-option-parser parse-options let-args <parseopt-error>))
 (select-module gauche.parseopt)
 
+;; This error is thrown when the given argument doesn't follow the spec.
+;; (An error in the spec itself is thrown as an ordinary error.)
+(define-condition-type <parseopt-error> <error> #f
+  (option-name))
+
 ;; Represents each option info
 (define-class <option-spec> ()
-  ((name :init-keyword :name) ;; option name
-   (args :init-keyword :args) ;; option agrspecs (list of chars)
-   (arg-optional? :init-keyword :arg-optional?) ;; option's arg optional?
-   (handler :init-keyword :handler) ;; handler closure
-   (optspec :init-keyword :optspec) ;; original <optspec> string
-   (help :init-keyword :help) ;; help string
+  ((name :init-keyword :name)        ; option name
+   (args :init-keyword :args)        ; option agrspecs (list of chars)
+   (arg-optional? :init-keyword :arg-optional?) ; option's arg optional?
+   (handler :init-keyword :handler)  ; handler closure
+   (plural? :init-keyword :plural?)  ; accept multiple options?
+   (optspec :init-keyword :optspec)  ; original <optspec> string
+   (help    :init-keyword :help)     ; help string
    ))
 
 ;; Helper functions
 
 ;; Parse optspec clause, and returns
-;;  ((option-string argument-specs optional? handler) ...)
+;;  (<option-spec> ...)
 ;; <a-spec> is (<optspec> <handler>) or
 ;; ((<optspec> <help-string>) <handler)
 (define (compose-entry a-spec)
@@ -65,13 +70,14 @@
         (handler (cadr a-spec)))
     (unless (string? optspec)
       (error "option spec must be a string, but got" optspec))
-    (rxmatch-if (rxmatch #/^-*([-+\w|]+)(?:([=:])(.+))?$/ optspec)
-        (#f optnames optional?  argspec)
+    (rxmatch-if (rxmatch #/^-*([-+\w|]+)(\*)?(?:([=:])(.+))?$/ optspec)
+        (#f optnames plural? optional?  argspec)
       (map (cute make <option-spec>
                  :name <>
                  :args (if argspec (string->list argspec) '())
                  :arg-optional? (equal? optional? ":")
                  :handler handler
+                 :plural? plural?
                  :optspec optspec
                  :help helpstr)
            (string-split optnames #\|))
@@ -93,52 +99,57 @@
 (define (get-optargs optspec args)
   (define (get-number arg)
     (or (string->number arg)
-        (errorf "a number is required for option ~a, but got ~a"
-                (ref optspec 'name) arg)))
+        (errorf <parseopt-error> :option-name (~ optspec 'name)
+                "a number is required for option ~a, but got ~a"
+                (~ optspec 'name) arg)))
   (define (get-real arg)
     (or (and-let* ([num (string->number arg)]
                    [ (real? num) ])
           (exact->inexact num))
-        (errorf "a real number is required for option ~a, but got ~a"
-                (ref optspec 'name) arg)))
+        (errorf <parseopt-error> :option-name (~ optspec'name)
+                "a real number is required for option ~a, but got ~a"
+                (~ optspec 'name) arg)))
   (define (get-integer arg)
     (or (and-let* ([num (string->number arg)]
                    [ (exact? num) ])
           num)
-        (errorf "an integer is required for option ~a, but got ~a"
-                (ref optspec 'name) arg)))
+        (errorf <parseopt-error> :option-name (~ optspec'name)
+                "an integer is required for option ~a, but got ~a"
+                (~ optspec 'name) arg)))
   (define (get-sexp arg)
     (guard (e [(<read-error> e)
-               (errorf "the argument for option ~a is not valid sexp: ~s"
-                       (ref optspec 'name) arg)])
+               (errorf <parseopt-error> :option-name (~ optspec'name)
+                       "the argument for option ~a is not valid sexp: ~s"
+                       (~ optspec 'name) arg)])
       (read-from-string arg)))
   (define (process-args)
-    (let loop ((spec (ref optspec 'args))
+    (let loop ((spec (~ optspec 'args))
                (args args)
                (optargs '()))
-      (cond ((null? spec) (values (reverse! optargs) args))
-            ((null? args) (error "running out the arguments for option"
-                                 (ref optspec 'name)))
-            (else
+      (cond [(null? spec) (values (reverse! optargs) args)]
+            [(null? args) (error <parseopt-error> :option-name (~ optspec'name)
+                                 "running out the arguments for option"
+                                 (~ optspec 'name))]
+            [else
              (case (car spec)
-               ((#\s) (loop (cdr spec) (cdr args) (cons (car args) optargs)))
-               ((#\n) (loop (cdr spec) (cdr args)
-                            (cons (get-number (car args)) optargs)))
-               ((#\f) (loop (cdr spec) (cdr args)
-                            (cons (get-real (car args)) optargs)))
-               ((#\i) (loop (cdr spec) (cdr args)
-                            (cons (get-integer (car args)) optargs)))
-               ((#\e) (loop (cdr spec) (cdr args)
-                            (cons (get-sexp (car args)) optargs)))
-               ((#\y) (loop (cdr spec) (cdr args)
-                            (cons (string->symbol (car args)) optargs)))
-               (else (error "unknown option argument spec:" (car spec))))))
+               [(#\s) (loop (cdr spec) (cdr args) (cons (car args) optargs))]
+               [(#\n) (loop (cdr spec) (cdr args)
+                            (cons (get-number (car args)) optargs))]
+               [(#\f) (loop (cdr spec) (cdr args)
+                            (cons (get-real (car args)) optargs))]
+               [(#\i) (loop (cdr spec) (cdr args)
+                            (cons (get-integer (car args)) optargs))]
+               [(#\e) (loop (cdr spec) (cdr args)
+                            (cons (get-sexp (car args)) optargs))]
+               [(#\y) (loop (cdr spec) (cdr args)
+                            (cons (string->symbol (car args)) optargs))]
+               [else (error "unknown option argument spec:" (car spec))])])
       ))
 
-  (if (ref optspec 'arg-optional?)
+  (if (~ optspec 'arg-optional?)
     (if (or (null? args)
             (#/^-/ (car args)))
-      (values (make-list (length (ref optspec 'args)) #f) args)
+      (values (make-list (length (~ optspec 'args)) #f) args)
       (process-args))
     (process-args))
   )
@@ -148,14 +159,14 @@
   (let loop ((args args))
     (receive (option nextargs) (next-option args)
       (if option
-          (cond ((find (lambda (e) (equal? option (ref e 'name))) speclist)
-                 => (lambda (entry)
-                      (receive (optargs nextargs)
-                          (get-optargs entry nextargs)
-                        (apply (ref entry 'handler) optargs)
-                        (loop nextargs))))
-                (else (fallback option nextargs loop)))
-          nextargs))))
+        (cond [(find (lambda (e) (equal? option (~ e'name))) speclist)
+               => (lambda (entry)
+                    (receive (optargs nextargs)
+                        (get-optargs entry nextargs)
+                      (apply (~ entry'handler) optargs)
+                      (loop nextargs)))]
+              [else (fallback option nextargs loop)])
+        nextargs))))
 
 ;; Build 
 (define (build-option-parser spec fallback)
@@ -177,7 +188,8 @@
     ((_ () specs)
      (build-option-parser (list . specs)
                           (lambda (option args looper)
-                            (error "unrecognized option:" option))))
+                            (error <parseopt-error> :option-name #f
+                                   "unrecognized option:" option))))
     ((_ ((else args . body)) specs)
      (build-option-parser (list . specs) (lambda args . body)))
     ((_ ((else => proc)) specs)
@@ -326,4 +338,3 @@
      (syntax-error "malformed let-args:" (let-args . otherwise)))
     ))
 
-(provide "gauche/parseopt")
