@@ -1,7 +1,7 @@
 ;;;
 ;;; port related utility functions.  to be autoloaded.
 ;;;  
-;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,8 +30,6 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: portutil.scm,v 1.10 2007/03/02 07:39:08 shirok Exp $
-;;;
 
 (define-module gauche.portutil
   (export port->string port->list port->string-list port->sexp-list
@@ -44,7 +42,7 @@
 ;; port->something
 ;;   TODO: allow caller to specify reading units
 (define (port->string port)
-  (let ((out (open-output-string :private? #t)))
+  (let1 out (open-output-string :private? #t)
     (with-port-locking port
       (lambda ()
         (let loop ((ch (read-char port)))
@@ -59,14 +57,11 @@
       (let loop ((obj (reader port))
                  (result '()))
         (if (eof-object? obj)
-            (reverse! result)
-            (loop (reader port) (cons obj result)))))))
+          (reverse! result)
+          (loop (reader port) (cons obj result)))))))
 
-(define (port->string-list port)
-  (port->list read-line port))
-
-(define (port->sexp-list port)
-  (port->list read port))
+(define (port->string-list port) (port->list read-line port))
+(define (port->sexp-list port)   (port->list read port))
 
 ;;-----------------------------------------------------
 ;; copy-port
@@ -77,29 +72,29 @@
 
 (define-macro (%do-copy reader writer incr)
   `(with-port-locking src
-    (lambda ()
-      (with-port-locking dst
-        (lambda ()
-          (let loop ((data  ,reader)
-                     (count 0))
-            (if (eof-object? data)
-                count
-                (begin ,writer
-                       (loop ,reader ,incr)))))))))
+     (lambda ()
+       (with-port-locking dst
+         (lambda ()
+           (let loop ((data  ,reader)
+                      (count 0))
+             (if (eof-object? data)
+               count
+               (begin ,writer
+                      (loop ,reader ,incr)))))))))
 
 (define-macro (%do-copy/limit1 reader writer limit)
   `(with-port-locking src
-    (lambda ()
-      (with-port-locking dst
-        (lambda ()
-          (let loop ((count 0))
-            (if (>= count ,limit)
-                count
-                (let ((data ,reader))
-                  (if (eof-object? data)
-                      count
-                      (begin ,writer
-                             (loop (+ count 1))))))))))))
+     (lambda ()
+       (with-port-locking dst
+         (lambda ()
+           (let loop ((count 0))
+             (if (>= count ,limit)
+               count
+               (let ((data ,reader))
+                 (if (eof-object? data)
+                   count
+                   (begin ,writer
+                          (loop (+ count 1))))))))))))
 
 (define (%do-copy/limitN src dst buf unit limit)
   (with-port-locking src
@@ -108,49 +103,35 @@
         (lambda ()
           (let loop ((count 0))
             (if (>= count limit)
-                count
-                (let1 nr (read-block! buf src 0
-                                      (if (>= (+ count unit) limit)
-                                          (- limit count)
-                                          unit))
-                  (if (eof-object? nr)
-                      count
-                      (begin (write-block buf dst 0 nr)
-                             (loop (+ count nr))))))))))))
+              count
+              (let1 nr (read-block! buf src 0
+                                    (if (>= (+ count unit) limit)
+                                      (- limit count)
+                                      unit))
+                (if (eof-object? nr)
+                  count
+                  (begin (write-block buf dst 0 nr)
+                         (loop (+ count nr))))))))))))
 
-(define (copy-port src dst . args)
+(define (copy-port src dst :key (unit 4096) (size -1))
   (check-arg input-port? src)
   (check-arg output-port? dst)
-  (let-keywords args ((unit 4096)
-                      (size 0))
-    (cond ((eq? unit 'byte)
-           (if (and (integer? size) (positive? size))
-               (%do-copy/limit1 (read-byte src)
-                                (write-byte data dst)
-                                size)
-               (%do-copy (read-byte src)
-                         (write-byte data dst)
-                         (+ count 1))))
-          ((eq? unit 'char)
-           (if (and (integer? size) (positive? size))
-               (%do-copy/limit1 (read-char src)
-                                (write-char data dst)
-                                size)
-               (%do-copy (read-char src)
-                         (write-char data dst)
-                         (+ count 1))))
-          ((integer? unit)
-           (let ((buf (make-u8vector (if (zero? unit) 4096 unit))))
-             (if (and (integer? size) (positive? size))
-                 (%do-copy/limitN src dst buf unit size)
-                 (%do-copy (read-block! buf src)
-                           (write-block buf dst 0 data)
-                           (+ count data)))))
-          (else
-           (error "unit must be 'char, 'byte, or non-negative integer" unit))
-          )
-    )
-  )
+  (cond [(eq? unit 'byte)
+         (if (and (integer? size) (not (negative? size)))
+           (%do-copy/limit1 (read-byte src) (write-byte data dst) size)
+           (%do-copy (read-byte src) (write-byte data dst) (+ count 1)))]
+        [(eq? unit 'char)
+         (if (and (integer? size) (not (negative? size)))
+           (%do-copy/limit1 (read-char src) (write-char data dst) size)
+           (%do-copy (read-char src) (write-char data dst) (+ count 1)))]
+        [(integer? unit)
+         (let ((buf (make-u8vector (if (zero? unit) 4096 unit))))
+           (if (and (integer? size) (not (negative? size)))
+             (%do-copy/limitN src dst buf unit size)
+             (%do-copy (read-block! buf src) (write-block buf dst 0 data)
+                       (+ count data))))]
+        [else (error "unit must be 'char, 'byte, or non-negative integer" unit)]
+        ))
 
 ;;-----------------------------------------------------
 ;; Iterators on the input stream
@@ -162,15 +143,15 @@
   (let loop ((item (reader))
              (r    knil))
     (if (eof-object? item)
-        r
-        (loop (reader) (fn item r)))))
+      r
+      (loop (reader) (fn item r)))))
 
 ;; This will consume large stack if input file is large.
 (define (port-fold-right fn knil reader)
   (let loop ((item (reader)))
     (if (eof-object? item)
-        knil
-        (fn item (loop (reader))))))
+      knil
+      (fn item (loop (reader))))))
 
 (define (port-for-each fn reader)
   (let loop ((item (reader)))
@@ -182,18 +163,18 @@
   (let loop ((item (reader))
              (r    '()))
     (if (eof-object? item)
-        (reverse! r)
-        (loop (reader) (cons (fn item) r)))))
+      (reverse! r)
+      (loop (reader) (cons (fn item) r)))))
 
 ;; useful for error messages
 (define (port-position-prefix port)
   (let ((n (port-name port))
         (l (port-current-line port)))
     (if n
-        (if (positive? l)
-            (format #f "~s:line ~a: " n l)
-            (format #f "~s: " n))
-        "")))
+      (if (positive? l)
+        (format #f "~s:line ~a: " n l)
+        (format #f "~s: " n))
+      "")))
 
 ;;-----------------------------------------------------
 ;; useful alias
@@ -201,4 +182,3 @@
 
 (define (port-tell p) (port-seek p 0 SEEK_CUR))
 
-(provide "gauche/portutil")
