@@ -1,7 +1,7 @@
 ;;;
 ;;; auxiliary string utilities.  to be autoloaded.
 ;;;  
-;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,8 +30,6 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: stringutil.scm,v 1.4 2007/03/02 07:39:08 shirok Exp $
-;;;
 
 (define-module gauche.stringutil
   (export string-split)
@@ -80,17 +78,17 @@
 (define (%string-split-scanner-each-char pred)
   (define (scan-in p)
     (let ((c (string-pointer-ref p)))
-      (cond ((eof-object? c) (values (string-pointer-substring p) #f))
-            ((pred c)
+      (cond [(eof-object? c) (values (string-pointer-substring p) #f)]
+            [(pred c)
              (let1 before (string-pointer-substring p)
                (string-pointer-next! p)
-               (scan-out p before)))
-            (else (string-pointer-next! p) (scan-in p)))))
+               (scan-out p before))]
+            [else (string-pointer-next! p) (scan-in p)])))
   (define (scan-out p before)
     (let ((c (string-pointer-ref p)))
-      (cond ((eof-object? c) (values before ""))
-            ((pred c) (string-pointer-next! p) (scan-out p before))
-            (else (values before (string-pointer-substring p :after #t))))))
+      (cond [(eof-object? c) (values before "")]
+            [(pred c) (string-pointer-next! p) (scan-out p before)]
+            [else (values before (string-pointer-substring p :after #t))])))
   (lambda (s) (scan-in (make-string-pointer s))))
 
 (define (%string-split string scanner)
@@ -101,4 +99,3 @@
           (loop after (cons before r))
           (reverse! (cons before r))))))
 
-(provide "gauche/stringutil")
