@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.test - test framework
 ;;;  
-;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,7 +30,14 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: test.scm,v 1.27 2008/02/04 05:04:48 shirok Exp $
+
+#!no-fold-case
+
+;; Note for developers: This module intentionally avoids using
+;; Gauche's convenience features and extended libraries; instead,
+;; we stick to the minimal primitives here as much as possible.
+;; It's because this module is used to test the convenience
+;; features and extended libraries.
 
 ;; Writing your own test
 ;;
@@ -76,7 +83,7 @@
 
 (define-module gauche.test
   (export test test* test-start test-end test-section
-          test-module
+          test-module test-error test-check test-record-file
           *test-error* *test-report-error* test-error? prim-test))
 (select-module gauche.test)
 
@@ -87,26 +94,38 @@
 
 ;; An object to represent error.
 (define-class <test-error> ()
-  ((message :init-keyword :message :initform #f)))
+  ((message :init-keyword :message :init-value #f)
+   (class   :init-keyword :class   :init-value #f)))
 
 (define-method write-object ((obj <test-error>) out)
-  (if (ref obj 'message)
-      (format out "#<error ~s>" (ref obj 'message))
-      (display "#<error>" out)))
-
-(define-method object-equal? ((x <test-error>) (y <test-error>))
-  #t)
-
-(define *test-error* (make <test-error>))
-
-(define *test-report-error*
-  (sys-getenv "GAUCHE_TEST_REPORT_ERROR"))
-
-(define *test-record-file*
-  (sys-getenv "GAUCHE_TEST_RECORD_FILE"))
+  (let1 cname (if (ref obj'class) (class-name (ref obj'class)) 'error)
+    (if (ref obj 'message)
+      (format out "#<~a ~s>" cname (ref obj 'message))
+      (format out "#<~a>" cname))))
 
 (define (test-error? obj) (is-a? obj <test-error>))
 
+(define (test-error . maybe-class)
+  (make <test-error> :class (and (pair? maybe-class) (car maybe-class))))
+
+(define (test-check expected result)
+  (cond [(test-error? expected)
+         (and (test-error? result)
+              (let ([ex (slot-ref expected'class)]
+                    [ey (slot-ref result'class)])
+                (or (not ex)
+                    (memq (slot-ref expected'class)
+                          (class-precedence-list (slot-ref result'class))))))]
+        [else (equal? expected result)]))
+
+(define *test-error* (make <test-error>)) ;DEPRECATED
+
+(define *test-report-error* (sys-getenv "GAUCHE_TEST_REPORT_ERROR"))
+
+(define *test-record-file* (sys-getenv "GAUCHE_TEST_RECORD_FILE"))
+
+(define (test-record-file file) (set! *test-record-file* file)) ;public API
+
 ;; List of discrepancies
 (define *discrepancy-list* '())
 
@@ -126,7 +145,7 @@
           (vector-ref *test-counts* 3)))
 (define (read-summary)
   (when (and (string? *test-record-file*)
-             (file-exists? *test-record-file*))
+             (sys-access *test-record-file* F_OK)) ; avoid file-exists? to trigger autoload
     (with-input-from-file *test-record-file*
       (lambda ()
         (let ((m (rxmatch #/Total:\s+(\d+)\s+tests,\s+(\d+)\s+passed,\s+(\d+)\s+failed,\s+(\d+)\s+aborted/ (read-line))))
@@ -155,7 +174,7 @@
 ;; Primitive test.  This doesn't use neither with-error-handler nor
 ;; object system, so it can be used _before_ those constructs are tested.
 (define (prim-test msg expect thunk . compare)
-  (let ((cmp (if (pair? compare) (car compare) equal?)))
+  (let ((cmp (if (pair? compare) (car compare) test-check)))
     (format/ss #t "test ~a, expects ~s ==> " msg expect)
     (flush)
     (test-count++)
@@ -179,6 +198,7 @@
                       (when *test-report-error*
                         (report-error e))
                       (make <test-error>
+                        :class (class-of e)
                         :message (if (is-a? e <message-condition>)
                                    (ref e 'message)
                                    e))))
@@ -196,69 +216,68 @@
 ;; Keyword argument :allow-undefined may take a list of symbols, which
 ;; is excluded from undefined variable check.
 
-(define (test-module module . opts)
+(define (test-module module :key (allow-undefined '()))
   (test-count++)
-  (let-keywords opts ((allow-undefined '()))
-    (let1 mod (cond ((module? module) module)
-                    ((symbol? module)
-                     (or (find-module module)
-                         (error "no such module" module)))
-                    (else
-                     (error "test-module requires module or symbol, but got"
-                            module)))
-      (format #t "testing bindings in ~a ... " mod) (flush)
-      (let ((bad-autoload '())
-            (bad-export '())
-            (bad-gref '())
-            (report '()))
-        ;; 1. Check if there's no dangling autoloads.
-        (hash-table-for-each (module-table mod)
-                             (lambda (sym val)
-                               (guard (_ (else (push! bad-autoload sym)))
-                                 (global-variable-ref mod sym))))
-        ;; 2. Check if all exported symbols are properly defined.
-        (when (pair? (module-exports mod))
-          (for-each (lambda (sym)
-                      (guard (_ (else (push! bad-export sym)))
-                        (global-variable-ref mod sym)))
-                    (module-exports mod)))
-        ;; 3. Check if all global references are resolvable.
-        (for-each
-         (lambda (closure)
-           (for-each (lambda (gref)
-                       (cond ((and (not (memq (slot-ref gref 'name)
-                                              allow-undefined))
-                                   (dangling-gref? gref closure))
-                              => (lambda (bad) (push! bad-gref bad)))))
-                     (closure-grefs closure)))
-         (toplevel-closures mod))
-        ;; report discrepancies
-        (unless (null? bad-autoload)
-          (push! report (format "found dangling autoloads: ~a" bad-autoload)))
-        (unless (null? bad-export)
-          (unless (null? report) (push! report " AND "))
-          (push! report
-                 (format "symbols exported but not defined: ~a" bad-export)))
-        (unless (null? bad-gref)
-          (unless (null? report) (push! report " AND "))
-          (push! report
-                 (format "symbols referenced but not defined: ~a"
-                         (string-join (map (lambda (z)
-                                             (format "~a(~a)" (car z) (cdr z)))
-                                           bad-gref)
-                                      ", "))))
-        (cond
-         ((null? report) (test-pass++) (format #t "ok\n"))
-         (else
-          (test-fail++)
-          (let ((s (apply string-append report)))
-            (format #t "ERROR: ~a\n" s)
-            (set! *discrepancy-list*
-                  (cons (list (format #f "bindings in module ~a" (module-name mod))
-                              '() s)
-                        *discrepancy-list*)))))
-        )
-      )))
+  (let1 mod (cond [(module? module) module]
+                  [(symbol? module)
+                   (or (find-module module)
+                       (error "no such module" module))]
+                  [else
+                   (error "test-module requires module or symbol, but got"
+                          module)])
+    (format #t "testing bindings in ~a ... " mod) (flush)
+    (let ((bad-autoload '())
+          (bad-export '())
+          (bad-gref '())
+          (report '()))
+      ;; 1. Check if there's no dangling autoloads.
+      (hash-table-for-each (module-table mod)
+                           (lambda (sym val)
+                             (guard (_ (else (push! bad-autoload sym)))
+                               (global-variable-ref mod sym))))
+      ;; 2. Check if all exported symbols are properly defined.
+      (when (pair? (module-exports mod))
+        (for-each (lambda (sym)
+                    (guard (_ (else (push! bad-export sym)))
+                      (global-variable-ref mod sym)))
+                  (module-exports mod)))
+      ;; 3. Check if all global references are resolvable.
+      (for-each
+       (lambda (closure)
+         (for-each (lambda (gref)
+                     (cond ((and (not (memq (slot-ref gref 'name)
+                                            allow-undefined))
+                                 (dangling-gref? gref closure))
+                            => (lambda (bad) (push! bad-gref bad)))))
+                   (closure-grefs closure)))
+       (toplevel-closures mod))
+      ;; report discrepancies
+      (unless (null? bad-autoload)
+        (push! report (format "found dangling autoloads: ~a" bad-autoload)))
+      (unless (null? bad-export)
+        (unless (null? report) (push! report " AND "))
+        (push! report
+               (format "symbols exported but not defined: ~a" bad-export)))
+      (unless (null? bad-gref)
+        (unless (null? report) (push! report " AND "))
+        (push! report
+               (format "symbols referenced but not defined: ~a"
+                       (string-join (map (lambda (z)
+                                           (format "~a(~a)" (car z) (cdr z)))
+                                         bad-gref)
+                                    ", "))))
+      (cond
+       [(null? report) (test-pass++) (format #t "ok\n")]
+       [else
+        (test-fail++)
+        (let ((s (apply string-append report)))
+          (format #t "ERROR: ~a\n" s)
+          (set! *discrepancy-list*
+                (cons (list (format #f "bindings in module ~a" (module-name mod))
+                            '() s)
+                      *discrepancy-list*)))])
+      )
+    ))
 
 ;; Auxiliary funcs to catch dangling grefs.  We use the fact that
 ;; an identifier embedded within the vm code is almost always an
@@ -346,4 +365,3 @@
     ;; Returns the number of failed tests.
     (length *discrepancy-list*)))
 
-(provide "gauche/test")
