@@ -3,8 +3,6 @@
 ;;;
 ;;; Written by Shiro Kawai
 ;;;
-;;; $Id: srfi-55.scm,v 1.3 2005/10/06 10:11:58 shirok Exp $
-;;;
 
 ;; This file is to be autoloaded
 
@@ -28,7 +26,7 @@
            (if (eq? (unwrap-syntax (caar clauses)) 'srfi)
              (require-srfi (cdar clauses) (cdr clauses))
              (error "require-extension: unknown extension identifier:"
-                    (car claues))))
+                    (car clauses))))
           (else
            (error "require-extension: bad clause:" (car clauses)))))
   (define (require-srfi ids rest)
@@ -41,7 +39,6 @@
                        ,(require-srfi (cdr ids) rest))))))
   (rec clauses))
 
-(provide "srfi-55")
 
 
 
