@@ -1,7 +1,7 @@
 ;;;
 ;;; html-lite.scm - lightweight HTML construction
 ;;;  
-;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,8 +30,6 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: html-lite.scm,v 1.18 2007/03/02 07:39:11 shirok Exp $
-;;;
 
 (define-module text.html-lite
   (use text.tree)
@@ -90,11 +88,10 @@
        \"http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd\">\n")
     ))
 
-(define (html-doctype . args)
-  (let-keywords args ((type :html-4.01-strict))
-    (cond ((find (lambda (e) (memq type (car e))) *doctype-alist*)
-           => caddr )
-          (else (error "Unknown doctype type spec" type)))))
+(define (html-doctype :key (type :html-4.01-strict))
+  (cond ((find (lambda (e) (memq type (car e))) *doctype-alist*)
+         => caddr )
+        (else (error "Unknown doctype type spec" type))))
 
 ;; Elements ------------------------------------------------
 
@@ -223,6 +220,5 @@
 ;; FRAMES
 (define-html-elements frameset frame noframes iframe)
 
-(provide "text/html-lite")
 
 
