@@ -1,7 +1,7 @@
 ;;;
 ;;; util/list.scm - more list library
 ;;;
-;;;  Copyright(C) 2003 by Shiro Kawai (shiro@acm.org)
+;;;  Copyright (c) 2003-2010  Shiro Kawai  <shiro@acm.org>
 ;;;  Copyright(C) 2003 by Alex Shinn (foof@synthcode.com)
 ;;;
 ;;;  Permission to use, copy, modify, distribute this software and
@@ -13,8 +13,6 @@
 ;;;  warranty.  In no circumstances the author(s) shall be liable
 ;;;  for any damages arising out of the use of this software.
 ;;;
-;;;  $Id: list.scm,v 1.12 2006/10/16 19:24:56 shirok Exp $
-;;;
 
 ;; This module adds useful list utility procedures that are not in SRFI-1.
 
@@ -34,21 +32,19 @@
 ;; permissive take and drop - if the length of given list is shorter
 ;; than index, returns shorter list or fills the rest.
 
-(define (split-at* lis k . args)
+(define (split-at* lis k :optional (fill? #f) (filler #f))
   (when (or (not (integer? k)) (negative? k))
     (error "index must be non-negative integer" k))
-  (let-optionals* args ((fill? #f)
-                        (filler #f))
-    (let loop ((i 0)
-               (lis lis)
-               (r '()))
-      (cond ((= i k) (values (reverse! r) lis))
-            ((null? lis)
-             (values (if fill?
-                         (append! (reverse! r) (make-list (- k i) filler))
-                         (reverse! r))
-                     lis))
-            (else (loop (+ i 1) (cdr lis) (cons (car lis) r)))))))
+  (let loop ((i 0)
+             (lis lis)
+             (r '()))
+    (cond [(= i k) (values (reverse! r) lis)]
+          [(null? lis)
+           (values (if fill?
+                     (append! (reverse! r) (make-list (- k i) filler))
+                     (reverse! r))
+                   lis)]
+          [else (loop (+ i 1) (cdr lis) (cons (car lis) r))])))
 
 (define (take* lis k . args)
   (receive (h t) (apply split-at* lis k args) h))
@@ -58,25 +54,21 @@
     (error "index must be non-negative integer" k))
   (let loop ((i 0)
              (lis lis))
-    (cond ((= i k) lis)
-          ((null? lis) '())
-          (else (loop (+ i 1) (cdr lis))))))
+    (cond [(= i k) lis]
+          [(null? lis) '()]
+          [else (loop (+ i 1) (cdr lis))])))
 
-(define (take-right* lis k . args)
+(define (take-right* lis k :optional (fill? #f) (filler #f))
   (when (or (not (integer? k)) (negative? k))
     (error "index must be non-negative integer" k))
-  (let-optionals* args ((fill? #f)
-                        (filler #f))
-    (let1 len (length lis)
-      (cond ((<= k len) (drop lis (- len k)))
-            (fill? (append! (make-list (- k len) filler) lis))
-            (else lis)))))
+  (let1 len (length lis)
+    (cond [(<= k len) (drop lis (- len k))]
+          [fill? (append! (make-list (- k len) filler) lis)]
+          [else lis])))
 
 (define (drop-right* lis k)
   (let1 len (length lis)
-    (if (<= k len)
-        (take lis (- len k))
-        '())))
+    (if (<= k len) (take lis (- len k)) '())))
 
 ;;-----------------------------------------------------------------
 ;; slices - split a list to a bunch of sublists of length k
@@ -154,11 +146,8 @@
   (hash-table-map h cons))
 
 ;; `reverse' alist search fn
-(define (rassoc key alist . opt-eq)
-  (let-optionals* opt-eq ((eq equal?))
-    (find (lambda (elt)
-            (and (pair? elt) (eq (cdr elt) key)))
-          alist)))
+(define (rassoc key alist :optional (eq equal?))
+  (find (lambda (elt) (and (pair? elt) (eq (cdr elt) key))) alist))
 
 (define rassq (cut rassoc <> <> eq?))
 (define rassv (cut rassoc <> <> eqv?))
@@ -166,22 +155,18 @@
 ;; 'assoc-ref', a shortcut of value retrieval w/ default value
 ;; Default parameter comes first, following the convention of
 ;; other *-ref functions.
-(define (assoc-ref alist key . opts)
-  (let-optionals* opts ((default #f)
-                        (eq      equal?))
-    (cond ((assoc key alist eq) => cdr)
-          (else default))))
+(define (assoc-ref alist key :optional (default #f) (eq equal?))
+  (cond [(assoc key alist eq) => cdr]
+        [else default]))
 
 (define (assq-ref alist key . opts)
   (assoc-ref alist key (get-optional opts #f) eq?))
 (define (assv-ref alist key . opts)
   (assoc-ref alist key (get-optional opts #f) eqv?))
 
-(define (rassoc-ref alist key . opts)
-  (let-optionals* opts ((default #f)
-                        (eq      equal?))
-    (cond ((rassoc key alist eq) => car)
-          (else default))))
+(define (rassoc-ref alist key :optional (default #f) (eq equal?))
+  (cond [(rassoc key alist eq) => car]
+        [else default]))
 
 (define (rassq-ref alist key . opts)
   (rassoc-ref alist key (get-optional opts #f) eq?))
@@ -189,13 +174,11 @@
   (rassoc-ref alist key (get-optional opts #f) eqv?))
 
 ;; 'assoc-set!'
-(define (assoc-set! alist key val . opt-eq)
-  (let-optionals* opt-eq ((eq equal?))
-    (cond ((assoc key alist eq)
-           => (lambda (p) (set-cdr! p val) alist))
-          (else (acons key val alist)))))
+(define (assoc-set! alist key val :optional (eq equal?))
+  (cond [(assoc key alist eq)
+         => (lambda (p) (set-cdr! p val) alist)]
+        [else (acons key val alist)]))
 
 (define assq-set!  (cut assoc-set! <> <> <> eq?))
 (define assv-set!  (cut assoc-set! <> <> <> eqv?))
 
-(provide "util/list")
