@@ -21,7 +21,6 @@
 ;;; makes error message a bit less direct, but cutting edges in such
 ;;; inner loop procedure has a considerable gain.
 ;;;
-;;; $Id: stream.scm,v 1.5 2005/08/23 10:44:05 shirok Exp $
 
 (define-module util.stream
   (use srfi-1)
@@ -82,9 +81,10 @@
     ((stream-cons obj strm)
      (%make-stream
       (delay
-        (if (not (stream? strm))
-          (error "attempt to stream-cons onto non-stream")
-          (cons obj strm)))))))
+        (let ((s strm))
+          (if (not (stream? s))
+            (error "attempt to stream-cons onto non-stream")
+            (cons obj s))))))))
 
 ;; STREAM-NULL? object -- #t if object is the null stream, #f otherwise
 (define-inline (stream-null? obj)
@@ -172,11 +172,13 @@
          (error "attempt to apply stream-filter to non-stream"))
         (else (stream-unfoldn
                (lambda (s)
-                 (values
-                  (stream-cdr s)
-                  (cond ((stream-null? s) '())
-                        ((pred? (stream-car s)) (list (stream-car s)))
-                        (else #f))))
+                 (cond
+                  ((stream-null? s)
+                   (values stream-null '()))
+                  ((pred? (stream-car s))
+                   (values (stream-cdr s) (list (stream-car s))))
+                  (else
+                   (values (stream-cdr s) #f))))
                strm
                1))))
 
@@ -236,13 +238,12 @@
      stream-null
      (stream-cons (car list) (list->stream (cdr list))))))
 
-(define (string->stream str . rest)
-  (let-optionals* rest ((tail stream-null))
-    (let loop ((i 0))
-      (stream-delay
-       (if (equal? i (string-length str))
-         tail
-         (stream-cons (string-ref str i) (loop (+ i 1))))))))
+(define (string->stream str :optional (tail stream-null))
+  (let loop ((i 0))
+    (stream-delay
+     (if (equal? i (string-length str))
+       tail
+       (stream-cons (string-ref str i) (loop (+ i 1)))))))
 
 (define stream->string (compose list->string stream->list))
 (define number->stream (compose string->stream number->string))
@@ -250,19 +251,17 @@
 (define stream->symbol (compose string->symbol stream->string))
 (define symbol->stream (compose string->stream symbol->string))
 
-(define (port->stream . rest)
-  (let-optionals* rest ((in (current-input-port))
-                        (reader read-char)
-                        (close-at-eof close-input-port))
-    (define (next)
-      (stream-delay
-       (let ((element (reader in)))
-         (cond
-          ((eof-object? element)
-           (when close-at-eof (close-at-eof in)) stream-null)
-          (else
-           (stream-cons element (next)))))))
-    (next)))
+(define (port->stream :optional (in (current-input-port))
+                      (reader read-char) (close-at-eof close-input-port))
+  (define (next)
+    (stream-delay
+     (let ((element (reader in)))
+       (cond
+        ((eof-object? element)
+         (when close-at-eof (close-at-eof in)) stream-null)
+        (else
+         (stream-cons element (next)))))))
+  (next))
 
 ;(define (make-output-port-char write close)
 ;  (make-output-port
@@ -503,13 +502,12 @@
      (stream-append (stream-car strs)
                     (stream-concatenate (stream-cdr strs))))))
 
-(define (stream-reverse str . args)
+(define (stream-reverse str :optional (tail stream-null))
   (stream-delay
-   (let-optionals* args ((tail stream-null))
-     (let loop ((head str) (tail tail))
-       (if (stream-null? head) 
-         tail
-         (loop (stream-cdr head) (stream-cons (stream-car head) tail)))))))
+   (let loop ((head str) (tail tail))
+     (if (stream-null? head) 
+       tail
+       (loop (stream-cdr head) (stream-cons (stream-car head) tail))))))
 
 ;; zip?
 
@@ -631,19 +629,18 @@
 ; (equal? tail stream-null) rather than (stream-null? tail) to avoid an
 ; off-by-one error (evaluating tail before obj is fully consumed).
 
-(define (->stream-char obj . rest)
+(define (->stream-char obj :optional (tail stream-null))
   (stream-delay
-   (let-optionals rest ((tail stream-null))
-     (cond
-      ((string? obj) (string->stream obj tail))
-      ((or (number? obj) (boolean? obj) (symbol? obj)) (->stream-char (->string obj) tail))
-      ((char? obj) (stream-cons obj tail))
-      ((port? obj) (port->stream obj))
-      ((stream? obj)
-       (if (equal? tail stream-null)
-           obj
-           (stream-append obj tail)))
-      (else (error "Unable to convert object to stream-char" obj))))))
+   (cond
+    ((string? obj) (string->stream obj tail))
+    ((or (number? obj) (boolean? obj) (symbol? obj)) (->stream-char (x->string obj) tail))
+    ((char? obj) (stream-cons obj tail))
+    ((port? obj) (port->stream obj))
+    ((stream? obj)
+     (if (equal? tail stream-null)
+       obj
+       (stream-append obj tail)))
+    (else (error "Unable to convert object to stream-char" obj)))))
 
 (define (stream-replace in reps)
   (if (stream-null? in)
@@ -656,12 +653,12 @@
 (define (stream-translate str from to)
   (stream-map (lambda (c) (if (equal? c from) to c)) str))
 
-(define (write-stream stream . rest)
-  (let-optionals rest ((port (current-output-port)) (writer write-char))
-    (let loop ((s stream))
-      (unless (stream-null? s)
-        (writer (stream-car s) port)
-        (loop (stream-cdr s))))))
+(define (write-stream stream :optional
+                      (port (current-output-port)) (writer write-char))
+  (let loop ((s stream))
+    (unless (stream-null? s)
+      (writer (stream-car s) port)
+      (loop (stream-cdr s)))))
 
 ;(define (stream-chomp stream . args)
 ;  (stream-delay
@@ -672,4 +669,3 @@
 ;        stream-null
 ;        (stream-cons (stream-car stream) (stream-chomp (stream-cdr stream) lastchar))))))
 
-(provide "util/stream")
