@@ -1,7 +1,7 @@
 ;;;
 ;;; cgi.scm - CGI utility
 ;;;  
-;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,8 +30,6 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: cgi.scm,v 1.34 2007/04/19 05:54:43 shirok Exp $
-;;;
 
 ;; Surprisingly, there's no ``formal'' definition of CGI.
 ;; The most reliable document I found is in <http://CGI-Spec.Golux.Com/>
@@ -177,30 +175,30 @@
 ;; API: cgi-parse-parameters &keyword query-string merge-cookies
 ;;                                    part-handlers content-type
 ;;                                    mime-input
-(define (cgi-parse-parameters . args)
-  (let-keywords args ((query-string #f)
-                      (merge-cookies #f)
-                      (part-handlers '())
-                      (content-type #f)
-                      (content-length #f)
-                      (mime-input #f))
-    (let* ((input (cond (query-string)
-                        ((or mime-input content-type) 'mime)
-                        (else
-                         (cgi-get-query content-length))))
-           (cookies (cond ((and merge-cookies (get-meta "HTTP_COOKIE"))
-                           => parse-cookie-string)
-                          (else '()))))
-      (append
-       (cond
-        ((eq? input 'mime)
-         (get-mime-parts part-handlers
-                         (or content-type (get-meta "CONTENT_TYPE"))
-                         (or content-length (get-meta "CONTENT_LENGTH"))
-                         (or mime-input (current-input-port))))
-        ((string-null? input) '())
-        (else (split-query-string input)))
-       (map (lambda (cookie) (list (car cookie) (cadr cookie))) cookies)))))
+(define (cgi-parse-parameters :key
+                              (query-string #f)
+                              (merge-cookies #f)
+                              (part-handlers '())
+                              (content-type #f)
+                              (content-length #f)
+                              (mime-input #f))
+  (let* ((input (cond (query-string)
+                      ((or mime-input content-type) 'mime)
+                      (else
+                       (cgi-get-query content-length))))
+         (cookies (cond ((and merge-cookies (get-meta "HTTP_COOKIE"))
+                         => parse-cookie-string)
+                        (else '()))))
+    (append
+     (cond
+      ((eq? input 'mime)
+       (get-mime-parts part-handlers
+                       (or content-type (get-meta "CONTENT_TYPE"))
+                       (or content-length (get-meta "CONTENT_LENGTH"))
+                       (or mime-input (current-input-port))))
+      ((string-null? input) '())
+      (else (split-query-string input)))
+     (map (lambda (cookie) (list (car cookie) (cadr cookie))) cookies))))
 
 (define (split-query-string input)
   (fold-right (lambda (elt params)
@@ -232,6 +230,8 @@
 ;;  <options>   keyword-value list. 
 ;;               :prefix "prefix"  - use "prefix" for temporary file
 ;;                                   (file and file+name only)
+;;               :mode   <mode>    - set tmpfile's mode to <mode>
+;;                                   (file and file+name only)
 ;;               :max-length integer - limit the content-length of the
 ;;                                   part.  if it is exceeded,
 ;;                                   <cgi-request-size-error> is
@@ -240,12 +240,13 @@
   (define (part-ref info name)
     (rfc822-header-ref (ref info 'headers) name))
 
-  (define (make-file-handler prefix honor-origfile?)
+  (define (make-file-handler prefix honor-origfile? mode)
     (lambda (name filename part-info inp)
       (receive (outp tmpfile) (sys-mkstemp prefix)
         (cgi-add-temporary-file tmpfile)
         (mime-retrieve-body part-info inp outp)
         (close-output-port outp)
+        (when mode (sys-chmod tmpfile mode))
         (if honor-origfile?
           (list tmpfile filename)
           tmpfile))))
@@ -267,26 +268,27 @@
                              (string=? (x->string (car entry)) part-name)))
                        part-handlers)
       (cond
-       ((or (not clause)
+       [(or (not clause)
             (not (pair? (cdr clause))))
-        (list string-handler)) ;; default action
-       ((and (= (length clause) 3)
+        (list string-handler)] ;; default action
+       [(and (= (length clause) 3)
              (memq (cadr clause) '(file file+name))
              (string? (caddr clause)))
         ;; backward compatibility - will be deleted soon
-        (list (cadr clause) :prefix (caddr clause)))
-       (else (cdr clause)))))
+        (list (cadr clause) :prefix (caddr clause))]
+       [else (cdr clause)])))
                
   (define (get-handler action . opts)
     (cond
-     ((not action) string-handler)
-     ((memq action '(file file+name))
+     [(not action) string-handler]
+     [(memq action '(file file+name))
       (make-file-handler (get-keyword* :prefix opts
                                        (build-path (temporary-directory)
                                                    "gauche-cgi-"))
-                         (eq? action 'file+name)))
-     ((eq? action 'ignore) ignore-handler)
-     (else action)))
+                         (eq? action 'file+name)
+                         (get-keyword :mode opts #f))]
+     [(eq? action 'ignore) ignore-handler]
+     [else action]))
 
   ;; The value of content-disposition must be a properly quoted string
   ;; according to RFC2183 and RFC2045.  However, IE sends a pathname including
@@ -298,17 +300,16 @@
     (let1 r (open-output-string :private? #t)
       (define (finish) (get-output-string r))
       (let loop ((c (read-char input)))
-        (cond ((eof-object? c) (finish)) ; tolerate missing closing DQUOTE
-              ((char=? c #\")  (finish)) ; discard ending DQUOTE
-              ((char=? c #\\)
+        (cond [(eof-object? c) (finish)] ; tolerate missing closing DQUOTE
+              [(char=? c #\")  (finish)] ; discard ending DQUOTE
+              [(char=? c #\\)
                (let1 c (read-char input)
-                 (cond ((eof-object? c) (finish)) ;; tolerate stray backslash
-                       (else
-                        (unless (char-set-contains? #[\\\"] c)
-                          (write-char #\\ r))
-                        (write-char c r)
-                        (loop (read-char input))))))
-              (else (write-char c r) (loop (read-char input)))))))
+                 (cond [(eof-object? c) (finish)] ;; tolerate stray backslash
+                       [else (unless (char-set-contains? #[\\\"] c)
+                               (write-char #\\ r))
+                             (write-char c r)
+                             (loop (read-char input))]))]
+              [else (write-char c r) (loop (read-char input))]))))
 
   (define (parse-content-disposition field)
     (if field
@@ -317,88 +318,92 @@
                               (,*rfc822-atext-chars* . ,rfc822-dot-atom)))
       '()))
 
-  (define (handle-part part-info inp)
-    (let* ((cd   (part-ref part-info "content-disposition"))
-           (opts (parse-content-disposition cd))
-           (name (cond ((member "name=" opts) => cadr) (else #f)))
-           (filename (cond ((member "filename=" opts) => cadr) (else #f)))
-           )
-      (cond
-       ((not name)      ;; if no name is given, just ignore this part.
-        (ignore-handler name filename part-info inp)
-        #f)
-       ((not filename)  ;; this is not a file uploading field.
-        (list name (string-handler name filename part-info inp)))
-       ((string-null? filename) ;; file field is empty
-        (list name (ignore-handler name filename part-info inp)))
-       (else
-        (let* ((action&opts (get-action&opts name))
-               (handler (apply get-handler action&opts))
-               (result (handler name filename part-info inp)))
-          (list name result))))))
+  ;; NB: This is not an ideal solution.  There could be a malformed
+  ;; option string (e.g. filename=c:\foo\bar\baz, in which the option
+  ;; value should be quoted but we have to accept those non-conforming
+  ;; clients).
+  (define (get-option optname optregex opts)
+    (cond [(member optname opts) => cadr]
+          [(any (^t (and (string? t) (rxmatch optregex t))) opts)
+           => (cut rxmatch-after <>)]
+          [else #f]))
 
-  (let* ((inp (if (and clength (<= 0 (x->integer clength)))
+  (define (handle-part part-info inp)
+    (let* ([cd   (part-ref part-info "content-disposition")]
+           [opts (parse-content-disposition cd)]
+           [name (get-option "name=" #/^name=/ opts)]
+           [filename (get-option "filename=" #/^filename=/ opts)])
+      (cond
+       [(not name)      ;; if no name is given, just ignore this part.
+        (ignore-handler name filename part-info inp)
+        #f]
+       [(not filename)  ;; this is not a file uploading field.
+        (list name (string-handler name filename part-info inp))]
+       [(string-null? filename) ;; file field is empty
+        (list name (ignore-handler name filename part-info inp))]
+       [else
+        (let* ([action&opts (get-action&opts name)]
+               [handler (apply get-handler action&opts)]
+               [result (handler name filename part-info inp)])
+          (list name result))])))
+
+  (let* ([inp (if (and clength (<= 0 (x->integer clength)))
                 (open-input-limited-length-port inp (x->integer clength))
-                inp))
-         (result (mime-parse-message inp `(("content-type" ,ctype))
-                                     handle-part)))
-    (filter-map (cut ref <> 'content) (ref result 'content)))
-  )
+                inp)]
+         [result (mime-parse-message inp `(("content-type" ,ctype))
+                                     handle-part)])
+    (filter-map (cut ~ <>'content) (~ result'content))))
 
 ;;----------------------------------------------------------------
 ;; API: cgi-get-parameter key params &keyword list default convert
 ;;
-(define (cgi-get-parameter key params . args)
-  (let* ((list?   (get-keyword :list args #f))
-         (default (get-keyword :default args (if list? '() #f)))
-         (convert (get-keyword :convert args identity)))
-    (cond ((assoc key params)
-           => (lambda (p)
-                (if list?
-                    (map convert (cdr p))
-                    (convert (cadr p)))))
-          (else default))))
+(define (cgi-get-parameter key params :key ((:list lis) #f)
+                                           (default (if lis '() #f))
+                                           ((:convert cv) identity))
+  (cond [(assoc key params) => (^p (if lis (map cv (cdr p)) (cv (cadr p))))]
+        [else default]))
 
 ;;----------------------------------------------------------------
 ;; API: cgi-header &keyword content-type cookies location &allow-other-keys
 ;;
-(define (cgi-header . args)
-  (let-keywords args ((content-type #f)
-                      (location     #f)
-                      (status       #f)
-                      (cookies      '())
-                      . rest)
-    (let ((ct (or content-type
-                  (and (not location) "text/html")))
-          (r '()))
-      (when status   (push! r #`"Status: ,status\r\n"))
-      (when ct       (push! r #`"Content-type: ,ct\r\n"))
-      (when location (push! r #`"Location: ,location\r\n"))
-      (for-each (lambda (cookie)
-                  (push! r #`"Set-cookie: ,cookie\r\n"))
-                cookies)
-      (for-each (lambda (p)
-                  (when (pair? (cdr p))
-                    (push! r #`",(car p): ,(cadr p)\r\n")))
-                (slices rest 2))
-      (push! r "\r\n")
-      (reverse r))))
+(define (cgi-header :key
+                    (content-type #f)
+                    (location     #f)
+                    (status       #f)
+                    (cookies      '())
+                    :allow-other-keys rest)
+  (let ([ct (or content-type
+                (and (not location) "text/html"))]
+        [r '()])
+    (when status   (push! r #`"Status: ,status\r\n"))
+    (when ct       (push! r #`"Content-type: ,ct\r\n"))
+    (when location (push! r #`"Location: ,location\r\n"))
+    (for-each (lambda (cookie)
+                (push! r #`"Set-cookie: ,cookie\r\n"))
+              cookies)
+    (for-each (lambda (p)
+                (when (pair? (cdr p))
+                  (push! r #`",(car p): ,(cadr p)\r\n")))
+              (slices rest 2))
+    (push! r "\r\n")
+    (reverse r)))
       
 ;;----------------------------------------------------------------
 ;; API: cgi-main proc &keyword on-error merge-cookies
 ;;                             output-proc part-handlers
-(define (cgi-main proc . args)
-  (let-keywords* args ((on-error cgi-default-error-proc)
-                       (output-proc cgi-default-output)
-                       (merge-cookies #f)
-                       (part-handlers '()))
-    (guard (e (else (output-proc (on-error e))))
-      (let1 params (cgi-parse-parameters :merge-cookies merge-cookies
-                                         :part-handlers part-handlers)
-        (output-proc (proc params))))
-    ;; remove any temporary files
-    (for-each sys-unlink (cgi-temporary-files))
-    0))
+(define (cgi-main proc :key
+                  (on-error cgi-default-error-proc)
+                  (output-proc cgi-default-output)
+                  (merge-cookies #f)
+                  (part-handlers '()))
+  (set! (port-buffering (current-error-port)) :line)
+  (guard (e (else (output-proc (on-error e))))
+    (let1 params (cgi-parse-parameters :merge-cookies merge-cookies
+                                       :part-handlers part-handlers)
+      (output-proc (proc params))))
+  ;; remove any temporary files
+  (for-each sys-unlink (cgi-temporary-files))
+  0)
 
 ;; aux fns
 (define (cgi-default-error-proc e)
@@ -421,4 +426,3 @@
      (cut write-tree tree)
      :encoding (cgi-output-character-encoding))))
 
-(provide "www/cgi")
