@@ -1,7 +1,7 @@
 ;;;
 ;;; parameter.scm - parameter support
 ;;;  
-;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,8 +30,6 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: parameter.scm,v 1.2 2007/03/02 07:39:12 shirok Exp $
-;;;
 
 ;; The API is upper-compatible to ChezScheme and Chicken's.
 
@@ -95,6 +93,10 @@
           (error "wrong number of arguments for parameter" maybe-newval))
       ((slot-ref self 'getter))))
 
+;; Allow (set! (parameter) value).  By KOGURO, Naoki
+(define-method (setter object-apply) ((obj <parameter>) value)
+  (obj value))
+
 (define (make-parameter value . maybe-filter)
   (let1 p (make <parameter> :filter (get-optional maybe-filter #f))
     (p value)
@@ -148,7 +150,6 @@
       (delete-hook! (parameter-post-observers self) proc))
     ))
 
-(provide "gauche/parameter")
 
      
      
