@@ -1,7 +1,7 @@
 ;;;
 ;;; srfi-13.scm - string library
 ;;;  
-;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,8 +30,6 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: srfi-13.scm,v 1.7 2007/08/01 08:03:31 shirok Exp $
-;;;
 
 ;; Natively implemented functions:
 ;;   string? make-string string string->list list->string
@@ -80,17 +78,17 @@
           ))
 (select-module srfi-13)
 
-;; utility macro to handle char/char-set/pred match.
-(define-syntax %get-char-pred
-  (syntax-rules ()
-    ((_ char/char-set/pred)
-     (cond ((char? char/char-set/pred)
-            (lambda (x) (char=? char/char-set/pred x)))
-           ((char-set? char/char-set/pred)
-            (lambda (x) (char-set-contains? char/char-set/pred x)))
-           ((procedure? char/char-set/pred) char/char-set/pred)
-           (else (error "argument needs to be either a character, a char-set, or a procedure:" char/char-set/pred))))
-    ))
+;; Utility macro to handle char/char-set/pred match.
+;; To avoid closure creation in inner loop, this returns a static procedure
+;; that takes char/char-set/pred and char.
+(define-inline (%get-char-pred char/char-set/pred)
+  (cond [(char? char/char-set/pred) char=?]
+        [(char-set? char/char-set/pred) char-set-contains?]
+        [(procedure? char/char-set/pred) %char-pred/pred]
+        [else (error "argument needs to be either a character, a char-set, \
+                      or a procedure:" char/char-set/pred)]))
+
+(define (%char-pred/pred c/s/p x) (c/s/p x))
 
 ;;;
 ;;; Predicates
@@ -107,14 +105,14 @@
                (r  #t))
       (cond ((not r) #f)
             ((eof-object? ch) r)
-            (else (loop (read-char src) (pred ch)))))))
+            (else (loop (read-char src) (pred c/s/p ch)))))))
 
 (define (string-any c/s/p s . args)
   (let* ((src  (open-input-string (apply %maybe-substring s args)))
          (pred (%get-char-pred c/s/p)))
     (let loop ((ch (read-char src)))
       (cond ((eof-object? ch) #f)
-            ((pred ch))
+            ((pred c/s/p ch))
             (else (loop (read-char src)))))))
 
 ;;;
@@ -154,27 +152,25 @@
                                                        (+ tstart slen)
                                                        tlen))))))
 
-(define (string-pad s len . args)
-  (let-optionals* args ((char #\space) start end)
-    (check-arg char? char)
-    (let* ((str (%maybe-substring s start end))
-           (slen (string-length str)))
-      (cond ((< slen len)
-             (string-append (make-string (- len slen) char) str))
-            ((> slen len)
-             (string-take-right str len))
-            (else str)))))
+(define (string-pad s len :optional (char #\space) start end)
+  (check-arg char? char)
+  (let* ((str (%maybe-substring s start end))
+         (slen (string-length str)))
+    (cond ((< slen len)
+           (string-append (make-string (- len slen) char) str))
+          ((> slen len)
+           (string-take-right str len))
+          (else str))))
 
-(define (string-pad-right s len . args)
-  (let-optionals* args ((char #\space) start end)
-    (check-arg char? char)
-    (let* ((str (%maybe-substring s start end))
-           (slen (string-length str)))
-      (cond ((< slen len)
-             (string-append str (make-string (- len slen) char)))
-            ((> slen len)
-             (string-take str len))
-            (else str)))))
+(define (string-pad-right s len :optional (char #\space) start end)
+  (check-arg char? char)
+  (let* ((str (%maybe-substring s start end))
+         (slen (string-length str)))
+    (cond ((< slen len)
+           (string-append str (make-string (- len slen) char)))
+          ((> slen len)
+           (string-take str len))
+          (else str))))
 
 (define (string-take s nchars)
   (check-arg string? s)
@@ -200,51 +196,45 @@
     (error "argument out of range:" nchars))
   (%maybe-substring s 0 (- (string-length s) nchars)))
 
-(define (string-trim s . args)
+(define (string-trim s :optional (c/s/p #[\s]) start end)
   (check-arg string? s)
-  (let-optionals* args ((c/s/p #[\s]) start end)
-    (let ((pred (%get-char-pred c/s/p))
-          (sp (make-string-pointer (%maybe-substring s start end))))
-      (let loop ((ch (string-pointer-next! sp)))
-        (cond ((eof-object? ch) "")
-              ((pred ch) (loop (string-pointer-next! sp)))
-              (else (string-pointer-prev! sp)
-                    (string-pointer-substring sp :after #t))))
-      ))
-  )
+  (let ((pred (%get-char-pred c/s/p))
+        (sp (make-string-pointer (%maybe-substring s start end))))
+    (let loop ((ch (string-pointer-next! sp)))
+      (cond ((eof-object? ch) "")
+            ((pred c/s/p ch) (loop (string-pointer-next! sp)))
+            (else (string-pointer-prev! sp)
+                  (string-pointer-substring sp :after #t))))
+    ))
 
-(define (string-trim-right s . args)
+(define (string-trim-right s :optional (c/s/p #[\s]) start end)
   (check-arg string? s)
-  (let-optionals* args ((c/s/p #[\s]) start end)
-    (let ((pred (%get-char-pred c/s/p))
-          (sp (make-string-pointer (%maybe-substring s start end) -1)))
-      (let loop ((ch (string-pointer-prev! sp)))
-        (cond ((eof-object? ch) "")
-              ((pred ch) (loop (string-pointer-prev! sp)))
-              (else (string-pointer-next! sp)
-                    (string-pointer-substring sp))))
-      ))
-  )
+  (let ((pred (%get-char-pred c/s/p))
+        (sp (make-string-pointer (%maybe-substring s start end) -1)))
+    (let loop ((ch (string-pointer-prev! sp)))
+      (cond ((eof-object? ch) "")
+            ((pred c/s/p ch) (loop (string-pointer-prev! sp)))
+            (else (string-pointer-next! sp)
+                  (string-pointer-substring sp))))
+    ))
 
-(define (string-trim-both s . args)
+(define (string-trim-both s :optional (c/s/p #[\s]) start end)
   (check-arg string? s)
-  (let-optionals* args ((c/s/p #[\s]) start end)
-    (let ((pred (%get-char-pred c/s/p))
-          (sp (make-string-pointer (%maybe-substring s start end))))
-      (let loop ((ch (string-pointer-next! sp)))
-        (cond ((eof-object? ch) "")
-              ((pred ch) (loop (string-pointer-next! sp)))
-              (else (string-pointer-prev! sp)
-                    (let ((sp (make-string-pointer
-                               (string-pointer-substring sp :after #t) -1)))
-                      (let loop ((ch (string-pointer-prev! sp)))
-                        (cond ((eof-object? ch) "")
-                              ((pred ch) (loop (string-pointer-prev! sp)))
-                              (else (string-pointer-next! sp)
-                                    (string-pointer-substring sp))))
-                      ))))
-      ))
-  )
+  (let ((pred (%get-char-pred c/s/p))
+        (sp (make-string-pointer (%maybe-substring s start end))))
+    (let loop ((ch (string-pointer-next! sp)))
+      (cond ((eof-object? ch) "")
+            ((pred c/s/p ch) (loop (string-pointer-next! sp)))
+            (else (string-pointer-prev! sp)
+                  (let ((sp (make-string-pointer
+                             (string-pointer-substring sp :after #t) -1)))
+                    (let loop ((ch (string-pointer-prev! sp)))
+                      (cond ((eof-object? ch) "")
+                            ((pred c/s/p ch) (loop (string-pointer-prev! sp)))
+                            (else (string-pointer-next! sp)
+                                  (string-pointer-substring sp))))
+                    ))))
+    ))
 
 ;;;
 ;;; Comparison functions
@@ -267,25 +257,25 @@
                         (string-pointer-next! sp2))))
       )))
 
-(define (string-compare s1 s2 proc< proc= proc> . args)
+(define (string-compare s1 s2 proc< proc= proc>
+                        :optional (start1 0) end1 start2 end2)
   (check-arg string? s1)
   (check-arg string? s2)
-  (let-optionals* args ((start1 0) end1 start2 end2)
-    (let ((str1 (%maybe-substring s1 start1 end1))
-          (str2 (%maybe-substring s2 start2 end2)))
-      (%string-compare-int (- start1 1) str1 str2
-                           char<? char>?
-                           proc< proc= proc>))))
+  (let ((str1 (%maybe-substring s1 start1 end1))
+        (str2 (%maybe-substring s2 start2 end2)))
+    (%string-compare-int (- start1 1) str1 str2
+                         char<? char>?
+                         proc< proc= proc>)))
 
-(define (string-compare-ci s1 s2 proc< proc= proc> . args)
+(define (string-compare-ci s1 s2 proc< proc= proc>
+                           :optional (start1 0) end1 start2 end2)
   (check-arg string? s1)
   (check-arg string? s2)
-  (let-optionals* args ((start1 0) end1 start2 end2)
-    (let ((str1 (%maybe-substring s1 start1 end1))
-          (str2 (%maybe-substring s2 start2 end2)))
-      (%string-compare-int (- start1 1) str1 str2
-                           char-ci<? char-ci>?
-                           proc< proc= proc>))))
+  (let ((str1 (%maybe-substring s1 start1 end1))
+        (str2 (%maybe-substring s2 start2 end2)))
+    (%string-compare-int (- start1 1) str1 str2
+                         char-ci<? char-ci>?
+                         proc< proc= proc>)))
 
 (define (string= s1 s2 . args)
   (check-arg string? s1)
@@ -375,16 +365,14 @@
 ;;; Hash functions
 ;;;
 
-(define (string-hash s . args)
+(define (string-hash s :optional bound start end)
   (check-arg string? s)
-  (let-optionals* args (bound start end)
-    (%hash-string (%maybe-substring s start end) bound)))
+  (%hash-string (%maybe-substring s start end) bound))
 
-(define (string-hash-ci s . args)
+(define (string-hash-ci s :optional bound start end)
   (check-arg string? s)
-  (let-optionals* args (bound start end)
-    ;; oops! optimization required!
-    (%hash-string (string-upcase (%maybe-substring s start end)) bound)))
+  ;; oops! optimization required!
+  (%hash-string (string-upcase (%maybe-substring s start end)) bound))
 
 ;;;
 ;;; Prefix and suffix
@@ -402,38 +390,33 @@
             (else (action (- (string-pointer-index sp1) 1) #f)))
       )))
 
-(define (string-prefix-length s1 s2 . args)
+(define (string-prefix-length s1 s2 :optional start1 end1 start2 end2)
   (check-arg string? s1)
   (check-arg string? s2)
-  (let-optionals* args (start1 end1 start2 end2)
-    (let ((str1 (%maybe-substring s1 start1 end1))
-          (str2 (%maybe-substring s2 start2 end2)))
-      (%string-prefix-int str1 str2 char=? (lambda (cnt flag) cnt)))))
+  (let ((str1 (%maybe-substring s1 start1 end1))
+        (str2 (%maybe-substring s2 start2 end2)))
+    (%string-prefix-int str1 str2 char=? (lambda (cnt flag) cnt))))
 
-(define (string-prefix-length-ci s1 s2 . args)
+(define (string-prefix-length-ci s1 s2 :optional start1 end1 start2 end2)
   (check-arg string? s1)
   (check-arg string? s2)
-  (let-optionals* args (start1 end1 start2 end2)
-    (let ((str1 (%maybe-substring s1 start1 end1))
-          (str2 (%maybe-substring s2 start2 end2)))
-      (%string-prefix-int str1 str2 char-ci=? (lambda (cnt flag) cnt)))))
+  (let ((str1 (%maybe-substring s1 start1 end1))
+        (str2 (%maybe-substring s2 start2 end2)))
+    (%string-prefix-int str1 str2 char-ci=? (lambda (cnt flag) cnt))))
 
-(define (string-prefix? s1 s2 . args)
+(define (string-prefix? s1 s2 :optional start1 end1 start2 end2)
   (check-arg string? s1)
   (check-arg string? s2)
-  (let-optionals* args (start1 end1 start2 end2)
-    (let ((str1 (%maybe-substring s1 start1 end1))
-          (str2 (%maybe-substring s2 start2 end2)))
-      (%string-prefix-int str1 str2 char=? (lambda (cnt flag) flag)))))
+  (let ((str1 (%maybe-substring s1 start1 end1))
+        (str2 (%maybe-substring s2 start2 end2)))
+    (%string-prefix-int str1 str2 char=? (lambda (cnt flag) flag))))
 
-(define (string-prefix-ci? s1 s2 . args)
+(define (string-prefix-ci? s1 s2 :optional start1 end1 start2 end2)
   (check-arg string? s1)
   (check-arg string? s2)
-  (let-optionals* args (start1 end1 start2 end2)
-    (let ((str1 (%maybe-substring s1 start1 end1))
-          (str2 (%maybe-substring s2 start2 end2)))
-      (%string-prefix-int str1 str2 char-ci=? (lambda (cnt flag) flag)))))
-
+  (let ((str1 (%maybe-substring s1 start1 end1))
+        (str2 (%maybe-substring s2 start2 end2)))
+    (%string-prefix-int str1 str2 char-ci=? (lambda (cnt flag) flag))))
 
 
 (define (%string-suffix-int str1 str2 = action)
@@ -450,37 +433,33 @@
             (else (action count #f)))
       )))
 
-(define (string-suffix-length s1 s2 . args)
+(define (string-suffix-length s1 s2 :optional start1 end1 start2 end2)
   (check-arg string? s1)
   (check-arg string? s2)
-  (let-optionals* args (start1 end1 start2 end2)
-    (let ((str1 (%maybe-substring s1 start1 end1))
-          (str2 (%maybe-substring s2 start2 end2)))
-      (%string-suffix-int str1 str2 char=? (lambda (cnt flag) cnt)))))
+  (let ((str1 (%maybe-substring s1 start1 end1))
+        (str2 (%maybe-substring s2 start2 end2)))
+    (%string-suffix-int str1 str2 char=? (lambda (cnt flag) cnt))))
 
-(define (string-suffix-length-ci s1 s2 . args)
+(define (string-suffix-length-ci s1 s2 :optional start1 end1 start2 end2)
   (check-arg string? s1)
   (check-arg string? s2)
-  (let-optionals* args (start1 end1 start2 end2)
-    (let ((str1 (%maybe-substring s1 start1 end1))
-          (str2 (%maybe-substring s2 start2 end2)))
-      (%string-suffix-int str1 str2 char-ci=? (lambda (cnt flag) cnt)))))
+  (let ((str1 (%maybe-substring s1 start1 end1))
+        (str2 (%maybe-substring s2 start2 end2)))
+    (%string-suffix-int str1 str2 char-ci=? (lambda (cnt flag) cnt))))
 
-(define (string-suffix? s1 s2 . args)
+(define (string-suffix? s1 s2 :optional start1 end1 start2 end2)
   (check-arg string? s1)
   (check-arg string? s2)
-  (let-optionals* args (start1 end1 start2 end2)
-    (let ((str1 (%maybe-substring s1 start1 end1))
-          (str2 (%maybe-substring s2 start2 end2)))
-      (%string-suffix-int str1 str2 char=? (lambda (cnt flag) flag)))))
+  (let ((str1 (%maybe-substring s1 start1 end1))
+        (str2 (%maybe-substring s2 start2 end2)))
+    (%string-suffix-int str1 str2 char=? (lambda (cnt flag) flag))))
 
-(define (string-suffix-ci? s1 s2 . args)
+(define (string-suffix-ci? s1 s2 :optional start1 end1 start2 end2)
   (check-arg string? s1)
   (check-arg string? s2)
-  (let-optionals* args (start1 end1 start2 end2)
-    (let ((str1 (%maybe-substring s1 start1 end1))
-          (str2 (%maybe-substring s2 start2 end2)))
-      (%string-suffix-int str1 str2 char-ci=? (lambda (cnt flag) flag)))))
+  (let ((str1 (%maybe-substring s1 start1 end1))
+        (str2 (%maybe-substring s2 start2 end2)))
+    (%string-suffix-int str1 str2 char-ci=? (lambda (cnt flag) flag))))
 
 ;;;
 ;;; Search
@@ -493,7 +472,7 @@
         (sp (apply make-string-pointer s 0 args)))
     (let loop ((ch (string-pointer-next! sp)))
       (cond ((eof-object? ch) #f)
-            ((pred ch) (+ offset (- (string-pointer-index sp) 1)))
+            ((pred c/s/p ch) (+ offset (- (string-pointer-index sp) 1)))
             (else (loop (string-pointer-next! sp)))))))
 
 (define (string-index-right s c/s/p . args)
@@ -503,7 +482,7 @@
         (sp (apply make-string-pointer s -1 args)))
     (let loop ((ch (string-pointer-prev! sp)))
       (cond ((eof-object? ch) #f)
-            ((pred ch) (+ offset (string-pointer-index sp)))
+            ((pred c/s/p ch) (+ offset (string-pointer-index sp)))
             (else (loop (string-pointer-prev! sp)))))))
 
 (define (string-skip s c/s/p . args)
@@ -513,7 +492,7 @@
         (sp (apply make-string-pointer s 0 args)))
     (let loop ((ch (string-pointer-next! sp)))
       (cond ((eof-object? ch) #f)
-            ((pred ch) (loop (string-pointer-next! sp)))
+            ((pred c/s/p ch) (loop (string-pointer-next! sp)))
             (else (+ offset (- (string-pointer-index sp) 1)))))))
 
 (define (string-skip-right s c/s/p . args)
@@ -523,7 +502,7 @@
         (sp (apply make-string-pointer s -1 args)))
     (let loop ((ch (string-pointer-prev! sp)))
       (cond ((eof-object? ch) #f)
-            ((pred ch) (loop (string-pointer-prev! sp)))
+            ((pred c/s/p ch) (loop (string-pointer-prev! sp)))
             (else (+ offset (string-pointer-index sp)))))))
 
 (define (string-count s c/s/p . args)
@@ -533,27 +512,25 @@
     (let loop ((ch (string-pointer-next! sp))
                (count 0))
       (cond ((eof-object? ch) count)
-            ((pred ch) (loop (string-pointer-next! sp) (+ count 1)))
+            ((pred c/s/p ch) (loop (string-pointer-next! sp) (+ count 1)))
             (else      (loop (string-pointer-next! sp) count))))))
 
-(define (string-contains s1 s2 . args)
+(define (string-contains s1 s2 :optional (start1 0) end1 start2 end2)
   (check-arg string? s1)
   (check-arg string? s2)
-  (let-optionals* args ((start1 0) end1 start2 end2)
-    (let* ((str1 (%maybe-substring s1 start1 end1))
-           (str2 (%maybe-substring s2 start2 end2))
-           (res  (string-scan str1 str2)))
-      (and res (+ start1 res)))))
+  (let* ((str1 (%maybe-substring s1 start1 end1))
+         (str2 (%maybe-substring s2 start2 end2))
+         (res  (string-scan str1 str2)))
+    (and res (+ start1 res))))
 
 ;; not tuned (maybe to be moved to native)
-(define (string-contains-ci s1 s2 . args)
+(define (string-contains-ci s1 s2 :optional (start1 0) end1 start2 end2)
   (check-arg string? s1)
   (check-arg string? s2)
-  (let-optionals* args ((start1 0) end1 start2 end2)
-    (let* ((str1 (string-upcase (%maybe-substring s1 start1 end1)))
-           (str2 (string-upcase (%maybe-substring s2 start2 end2)))
-           (res  (string-scan str1 str2)))
-      (and res (+ start1 res)))))
+  (let* ((str1 (string-upcase (%maybe-substring s1 start1 end1)))
+         (str2 (string-upcase (%maybe-substring s2 start2 end2)))
+         (res  (string-scan str1 str2)))
+    (and res (+ start1 res))))
 
 ;;;
 ;;; Case mapping
@@ -596,11 +573,10 @@
       )))
 
 (define (%string-*case! converter)
-  (lambda (s . args)
-    (let-optionals* args ((start 0) (end (string-length s)))
-      (if (and (= start 0) (= end (string-length s)))
-        (%string-replace-body! s (converter s))
-        (string-copy! s start (converter s start end))))))
+  (lambda (s :optional (start 0) (end (string-length s)))
+    (if (and (= start 0) (= end (string-length s)))
+      (%string-replace-body! s (converter s))
+      (string-copy! s start (converter s start end)))))
 
 (define string-upcase!    (%string-*case! string-upcase))
 (define string-downcase!  (%string-*case! string-downcase))
@@ -621,9 +597,8 @@
     ))
 
 (define (string-reverse! s . args)
-  (let-optionals* args ((start 0) end)
-    (let ((rev (apply string-reverse s args)))
-      (string-copy! s start rev))))
+  (let ((rev (apply string-reverse s args)))
+    (string-copy! s (get-optional args 0) rev)))
 
 (define (string-concatenate lis)
   (cond
@@ -682,9 +657,8 @@
 (define (string-map! proc s . args)
   (check-arg procedure? proc)
   (check-arg string? s)
-  (let-optionals* args ((start 0) end)
-     (let ((mapped (apply string-map proc s args)))
-       (string-copy! s start mapped))))
+  (let ((mapped (apply string-map proc s args)))
+    (string-copy! s (get-optional args 0) mapped)))
 
 (define (string-fold kons knil s . args)
   (check-arg procedure? kons)
@@ -708,35 +682,33 @@
         (loop (string-pointer-prev! src) (kons ch r))))
     ))
 
-(define (string-unfold p f g seed . args)
+(define (string-unfold p f g seed
+                       :optional (base "") (make-final (lambda (_) "")))
   (check-arg procedure? p)
   (check-arg procedure? f)
   (check-arg procedure? g)
-  (let-optionals* args ((base "") (make-final (lambda (_) "")))
-    (let ((dest (open-output-string)))
-      (display base dest)
-      (let loop ((seed seed))
-        (if (p seed)
-          (begin (display (make-final seed) dest)
-                 (get-output-string dest))
-          (begin (write-char (f seed) dest)
-                 (loop (g seed))))))
-    ))
+  (let ((dest (open-output-string)))
+    (display base dest)
+    (let loop ((seed seed))
+      (if (p seed)
+        (begin (display (make-final seed) dest)
+               (get-output-string dest))
+        (begin (write-char (f seed) dest)
+               (loop (g seed)))))))
 
-(define (string-unfold-right p f g seed . args)
+(define (string-unfold-right p f g seed
+                             :optional (base "") (make-final (lambda (_) "")))
   (check-arg procedure? p)
   (check-arg procedure? f)
   (check-arg procedure? g)
-  (let-optionals* args ((base "") (make-final (lambda (_) "")))
-    (let ((dest (open-output-string)))
-      (let loop ((seed seed))
-        (if (p seed)
-          (string-append (make-final seed)
-                         (string-reverse (get-output-string dest))
-                         base)
-          (begin (write-char (f seed) dest)
-                 (loop (g seed))))))
-    ))
+  (let ((dest (open-output-string)))
+    (let loop ((seed seed))
+      (if (p seed)
+        (string-append (make-final seed)
+                       (string-reverse (get-output-string dest))
+                       base)
+        (begin (write-char (f seed) dest)
+               (loop (g seed)))))))
 
 (define (string-for-each proc s . args)
   (check-arg procedure? proc)
@@ -748,54 +720,50 @@
         (loop (string-pointer-next! src)))))
   )
 
-(define (string-for-each-index proc s . args)
+(define (string-for-each-index proc s :optional (start 0) (end (string-length s)))
   (check-arg procedure? proc)
   (check-arg string? s)
-  (let-optionals* args ((start 0) (end (string-length s)))
-    (do ((i start (+ i 1)))
-        ((>= i end))
-      (proc i)))
-  )
+  (do ((i start (+ i 1)))
+      ((>= i end))
+    (proc i)))
 
 ;;;
 ;;; Rotator
 ;;;
 
-(define (xsubstring s from . args)
+(define (xsubstring s from :optional to start end)
   (check-arg string? s)
   (check-arg (lambda (x) (and (integer? x) (exact? x))) from)
-  (let-optionals* args (to start end)
-    (let* ((str (%maybe-substring s start end))
-           (len (string-length str))
-           (from-rank (quotient from len))
-           (from-mod  (modulo from len))
-           (dest (open-output-string)))
-      (cond ((undefined? to)
-             (set! from from-mod)
-             (set! to (+ from len)))
-            ((not (and (integer? to) (exact? to)))
-             (error "argument out of domain:" to))
-            ((= len 0)
-             (error "zero length source string is not allowed"))
-            ((< to from)
-             (errorf "argument out of range (from, to): (~s, ~s)" from to))
-            (else
-             (set! from from-mod)
-             (set! to (- to (* from-rank len)))))
-      (let ((sp (make-string-pointer str from)))
-        (let loop ((count from)
-                   (ch (string-pointer-next! sp)))
-          (cond ((>= count to) (get-output-string dest))
-                ((eof-object? ch)
-                 (string-pointer-set! sp 0)
-                 (write-char (string-pointer-next! sp) dest)
-                 (loop (+ count 1) (string-pointer-next! sp)))
-                (else
-                 (write-char ch dest)
-                 (loop (+ count 1) (string-pointer-next! sp)))))
-        )
-      ))
-  )
+  (let* ((str (%maybe-substring s start end))
+         (len (string-length str))
+         (from-rank (quotient from len))
+         (from-mod  (modulo from len))
+         (dest (open-output-string)))
+    (cond ((undefined? to)
+           (set! from from-mod)
+           (set! to (+ from len)))
+          ((not (and (integer? to) (exact? to)))
+           (error "argument out of domain:" to))
+          ((= len 0)
+           (error "zero length source string is not allowed"))
+          ((< to from)
+           (errorf "argument out of range (from, to): (~s, ~s)" from to))
+          (else
+           (set! from from-mod)
+           (set! to (- to (* from-rank len)))))
+    (let ((sp (make-string-pointer str from)))
+      (let loop ((count from)
+                 (ch (string-pointer-next! sp)))
+        (cond ((>= count to) (get-output-string dest))
+              ((eof-object? ch)
+               (string-pointer-set! sp 0)
+               (write-char (string-pointer-next! sp) dest)
+               (loop (+ count 1) (string-pointer-next! sp)))
+              (else
+               (write-char ch dest)
+               (loop (+ count 1) (string-pointer-next! sp)))))
+      )
+    ))
 
 (define (string-xcopy! target tstart s sfrom . args)
   (check-arg string? target)
@@ -814,28 +782,27 @@
                  (apply %maybe-substring s2 args)
                  (substring s1 end1 (string-length s1))))
 
-(define (string-tokenize s . args)
+(define (string-tokenize s :optional (token-set #[\S]) start end)
   (check-arg string? s)
-  (let-optionals* args ((token-set #[\S]) start end)
-    (letrec ((src (open-input-string (%maybe-substring s start end)))
-             (in-word (lambda (ch dst r)
-                        (cond ((eof-object? ch)
-                               (reverse! (cons (get-output-string dst) r)))
-                              ((char-set-contains? token-set ch)
-                               (write-char ch dst)
-                               (in-word (read-char src) dst r))
-                              (else
-                               (out-word (read-char src)
-                                         (cons (get-output-string dst) r))))))
-             (out-word (lambda (ch r)
-                         (cond ((eof-object? ch) (reverse! r))
-                               ((char-set-contains? token-set ch)
-                                (let ((dst (open-output-string)))
-                                  (write-char ch dst)
-                                  (in-word (read-char src) dst r)))
-                               (else
-                                (out-word (read-char src) r))))))
-      (out-word (read-char src) '()))))
+  (letrec ((src (open-input-string (%maybe-substring s start end)))
+           (in-word (lambda (ch dst r)
+                      (cond ((eof-object? ch)
+                             (reverse! (cons (get-output-string dst) r)))
+                            ((char-set-contains? token-set ch)
+                             (write-char ch dst)
+                             (in-word (read-char src) dst r))
+                            (else
+                             (out-word (read-char src)
+                                       (cons (get-output-string dst) r))))))
+           (out-word (lambda (ch r)
+                       (cond ((eof-object? ch) (reverse! r))
+                             ((char-set-contains? token-set ch)
+                              (let ((dst (open-output-string)))
+                                (write-char ch dst)
+                                (in-word (read-char src) dst r)))
+                             (else
+                              (out-word (read-char src) r))))))
+    (out-word (read-char src) '())))
 
 ;;;
 ;;; Filter
@@ -848,7 +815,7 @@
         (pred (%get-char-pred c/s/p)))
     (let loop ((ch (read-char src)))
       (cond ((eof-object? ch) (get-output-string dest))
-            ((pred ch) (write-char ch dest) (loop (read-char src)))
+            ((pred c/s/p ch) (write-char ch dest) (loop (read-char src)))
             (else (loop (read-char src)))))
     ))
 
@@ -859,7 +826,7 @@
         (pred (%get-char-pred c/s/p)))
     (let loop ((ch (read-char src)))
       (cond ((eof-object? ch) (get-output-string dest))
-            ((pred ch) (loop (read-char src)))
+            ((pred c/s/p ch) (loop (read-char src)))
             (else (write-char ch dest) (loop (read-char src)))))
     ))
 
@@ -939,21 +906,20 @@
 ;; may not be very efficient, in spite of the efforts for efficiency put
 ;; in the original SRFI design.
 
-(define (make-kmp-restart-vector s . args)
-  (let-optionals* args ((c= char=?) start end)
-    (let* ((pat (%maybe-substring s start end))
-           (rv (make-vector (string-length pat) -1))
-           (plen (string-length pat))
-           (plen-1 (- plen 1)))
-      (do ((i 0 (+ i 1)))
-          ((= i plen-1) rv)
-        (let loop ((k (+ (vector-ref rv i) 1)))
-          (if (and (> k 0)
-                   (not (c= (string-ref pat i) (string-ref pat (- k 1)))))
-            (loop (+ (vector-ref rv (- k 1)) 1))
-            (vector-set! rv (+ i 1) k)))
-        )
-      )))
+(define (make-kmp-restart-vector s :optional (c= char=?) start end)
+  (let* ((pat (%maybe-substring s start end))
+         (rv (make-vector (string-length pat) -1))
+         (plen (string-length pat))
+         (plen-1 (- plen 1)))
+    (do ((i 0 (+ i 1)))
+        ((= i plen-1) rv)
+      (let loop ((k (+ (vector-ref rv i) 1)))
+        (if (and (> k 0)
+                 (not (c= (string-ref pat i) (string-ref pat (- k 1)))))
+          (loop (+ (vector-ref rv (- k 1)) 1))
+          (vector-set! rv (+ i 1) k)))
+      )
+    ))
 
 (define (kmp-step pat rv c i c= p-start)
   (let loop ((i i))
@@ -963,17 +929,16 @@
         (if (= i -1) 0 (loop i))))))
 
 ;; This is inefficient if input string s contains multibyte characters.
-(define (string-kmp-partial-search pat rv s i . args)
-  (let-optionals* args ((c= char=?) (p-start 0) start end)
-    (let ((patlen (vector-length rv)))
-      (let lp ((si start)
-               (vi i))
-        (cond ((= vi patlen) (- si)) 
-              ((= si end) vi)
-              (else (lp (+ si 1)
-                        (kmp-step pat rv (string-ref s si)
-                                  vi c= p-start))))))))
+(define (string-kmp-partial-search pat rv s i :optional
+                                   (c= char=?) (p-start 0) start end)
+  (let ((patlen (vector-length rv)))
+    (let lp ((si start)
+             (vi i))
+      (cond ((= vi patlen) (- si)) 
+            ((= si end) vi)
+            (else (lp (+ si 1)
+                      (kmp-step pat rv (string-ref s si)
+                                vi c= p-start)))))))
     
-(provide "srfi-13")
 
           
