@@ -5,7 +5,6 @@
 ;;   Modified to work with Gauche's object system instead of the original
 ;;   structure model.
 ;;
-;; $Id: match.scm,v 1.3 2006/11/08 21:14:51 shirok Exp $
 
 (define-module util.match
   (use srfi-1)
@@ -141,6 +140,34 @@
 ;; End of user visible/modifiable stuff.
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+;;; [SK] NOTE NOTE NOTE
+;;; match macro is unhygienic, and it is possible that it has undesired
+;;; interference with hygienic maros.  I put certain ad-hoc hack in it
+;;; to avoid some unexpected behavior.  It is still a hack, and ultimately
+;;; this should be rewritten in hygienic macro.  Here's some points:
+;;;
+;;; - In patterns, treat symbols and identifiers the same as pattern vars.
+;;; - When inserting references to global variables defined in this module,
+;;;   use identifiers instead of symbols.
+;;;
+;;; For the latter I needed to use make-identifier which is a private
+;;; procedure in gauche.internal.  If you're reading this code out of
+;;; curiousity, DO NOT USE IT in your code.  The identifier stuff may be
+;;; changed greatly when I implement proper low-level hygienic stuff.
+
+(define (%match-id sym)
+  ((with-module gauche.internal make-identifier) sym (current-module) '()))
+
+(define match.         (%match-id 'match))
+(define match-let*.    (%match-id 'match-let*))
+(define match:error.   (%match-id 'match:error))
+(define match-lambda*. (%match-id 'match-lambda*))
+(define match-letrec.  (%match-id 'match-letrec))
+(define match-define.  (%match-id 'match-define))
+
+;;; [SK] End of black magic
+
+
 (define match:error
   (lambda (val . args)
     (errorf "~a: no matching clause for ~a"
@@ -157,10 +184,12 @@
 (define match:set-error-control
   (lambda (v) (set! match:error-control v)))
 
+(define-inline (symid? x) (or (symbol? x) (identifier? x)))
+
 (define match:disjoint-predicates
   (cons 'null
         '(pair?
-          symbol?
+          symid?
           boolean?
           number?
           string?
@@ -191,8 +220,8 @@
                               (code (gensym))
                               (fail (and (pair? (cdr c))
                                          (pair? (cadr c))
-                                         (eq? (caadr c) '=>)
-                                         (symbol? (cadadr c))
+                                         (equal? (caadr c) '=>)
+                                         (symid? (cadadr c))
                                          (pair? (cdadr c))
                                          (null? (cddadr c))
                                          (pair? (cddr c))
@@ -265,16 +294,19 @@
                  ,m)))))
 
 (define (pattern-var? x)
-  (and (symbol? x)
-       (not (dot-dot-k? x))
-       (not (memq x '(quasiquote quote unquote unquote-splicing
-                      ? _ $ struct @ object = and or not set! get! ... ___)))))
+  (and-let* ([x (cond [(symbol? x) x]
+                      [(identifier? x) (unwrap-syntax x)]
+                      [else #f])]
+             [ (not (dot-dot-k? x)) ])
+    (not (memq x '(quasiquote quote unquote unquote-splicing
+                   ? _ $ struct @ object = and or not set! get! ... ___)))))
 
 (define (dot-dot-k? s)
-  (and (symbol? s)
-       (if (memq s '(... ___))
+  (and (symid? s)
+       (if (member s '(... ___))
          0
-         (and-let* ((m (#/^(?:\.\.|__)(\d+)$/ (symbol->string s))))
+         (and-let* ([s (if (symbol? s) s (unwrap-syntax s))]
+                    [m (#/^(?:\.\.|__)(\d+)$/ (symbol->string s))])
            (string->number (m 1))))))
 
 (define (error-maker match-expr)
@@ -282,12 +314,12 @@
    ((eq? match:error-control 'unspecified)
     (cons '() (lambda (x) `(cond (#f #f)))))
    ((memq match:error-control '(error fail))
-    (cons '() (lambda (x) `(match:error ,x))))
+    (cons '() (lambda (x) `(,match:error. ,x))))
    ((eq? match:error-control 'match)
     (let ((errf (gensym))
           (arg (gensym)))
       (cons `((,errf (lambda (,arg)
-                       (match:error ,arg ',match-expr))))
+                       (,match:error. ,arg ',match-expr))))
             (lambda (x) `(,errf ,x)))))
    (else (match:syntax-err
           '(unspecified error fail match)
@@ -340,11 +372,11 @@
              `(not ,@(map ordinary (cdr p)))
              (cons-ordinary (car p) (cdr p))))
           (($ struct)
-           (if (and (pair? (cdr p)) (symbol? (cadr p)) (list? (cddr p)))
+           (if (and (pair? (cdr p)) (symid? (cadr p)) (list? (cddr p)))
              `($ ,(cadr p) ,@(map ordinary (cddr p)))
              (cons-ordinary (car p) (cdr p))))
           ((@ object)
-           (if (and (pair? (cdr p)) (symbol? (cadr p)) (list? (cddr p))
+           (if (and (pair? (cdr p)) (symid? (cadr p)) (list? (cddr p))
                     (every (lambda (p) (and (list? p) (= (length p) 2)))
                            (cddr p)))
              `(object ,(cadr p) ,@(map (lambda (p)
@@ -381,7 +413,7 @@
     (let ((cons-quasi (lambda (x y) (cons (quasi x) (quasi y)))))
       (cond
        ((simple? p) p)
-       ((symbol? p) `',p)
+       ((symid? p) `',p)
        ((pair? p)
         (cond 
          ((eq? (car p) 'unquote)
@@ -423,7 +455,7 @@
   (define (bound p a k)
     (cond
      ((eq? '_ p) (k p a))
-     ((symbol? p)
+     ((symid? p)
       (when (memq p a)
         (match:syntax-err pattern "duplicate variable in pattern"))
       (k p (cons p a)))
@@ -433,7 +465,7 @@
       (cond
        ((not (null? (cddr p)))
         (bound `(and (? ,(cadr p)) ,@(cddr p)) a k))
-       ((or (not (symbol? (cadr p)))
+       ((or (not (symid? (cadr p)))
             (memq (cadr p) a))
         (let ((g (gensym)))
           (push! pred-bodies `(,g ,(cadr p)))
@@ -441,7 +473,7 @@
        (else (k p a))))
      ((and (pair? p) (eq? '= (car p)))
       (cond
-       ((or (not (symbol? (cadr p)))
+       ((or (not (symid? (cadr p)))
             (memq (cadr p) a))
         (let ((g (gensym)))
           (push! pred-bodies `(,g ,(cadr p)))
@@ -590,7 +622,7 @@
                        ((eq? old e) new)
                        (else e)))))
            (const? (lambda (sexp)
-                     (or (symbol? sexp)
+                     (or (symid? sexp)
                          (boolean? sexp)
                          (string? sexp)
                          (char? sexp)
@@ -599,7 +631,7 @@
                          (and (pair? sexp)
                               (eq? (car sexp) 'quote)
                               (pair? (cdr sexp))
-                              (symbol? (cadr sexp))
+                              (symid? (cadr sexp))
                               (null? (cddr sexp))))))
            (isval? (lambda (sexp)
                      (or (const? sexp)
@@ -638,141 +670,213 @@
        (else (loop (cdr b) (cons (car b) new-b) e))))))
 
 (define (gen x sf plist erract length>= eta)
-  (if (null? plist)
-    (erract x)
-    (let* ((v '())
-           (val (lambda (x) (cdr (assq x v))))
-           (fail (lambda (sf)
-                   (gen x sf (cdr plist) erract length>= eta)))
-           (success (lambda (sf)
-                      (set-car! (cddddr (car plist)) #t)
-                      (let* ((code (cadr (car plist)))
-                             (bv (caddr (car plist)))
-                             (fail-sym (cadddr (car plist))))
-                        (if fail-sym
-                          (let ((ap `(,code ,fail-sym ,@(map val bv))))
-                            `(call-with-current-continuation
-                              (lambda (,fail-sym)
-                                (let ((,fail-sym
-                                       (lambda ()
-                                         (call-with-values
-                                             (lambda () ,(fail sf))
-                                           ,fail-sym))))
-                                  ,ap))))
-                          `(,code ,@(map val bv)))))))
-      (let next ((p (caar plist))
-                 (e x)
-                 (sf sf)
-                 (kf fail)
-                 (ks success))
-        (cond
-         ((eq? '_ p) (ks sf))
-         ((symbol? p) (set! v (cons (cons p e) v))
-          (ks sf))
-         ((null? p) (emit `(null? ,e) sf kf ks))
-         ((equal? p ''()) (emit `(null? ,e) sf kf ks))
-         ((string? p) (emit `(equal? ,e ,p) sf kf ks))
-         ((boolean? p) (emit `(equal? ,e ,p) sf kf ks))
-         ((char? p) (emit `(equal? ,e ,p) sf kf ks))
-         ((number? p) (emit `(equal? ,e ,p) sf kf ks))
-         ((keyword? p) (emit `(equal? ,e ,p) sf kf ks))
-         ((and (pair? p) (eq? 'quote (car p)))
-          (emit `(equal? ,e ,p) sf kf ks))
-         ((and (pair? p) (eq? '? (car p)))
-          (let ((tst `(,(cadr p) ,e)))
-            (emit tst sf kf ks)))
-         ((and (pair? p) (eq? '= (car p)))
-          (if (and (pair? (cadr p))
-                   (eq? (caadr p) 'quote))
-            (next (caddr p) `(ref ,(cadr p) ,e) sf kf ks)
-            (next (caddr p) `(,(cadr p) ,e) sf kf ks)))
-         ((and (pair? p) (eq? 'and (car p)))
-          (let loop ((p (cdr p))
-                     (sf sf))
-            (if (null? p)
-              (ks sf)
-              (next (car p) e sf kf (lambda (sf) (loop (cdr p) sf))))))
-         ((and (pair? p) (eq? 'or (car p)))
-          (let ((or-v v))
+  (define (gen-rec plist memo)
+    (if (null? plist)
+      (erract x)
+      (let* ((v '())
+             (val (lambda (x) (cdr (assq x v))))
+             (fail (lambda (sf)
+                     (or (hash-table-get memo (cdr plist) #f)
+                         (rlet1 code (gen-rec (cdr plist) memo)
+                           (hash-table-put! memo (cdr plist) code)))))
+             (success (lambda (sf)
+                        (set-car! (cddddr (car plist)) #t)
+                        (let* ((code (cadr (car plist)))
+                               (bv (caddr (car plist)))
+                               (fail-sym (cadddr (car plist))))
+                          (if fail-sym
+                            (let ((ap `(,code ,fail-sym ,@(map val bv))))
+                              `(call-with-current-continuation
+                                (lambda (,fail-sym)
+                                  (let ((,fail-sym
+                                         (lambda ()
+                                           (call-with-values
+                                               (lambda () ,(fail sf))
+                                             ,fail-sym))))
+                                    ,ap))))
+                            `(,code ,@(map val bv)))))))
+        (let next ((p (caar plist))
+                   (e x)
+                   (sf sf)
+                   (kf fail)
+                   (ks success))
+          (cond
+           ((eq? '_ p) (ks sf))
+           ((symid? p) (set! v (cons (cons p e) v))
+            (ks sf))
+           ((null? p) (emit `(null? ,e) sf kf ks))
+           ((equal? p ''()) (emit `(null? ,e) sf kf ks))
+           ((string? p) (emit `(equal? ,e ,p) sf kf ks))
+           ((boolean? p) (emit `(equal? ,e ,p) sf kf ks))
+           ((char? p) (emit `(equal? ,e ,p) sf kf ks))
+           ((number? p) (emit `(equal? ,e ,p) sf kf ks))
+           ((keyword? p) (emit `(equal? ,e ,p) sf kf ks))
+           ((and (pair? p) (eq? 'quote (car p)))
+            (emit `(equal? ,e ,p) sf kf ks))
+           ((and (pair? p) (eq? '? (car p)))
+            (let ((tst `(,(cadr p) ,e)))
+              (emit tst sf kf ks)))
+           ((and (pair? p) (eq? '= (car p)))
+            (if (and (pair? (cadr p))
+                     (eq? (caadr p) 'quote))
+              (next (caddr p) `(ref ,(cadr p) ,e) sf kf ks)
+              (next (caddr p) `(,(cadr p) ,e) sf kf ks)))
+           ((and (pair? p) (eq? 'and (car p)))
             (let loop ((p (cdr p))
                        (sf sf))
               (if (null? p)
-                (kf sf)
-                (begin (set! v or-v)
-                       (next (car p) e sf (lambda (sf) (loop (cdr p) sf))
-                             ks))))))
-         ((and (pair? p) (eq? 'not (car p)))
-          (next (cadr p) e sf ks kf))
-         ((and (pair? p) (eq? '$ (car p)))
-          (let* ((tag (cadr p))
-                 (fields (cdr p))
-                 (rlen (length fields))
-                 (tst `(is-a? ,e ,tag)))
-            (emit tst sf kf
-                  (let rloop ((n 1))
+                (ks sf)
+                (next (car p) e sf kf (lambda (sf) (loop (cdr p) sf))))))
+           ((and (pair? p) (eq? 'or (car p)))
+            (let ((or-v v))
+              (let loop ((p (cdr p))
+                         (sf sf))
+                (if (null? p)
+                  (kf sf)
+                  (begin (set! v or-v)
+                         (next (car p) e sf (lambda (sf) (loop (cdr p) sf))
+                               ks))))))
+           ((and (pair? p) (eq? 'not (car p)))
+            (next (cadr p) e sf ks kf))
+           ((and (pair? p) (eq? '$ (car p)))
+            (let* ((tag (cadr p))
+                   (fields (cdr p))
+                   (rlen (length fields))
+                   (tst `(is-a? ,e ,tag)))
+              (emit tst sf kf
+                    (let rloop ((n 1))
+                      (lambda (sf)
+                        (if (= n rlen)
+                          (ks sf)
+                          (next (list-ref fields n)
+                                `(match:$-ref ,tag ,(- n 1) ,e)
+                                sf kf (rloop (+ 1 n)))))))))
+           ((and (pair? p) (eq? 'object (car p)))  ;; Gauche extension
+            (let* ((tag (cadr p))
+                   (fields (cddr p))
+                   (tst `(is-a? ,e ,tag)))
+              (emit tst sf kf
+                    (let rloop ((fields fields))
+                      (lambda (sf)
+                        (if (null? fields)
+                          (ks sf)
+                          (next (cadar fields)
+                                `(ref ,e ',(caar fields))
+                                sf kf (rloop (cdr fields)))))))))
+           ((and (pair? p) (eq? 'set! (car p)))
+            (set! v (cons (cons (cadr p) (get-setter e p)) v))
+            (ks sf))
+           ((and (pair? p) (eq? 'get! (car p)))
+            (set! v (cons (cons (cadr p) (get-getter e p)) v))
+            (ks sf))
+           ((and (pair? p)
+                 (pair? (cdr p))
+                 (dot-dot-k? (cadr p)))
+            (emit `(list? ,e) sf kf
+                  (lambda (sf)
+                    (let* ((k (dot-dot-k? (cadr p)))
+                           (ks (lambda (sf)
+                                 (let ((bound (list-ref p 2)))
+                                   (cond
+                                    ((eq? (car p) '_) (ks sf))
+                                    ((null? bound)
+                                     (let* ((eta (gensym))
+                                            (ptst (next (car p) eta sf
+                                                        (lambda (sf) #f)
+                                                        (lambda (sf) #t)))
+                                            (tst (if (and (pair? ptst)
+                                                          (symid? (car ptst))
+                                                          (pair? (cdr ptst))
+                                                          (eq? eta (cadr ptst))
+                                                          (null? (cddr ptst)))
+                                                   (car ptst)
+                                                   `(lambda (,eta) ,ptst))))
+                                       (assm `(match:every ,tst ,e)
+                                             (kf sf)
+                                             (ks sf))))
+                                    ((and (symid? (car p))
+                                          (equal? (list (car p)) bound))
+                                     (next (car p) e sf kf ks))
+                                    (else
+                                     (let* ((gloop (list-ref p 3))
+                                            (ge (list-ref p 4))
+                                            (fresh (list-ref p 5))
+                                            (p1 (next (car p)
+                                                      `(car ,ge)
+                                                      sf
+                                                      kf
+                                                      (lambda (sf)
+                                                        `(,gloop
+                                                          (cdr ,ge)
+                                                          ,@(map (lambda (b f)
+                                                                   `(cons ,(val b)
+                                                                          ,f))
+                                                                 bound
+                                                                 fresh))))))
+                                       (set! v
+                                             (append
+                                              (map cons
+                                                   bound
+                                                   (map (lambda (x)
+                                                          `(reverse ,x))
+                                                        fresh))
+                                              v))
+                                       `(let ,gloop ((,ge ,e)
+                                                     ,@(map (lambda (x)
+                                                              `(,x '()))
+                                                            fresh))
+                                             (if (null? ,ge)
+                                               ,(ks sf)
+                                               ,p1)))))))))
+                      (case k
+                        ((0) (ks sf))
+                        ((1) (emit `(pair? ,e) sf kf ks))
+                        (else (emit `((,length>= ,k) ,e) sf kf ks)))))))
+           ((pair? p) (emit `(pair? ,e)
+                            sf
+                            kf
+                            (lambda (sf)
+                              (next (car p)
+                                    (add-a e)
+                                    sf
+                                    kf
+                                    (lambda (sf)
+                                      (next (cdr p) (add-d e) sf kf ks))))))
+           ((and (vector? p)
+                 (>= (vector-length p) 6)
+                 (dot-dot-k? (vector-ref p (- (vector-length p) 5))))
+            (let* ((vlen (- (vector-length p) 6))
+                   (k (dot-dot-k? (vector-ref p (+ vlen 1))))
+                   (minlen (+ vlen k))
+                   (bound (vector-ref p (+ vlen 2))))
+              (emit `(vector? ,e)
+                    sf
+                    kf
                     (lambda (sf)
-                      (if (= n rlen)
-                        (ks sf)
-                        (next (list-ref fields n)
-                              `(match:$-ref ,tag ,(- n 1) ,e)
-                              sf kf (rloop (+ 1 n)))))))))
-         ((and (pair? p) (eq? 'object (car p)))  ;; Gauche extension
-          (let* ((tag (cadr p))
-                 (fields (cddr p))
-                 (tst `(is-a? ,e ,tag)))
-            (emit tst sf kf
-                  (let rloop ((fields fields))
-                    (lambda (sf)
-                      (if (null? fields)
-                        (ks sf)
-                        (next (cadar fields)
-                              `(ref ,e ',(caar fields))
-                              sf kf (rloop (cdr fields)))))))))
-         ((and (pair? p) (eq? 'set! (car p)))
-          (set! v (cons (cons (cadr p) (get-setter e p)) v))
-          (ks sf))
-         ((and (pair? p) (eq? 'get! (car p)))
-          (set! v (cons (cons (cadr p) (get-getter e p)) v))
-          (ks sf))
-         ((and (pair? p)
-               (pair? (cdr p))
-               (dot-dot-k? (cadr p)))
-          (emit `(list? ,e) sf kf
-                (lambda (sf)
-                  (let* ((k (dot-dot-k? (cadr p)))
-                         (ks (lambda (sf)
-                               (let ((bound (list-ref p 2)))
+                      (assm `(>= (vector-length ,e) ,minlen)
+                            (kf sf)
+                            ((let vloop ((n 0))
+                               (lambda (sf)
                                  (cond
-                                  ((eq? (car p) '_) (ks sf))
-                                  ((null? bound)
-                                   (let* ((eta (gensym))
-                                          (ptst (next (car p) eta sf
-                                                      (lambda (sf) #f)
-                                                      (lambda (sf) #t)))
-                                          (tst (if (and (pair? ptst)
-                                                        (symbol? (car ptst))
-                                                        (pair? (cdr ptst))
-                                                        (eq? eta (cadr ptst))
-                                                        (null? (cddr ptst)))
-                                                 (car ptst)
-                                                 `(lambda (,eta) ,ptst))))
-                                     (assm `(match:every ,tst ,e)
-                                           (kf sf)
-                                           (ks sf))))
-                                  ((and (symbol? (car p))
-                                        (equal? (list (car p)) bound))
-                                   (next (car p) e sf kf ks))
+                                  ((not (= n vlen))
+                                   (next (vector-ref p n)
+                                         `(vector-ref ,e ,n)
+                                         sf
+                                         kf
+                                         (vloop (+ 1 n))))
+                                  ((eq? (vector-ref p vlen) '_)
+                                   (ks sf))
                                   (else
-                                   (let* ((gloop (list-ref p 3))
-                                          (ge (list-ref p 4))
-                                          (fresh (list-ref p 5))
-                                          (p1 (next (car p)
-                                                    `(car ,ge)
+                                   (let* ((gloop (vector-ref p (+ vlen 3)))
+                                          (ind (vector-ref p (+ vlen 4)))
+                                          (fresh (vector-ref p (+ vlen 5)))
+                                          (p1 (next (vector-ref p vlen)
+                                                    `(vector-ref ,e ,ind)
                                                     sf
                                                     kf
                                                     (lambda (sf)
                                                       `(,gloop
-                                                        (cdr ,ge)
+                                                        (- ,ind 1)
                                                         ,@(map (lambda (b f)
                                                                  `(cons ,(val b)
                                                                         ,f))
@@ -780,106 +884,38 @@
                                                                fresh))))))
                                      (set! v
                                            (append
-                                            (map cons
-                                                 bound
-                                                 (map (lambda (x)
-                                                        `(reverse ,x))
-                                                      fresh))
+                                            (map cons bound fresh)
                                             v))
-                                     `(let ,gloop ((,ge ,e)
-                                                   ,@(map (lambda (x)
-                                                            `(,x '()))
-                                                          fresh))
-                                        (if (null? ,ge)
+                                     `(let ,gloop
+                                        ((,ind (- (vector-length ,e) 1))
+                                         ,@(map (lambda (x) `(,x '()))
+                                                fresh))
+                                        (if (> ,minlen ,ind)
                                           ,(ks sf)
-                                          ,p1)))))))))
-                    (case k
-                      ((0) (ks sf))
-                      ((1) (emit `(pair? ,e) sf kf ks))
-                      (else (emit `((,length>= ,k) ,e) sf kf ks)))))))
-         ((pair? p) (emit `(pair? ,e)
-                          sf
-                          kf
-                          (lambda (sf)
-                            (next (car p)
-                                  (add-a e)
-                                  sf
-                                  kf
-                                  (lambda (sf)
-                                    (next (cdr p) (add-d e) sf kf ks))))))
-         ((and (vector? p)
-               (>= (vector-length p) 6)
-               (dot-dot-k? (vector-ref p (- (vector-length p) 5))))
-          (let* ((vlen (- (vector-length p) 6))
-                 (k (dot-dot-k? (vector-ref p (+ vlen 1))))
-                 (minlen (+ vlen k))
-                 (bound (vector-ref p (+ vlen 2))))
-            (emit `(vector? ,e)
-                  sf
-                  kf
-                  (lambda (sf)
-                    (assm `(>= (vector-length ,e) ,minlen)
-                          (kf sf)
-                          ((let vloop ((n 0))
-                             (lambda (sf)
-                               (cond
-                                ((not (= n vlen))
-                                 (next (vector-ref p n)
-                                       `(vector-ref ,e ,n)
-                                       sf
-                                       kf
-                                       (vloop (+ 1 n))))
-                                ((eq? (vector-ref p vlen) '_)
-                                 (ks sf))
-                                (else
-                                 (let* ((gloop (vector-ref p (+ vlen 3)))
-                                        (ind (vector-ref p (+ vlen 4)))
-                                        (fresh (vector-ref p (+ vlen 5)))
-                                        (p1 (next (vector-ref p vlen)
-                                                  `(vector-ref ,e ,ind)
-                                                  sf
-                                                  kf
-                                                  (lambda (sf)
-                                                    `(,gloop
-                                                      (- ,ind 1)
-                                                      ,@(map (lambda (b f)
-                                                               `(cons ,(val b)
-                                                                      ,f))
-                                                             bound
-                                                             fresh))))))
-                                        (set! v
-                                              (append
-                                               (map cons bound fresh)
-                                               v))
-                                        `(let ,gloop
-                                           ((,ind (- (vector-length ,e) 1))
-                                            ,@(map (lambda (x) `(,x '()))
-                                                   fresh))
-                                           (if (> ,minlen ,ind)
-                                             ,(ks sf)
-                                             ,p1)))))))
-                           sf))))))
-         ((vector? p)
-          (let ((vlen (vector-length p)))
-            (emit `(vector? ,e)
-                  sf
-                  kf
-                  (lambda (sf)
-                    (emit `(equal? (vector-length ,e) ,vlen)
-                          sf
-                          kf
-                          (let vloop ((n 0))
-                            (lambda (sf)
-                              (if (= n vlen)
-                                (ks sf)
-                                (next (vector-ref p n)
-                                      `(vector-ref ,e ,n)
-                                      sf
-                                      kf
-                                      (vloop (+ 1 n)))))))))))
-         (else (display "FATAL ERROR IN PATTERN MATCHER")
-               (newline)
-               (error #f "THIS NEVER HAPPENS")))))))
+                                          ,p1)))))))
+                             sf))))))
+           ((vector? p)
+            (let ((vlen (vector-length p)))
+              (emit `(vector? ,e)
+                    sf
+                    kf
+                    (lambda (sf)
+                      (emit `(equal? (vector-length ,e) ,vlen)
+                            sf
+                            kf
+                            (let vloop ((n 0))
+                              (lambda (sf)
+                                (if (= n vlen)
+                                  (ks sf)
+                                  (next (vector-ref p n)
+                                        `(vector-ref ,e ,n)
+                                        sf
+                                        kf
+                                        (vloop (+ 1 n)))))))))))
+           (else (display "FATAL ERROR IN PATTERN MATCHER")
+                 (newline)
+                 (error #f "THIS NEVER HAPPENS")))))))
+  (gen-rec plist (make-hash-table 'eq?)))
 
 (define (emit tst sf kf ks)
   (cond
@@ -1013,15 +1049,15 @@
     (let loop ((code code))
       (cond
        ((not (pair? code)) #f)
-       ((memq (car code) '(cond match:error)) #t)
+       ((member (car code) '(cond match:error)) #t)
        ((or (equal? code a) (equal? code d)) #t)
        ((eq? (car code) 'if)
         (or (loop (cadr code))
             (and (loop (caddr code))
                  (loop (cadddr code)))))
-       ((eq? (car code) 'lambda) #f)
+       ((equal? (car code) 'lambda) #f)
        ((and (eq? (car code) 'let)
-             (symbol? (cadr code)))
+             (symid? (cadr code)))
         #f)
        (else (or (loop (car code))
                  (loop (cdr code))))))))
@@ -1080,7 +1116,7 @@
                 (pair? (cdr p))
                 (null? (cddr p))
                 (eq? 'quote (car p))
-                (symbol? (cadr p)))
+                (symid? (cadr p)))
            'symbol?)
           (else #f)))))
 
@@ -1132,17 +1168,17 @@
             (lambda (y) (and (list? y) (<= 2 (length y))))
             (cdr args))) (let* ((exp (car args))
                                 (clauses (cdr args))
-                                (e (if (symbol? exp) exp (gensym))))
-                           (if (symbol? exp)
+                                (e (if (symid? exp) exp (gensym))))
+                           (if (symid? exp)
                                ((car match:expanders)
                                 e
                                 clauses
-                                `(match ,@args))
+                                `(,match. ,@args))
                                `(let ((,e ,exp))
                                   ,((car match:expanders)
                                     e
                                     clauses
-                                    `(match ,@args))))))
+                                    `(,match. ,@args))))))
     (else (match:syntax-err `(match ,@args) "syntax error in"))))
 
 (define-macro (match-lambda . args)
@@ -1154,7 +1190,7 @@
                    #f))
              args))
       ((lambda ()
-         (let ((e (gensym))) `(lambda (,e) (match ,e ,@args)))))
+         (let ((e (gensym))) `(lambda (,e) (,match. ,e ,@args)))))
       ((lambda ()
          (match:syntax-err
            `(match-lambda ,@args)
@@ -1169,7 +1205,7 @@
                    #f))
              args))
       ((lambda ()
-         (let ((e (gensym))) `(lambda ,e (match ,e ,@args)))))
+         (let ((e (gensym))) `(lambda ,e (,match. ,e ,@args)))))
       ((lambda ()
          (match:syntax-err
            `(match-lambda* ,@args)
@@ -1177,21 +1213,21 @@
 
 (define-macro (match-let . args)
   (let ((g158 (lambda (pat exp body)
-                `(match ,exp (,pat ,@body))))
+                `(,match. ,exp (,pat ,@body))))
         (g154 (lambda (pat exp body)
                 (let ((g (map (lambda (x) (gensym)) pat))
                       (vpattern (list->vector pat)))
                   `(let ,(map list g exp)
-                     (match (vector ,@g) (,vpattern ,@body))))))
+                     (,match. (vector ,@g) (,vpattern ,@body))))))
         (g146 (lambda ()
                 (match:syntax-err `(match-let ,@args) "syntax error in")))
         (g145 (lambda (p1 e1 p2 e2 body)
                 (let ((g1 (gensym)) (g2 (gensym)))
                   `(let ((,g1 ,e1) (,g2 ,e2))
-                     (match (cons ,g1 ,g2) ((,p1 . ,p2) ,@body))))))
+                     (,match. (cons ,g1 ,g2) ((,p1 . ,p2) ,@body))))))
         (g136 (cadddr match:expanders)))
     (if (pair? args)
-        (if (symbol? (car args))
+        (if (symid? (car args))
             (if (and (pair? (cdr args)) (list? (cadr args)))
                 (let g161 ((g162 (cadr args)) (g160 '()) (g159 '()))
                   (if (null? g162)
@@ -1201,7 +1237,7 @@
                                    (cadddr match:expanders)
                                    pat)
                                  `(let ,@args)
-                                 `(letrec ((,name (match-lambda*
+                                 `(letrec ((,name (,match-lambda*.
                                                     (,pat ,@body))))
                                     (,name ,@exp))))
                            (car args)
@@ -1400,7 +1436,7 @@
         (g146))))
 
 (define-macro (match-let1 pat exp . body) ;; Gauche extension
-  `(match ,exp (,pat ,@body)))
+  `(,match. ,exp (,pat ,@body)))
 
 (define-macro (match-let* . args)
   (let ((g176 (lambda ()
@@ -1419,8 +1455,8 @@
                      (pair? (cdr args)))
                 ((lambda (pat exp rest body)
                    (if ((cadddr match:expanders) pat)
-                       `(let ((,pat ,exp)) (match-let* ,rest ,@body))
-                       `(match ,exp (,pat (match-let* ,rest ,@body)))))
+                       `(let ((,pat ,exp)) (,match-let*. ,rest ,@body))
+                       `(,match. ,exp (,pat (,match-let*. ,rest ,@body)))))
                  (caaar args)
                  (cadaar args)
                  (cdar args)
@@ -1431,13 +1467,13 @@
 (define-macro (match-letrec . args)
   (let ((g200 (cadddr match:expanders))
         (g199 (lambda (p1 e1 p2 e2 body)
-                `(match-letrec (((,p1 . ,p2) (cons ,e1 ,e2))) ,@body)))
+                `(,match-letrec. (((,p1 . ,p2) (cons ,e1 ,e2))) ,@body)))
         (g195 (lambda ()
                 (match:syntax-err
                   `(match-letrec ,@args)
                   "syntax error in")))
         (g194 (lambda (pat exp body)
-                `(match-letrec
+                `(,match-letrec.
                    ((,(list->vector pat) (vector ,@exp)))
                    ,@body)))
         (g186 (lambda (pat exp body)
@@ -1445,7 +1481,7 @@
                  pat
                  exp
                  body
-                 `(match-letrec ((,pat ,exp)) ,@body)))))
+                 `(,match-letrec. ((,pat ,exp)) ,@body)))))
     (if (pair? args)
         (if (list? (car args))
             (if (match:every
@@ -1633,10 +1669,9 @@
                    ((caddr match:expanders)
                     pat
                     exp
-                    `(match-define ,@args)))
+                    `(,match-define. ,@args)))
                  (car args)
                  (cadr args))
                 (g209)))
         (g209))))
 
-(provide "util/match")
