@@ -1,7 +1,7 @@
 /*
  * bignum.c - multiple precision exact integer arithmetic
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: bignum.c,v 1.69 2008/02/03 13:07:46 shirok Exp $
  */
 
 /* Bignum library.  Not optimized well yet---I think bignum performance
@@ -404,6 +402,8 @@ ScmUInt64 Scm_BignumToUI64(ScmBignum *b, int clamp, int *oor)
 #endif /* SIZEOF_LONG == 4 */
 
 
+extern double Scm__EncodeDouble(u_long, u_long, int, int);
+
 /* Converts a bignum b to a double.  b must be normalized.
    We don't rely on double arithmetic, for it may result
    an error in the LSB from multiple roundings.  Instead we
@@ -412,7 +412,6 @@ ScmUInt64 Scm_BignumToUI64(ScmBignum *b, int clamp, int *oor)
  */
 double Scm_BignumToDouble(ScmBignum *b)
 {
-    ScmIEEEDouble dd;
     ScmBits *bits = (ScmBits*)b->values;
     ScmBits dst[2];
     int maxbit, exponent;
@@ -430,7 +429,7 @@ double Scm_BignumToDouble(ScmBignum *b)
        the special case where maxbit-52==0, maxbit-53==1, and all
        bits below are 0 (round-to-even rule) */
     if (SCM_BITS_TEST(bits, maxbit-53)
-        && (dst[0]&1 == 1
+        && ((dst[0]&1) == 1
             || (Scm_BitsCount1(bits, 0, maxbit-53) > 0))) {
         dst[0]++;
         if (dst[0] >= (1UL<<52)) {
@@ -440,14 +439,8 @@ double Scm_BignumToDouble(ScmBignum *b)
             exponent++;
         }
     }
-    if (exponent > 2046) {
-        dd.components.mant = 0;
-        dd.components.exp = 2047;
-    } else {
-        dd.components.mant = dst[0];
-        dd.components.exp  = exponent;
-    }
-    dd.components.sign = (b->sign < 0);
+    if (exponent > 2046) return Scm__EncodeDouble(0, 0, 2047, (b->sign < 0));
+    else return Scm__EncodeDouble(0, dst[0], exponent, (b->sign < 0));
 #else  /*SIZEOF_LONG == 4 */
     dst[0] = dst[1] = 0;
     if (maxbit < 53) {
@@ -470,18 +463,9 @@ double Scm_BignumToDouble(ScmBignum *b)
             }
         }
     }
-    if (exponent > 2046) {
-        dd.components.mant0 = 0;
-        dd.components.mant1 = 0;
-        dd.components.exp = 2047;
-    } else {
-        dd.components.mant0 = dst[1];
-        dd.components.mant1 = dst[0];
-        dd.components.exp = exponent;
-    }
-    dd.components.sign = (b->sign < 0);
+    if (exponent > 2046) return Scm__EncodeDouble(0, 0, 2047, (b->sign < 0));
+    else return Scm__EncodeDouble(dst[0], dst[1], exponent, (b->sign < 0));
 #endif /*SIZEOF_LONG==4*/
-    return dd.d;
 }
 
 /* return -b, normalized */
