@@ -1,7 +1,7 @@
 /*
  * char.c - character and character set operations
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: char.c,v 1.61 2008/02/08 08:06:32 shirok Exp $
  */
 
 #include <ctype.h>
@@ -192,9 +190,9 @@ SCM_DEFINE_BUILTIN_CLASS(Scm_CharSetClass,
  * Printer
  */
 
-static void charset_print_ch(ScmPort *out, ScmChar ch)
+static void charset_print_ch(ScmPort *out, ScmChar ch, int firstp)
 {
-    if (ch == '[' || ch == ']' || ch == '-') {
+    if (ch == '[' || ch == ']' || ch == '-' || (ch == '^' && firstp)) {
         Scm_Printf(out, "\\%c", ch);
     } else if (ch < 0x20 || ch == 0x7f) {
         Scm_Printf(out, "\\x%02x", ch);
@@ -209,7 +207,7 @@ static void charset_print_ch(ScmPort *out, ScmChar ch)
 
 static void charset_print(ScmObj obj, ScmPort *out, ScmWriteContext *ctx)
 {
-    int prev, code;
+    int prev, code, first = TRUE;
     ScmCharSet *cs = SCM_CHAR_SET(obj);
     ScmTreeIter iter;
     ScmDictEntry *e;
@@ -217,13 +215,14 @@ static void charset_print(ScmObj obj, ScmPort *out, ScmWriteContext *ctx)
     Scm_Printf(out, "#[");
     for (prev = -1, code = 0; code < SCM_CHAR_SET_SMALL_CHARS; code++) {
         if (MASK_ISSET(cs, code) && prev < 0) {
-            charset_print_ch(out, code);
+            charset_print_ch(out, code, first);
             prev = code;
+            first = FALSE;
         } 
         else if (!MASK_ISSET(cs, code) && prev >= 0) {
             if (code - prev > 1) {
                 if (code - prev > 2) Scm_Printf(out, "-");
-                charset_print_ch(out, code-1);
+                charset_print_ch(out, code-1, FALSE);
             }
             prev = -1;
         }
@@ -231,15 +230,15 @@ static void charset_print(ScmObj obj, ScmPort *out, ScmWriteContext *ctx)
     if (prev >= 0) {
         if (code - prev > 1) {
             if (prev < 0x7e) Scm_Printf(out, "-");
-            charset_print_ch(out, code-1);
+            charset_print_ch(out, code-1, FALSE);
         }
     }
     Scm_TreeIterInit(&iter, &cs->large, NULL);
     while ((e = Scm_TreeIterNext(&iter)) != NULL) {
-        charset_print_ch(out, (int)e->key);
+        charset_print_ch(out, (int)e->key, FALSE);
         if (e->value != e->key) {
             if (e->value - e->key > 2) Scm_Printf(out, "-");
-            charset_print_ch(out, (int)e->value);
+            charset_print_ch(out, (int)e->value, FALSE);
         }
     }
     Scm_Printf(out, "]", obj);
@@ -461,18 +460,18 @@ ScmObj Scm_CharSetComplement(ScmCharSet *cs)
 
     Scm_BitsOperate(cs->small, SCM_BIT_NOT1, cs->small, NULL,
                     0, SCM_CHAR_SET_SMALL_CHARS);
-    last = SCM_CHAR_SET_SMALL_CHARS;
+    last = SCM_CHAR_SET_SMALL_CHARS-1;
     /* we can't use treeiter, since we modify the tree while traversing it. */
     while ((e = Scm_TreeCoreNextEntry(&cs->large, last)) != NULL) {
         Scm_TreeCoreSearch(&cs->large, e->key, SCM_DICT_DELETE);
         if (last < e->key-1) {
-            n = Scm_TreeCoreSearch(&cs->large, last, SCM_DICT_CREATE);
+            n = Scm_TreeCoreSearch(&cs->large, last+1, SCM_DICT_CREATE);
             n->value = e->key-1;
         }
-        last = (int)e->value+1;
+        last = (int)e->value;
     }
     if (last < SCM_CHAR_MAX) {
-        n = Scm_TreeCoreSearch(&cs->large, last, SCM_DICT_CREATE);
+        n = Scm_TreeCoreSearch(&cs->large, last+1, SCM_DICT_CREATE);
         n->value = SCM_CHAR_MAX;
     }
     return SCM_OBJ(cs);
