@@ -1,7 +1,7 @@
 /*
  * core.c - core kernel interface
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: core.c,v 1.82 2007/09/13 12:30:27 shirok Exp $
  */
 
 #define LIBGAUCHE_BODY
@@ -39,13 +37,18 @@
 #include "gauche/paths.h"
 #include "gauche/builtin-syms.h"
 
+/* GC_print_static_roots() is declared in private/gc_priv.h.  It is too much
+   hassle to include it with other GC internal baggages, so we just declare
+   it.  It's a private funciton of GC, so watch out the changes in GC. */
+extern void GC_print_static_roots(void);
+
 /*
  * out-of-memory handler.  this will be called by GC.
  */
 
-static GC_PTR oom_handler(size_t bytes)
+static void *oom_handler(size_t bytes)
 {
-    Scm_Panic("out of memory (%d).  aborting...", bytes);
+    Scm_Panic("out of memory (%lu).  aborting...", bytes);
     return NULL;                /* dummy */
 }
 
@@ -62,6 +65,7 @@ static struct {
  */
 
 extern void Scm__InitModule(void);
+extern void Scm__InitModulePost(void);
 extern void Scm__InitSymbol(void);
 extern void Scm__InitKeyword(void);
 extern void Scm__InitNumber(void);
@@ -81,14 +85,12 @@ extern void Scm__InitSystem(void);
 extern void Scm__InitCode(void);
 extern void Scm__InitVM(void);
 extern void Scm__InitRepl(void);
-extern void Scm__InitParameter(void);
 extern void Scm__InitAutoloads(void);
 extern void Scm__InitCollection(void);
 
 extern void Scm_Init_stdlib(ScmModule *);
 extern void Scm_Init_extlib(ScmModule *);
 extern void Scm_Init_syslib(ScmModule *);
-extern void Scm_Init_moplib(ScmModule *);
 extern void Scm_Init_intlib(ScmModule *);
 
 extern void Scm_Init_scmlib(void);
@@ -110,7 +112,7 @@ void Scm_Init(const char *signature)
 {
     /* make sure the main program links the same version of libgauche */
     if (strcmp(signature, GAUCHE_SIGNATURE) != 0) {
-        Scm_Panic("libgauche version mismatch: libgauche %s, expected %s",
+        Scm_Panic("libgauche ABI version mismatch: libgauche %s, expected %s",
                   GAUCHE_SIGNATURE, signature);
     }
 
@@ -129,20 +131,20 @@ void Scm_Init(const char *signature)
 
     /* Initialize components.  The order is important, for some components
        rely on the other components to be initialized. */
+    Scm__InitVM();
     Scm__InitSymbol();
     Scm__InitModule();
     Scm__InitKeyword();
     Scm__InitNumber();
     Scm__InitChar();
     Scm__InitClass();
+    Scm__InitModulePost();
     Scm__InitCollection();
     Scm__InitExceptions();
     Scm__InitProc();
     Scm__InitPort();
     Scm__InitWrite();
     Scm__InitCode();
-    Scm__InitVM();
-    Scm__InitParameter();
     Scm__InitMacro();
     Scm__InitLoad();
     Scm__InitRegexp();
@@ -150,11 +152,10 @@ void Scm_Init(const char *signature)
     Scm__InitSignal();
     Scm__InitSystem();
     Scm__InitRepl();
-    
+
     Scm_Init_stdlib(Scm_SchemeModule());
     Scm_Init_extlib(Scm_GaucheModule());
     Scm_Init_syslib(Scm_GaucheModule());
-    Scm_Init_moplib(Scm_GaucheModule());
     Scm_Init_intlib(Scm_GaucheInternalModule());
 
     Scm_Init_scmlib();
@@ -186,6 +187,11 @@ void Scm_GC()
     GC_gcollect();
 }
 
+void Scm_PrintStaticRoots()
+{
+    GC_print_static_roots();
+}
+
 /*
  * External API to register root set in dynamically loaded library.
  * Boehm GC doesn't do this automatically on some platforms.
@@ -199,10 +205,10 @@ void Scm_RegisterDL(void *data_start, void *data_end,
                     void *bss_start, void *bss_end)
 {
     if (data_start < data_end) {
-        GC_add_roots((GC_PTR)data_start, (GC_PTR)data_end);
+        GC_add_roots(data_start, data_end);
     }
     if (bss_start < bss_end) {
-        GC_add_roots((GC_PTR)bss_start, (GC_PTR)bss_end);
+        GC_add_roots(bss_start, bss_end);
     }
 }
 
@@ -226,14 +232,14 @@ void Scm_GCSentinel(void *obj, const char *name)
  */
 void Scm_RegisterFinalizer(ScmObj z, ScmFinalizerProc finalizer, void *data)
 {
-    GC_finalization_proc ofn; GC_PTR ocd;
+    GC_finalization_proc ofn; void *ocd;
     GC_REGISTER_FINALIZER_NO_ORDER(z, (GC_finalization_proc)finalizer,
                                    data, &ofn, &ocd);
 }
 
 void Scm_UnregisterFinalizer(ScmObj z)
 {
-    GC_finalization_proc ofn; GC_PTR ocd;
+    GC_finalization_proc ofn; void *ocd;
     GC_REGISTER_FINALIZER_NO_ORDER(z, (GC_finalization_proc)NULL, NULL,
                                    &ofn, &ocd);
 }
@@ -242,14 +248,15 @@ void Scm_UnregisterFinalizer(ScmObj z)
 void finalizable(void)
 {
     ScmVM *vm = Scm_VM();
-    vm->queueNotEmpty |= SCM_VM_FINQ_MASK;
+    vm->finalizerPending = TRUE;
+    vm->attentionRequest = TRUE;
 }
 
 /* Called from VM loop.  Queue is not empty. */
 ScmObj Scm_VMFinalizerRun(ScmVM *vm)
 {
     GC_invoke_finalizers();
-    vm->queueNotEmpty &= ~SCM_VM_FINQ_MASK;
+    vm->finalizerPending = FALSE;
     return SCM_UNDEFINED;
 }
 
@@ -298,16 +305,29 @@ void Scm_DeleteCleanupHandler(void *handle)
     }
 }
 
+/* pthread specific cleanup handler to unlock mutex */
+#ifdef GAUCHE_USE_PTHREADS
+void Scm__MutexCleanup(void *mutex_)
+{
+    ScmInternalMutex *mutex = mutex_;
+    (void)SCM_INTERNAL_MUTEX_UNLOCK(*mutex);
+}
+#endif /* GAUCHE_USE_PTHREADS */
+
+
 /* Scm_Cleanup and Scm_Exit
    Usually calling Scm_Exit is the easiest way to terminate Gauche
    application safely.  If the application wants to continue operation
    after shutting down the Scheme part, however, it can call Scm_Cleanup().
 */
 
+/* To avoid complication in supporting different platforms */
+#define EXIT_CODE(code) ((code)&0xff)
+
 void Scm_Exit(int code)
 {
     Scm_Cleanup();
-    exit(code);
+    exit(EXIT_CODE(code));
 }
 
 void Scm_Cleanup(void)
@@ -346,15 +366,16 @@ void Scm_Panic(const char *msg, ...)
     va_end(args);
     fputc('\n', stderr);
     fflush(stderr);
-    _exit(1);
+    _exit(EXIT_CODE(1));
 }
 
 /* Use this for absolute emergency.  Newline is not attached to msg. */
 void Scm_Abort(const char *msg)
 {
     int size = (int)strlen(msg);
-    write(2, msg, size); /* this may return an error, but we don't care */
-    _exit(1);
+    /* this may return an error, but we don't care, since we exit anyway. */
+    SCM_IGNORE_RESULT(write(2, msg, size));
+    _exit(EXIT_CODE(1));
 }
 
 /*=============================================================
@@ -411,6 +432,9 @@ init_cond_features()
         { "gauche.os.windows", NULL },
         { "gauche-windows", NULL }, /* for backward compatibility */
 #endif
+#ifdef __CYGWIN__              /* cygwin is different enough to deserve this */
+        { "gauche.os.cygwin", NULL },
+#endif
 #ifdef GAUCHE_USE_PTHREADS
         { "gauche.sys.pthreads", "gauche.threads" },
 #endif
@@ -422,7 +446,7 @@ init_cond_features()
         { "srfi-6", NULL },         /* builtin */
         { "srfi-7", NULL },         /* autoloaded */
         { "srfi-8", NULL },         /* builtin */
-        { "srfi-9", "srfi-9" },
+        { "srfi-9", "gauche.record" },
         { "srfi-10", NULL },
         { "srfi-11", "srfi-11" },
         { "srfi-13", "srfi-13" },
@@ -454,6 +478,8 @@ init_cond_features()
         { "srfi-61", NULL },
         { "srfi-62", NULL },
         { "srfi-87", NULL },
+        { "srfi-98", "srfi-98" },
+        { "srfi-99", "gauche.record" },
         { NULL, NULL }
     };
 
@@ -519,6 +545,33 @@ ScmObj Scm_SiteArchitectureDirectory(void)
     return dir;
 }
 
+/* Temporary hack - this only works on Windows native and Darwin.
+   Returns a pathname that can be replaced for '@' prefix in various
+   configuration paths.  Note that replacement is already done in
+   the returned values of other Scm_*Directory functions above.
+   This is needed to augument the returned values of gauche.config
+   module.  Some refactorization is needed in future.
+   
+   Aside from the current purpose, it will be useful to have a procedure
+   to obtain the directory of the running executable.  This function
+   can eventually evlove to it.
+ */
+ScmObj Scm__RuntimeDirectory(void)
+{
+    static ScmObj dir = SCM_FALSE;
+    if (SCM_FALSEP(dir)) {
+        char buf[PATH_MAX];
+        Scm_GetRuntimeDirectory(buf, PATH_MAX, Scm_Error);
+        dir = Scm_MakeString(buf, -1, -1,
+                             SCM_STRING_COPYING|SCM_STRING_IMMUTABLE);
+    }
+    return dir;
+}
+
+/*=============================================================
+ * 'Main'
+ */
+
 /*
  * When creating DLL under Cygwin, we need the following dummy main()
  * or we get "undefined reference _WinMain@16" error.
@@ -530,3 +583,44 @@ int main(void)
     return 0;
 }
 #endif /*__CYGWIN__*/
+
+/*
+ * A simple main routine useful to build a binary executable.
+ */
+
+void Scm_SimpleMain(int argc, const char *argv[],
+                    const char *script, u_long flags)
+{
+    ScmModule *user = Scm_UserModule();
+    ScmObj mainproc, args;
+    ScmLoadPacket lpak;
+
+    SCM_ASSERT(argc > 0);
+    if (Scm_Load("gauche-init.scm", 0, NULL)) {
+        Scm_Printf(SCM_CURERR, "%s: Couldn't load gauche-init.scm: %A(%A).\n",
+                   argv[0], 
+                   Scm_ConditionMessage(lpak.exception),
+                   Scm_ConditionTypeName(lpak.exception));
+        Scm_Exit(1);
+    }
+
+    args = Scm_CStringArrayToList(argv, argc, SCM_STRING_IMMUTABLE);
+    SCM_DEFINE(user, "*program-name*", SCM_CAR(args));
+    SCM_DEFINE(user, "*argv*", SCM_CDR(args));
+
+    if (script) {
+        ScmObj s = SCM_MAKE_STR(script);
+        ScmObj p = Scm_MakeInputStringPort(SCM_STRING(s), TRUE);
+        Scm_LoadFromPort(SCM_PORT(p), SCM_LOAD_PROPAGATE_ERROR, NULL);
+    }
+
+    mainproc = Scm_GlobalVariableRef(user, SCM_SYMBOL(SCM_INTERN("main")), 0);
+    if (SCM_PROCEDUREP(mainproc)) {
+        ScmObj r = Scm_ApplyRec(mainproc, SCM_LIST1(args));
+        if (SCM_INTP(r)) Scm_Exit(SCM_INT_VALUE(r));
+        else             Scm_Exit(70);
+    } else {
+        Scm_Exit(70);
+    }
+}
+
