@@ -1,7 +1,7 @@
 /*
  * char-euc-jp.h
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: char_euc_jp.h,v 1.16 2007/03/22 11:20:30 shirok Exp $
  */
 
 #ifndef SCM_CHAR_ENCODING_BODY
@@ -117,6 +115,14 @@
 
 SCM_EXTERN const char *Scm_CharBackwardEUC(const char *cp, const char *start);
 
+/* C is an ScmChar > 0x80.  Returns true if C is a whitespace character. */
+#define SCM_CHAR_EXTRA_WHITESPACE(c) ((c) == 0xa1a1) /* zenkaku space */
+/* Like SCM_CHAR_EXTRA_WHITESPACE, but excludes Zl and Zp.
+   See R6RS on the intraline whitespaces. */
+#define SCM_CHAR_EXTRA_WHITESPACE_INTRALINE(c) SCM_CHAR_EXTRA_WHITESPACE(c)
+
+
+
 #else  /* !SCM_CHAR_ENCODING_BODY */
 /*==================================================================
  * This part is included in char.c
