@@ -1,7 +1,7 @@
 /*
  * char-none.h
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: char_none.h,v 1.8 2007/03/22 11:20:30 shirok Exp $
  */
 
 #ifndef SCM_CHAR_ENCODING_BODY
@@ -68,6 +66,12 @@
         else (result) = NULL;                   \
     } while (0)
 
+/* C is an ScmChar > 0x80.  Returns true if C is a whitespace character. */
+#define SCM_CHAR_EXTRA_WHITESPACE(c)  ((c) == 0xa0) /* nbws */
+/* Like SCM_CHAR_EXTRA_WHITESPACE, but excludes Zl and Zp.
+   See R6RS on the intraline whitespaces. */
+#define SCM_CHAR_EXTRA_WHITESPACE_INTRALINE(c) SCM_CHAR_EXTRA_WHITESPACE(c)
+
 #else  /* !SCM_CHAR_ENCODING_BODY */
 /*==================================================================
  * This part is included in char.c
