@@ -1,7 +1,7 @@
 /*
  * code.h - Virtual machine code
  *
- *   Copyright (c) 2005-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2005-2010  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,13 +29,13 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: code.h,v 1.10 2007/09/13 12:30:28 shirok Exp $
  */
 
 #ifndef GAUCHE_CODE_H
 #define GAUCHE_CODE_H
 
+SCM_DECL_BEGIN
+
 #include <gauche/vm.h>
 
 /*
@@ -85,7 +85,7 @@ struct ScmCompiledCodeRec {
  *       is NULL.
  *   *3) This info isn't set for the time being.
  *   *4) This IForm is a direct result of Pass1, i.e. non-optimized form.
- *       Pass2 scans it When IForm is inlined into the caller site.
+ *       Pass2 scans it when IForm is inlined into the caller site.
  */
 
 SCM_CLASS_DECL(Scm_CompiledCodeClass);
@@ -146,15 +146,15 @@ SCM_EXTERN void   Scm_CompiledCodeEmit(ScmCompiledCode *cc,
 /* Macros for transition to the packed code vector of NVM.
    In the packed code vector, VM insns are stored untagged.
    It eliminates the shift in the dispatcher. */
-#define SCM_VM_INSN_CODE(obj)       ((u_int)(SCM_WORD(obj)&0x0ff))
-#define SCM_VM_INSN_ARG(obj)        ((signed long)SCM_WORD(obj) >> 8)
-#define SCM_VM_INSN_ARG0(obj)       ((int)((SCM_WORD(obj) >>  8) & 0x03ff))
-#define SCM_VM_INSN_ARG1(obj)       ((int)((SCM_WORD(obj) >> 18) & 0x03ff))
+#define SCM_VM_INSN_CODE(obj)       ((u_int)(SCM_WORD(obj)&0x0fff))
+#define SCM_VM_INSN_ARG(obj)        ((signed long)SCM_WORD(obj) >> 12)
+#define SCM_VM_INSN_ARG0(obj)       ((int)((SCM_WORD(obj) >> 12) & 0x03ff))
+#define SCM_VM_INSN_ARG1(obj)       ((int)((SCM_WORD(obj) >> 22) & 0x03ff))
 
 #define SCM_VM_INSN(code)           SCM_WORD(code)
-#define SCM_VM_INSN1(code, arg)     SCM_WORD((long)((arg)<<8) | (code))
+#define SCM_VM_INSN1(code, arg)     SCM_WORD((long)((arg)<<12) | (code))
 #define SCM_VM_INSN2(code, arg0, arg1)  \
-    SCM_WORD((long)((arg1) << 18) | ((arg0) << 8) | (code))
+    SCM_WORD((long)((arg1) << 22) | ((arg0) << 12) | (code))
 
 /* Operand type */
 enum {
@@ -172,6 +172,6 @@ SCM_EXTERN int Scm_VMInsnOperandType(u_int code);
 SCM_EXTERN int Scm_VMInsnNameToCode(ScmObj name);
 SCM_EXTERN ScmWord Scm_VMInsnBuild(ScmObj insn);
 
-
+SCM_DECL_END
 
 #endif /* GAUCHE_CODE_H */
