@@ -1,7 +1,7 @@
 /*
  * float.h - auxilirary floating-point number support
  *
- *   Copyright (c) 2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2007-2010  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: float.h,v 1.4 2007/08/24 23:55:45 shirok Exp $
  */
 
 /*
@@ -55,6 +53,12 @@ typedef unsigned short  ScmHalfFloat;
 #define SCM_HALF_FLOAT_SIGN_BIT(hf)  ((hf)&0x8000U)
 #define SCM_HALF_FLOAT_EXPONENT(hf)  (((hf)&0x7c00U)>>10)
 #define SCM_HALF_FLOAT_MANTISSA(hf)  ((hf)&0x03ffU)
+#define SCM_HALF_FLOAT_IS_NAN(hf)                       \
+    ((((~(hf))&0x7c00U) == 0) && (((hf)&0x03ffU) != 0))
+#define SCM_HALF_FLOAT_CMP(op, hf1, hf2)        \
+    (!SCM_HALF_FLOAT_IS_NAN(hf1)                \
+     && !SCM_HALF_FLOAT_IS_NAN(hf2)             \
+     && ((hf1) op (hf2)))
 
 /*
  * Long double support
@@ -66,47 +70,6 @@ typedef long double     ScmLongDouble;
 typedef double          ScmLongDouble;
 #endif
 
-/* IEEE754 double flonum structure.  This info may be provided by
- * a system header (e.g. ieee754.h) but for the portability we
- * define by ourselves.
- */
-typedef union {
-    double d;
-    struct {
-#ifdef DOUBLE_ARMENDIAN
-        /* ARM's mixed endian.  TODO: what if we have LP64 ARM? */
-        unsigned long mant0:20;
-        unsigned int exp:11;
-        unsigned int sign:1;
-        unsigned long mant1:32;
-#else  /*!DOUBLE_ARMENDIAN*/
-#ifdef WORDS_BIGENDIAN
-#if SIZEOF_LONG >= 8
-        unsigned int sign:1;
-        unsigned int exp:11;
-        unsigned long mant:52;
-#else  /*SIZEOF_LONG < 8*/
-        unsigned int sign:1;
-        unsigned int exp:11;
-        unsigned long mant0:20;
-        unsigned long mant1:32;
-#endif /*SIZEOF_LONG < 8*/
-#else  /*!WORDS_BIGENDIAN*/
-#if SIZEOF_LONG >= 8
-        unsigned long mant:52;
-        unsigned int  exp:11;
-        unsigned int  sign:1;
-#else  /*SIZEOF_LONG < 8*/
-        unsigned long mant1:32;
-        unsigned long mant0:20;
-        unsigned int  exp:11;
-        unsigned int  sign:1;
-#endif /*SIZEOF_LONG < 8*/
-#endif /*!WORDS_BIGENDIAN*/
-#endif /*!DOUBLE_ARMENDIAN*/
-    } components;
-} ScmIEEEDouble;
-
 /* NaN and Infinities.  The following works for most Unix platforms w/gcc.
    However, MSVC requires a different treatment. */
 #ifndef SCM_DBL_POSITIVE_INFINITY
