@@ -1,7 +1,7 @@
 /*
  * load.h - Public API for loading files
  *
- *   Copyright (c) 2000-2007 Shiro Kawai <shiro@acm.org>
+ *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: load.h,v 1.3 2007/08/12 03:16:55 shirok Exp $
  */
 
 /* This file is included from gauche.h */
@@ -42,7 +40,7 @@
  * Loading Scheme files
  */
 
-/* Flags for Scm_VMLoad, Scm_Load, amd Scm_Requre. (not for Scm_VMLoadPort) */
+/* Flags for Scm_VMLoad, Scm_Load, amd Scm_Require. (not for Scm_VMLoadPort) */
 typedef enum {
     SCM_LOAD_QUIET_NOFILE = (1L<<0),  /* do not signal an error if the file
                                          does not exist; just return #f. */
@@ -57,7 +55,6 @@ typedef enum {
 typedef struct ScmLoadPacketRec {
     ScmObj exception; /* OUT: exception object in case of LOAD_EVAL_ERROR */
     int    loaded;    /* OUT: TRUE iff file is successfully loaded.  */
-    ScmObj paths;     /* reserved */
 } ScmLoadPacket;
 
 SCM_EXTERN ScmObj Scm_VMLoadFromPort(ScmPort *port, ScmObj next_paths,
@@ -65,15 +62,16 @@ SCM_EXTERN ScmObj Scm_VMLoadFromPort(ScmPort *port, ScmObj next_paths,
 SCM_EXTERN ScmObj Scm_VMLoad(ScmString *file, ScmObj paths, ScmObj env,
 			     int flags);
 
-#if defined(GAUCHE_API_0_8_10) || defined(GAUCHE_API_0_9) || defined(LIBGAUCHE_BODY)
-SCM_EXTERN int Scm_LoadFromPort(ScmPort *port, int flags, ScmLoadPacket *p);
-SCM_EXTERN int Scm_Load(const char *file, int flags, ScmLoadPacket *p);
-#else  /*!GAUCHE_API_0_9*/
+#if !defined(GAUCHE_API_PRE_0_9)
+SCM_EXTERN void Scm_LoadPacketInit(ScmLoadPacket *p);
+SCM_EXTERN int Scm_LoadFromPort(ScmPort *port, u_long flags, ScmLoadPacket *p);
+SCM_EXTERN int Scm_Load(const char *file, u_long flags, ScmLoadPacket *p);
+#else  /*!GAUCHE_API_PRE_0_9*/
 #define Scm_LoadFromPort(port, flags)  Scm__LoadFromPortCompat(port, flags) 
 #define Scm_Load(port, flags)          Scm__LoadCompat(port, flags) 
 SCM_EXTERN void Scm__LoadFromPortCompat(ScmPort *port, int flags);
 SCM_EXTERN int  Scm__LoadCompat(const char *file, int flags);
-#endif /*!GAUCHE_API_0_9*/
+#endif /*!GAUCHE_API_PRE_0_9*/
 
 /*=================================================================
  * Load path management
@@ -86,18 +84,18 @@ SCM_EXTERN ScmObj Scm_AddLoadPath(const char *cpath, int afterp);
  * Dynamic Loading
  */
 
-SCM_EXTERN ScmObj Scm_DynLoad(ScmString *path, ScmObj initfn, int export_);
+SCM_EXTERN ScmObj Scm_DynLoad(ScmString *path, ScmObj initfn, u_long flags);
 
 /*=================================================================
  * Require & Provide
  */
 
-#if defined(GAUCHE_API_0_9) || defined(LIBGAUCHE_BODY)
+#if !defined(GAUCHE_API_PRE_0_9)
 SCM_EXTERN int Scm_Require(ScmObj feature, int flags, ScmLoadPacket *p);
-#else  /*!GAUCHE_API_0_9*/
+#else  /*GAUCHE_API_PRE_0_9*/
 #define Scm_Require(feature) Scm__RequireCompat(feature)
 SCM_EXTERN ScmObj Scm__RequireCompat(ScmObj feature);
-#endif /*!GAUCHE_API_0_9*/
+#endif /*GAUCHE_API_PRE_0_9*/
 SCM_EXTERN ScmObj Scm_Provide(ScmObj feature);
 SCM_EXTERN int    Scm_ProvidedP(ScmObj feature);
 
