@@ -1,7 +1,7 @@
 /*
  * number.h - Public API for Scheme numbers
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: number.h,v 1.8 2007/03/22 11:20:36 shirok Exp $
  */
 
 /* This file is included from gauche.h */
@@ -111,15 +109,6 @@ struct ScmRatnumRec {
 #define SCM_RATNUM_NUMER(obj)      SCM_RATNUM(obj)->numerator
 #define SCM_RATNUM_DENOM(obj)      SCM_RATNUM(obj)->denominator
 
-struct ScmFlonumRec {
-    SCM_HEADER;
-    double value;
-};
-
-#define SCM_FLONUM(obj)            ((ScmFlonum*)(obj))
-#define SCM_FLONUMP(obj)           SCM_XTYPEP(obj, SCM_CLASS_REAL)
-#define SCM_FLONUM_VALUE(obj)      (SCM_FLONUM(obj)->value)
-
 struct ScmCompnumRec {
     SCM_HEADER;
     double real;
@@ -131,6 +120,47 @@ struct ScmCompnumRec {
 #define SCM_COMPNUM_REAL(obj)      SCM_COMPNUM(obj)->real
 #define SCM_COMPNUM_IMAG(obj)      SCM_COMPNUM(obj)->imag
 
+/*
+   Internal macros concerning FFX (fast flonum extension).  Normal
+   users doesn't need to care about these.
+
+   When GAUCHE_FFX is defined,
+   new flonums generated by the system is put in the designated area
+   of the VM instead of the heap, thus eliminating most of
+   intermediate flonum allocations.  Flonum pointers have two flavors;
+   if SCM_FLONUM_REG_P is true, it is allocated in the VM registers,
+   and if SCM_FLONUM_MEM_P is true, it is allocated in heap.  An
+   invariance: only ScmObjs in VM stack are allowed to be FLONUM_REGs.
+   If such an ScmObj is moved to the heap, a new FLONUM_MEM is
+   allocated and the pointer is fixed.
+
+   User routines don't usually receive or return FLONUM_REGs.  If the
+   routine is directly called from VM as SUBR, and it never stores
+   received ScmObjs in the heap (in other words, if the routine
+   immediately retrieves doubles from the received ScmObjs), the
+   routine denotes the fact by marking its stub with "immediate-arg"
+   so that VM can pass FLONUM_REGs.  If the routine is sure that its
+   return value is used directly by the VM, it can call
+   Scm_VMReturnFlonum to generate FLONUM_REGs.
+ */
+
+#define SCM_MAKE_FLONUM_REG(dptr)  SCM_OBJ(SCM_WORD(dptr)+0x02)
+#define SCM_MAKE_FLONUM_MEM(dptr)  SCM_OBJ(SCM_WORD(dptr)+0x06)
+
+#define SCM_FLONUM_REG_P(obj)      ((SCM_WORD(obj)&0x7) == 0x02)
+#define SCM_FLONUM_MEM_P(obj)      ((SCM_WORD(obj)&0x7) == 0x06)
+
+#if GAUCHE_FFX
+#define SCM_FLONUM_ENSURE_MEM(var)                              \
+    do {                                                        \
+        if (SCM_FLONUM_REG_P(var)) {                            \
+            var = Scm_MakeFlonum(SCM_FLONUM_VALUE(var));        \
+        }                                                       \
+    } while (0)
+#else  /*!GAUCHE_FFX*/
+#define SCM_FLONUM_ENSURE_MEM(var) /* empty */
+#endif /*!GAUCHE_FFX*/
+
 /* Converting a Scheme number to a C number:
  *
  * It's a tricky business.  It's always possible that the Scheme number
@@ -210,13 +240,22 @@ SCM_EXTERN ScmObj Scm_MakeComplexPolar(double magnitude, double angle);
 
 SCM_EXTERN int    Scm_IntegerP(ScmObj obj);
 SCM_EXTERN int    Scm_OddP(ScmObj obj);
+SCM_EXTERN int    Scm_FiniteP(ScmObj obj);
+SCM_EXTERN int    Scm_InfiniteP(ScmObj obj);
+SCM_EXTERN int    Scm_NanP(ScmObj obj);
 SCM_EXTERN ScmObj Scm_Abs(ScmObj obj);
 SCM_EXTERN int    Scm_Sign(ScmObj obj);
 SCM_EXTERN ScmObj Scm_Negate(ScmObj obj);
 SCM_EXTERN ScmObj Scm_Reciprocal(ScmObj obj);
 SCM_EXTERN ScmObj Scm_ReciprocalInexact(ScmObj obj);
-SCM_EXTERN ScmObj Scm_ExactToInexact(ScmObj obj);
+
+/* TODO: We'll switch to make Scm_Exact/Scm_Inexact official in 1.0
+   and obsolete longer versions.  During 0.9.x we keep the old name
+   for binary compatibility. */
 SCM_EXTERN ScmObj Scm_InexactToExact(ScmObj obj);
+SCM_EXTERN ScmObj Scm_ExactToInexact(ScmObj obj);
+#define Scm_Exact    Scm_InexactToExact
+#define Scm_Inexact  Scm_ExactToInexact
 
 SCM_EXTERN ScmObj Scm_Add(ScmObj arg1, ScmObj arg2);
 SCM_EXTERN ScmObj Scm_Sub(ScmObj arg1, ScmObj arg2);
@@ -231,6 +270,10 @@ SCM_EXTERN ScmObj Scm_Gcd(ScmObj x, ScmObj y);
 SCM_EXTERN ScmObj Scm_Expt(ScmObj x, ScmObj y);
 
 SCM_EXTERN int    Scm_NumEq(ScmObj x, ScmObj y);
+SCM_EXTERN int    Scm_NumLT(ScmObj x, ScmObj y);
+SCM_EXTERN int    Scm_NumLE(ScmObj x, ScmObj y);
+SCM_EXTERN int    Scm_NumGT(ScmObj x, ScmObj y);
+SCM_EXTERN int    Scm_NumGE(ScmObj x, ScmObj y);
 SCM_EXTERN int    Scm_NumCmp(ScmObj x, ScmObj y);
 SCM_EXTERN void   Scm_MinMax(ScmObj arg0, ScmObj args, ScmObj *min, ScmObj *max);
 
@@ -264,5 +307,27 @@ SCM_EXTERN ScmObj Scm_StringToNumber(ScmString *str, int radix, int strict);
 
 SCM_EXTERN void   Scm_PrintDouble(ScmPort *port, double d, int flags);
 
+/* This is here, for we need to check double endianness on ARM. */
+SCM_EXTERN ScmObj Scm_NativeEndian(void);
+SCM_EXTERN ScmObj Scm_DefaultEndian(void);
+SCM_EXTERN void   Scm_SetDefaultEndian(ScmObj endian);
+
+/* The Scm_VM* version leaves unboxed flonum (FLONUM_REG) in VM's VAL0
+   register.   They can only be called "on VM", that is, when its return
+   value is used immediately by VM. */
+
+SCM_EXTERN ScmObj Scm_VMNegate(ScmObj obj);
+SCM_EXTERN ScmObj Scm_VMReciprocal(ScmObj obj);
+SCM_EXTERN ScmObj Scm_VMReciprocalInexact(ScmObj obj);
+SCM_EXTERN ScmObj Scm_VMInexact(ScmObj obj);
+SCM_EXTERN ScmObj Scm_VMAbs(ScmObj obj);
+SCM_EXTERN ScmObj Scm_VMAdd(ScmObj arg1, ScmObj arg2);
+SCM_EXTERN ScmObj Scm_VMSub(ScmObj arg1, ScmObj arg2);
+SCM_EXTERN ScmObj Scm_VMMul(ScmObj arg1, ScmObj arg2);
+SCM_EXTERN ScmObj Scm_VMDiv(ScmObj arg1, ScmObj arg2);
+SCM_EXTERN ScmObj Scm_VMDivInexact(ScmObj arg1, ScmObj arg2);
+SCM_EXTERN ScmObj Scm_VMExpt(ScmObj x, ScmObj y);
+SCM_EXTERN ScmObj Scm_VMRound(ScmObj num, int mode);
+
 #endif /* GAUCHE_NUMBER_H */
 
