@@ -1,7 +1,7 @@
 /*
  * gauche/port.h - Port API
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: port.h,v 1.27 2007/09/10 22:07:50 shirok Exp $
  */
 
 #ifndef GAUCHE_PORT_H
@@ -103,18 +101,18 @@ typedef struct ScmPortBufferRec {
 /* The funtion table of procedural port. */
 
 typedef struct ScmPortVTableRec {
-    int       (*Getb)(ScmPort *p);
-    int       (*Getc)(ScmPort *p);
-    int       (*Getz)(char *buf, int buflen, ScmPort *p);
-    int       (*Ready)(ScmPort *p, int charp);
-    void      (*Putb)(ScmByte b, ScmPort *p);
-    void      (*Putc)(ScmChar c, ScmPort *p);
-    void      (*Putz)(const char *buf, int size, ScmPort *p);
-    void      (*Puts)(ScmString *s, ScmPort *p);
-    void      (*Flush)(ScmPort *p);
-    void      (*Close)(ScmPort *p);
-    off_t     (*Seek)(ScmPort *p, off_t off, int whence);
-    void      *data;
+    int    (*Getb)(ScmPort *p);
+    int    (*Getc)(ScmPort *p);
+    int    (*Getz)(char *buf, int buflen, ScmPort *p);
+    int    (*Ready)(ScmPort *p, int charp);
+    void   (*Putb)(ScmByte b, ScmPort *p);
+    void   (*Putc)(ScmChar c, ScmPort *p);
+    void   (*Putz)(const char *buf, int size, ScmPort *p);
+    void   (*Puts)(ScmString *s, ScmPort *p);
+    void   (*Flush)(ScmPort *p);
+    void   (*Close)(ScmPort *p);
+    off_t  (*Seek)(ScmPort *p, off_t off, int whence);
+    void    *data;
 } ScmPortVTable;
 
 /* The main port structure.
@@ -127,17 +125,17 @@ typedef struct ScmPortVTableRec {
 
 struct ScmPortRec {
     SCM_INSTANCE_HEADER;
-    unsigned int direction : 2; /* SCM_PORT_INPUT or SCM_PORT_OUTPUT.
+    u_int direction : 2;        /* SCM_PORT_INPUT or SCM_PORT_OUTPUT.
                                    There may be I/O port in future. */
-    unsigned int type      : 2; /* SCM_PORT_{FILE|ISTR|OSTR|PROC} */
-    unsigned int scrcnt    : 3; /* # of bytes in the scratch buffer */
+    u_int type      : 2;        /* SCM_PORT_{FILE|ISTR|OSTR|PROC} */
+    u_int scrcnt    : 3;        /* # of bytes in the scratch buffer */
 
-    unsigned int ownerp    : 1; /* TRUE if this port owns underlying
+    u_int ownerp    : 1;        /* TRUE if this port owns underlying
                                    file pointer */
-    unsigned int closed    : 1; /* TRUE if this port is closed */
-    unsigned int error     : 1; /* Error has been occurred */
+    u_int closed    : 1;        /* TRUE if this port is closed */
+    u_int error     : 1;        /* Error has been occurred */
 
-    unsigned int flags     : 5; /* see ScmPortFlags below */
+    u_int flags     : 5;        /* see ScmPortFlags below */
     
     char scratch[SCM_CHAR_MAX_BYTES]; /* incomplete buffer */
 
@@ -151,8 +149,18 @@ struct ScmPortRec {
 
     ScmObj data;                /* used internally */
 
-    unsigned int line;          /* line counter */
+    /* Input counters.  these doesn't take account of ungetting and
+       seeking: Ungetting doesn't affect those counters (you can think
+       that ungetting are handled above the counting layer).
+       Seeking invalidates counters; if you seek, the values of the counters
+       become bogus.
+       We don't have character counter, since it is difficult to track
+       (read-line uses byte read; see Scm_ReadLine in portapi.c).
+     */
+    u_long line;                /* line counter */
+    u_long bytes;               /* byte counter */
 
+    /* The source or the sink of the port. */
     union {
         ScmPortBuffer buf;      /* buffered port */
         struct {
@@ -200,8 +208,10 @@ enum ScmPortFlags {
     SCM_PORT_WRITESS = (1L<<0), /* write/ss on by default? */
     SCM_PORT_WALKING = (1L<<1), /* this port is a special port only used in
                                    the 'walk' phase of write/ss. */
-    SCM_PORT_PRIVATE = (1L<<2)  /* this port is for 'private' use within
+    SCM_PORT_PRIVATE = (1L<<2), /* this port is for 'private' use within
                                    a thread, so never need to be locked. */
+    SCM_PORT_CASE_FOLD = (1L<<3) /* read from or write to this port should
+                                    be case folding. */
 };
 
 #if 0 /* not implemented */
@@ -231,6 +241,8 @@ enum ScmPortICPolicy {
 #define SCM_PORT_FLAGS(obj)     (SCM_PORT(obj)->flags)
 #define SCM_PORT_ICPOLICY(obj)  (SCM_PORT(obj)->icpolicy)
 
+#define SCM_PORT_CASE_FOLD(obj) (SCM_PORT_FLAGS(obj)&SCM_PORT_CASE_FOLD)
+
 #define SCM_PORT_CLOSED_P(obj)  (SCM_PORT(obj)->closed)
 #define SCM_PORT_OWNER_P(obj)   (SCM_PORT(obj)->ownerp)
 #define SCM_PORT_ERROR_OCCURRED_P(obj) (SCM_PORT(obj)->error)
@@ -335,6 +347,10 @@ SCM_EXTERN ScmObj Scm_Stdin(void);
 SCM_EXTERN ScmObj Scm_Stdout(void);
 SCM_EXTERN ScmObj Scm_Stderr(void);
 
+SCM_EXTERN ScmObj Scm_SetStdin(ScmPort *port);
+SCM_EXTERN ScmObj Scm_SetStdout(ScmPort *port);
+SCM_EXTERN ScmObj Scm_SetStderr(ScmPort *port);
+
 #define SCM_CURIN    SCM_VM_CURRENT_INPUT_PORT(Scm_VM())
 #define SCM_CUROUT   SCM_VM_CURRENT_OUTPUT_PORT(Scm_VM())
 #define SCM_CURERR   SCM_VM_CURRENT_ERROR_PORT(Scm_VM())
@@ -350,12 +366,12 @@ SCM_EXTERN ScmObj Scm_SetCurrentErrorPort(ScmPort *port);
 SCM_EXTERN ScmObj Scm_MakeInputStringPort(ScmString *str, int privatep);
 SCM_EXTERN ScmObj Scm_MakeOutputStringPort(int privatep);
 
-#if defined(GAUCHE_API_0_8_10) || defined(GAUCHE_API_0_9) || defined(LIBGAUCHE_BODY)
+#if !defined(GAUCHE_API_PRE_0_9)
 SCM_EXTERN ScmObj Scm_GetOutputString(ScmPort *port, int flags);
 SCM_EXTERN ScmObj Scm_GetOutputStringUnsafe(ScmPort *port, int flags);
 SCM_EXTERN ScmObj Scm_GetRemainingInputString(ScmPort *port, int flags);
 
-#else  /* !defined(GAUCHE_API_0_8_10) && !defined(GAUCHE_API_0_9) && !defined(LIBGAUCHE_BODY) */
+#else  /* GAUCHE_API_PRE_0_9 */
 #define Scm_GetOutputString(p) Scm__GetOutputStringCompat(p)
 #define Scm_GetOutputStringUnsafe(p) Scm__GetOutputStringUnsafeCompat(p)
 #define Scm_GetRemainingInputString(p) Scm__GetRemainingInputStringCompat(p)
@@ -364,7 +380,7 @@ SCM_EXTERN ScmObj Scm_GetRemainingInputString(ScmPort *port, int flags);
 SCM_EXTERN ScmObj Scm__GetOutputStringCompat(ScmPort *port);
 SCM_EXTERN ScmObj Scm__GetOutputStringUnsafeCompat(ScmPort *port);
 SCM_EXTERN ScmObj Scm__GetRemainingInputStringCompat(ScmPort *port);
-#endif /* !defined(GAUCHE_API_0_8_10) && !defined(GAUCHE_API_0_9) && !defined(LIBGAUCHE_BODY) */
+#endif /* GAUCHE_API_PRE_0_9 */
 
 /*================================================================
  * Other type of ports
@@ -387,6 +403,11 @@ SCM_EXTERN ScmObj Scm_MakeCodingAwarePort(ScmPort *iport);
 
 SCM_EXTERN void Scm__InstallCodingAwarePortHook(ScmPort *(*)(ScmPort*, const char*));
 
+/* Windows-specific initialization */
+#if defined(GAUCHE_WINDOWS)
+void Scm__SetupPortsForWindows(int has_console);
+#endif /*defined(GAUCHE_WINDOWS)*/
+
 /*================================================================
  * Locking the ports
  *
