@@ -1,7 +1,7 @@
 /*
  * pthread.h - pthread primitives
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,40 +29,50 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: pthread.h,v 1.12 2007/03/22 11:20:36 shirok Exp $
  */
 
 #ifndef GAUCHE_PTHREAD_H
 #define GAUCHE_PTHREAD_H
 
+/* Abstract thread implementation based on pthreads.
+ * See gauche/uthread.h for the detailed semantics of those macros.
+ */
+
 #include <pthread.h>
 
+/* Thread */
+typedef pthread_t ScmInternalThread;
+#define SCM_INTERNAL_THREAD_INIT(thr)   ((thr) = (pthread_t)NULL)
+#define SCM_INTERNAL_THREAD_INITIALIZED_P(thr)  ((thr) != (pthread_t)NULL)
+
 /* Mutex */
 typedef pthread_mutex_t ScmInternalMutex;
-#define SCM_INTERNAL_MUTEX_INIT(mutex) \
-    pthread_mutex_init(&(mutex), NULL)
-#define SCM_INTERNAL_MUTEX_LOCK(mutex) \
-    pthread_mutex_lock(&(mutex))
-#define SCM_INTERNAL_MUTEX_UNLOCK(mutex) \
-    pthread_mutex_unlock(&(mutex))
-#define SCM_INTERNAL_MUTEX_INITIALIZER \
-    PTHREAD_MUTEX_INITIALIZER
+#define SCM_INTERNAL_MUTEX_INIT(mutex)    pthread_mutex_init(&(mutex), NULL)
+#define SCM_INTERNAL_MUTEX_LOCK(mutex)    pthread_mutex_lock(&(mutex))
+#define SCM_INTERNAL_MUTEX_UNLOCK(mutex)  pthread_mutex_unlock(&(mutex))
+#define SCM_INTERNAL_MUTEX_DESTROY(mutex) pthread_mutex_destroy(&(mutex))
+#define SCM_INTERNAL_MUTEX_INITIALIZER    PTHREAD_MUTEX_INITIALIZER
+
+SCM_EXTERN void Scm__MutexCleanup(void *); /* in core.c */
+
+/* Mutex operation with cleanup.  The dummy statement before
+   pthread_cleanup_pop() allows a label to be placed before
+   SAFE_LOCK_END macro.  Without that, it could be an error
+   if pthread_cleanup_pop expands into something beginning with
+   closing brace. */
+#define SCM_INTERNAL_MUTEX_SAFE_LOCK_BEGIN(mutex)               \
+    pthread_mutex_lock(&(mutex));                               \
+    pthread_cleanup_push(Scm__MutexCleanup, &(mutex))
+#define SCM_INTERNAL_MUTEX_SAFE_LOCK_END() /*dummy*/; pthread_cleanup_pop(1)
 
 /* Condition variable */
 typedef pthread_cond_t ScmInternalCond;
-#define SCM_INTERNAL_COND_INIT(cond) \
-    pthread_cond_init(&(cond), NULL)
-#define SCM_INTERNAL_COND_SIGNAL(cond) \
-    pthread_cond_signal(&(cond))
-#define SCM_INTERNAL_COND_BROADCAST(cond) \
-    pthread_cond_broadcast(&(cond))
-#define SCM_INTERNAL_COND_WAIT(cond, mutex) \
-    pthread_cond_wait(&(cond), &(mutex))
-#define SCM_INTERNAL_COND_DESTROY(cond) \
-    pthread_cond_destroy(&(cond))
-#define SCM_INTERNAL_COND_INITIALIZER \
-    PTHREAD_COND_INITIALIZER
+#define SCM_INTERNAL_COND_INIT(cond)        pthread_cond_init(&(cond), NULL)
+#define SCM_INTERNAL_COND_SIGNAL(cond)      pthread_cond_signal(&(cond))
+#define SCM_INTERNAL_COND_BROADCAST(cond)   pthread_cond_broadcast(&(cond))
+#define SCM_INTERNAL_COND_WAIT(cond, mutex) pthread_cond_wait(&(cond), &(mutex))
+#define SCM_INTERNAL_COND_DESTROY(cond)     pthread_cond_destroy(&(cond))
+#define SCM_INTERNAL_COND_INITIALIZER       PTHREAD_COND_INITIALIZER
 
 /* Fast lock */
 #ifdef HAVE_PTHREAD_SPINLOCK_T
