@@ -1,7 +1,7 @@
 /*
  * vm.h - Virtual machine
  *
- *   Copyright (c) 2000-2005 Shiro Kawai, All rights reserved.
+ *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: vm.h,v 1.107 2006/11/09 10:32:20 shirok Exp $
  */
 
 #ifndef GAUCHE_VM_H
@@ -42,16 +40,9 @@
 /* Maximum # of values allowed for multiple value return */
 #define SCM_VM_MAX_VALUES      20
 
-/* Signal queue size */
-#define SCM_VM_SIGQ_SIZE       32
-
 /* Finalizer queue size */
 #define SCM_VM_FINQ_SIZE       32
 
-/* QueueNotEmpty flag */
-#define SCM_VM_SIGQ_MASK       1
-#define SCM_VM_FINQ_MASK       2
-
 #define SCM_PCTYPE ScmWord*
 
 #if defined(ITIMER_PROF) && defined(SIGPROF)
@@ -59,6 +50,26 @@
 #define GAUCHE_PROFILE
 #endif /* defined(ITIMER_PROF) && defined(SIGPROF) */
 
+
+#ifdef __GNUC__
+/* in GNUC, we inline Scm_VMReturnFlonum using statement expressions. */
+# if GAUCHE_FFX
+SCM_EXTERN void Scm_VMFlushFPStack(ScmVM *vm);
+#   define Scm_VMReturnFlonum(_val)                                     \
+        ({ ScmVM *vm__ = Scm_VM(); ScmFlonum *fp;                       \
+           if (vm__->fpsp == vm__->fpstackEnd) Scm_VMFlushFPStack(vm__);\
+           fp = vm__->fpsp++;                                           \
+           SCM_FLONUM_VALUE(fp) = (_val);                               \
+           SCM_MAKE_FLONUM_REG(fp);                                     \
+        })
+# else  /* !GAUCHE_FFX */
+#   define Scm_VMReturnFlonum(val)  Scm_MakeFlonum(val)
+# endif /* !GAUCHE_FFX */
+#else  /* !__GNUC__ */
+#   define Scm_VMReturnFlonum(val)  Scm_MakeFlonum(val)
+#endif /* !__GNUC__ */
+
+
 /* Actual structure is defined in code.h */
 typedef struct ScmCompiledCodeRec ScmCompiledCode;
 
@@ -171,11 +182,15 @@ SCM_EXTERN int    Scm_FreeVariableEqv(ScmObj var, ScmObj sym, ScmObj env);
 
 /*
  * C stack record
+ *
+ *  A chain of C stack to rewind.  C stack is captured by Scm_Sigsetjmp
+ *  and rewound by Scm_Siglongjmp; see signal.c.
  */
 typedef struct ScmCStackRec {
     struct ScmCStackRec *prev;
     ScmContFrame *cont;
     sigjmp_buf jbuf;
+    sigset_t mask;
 } ScmCStack;
 
 /*
@@ -221,7 +236,11 @@ typedef struct ScmEscapePointRec {
     ScmObj ehandler;            /* handler closure */
     ScmContFrame *cont;         /* saved continuation */
     ScmObj handlers;            /* saved dynamic handler chain */
-    ScmCStack *cstack;          /* C stack */
+    ScmCStack *cstack;          /* vm->cstack when escape point is created.
+                                   this will be used to rewind cstack.
+                                   this is NULL for partial continuations,
+                                   for they can be executed on anywhere
+                                   w.r.t. cstack. */
     ScmObj xhandler;            /* saved exception handler */
     int errorReporting;         /* state of SCM_VM_ERROR_REPORTING flag
                                    when this ep is captured.  The flag status
@@ -256,47 +275,31 @@ typedef struct ScmEscapePointRec {
 #define SCM_VM_ESCAPE_CONT   2
 #define SCM_VM_ESCAPE_EXIT   3
 
-/*
- * Parameters
- *
- *  Parameters keep thread-local state.   It is called 'fluids' in some
- *  Scheme implementations.  A thread inherits the parameters from its
- *  creator.   
- */
-
-typedef struct ScmVMParameterTableRec {
-    int numParameters;
-    int numAllocated;
-    ScmObj *vector;
-    int *ids;
-} ScmVMParameterTable;
-
-SCM_EXTERN void Scm_ParameterTableInit(ScmVMParameterTable *table,
-                                       ScmVM *base);
-
-SCM_EXTERN int Scm_MakeParameterSlot(ScmVM *vm, int *newid);
-SCM_EXTERN ScmObj Scm_ParameterRef(ScmVM *vm, int index, int id);
-SCM_EXTERN ScmObj Scm_ParameterSet(ScmVM *vm, int index, int id, ScmObj value);
-
 /*
  * Signal queue
  *
- *  Gauche installs singnal handlers that simply queues the signal.
- *  See signal.c for the implementaiton.
- *  Signal queue is a fixed size, since we can't really extend it
- *  within the system's signal handler.  For the time being, overflowed
- *  signals are simply discarded.
+ *  Following the Unix tradition, gauche doesn't really queue the signals;
+ *  the C-level signal handler only records the fact that a specific
+ *  signal has been delivered.  VM loop examines the record and invoke
+ *  appropriate Scheme-level signal handlers.  Even if the same signal
+ *  arrives more than one time before VM loop checks the record, the handler
+ *  is invoked only once.
+ *
+ *  The C-level signal handler does count each signal until it is cleared
+ *  by VM loop.  If one kind of signal arraives more than a certain limit
+ *  (set/get by Scm_{Set|Get}SignalPendingLimit), Gauche thinks something
+ *  went wrong and bail out.   It is useful, for example, to interrupt
+ *  unresponsive program from the terminal.
  */
 
 typedef struct ScmSignalQueueRec {
-    int queue[SCM_VM_SIGQ_SIZE];/* Ring buffer for pending signals */
-    unsigned int head;     /* points to the queue head */
-    unsigned int tail;     /* points to the queue tail */
-    unsigned int overflow; /* flag to indicate queue overflow */
+    unsigned char sigcounts[NSIG];
     ScmObj pending;        /* pending signal handlers */
 } ScmSignalQueue;
 
 SCM_EXTERN void Scm_SignalQueueInit(ScmSignalQueue* q);
+SCM_EXTERN int  Scm_GetSignalPendingLimit(void);
+SCM_EXTERN void Scm_SetSignalPendingLimit(int val);
 
 /* NB: Obsoleted macro, but kept for backward compatibility.
    Better API should be provided in future. */
@@ -304,7 +307,7 @@ SCM_EXTERN void Scm_SignalQueueInit(ScmSignalQueue* q);
     ((vm)->queueNotEmpty&SCM_VM_SIGQ_MASK)
 
 #define SCM_SIGCHECK(vm) \
-    do { if (vm->queueNotEmpty&SCM_VM_SIGQ_MASK) Scm_SigCheck(vm); } while (0)
+    do { if (vm->signalPending) Scm_SigCheck(vm); } while (0)
 
 SCM_EXTERN void   Scm_SigCheck(ScmVM *vm);
 
@@ -351,27 +354,28 @@ typedef struct ScmVMProfilerRec ScmVMProfiler;
  *  Most fields of VM are private to the thread that owns the VM.
  *  Only the fields marked as "PUBLIC" should be modified by other
  *  thread, and only with obtaining the lock by VMLOCK mutex.
- *
- *  Note that some fields like "name" and "specific" are not marked
+ *  (Note that some fields like "name" and "specific" are not marked
  *  as PUBLIC although they can be referenced or modified by other
  *  thread (via Scheme call thread-specific-set! etc.)   It is the
- *  user program's responsibility to use a mutex.
- *  When you should introspect other thread (like stack trace), make
- *  sure you stopped that thread, or you may get inconsistent result.
+ *  user program's responsibility to use a mutex.)
+ *
+ *  When you need inspect other thread's private data (like stack
+ *  trace), make the thread is either stopped or terminated, or
+ *  you may get inconsistent result.
  */
 
 struct ScmVMRec {
     SCM_HEADER;
-#ifdef GAUCHE_USE_PTHREADS
-    pthread_t thread;           /* the thread executing this VM. */
-#endif /*!GAUCHE_USE_PTHREADS*/
+    ScmInternalThread thread;   /* the system thread executing this VM. */
     int state;                  /* thread state. PUBLIC. */
     ScmInternalMutex  vmlock;   /* mutex to be used to lock this VM
                                    structure.  PUBLIC. */
     ScmInternalCond cond;       /* the condition variable to wait for state
                                    change of this VM.  PUBLIC. */
-    ScmVM *canceller;           /* the thread that called thread-terminate!
+    ScmVM *canceller;           /* the thread which called thread-terminate!
                                    on this thread.  PUBLIC. */
+    ScmVM *inspector;           /* the thread which requested to stop this
+                                   thread.  PUBLIC. */
     ScmObj name;                /* Scheme thread name. */
     ScmObj specific;            /* Scheme thread specific data. */
     ScmProcedure *thunk;        /* Entry point of this VM. */
@@ -383,7 +387,21 @@ struct ScmVMRec {
                                    "C stack rewinding" below. */
     unsigned int runtimeFlags;  /* Runtime flags */
     unsigned int compilerFlags; /* Compiler flags */
-    unsigned int queueNotEmpty; /* Bitmask if sigq or finq is not empty */
+    intptr_t attentionRequest;  /* Flag if VM needs to check signal queue,
+                                   finalizer queue, or stop request.
+                                   This flag can be turned on asynchronously.
+                                   Only this VM can turn off this flag. */
+    intptr_t signalPending;     /* Flag if there are pending signals.
+                                   Turned on by sig_handle(), turned off
+                                   by Scm_SigCheck(), both in signal.c. */
+    intptr_t finalizerPending;  /* Flag if there are pending finalizers.
+                                   Turned on by finalizable() callback,
+                                   and turned off by Scm_VMFinalizerRun(),
+                                   both in core.c */
+    intptr_t stopRequest;       /* Flag if there is a pending stop request.
+                                   Turned on by Scm_ThreadStop() in
+                                   ext/threads/threads.c, and turned off by
+                                   process_queued_requests() in vm.c */
 
     ScmPort *curin;             /* current input port */
     ScmPort *curout;            /* current output port */
@@ -411,6 +429,14 @@ struct ScmVMRec {
     ScmObj *stackBase;          /* base of current stack area  */
     ScmObj *stackEnd;           /* end of current stack area */
 
+#if GAUCHE_FFX
+    ScmFlonum *fpsp;            /* flonum stack pointer.  we call it 'stack'
+                                   for historical reasons, but it's more like
+                                   a nursery. */
+    ScmFlonum *fpstack;         /* flonum stack */
+    ScmFlonum *fpstackEnd;      /* flonum stack limit */
+#endif
+
     /* Escape handling */
     ScmObj exceptionHandler;    /* the current exception handler installed by
                                    with-exception-handler. */
@@ -452,6 +478,7 @@ SCM_EXTERN void   Scm_VMDefaultExceptionHandler(ScmObj exc);
 SCM_EXTERN ScmObj Scm_VMThrowException(ScmVM *vm, ScmObj exc);
 SCM_EXTERN ScmObj Scm_VMGetSourceInfo(ScmCompiledCode *code, SCM_PCTYPE pc);
 SCM_EXTERN ScmObj Scm_VMGetBindInfo(ScmCompiledCode *code, SCM_PCTYPE pc);
+SCM_EXTERN void   Scm_VMSetResult(ScmObj obj);
 
 SCM_CLASS_DECL(Scm_VMClass);
 #define SCM_CLASS_VM              (&Scm_VMClass)
@@ -467,10 +494,8 @@ enum {
                                    initialized. */
     SCM_VM_RUNNABLE,            /* This VM is attached to a thread which is
                                    runnable or blocked. */
-    SCM_VM_BLOCKED,             /* The thread attached to this VM is stopped
-                                   because of thread-yield! or thread-sleep!.
-                                   Note that if the thread is blocked by
-                                   system call, VM's state is still RUNNABLE.*/
+    SCM_VM_STOPPED,             /* The thread attached to this VM is stopped
+                                   by the inspector thread for debugging. */
     SCM_VM_TERMINATED           /* The thread attached to this VM is
                                    terminated. */
 };
@@ -508,13 +533,15 @@ enum {
  *  part is called while the C stack is rewind.
  *  If you want to handle error as well, you should install error handler
  *  by yourself (and deinstall it in the cleanup code).
- *  
- *  If you don't call SCM_NEXT_HANDLER in the SCM_WHEN_ERROR clause,
- *  the control is transferred after SCM_END_PROTECT.  It is not recommended
- *  unless you know what you're doing.   The C stack is rewind not only
- *  at the error situation, but also a continuation is thrown in the main
- *  operation.  Except certain special occasions, stopping C-stack rewinding
- *  may cause semantic inconsistency.
+ * 
+ *  In general, you MUST call SCM_NEXT_HANDLER in the SCM_WHEN_ERROR clause.
+ *  In other words, you shouldn't use SCM_UNWIND_PROTECT as "ignore-errors"
+ *  construct.  The C stack is rewind not only at the error situation, but
+ *  also when acontinuation is thrown in the main operation.  Except certain
+ *  special occasions, stopping C-stack rewinding may cause semantic
+ *  inconsistency.   Besides, we don't save signal mask in SCM_UNWIND_PROTECT
+ *  for the performance reason; assuming the mask is eventually recovered
+ *  by the core exception handling mechanism (see vm.c).
  */
 
 #define SCM_UNWIND_PROTECT                      \
@@ -573,9 +600,11 @@ enum {
 
 #define SCM_CCONT_DATA_SIZE 6
 
+typedef ScmObj ScmCContinuationProc(ScmObj result, void **data);
+
 typedef struct ScmCContinuation {
     SCM_HEADER;
-    ScmObj (*func)(ScmObj value, void **data);
+    ScmCContinuationProc *func;
     void *data[SCM_CCONT_DATA_SIZE];
 } ScmCContinuation;
 
@@ -585,7 +614,7 @@ typedef struct ScmCContinuation {
 SCM_CLASS_DECL(Scm_CContClass);
 #define SCM_CLASS_CCONT           (&Scm_CContClass)
 
-SCM_EXTERN void Scm_VMPushCC(ScmObj (*func)(ScmObj value, void **data),
+SCM_EXTERN void Scm_VMPushCC(ScmCContinuationProc *func,
 			     void **data,
 			     int datasize);
 
@@ -600,7 +629,8 @@ enum {
     SCM_COMPILE_NOSOURCE = (1L<<3),        /* Do not insert source info */
     SCM_COMPILE_SHOWRESULT = (1L<<4),      /* Display each result of
                                               compilation */
-    SCM_COMPILE_NOCOMBINE = (1L<<5)        /* Do not combine instructions */
+    SCM_COMPILE_NOCOMBINE = (1L<<5),       /* Do not combine instructions */
+    SCM_COMPILE_NO_PASS2POST = (1L<<6)     /* Do not run pass2-post. */
 };
 
 #define SCM_VM_COMPILER_FLAG_IS_SET(vm, flag) ((vm)->compilerFlags & (flag))
