@@ -1,7 +1,7 @@
 /*
  * gloc.c - gloc implementation
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: gloc.c,v 1.2 2007/03/02 07:39:13 shirok Exp $
  */
 
 #define LIBGAUCHE_BODY
@@ -43,9 +41,12 @@
 static void gloc_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
 {
     ScmGloc *g = SCM_GLOC(obj);
-    Scm_Printf(port, "#<gloc %S%s%S>", g->module->name,
+    Scm_Printf(port, "#<gloc %S%s%S%s>", g->module->name,
                (g->exported?"#":"##"),
-               g->name);
+               g->name,
+               (Scm_GlocConstP(g)
+                ? " const"
+                : (Scm_GlocInlinableP(g) ? " inlinable" : "")));
 }
 
 SCM_DEFINE_BUILTIN_CLASS_SIMPLE(Scm_GlocClass, gloc_print);
@@ -58,18 +59,13 @@ ScmObj Scm_MakeGloc(ScmSymbol *sym, ScmModule *module)
     g->module = module;
     g->value = SCM_UNBOUND;
     g->exported = FALSE;
+    g->hidden = FALSE;
     g->getter = NULL;
     g->setter = NULL;
     return SCM_OBJ(g);
 }
 
-ScmObj Scm_MakeConstGloc(ScmSymbol *sym, ScmModule *module)
-{
-    ScmGloc *g = SCM_GLOC(Scm_MakeGloc(sym, module));
-    g->setter = Scm_GlocConstSetter;
-    return SCM_OBJ(g);
-}
-
+/* special setters for const and inlinable bindings. */
 ScmObj Scm_GlocConstSetter(ScmGloc *gloc, ScmObj val)
 {
     Scm_Error("cannot change constant value of %S#%S",
@@ -77,3 +73,46 @@ ScmObj Scm_GlocConstSetter(ScmGloc *gloc, ScmObj val)
     return SCM_UNDEFINED;       /* dummy */
 }
 
+ScmObj Scm_GlocInlinableSetter(ScmGloc *gloc, ScmObj val)
+{
+    Scm_Warn("altering binding of inlinable procedure: %S#%S",
+             gloc->module->name, gloc->name);
+    return val;
+}
+
+int Scm_GlocConstP(ScmGloc *gloc)
+{
+    return ((gloc)->setter == Scm_GlocConstSetter);
+}
+
+int Scm_GlocInlinableP(ScmGloc *gloc)
+{
+    return ((gloc)->setter == Scm_GlocInlinableSetter);
+}
+
+/* Change binding flags.  Do not use casually. */
+void Scm_GlocMark(ScmGloc *gloc, int flags)
+{
+    if (flags & SCM_BINDING_CONST) {
+        gloc->setter = Scm_GlocConstSetter;
+    } else if (flags & SCM_BINDING_INLINABLE) {
+        gloc->setter = Scm_GlocInlinableSetter;
+    } else {
+        gloc->setter = NULL;
+    }
+}
+
+/* For the backward ABI compatibility */
+ScmObj Scm_GlocMarkConst(ScmGloc *gloc)
+{
+    Scm_GlocMark(gloc, SCM_BINDING_CONST);
+    return SCM_OBJ(gloc);
+}
+
+/* For the backward ABI compatibility */
+ScmObj Scm_GlocUnmarkConst(ScmGloc *gloc)
+{
+    Scm_GlocMark(gloc, 0);
+    return SCM_OBJ(gloc);
+}
+
