@@ -1,7 +1,7 @@
 /*
  * hash.c - hash table implementation
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: hash.c,v 1.58 2007/08/24 23:55:42 shirok Exp $
  */
 
 #define LIBGAUCHE_BODY
@@ -218,7 +216,8 @@ u_long Scm_HashString(ScmString *str, u_long modulo)
     const ScmStringBody *b = SCM_STRING_BODY(str);
     p = SCM_STRING_BODY_START(b);
     STRING_HASH(hashval, p, SCM_STRING_BODY_SIZE(b));
-    return (hashval % modulo);
+    if (modulo == 0) return hashval;
+    else return (hashval % modulo);
 }
 
 /*------------------------------------------------------------
@@ -283,6 +282,9 @@ static Entry *insert_entry(ScmHashCore *table,
             f->next = newb[index];
             newb[index] = f;
         }
+        /* gc friendliness */
+        for (i=0; i<table->numBuckets; i++) table->buckets[i] = NULL;
+        
         table->numBuckets = newsize;
         table->numBucketsLog2 = newbits;
         table->buckets = (void**)newb;
@@ -747,7 +749,7 @@ ScmObj Scm_HashTableDelete(ScmHashTable *ht, ScmObj key)
     ScmDictEntry *e = Scm_HashCoreSearch(SCM_HASH_TABLE_CORE(ht),
                                          (intptr_t)key, SCM_DICT_DELETE);
     if (e && e->value) return SCM_DICT_VALUE(e);
-    else              return SCM_UNBOUND;
+    else               return SCM_UNBOUND;
 }
 
 ScmObj Scm_HashTableKeys(ScmHashTable *table)
@@ -873,7 +875,7 @@ ScmHashEntry *Scm_HashTableAdd(ScmHashTable *ht, ScmObj key, ScmObj value)
     ScmDictEntry *e = Scm_HashCoreSearch(SCM_HASH_TABLE_CORE(ht),
                                          (intptr_t)key, SCM_DICT_CREATE);
     if (sizeof(intptr_t) != sizeof(void*)) {
-        Scm_Error("[internal] Scm_HashTableGet is obsoleted on this platform.  You should use the new hashtable API.");
+        Scm_Error("[internal] Scm_HashTableAdd is obsoleted on this platform.  You should use the new hashtable API.");
     }
     if (!e->value) (void)SCM_DICT_SET_VALUE(e, value);
     return (ScmHashEntry*)e;
@@ -884,7 +886,7 @@ ScmHashEntry *Scm_HashTablePut(ScmHashTable *ht, ScmObj key, ScmObj value)
     ScmDictEntry *e = Scm_HashCoreSearch(SCM_HASH_TABLE_CORE(ht),
                                          (intptr_t)key, SCM_DICT_CREATE);
     if (sizeof(intptr_t) != sizeof(void*)) {
-        Scm_Error("[internal] Scm_HashTableGet is obsoleted on this platform.  You should use the new hashtable API.");
+        Scm_Error("[internal] Scm_HashTablePut is obsoleted on this platform.  You should use the new hashtable API.");
     }
     (void)SCM_DICT_SET_VALUE(e, value);
     return (ScmHashEntry*)e;
