@@ -1,7 +1,7 @@
 /*
  * keyword.c - keyword implementation
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: keyword.c,v 1.17 2007/03/02 07:39:13 shirok Exp $
  */
 
 #define LIBGAUCHE_BODY
@@ -42,11 +40,25 @@
 
 static void keyword_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
 {
-    if (SCM_WRITE_MODE(ctx) != SCM_WRITE_DISPLAY) {
+    if (SCM_WRITE_MODE(ctx) == SCM_WRITE_DISPLAY) {
+        SCM_PUTS(SCM_KEYWORD(obj)->name, port);
+    } else {
         SCM_PUTC(':', port);
+        /* We basically print keyword names in the same way as symbols
+           (i.e. using |-escape if necessary).  However, as a convention,
+           two things are different from the default symbol writer.
+           (1) We don't check the noninitials; :1 is unambiguously a
+           keyword, so we don't need to print :|1|.
+           (2) A keyword with an empty name can be printed just as :,
+           instead of :||.
+           These conventions are useful if we pass the S-expression with
+           these keywords to other Scheme implementations that don't support
+           CL-style keywords; they would just read those ones as symbols.
+        */ 
+        Scm_WriteSymbolName(SCM_KEYWORD(obj)->name, port, ctx,
+                            (SCM_SYMBOL_WRITER_NOESCAPE_INITIAL
+                             |SCM_SYMBOL_WRITER_NOESCAPE_EMPTY));
     }
-    SCM_PUTS(SCM_KEYWORD(obj)->name, port);
-    return;
 }
 
 SCM_DEFINE_BUILTIN_CLASS_SIMPLE(Scm_KeywordClass, keyword_print);
@@ -62,19 +74,21 @@ static struct {
  */
 ScmObj Scm_MakeKeyword(ScmString *name)
 {
-    ScmHashEntry *e;
     ScmObj r;
+    ScmKeyword *k;
 
     (void)SCM_INTERNAL_MUTEX_LOCK(keywords.mutex);
-    e = Scm_HashTableGet(keywords.table, SCM_OBJ(name));
-    if (e) r = e->value;
-    else {
-        ScmKeyword *k = SCM_NEW(ScmKeyword);
-        SCM_SET_CLASS(k, SCM_CLASS_KEYWORD);
-        k->name = SCM_STRING(Scm_CopyString(name));
-        Scm_HashTablePut(keywords.table, SCM_OBJ(name), SCM_OBJ(k));
-        r = SCM_OBJ(k);
-    }
+    r = Scm_HashTableRef(keywords.table, SCM_OBJ(name), SCM_FALSE);
+    (void)SCM_INTERNAL_MUTEX_UNLOCK(keywords.mutex);
+
+    if (SCM_KEYWORDP(r)) return r;
+
+    k = SCM_NEW(ScmKeyword);
+    SCM_SET_CLASS(k, SCM_CLASS_KEYWORD);
+    k->name = SCM_STRING(Scm_CopyString(name));
+    (void)SCM_INTERNAL_MUTEX_LOCK(keywords.mutex);
+    r = Scm_HashTableSet(keywords.table, SCM_OBJ(name), SCM_OBJ(k),
+                         SCM_DICT_NO_OVERWRITE);
     (void)SCM_INTERNAL_MUTEX_UNLOCK(keywords.mutex);
     return r;
 }
