@@ -1,7 +1,7 @@
 /*
  * list.c - List related functions
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,13 +29,10 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: list.c,v 1.47 2007/03/02 07:39:13 shirok Exp $
  */
 
 #define LIBGAUCHE_BODY
 #include "gauche.h"
-#include "gauche/memory.h"
 
 /*
  * Classes
@@ -59,8 +56,13 @@ SCM_DEFINE_BUILTIN_CLASS(Scm_NullClass, NULL, NULL, NULL, NULL, list_cpl);
 
 ScmObj Scm_Cons(ScmObj car, ScmObj cdr)
 {
-    ScmPair *z;
-    SCM_MALLOC_WORDS(z, sizeof(ScmPair)/sizeof(GC_word), ScmPair*);
+    ScmPair *z = SCM_NEW(ScmPair);
+    /* NB: these ENSURE_MEMs are moved here from vm loop to reduce
+       the register pressure there.  In most cases these increases
+       just a couple of mask-and-test instructions on the data on
+       the register. */
+    SCM_FLONUM_ENSURE_MEM(car);
+    SCM_FLONUM_ENSURE_MEM(cdr);
     SCM_SET_CAR(z, car);
     SCM_SET_CDR(z, cdr);
     return SCM_OBJ(z);
@@ -68,9 +70,8 @@ ScmObj Scm_Cons(ScmObj car, ScmObj cdr)
 
 ScmObj Scm_Acons(ScmObj caar, ScmObj cdar, ScmObj cdr)
 {
-    ScmPair *y, *z;
-    SCM_MALLOC_WORDS(y, sizeof(ScmPair)/sizeof(GC_word), ScmPair*);
-    SCM_MALLOC_WORDS(z, sizeof(ScmPair)/sizeof(GC_word), ScmPair*);
+    ScmPair *y = SCM_NEW(ScmPair);
+    ScmPair *z = SCM_NEW(ScmPair);
     SCM_SET_CAR(y, caar);
     SCM_SET_CDR(y, cdar);
     SCM_SET_CAR(z, SCM_OBJ(y));
@@ -117,14 +118,12 @@ ScmObj Scm_VaList(va_list pvar)
     {
 	if (SCM_NULLP(start)) {
             start = SCM_OBJ(SCM_NEW(ScmPair));
-            /*SCM_SET_CLASS(start, SCM_CLASS_PAIR);*/
             SCM_SET_CAR(start, obj);
             SCM_SET_CDR(start, SCM_NIL);
             cp = start;
         } else {
             ScmObj item;
             item = SCM_OBJ(SCM_NEW(ScmPair));
-            /*SCM_SET_CLASS(item, SCM_CLASS_PAIR);*/
             SCM_SET_CDR(cp, item);
             SCM_SET_CAR(item, obj);
             SCM_SET_CDR(item, SCM_NIL);
@@ -142,14 +141,18 @@ ScmObj Scm_VaCons(va_list pvar)
 }
 
 ScmObj Scm_ArrayToList(ScmObj *elts, int nelts)
+{
+    return Scm_ArrayToListWithTail(elts, nelts, SCM_NIL);
+}
+
+ScmObj Scm_ArrayToListWithTail(ScmObj *elts, int nelts, ScmObj tail)
 {
     ScmObj h = SCM_NIL, t = SCM_NIL;
     if (elts) {
         int i;
-        for (i=0; i<nelts; i++) {
-            SCM_APPEND1(h, t, *elts++);
-        }
+        for (i=0; i<nelts; i++) SCM_APPEND1(h, t, *elts++);
     }
+    if (!SCM_NULLP(tail)) SCM_APPEND(h, t, tail);
     return h;
 }
 
@@ -335,11 +338,15 @@ ScmObj Scm_Reverse(ScmObj list)
 
     if (!SCM_PAIRP(list)) return list;
 
-    SCM_NEW_PAIR(p, SCM_NIL, SCM_NIL);
+    p = SCM_NEW(ScmPair);
+    SCM_SET_CAR(p, SCM_NIL);
+    SCM_SET_CDR(p, SCM_NIL);
     result = SCM_OBJ(p);
     SCM_FOR_EACH(cp, list) {
 	SCM_SET_CAR(result, SCM_CAR(cp));
-        SCM_NEW_PAIR(p, SCM_NIL, result);
+        p = SCM_NEW(ScmPair);
+        SCM_SET_CAR(p, SCM_NIL);
+        SCM_SET_CDR(p, result);
         result = SCM_OBJ(p);
     }
     return SCM_CDR(result);
@@ -583,7 +590,7 @@ ScmObj Scm_DeleteDuplicates(ScmObj list, int cmpmode)
             SCM_APPEND1(result, tail, SCM_CAR(lp));
         }
     }
-    if (!SCM_NULLP(lp)) SCM_SET_CDR(lp, tail);
+    if (!SCM_NULLP(lp) && !SCM_NULLP(tail)) SCM_SET_CDR(tail, lp);
     return result;
 }
 
