@@ -1,7 +1,7 @@
 /*
  * load.c - load a program
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: load.c,v 1.118 2008/02/12 14:24:10 shirok Exp $
  */
 
 #define LIBGAUCHE_BODY
@@ -42,12 +40,13 @@
 #include <ctype.h>
 #include <fcntl.h>
 
-#define LOAD_SUFFIX ".scm"      /* default load suffix */
-
 /*
  * Load file.
  */
 
+typedef struct dlobj_rec dlobj;
+typedef struct dlobj_initfn_rec dlobj_initfn;
+
 /* Static parameters */
 static struct {
     /* Load path list */
@@ -68,19 +67,21 @@ static struct {
 
     /* Dynamic linking */
     ScmObj dso_suffixes;
-    ScmObj dso_list;              /* List of dynamically loaded objects. */
+    dlobj *dso_list;              /* List of dynamically loaded objects. */
     ScmInternalMutex dso_mutex;
 } ldinfo = { (ScmGloc*)&ldinfo, };  /* trick to put ldinfo in .data section */
 
 /* keywords used for load and load-from-port surbs */
-static ScmObj key_paths              = SCM_UNBOUND;
 static ScmObj key_error_if_not_found = SCM_UNBOUND;
-static ScmObj key_environment        = SCM_UNBOUND;
 static ScmObj key_macro              = SCM_UNBOUND;
 static ScmObj key_ignore_coding      = SCM_UNBOUND;
 
-/* small utility */
-static void load_packet_init(ScmLoadPacket *packet)
+/*
+ * ScmLoadPacket is the way to communicate to Scm_Load facility.
+ */
+
+/* small utility.  initializes OUT fields of the load packet. */
+static void load_packet_prepare(ScmLoadPacket *packet)
 {
     if (packet) {
         packet->exception = SCM_FALSE;
@@ -88,8 +89,21 @@ static void load_packet_init(ScmLoadPacket *packet)
     }
 }
 
+/* for applications to initialize ScmLoadPacket before passing it to
+   Scm_Load or Scm_LoadFromPort.   As of 0.9, ScmLoadPacket only has
+   fields to be filled by those APIs, so applications don't need to
+   initialize it explicitly.  However, it is possible in future that
+   we add some fields to pass info from applications to APIs, in which
+   case it is necessary for this function to set appropriate initial
+   values for such fields. */
+void Scm_LoadPacketInit(ScmLoadPacket *p)
+{
+    load_packet_prepare(p);
+}
+
 /*--------------------------------------------------------------------
  * Scm_LoadFromPort
+ * Scm_VMLoadFromPort
  * 
  *   The most basic function in the load()-family.  Read an expression
  *   from the given port and evaluates it repeatedly, until it reaches
@@ -106,14 +120,22 @@ static void load_packet_init(ScmLoadPacket *packet)
  *   extension.  SCM_LOAD_QUIET_NOFILE and SCM_LOAD_IGNORE_CODING
  *   won't have any effect for LoadFromPort; see Scm_Load below.
  *
+ *   Scm_VMLoadFromPort returns a Scheme object that encodes result
+ *   load.  Right now it is used to propagete ScmLoadProvideStatus
+ *   to Scm_LoadFromPort, but its meaning may be changed in future;
+ *   the application must treat the returned value opaque, and usually
+ *   should discard it.
+ *
  *   TODO: if we're using coding-aware port, how should we propagate
  *   locking into the wrapped (original) port?
  */
 
-struct load_packet {
+/* A structure to keep around the transient state of the current loading.
+   Created by Scm_VMLoadFromPort and discarded once the loading ends. */
+struct load_info {
     ScmPort *port;
     ScmModule *prev_module;
-    ScmReadContext ctx;
+    ScmReadContext *ctx;
     ScmObj prev_port;
     ScmObj prev_history;
     ScmObj prev_next;
@@ -123,7 +145,7 @@ struct load_packet {
 /* Clean up */
 static ScmObj load_after(ScmObj *args, int nargs, void *data)
 {
-    struct load_packet *p = (struct load_packet *)data;
+    struct load_info *p = (struct load_info *)data;
     ScmVM *vm = Scm_VM();
 
 #ifdef HAVE_GETTIMEOFDAY
@@ -149,8 +171,8 @@ static ScmObj load_after(ScmObj *args, int nargs, void *data)
 /* C-continuation of the loading */
 static ScmObj load_cc(ScmObj result, void **data)
 {
-    struct load_packet *p = (struct load_packet*)(data[0]);
-    ScmObj expr = Scm_ReadWithContext(SCM_OBJ(p->port), &(p->ctx));
+    struct load_info *p = (struct load_info*)(data[0]);
+    ScmObj expr = Scm_ReadWithContext(SCM_OBJ(p->port), p->ctx);
 
     if (!SCM_EOFP(expr)) {
         Scm_VMPushCC(load_cc, data, 1);
@@ -168,7 +190,7 @@ static ScmObj load_body(ScmObj *args, int nargs, void *data)
 ScmObj Scm_VMLoadFromPort(ScmPort *port, ScmObj next_paths,
                           ScmObj env, int flags)
 {
-    struct load_packet *p;
+    struct load_info *p;
     ScmObj port_info;
     ScmVM *vm = Scm_VM();
     ScmModule *module = vm->module;
@@ -186,7 +208,7 @@ ScmObj Scm_VMLoadFromPort(ScmPort *port, ScmObj next_paths,
         Scm_Error("bad load environment (must be a module or #f): %S", env);
     }
 
-    p = SCM_NEW(struct load_packet);
+    p = SCM_NEW(struct load_info);
     p->port = port;
     p->prev_module = vm->module;
     p->prev_port = vm->load_port;
@@ -194,11 +216,8 @@ ScmObj Scm_VMLoadFromPort(ScmPort *port, ScmObj next_paths,
     p->prev_next = vm->load_next;
     p->prev_situation = vm->evalSituation;
 
-    SCM_READ_CONTEXT_INIT(&(p->ctx));
-    p->ctx.flags = SCM_READ_LITERAL_IMMUTABLE | SCM_READ_SOURCE_INFO;
-    if (SCM_VM_RUNTIME_FLAG_IS_SET(vm, SCM_CASE_FOLD)) {
-        p->ctx.flags |= SCM_READ_CASE_FOLD;
-    }
+    p->ctx = Scm_MakeReadContext(NULL);
+    p->ctx->flags = SCM_READ_LITERAL_IMMUTABLE | SCM_READ_SOURCE_INFO;
     
     vm->load_next = next_paths;
     vm->load_port = SCM_OBJ(port);
@@ -216,40 +235,28 @@ ScmObj Scm_VMLoadFromPort(ScmPort *port, ScmObj next_paths,
     return Scm_VMDynamicWindC(NULL, load_body, load_after, p);
 }
 
-/* Scheme subr (load-from-port subr &keyword paths environment) */
-static ScmObj load_from_port(ScmObj *args, int argc, void *data)
-{
-    ScmPort *port;
-    ScmObj paths, env;
-    int flags = 0;
-    
-    if (!SCM_IPORTP(args[0])) {
-        Scm_Error("input port required, but got %S", args[0]);
-    }
-    port = SCM_PORT(args[0]);
-    paths = Scm_GetKeyword(key_paths, args[1], SCM_FALSE);
-    env   = Scm_GetKeyword(key_environment, args[1], SCM_FALSE);
-    return Scm_VMLoadFromPort(port, paths, env, flags);
-}
-
-static SCM_DEFINE_STRING_CONST(load_from_port_NAME, "load-from-port", 14, 14);
-static SCM_DEFINE_SUBR(load_from_port_STUB, 1, 1,
-                       SCM_OBJ(&load_from_port_NAME), load_from_port,
-                       NULL, NULL);
-
-int Scm_LoadFromPort(ScmPort *port, int flags, ScmLoadPacket *packet)
+int Scm_LoadFromPort(ScmPort *port, u_long flags, ScmLoadPacket *packet)
 {
+    static ScmObj load_from_port = SCM_UNDEFINED;
     ScmEvalPacket eresult;
     int r;
 
-    load_packet_init(packet);
+    if (SCM_UNDEFINEDP(load_from_port)) {
+        /* This should be an idempotent operation, so we don't need to
+           worry about MT-safety. */
+        load_from_port =
+            Scm_GlobalVariableRef(Scm_GaucheModule(),
+                                  SCM_SYMBOL(SCM_INTERN("load-from-port")),
+                                  0);
+    }
+
+    load_packet_prepare(packet);
     if (flags&SCM_LOAD_PROPAGATE_ERROR) {
-        Scm_ApplyRec(SCM_OBJ(&load_from_port_STUB), SCM_LIST1(SCM_OBJ(port)));
+        Scm_ApplyRec(load_from_port, SCM_LIST1(SCM_OBJ(port)));
         if (packet) packet->loaded = TRUE;
         return 0;
     } else {
-        r = Scm_Apply(SCM_OBJ(&load_from_port_STUB), SCM_LIST1(SCM_OBJ(port)),
-                      &eresult);
+        r = Scm_Apply(load_from_port, SCM_LIST1(SCM_OBJ(port)), &eresult);
         if (packet) {
             packet->exception = eresult.exception;
             packet->loaded = (r >= 0);
@@ -354,6 +361,7 @@ ScmObj Scm_FindFile(ScmString *filename, ScmObj *paths,
 
 /*---------------------------------------------------------------------
  * Scm_Load
+ * Scm_VMLoad
  *
  *  Scheme's load().
  * 
@@ -408,36 +416,20 @@ ScmObj Scm_VMLoad(ScmString *filename, ScmObj load_paths,
     return Scm_VMLoadFromPort(SCM_PORT(port), load_paths, env, flags);
 }
 
-/* Scheme subr (%load filename &keyword paths error-if-not-found
-                                        environment aware-coding) */
-static ScmObj load(ScmObj *args, int argc, void *data)
-{
-    ScmString *file;
-    ScmObj paths, env;
-    int flags = 0;
-
-    if (!SCM_STRINGP(args[0])) {
-        Scm_Error("string required, but got %S", args[0]);
-    }
-    file = SCM_STRING(args[0]);
-    paths = Scm_GetKeyword(key_paths, args[1], SCM_FALSE);
-    env   = Scm_GetKeyword(key_environment, args[1], SCM_FALSE);
-    if (SCM_FALSEP(Scm_GetKeyword(key_error_if_not_found, args[1], SCM_TRUE)))
-        flags |= SCM_LOAD_QUIET_NOFILE;
-    if (!SCM_FALSEP(Scm_GetKeyword(key_ignore_coding, args[1], SCM_FALSE)))
-        flags |= SCM_LOAD_IGNORE_CODING;
-    return Scm_VMLoad(file, paths, env, flags);
-}
-
-static SCM_DEFINE_STRING_CONST(load_NAME, "load", 4, 4);
-static SCM_DEFINE_SUBR(load_STUB, 1, 1, SCM_OBJ(&load_NAME), load, NULL, NULL);
-
-
-int Scm_Load(const char *cpath, int flags, ScmLoadPacket *packet)
+int Scm_Load(const char *cpath, u_long flags, ScmLoadPacket *packet)
 {
+    static ScmObj load_stub = SCM_UNDEFINED;
     ScmObj f = SCM_MAKE_STR_COPYING(cpath);
     ScmObj options = SCM_NIL;
     ScmEvalPacket eresult;
+
+    if (load_stub == SCM_UNDEFINED) {
+        /* This should be an idempotent operation, so we don't need to
+           worry about MT-safety. */
+        load_stub = Scm_GlobalVariableRef(Scm_SchemeModule(),
+                                          SCM_SYMBOL(SCM_INTERN("load")),
+                                          0);
+    }
     
     if (flags&SCM_LOAD_QUIET_NOFILE) {
         options = Scm_Cons(key_error_if_not_found,
@@ -448,15 +440,15 @@ int Scm_Load(const char *cpath, int flags, ScmLoadPacket *packet)
                            Scm_Cons(SCM_TRUE, options));
     }
 
-    load_packet_init(packet);
+    load_packet_prepare(packet);
     if (flags&SCM_LOAD_PROPAGATE_ERROR) {
-        ScmObj r = Scm_ApplyRec(SCM_OBJ(&load_STUB), Scm_Cons(f, options));
+        ScmObj r = Scm_ApplyRec(load_stub, Scm_Cons(f, options));
         if (packet) {
             packet->loaded = !SCM_FALSEP(r);
         }
         return 0;
     } else {
-        int r = Scm_Apply(SCM_OBJ(&load_STUB), Scm_Cons(f, options), &eresult);
+        int r = Scm_Apply(load_stub, Scm_Cons(f, options), &eresult);
         if (packet) {
             packet->exception = eresult.exception;
             packet->loaded = (r > 0 && !SCM_FALSEP(eresult.results[0]));
@@ -562,7 +554,7 @@ ScmObj Scm_AddLoadPath(const char *cpath, int afterp)
 }
 
 /*------------------------------------------------------------------
- * Dynamic link
+ * Dynamic linking
  */
 
 /* The API to load object file dynamically differ among platforms.
@@ -597,7 +589,7 @@ ScmObj Scm_AddLoadPath(const char *cpath, int afterp)
  *     Returns the last error occurred on HANDLE in the dl_* function.
  *
  * Notes:
- *   - The caller takes care of mutex so that dl_ won't be called from
+ *   - The caller must take care of mutex so that dl_ won't be called from
  *     more than one thread at a time, and no other thread calls
  *     dl_* functions between dl_open and dl_error (so that dl_open
  *     can store the error info in global variable).
@@ -607,8 +599,51 @@ ScmObj Scm_AddLoadPath(const char *cpath, int afterp)
  * functions.
  */
 
+/* The implementation of dynamic loader is a bit complicated in the presense
+   of multiple threads and multiple initialization routines.
+
+   We keep struct dlobj_rec record for each DYNAMIC-LOADed files (keyed
+   by pathname including suffix) to track the state of loading.  The thread
+   must lock the structure first to operate on the particluar DSO.
+
+   By default, a DSO has one initialization function (initfn) whose name
+   can be derived from DSO's basename (if DSO is /foo/bar/baz.so, the
+   initfn is Scm_Init_baz).  DSO may have more than one initfn, if it is
+   made from multiple Scheme files via precompiler; in which case, each
+   initfn initializes a part of DSO corresponding to a Scheme module.
+   Each *.sci file contains dynamic-load form of the DSO with :init-function
+   keyword arguments.
+ */
+
 typedef void (*ScmDynLoadInitFn)(void);
 
+enum dlobj_state {
+    DLOBJ_NONE,             /* dlopen and dlsym haven't completed. */
+    DLOBJ_LOADED,           /* dlopened and initfn found, but not init'ed */
+    DLOBJ_INITIALIZED       /* initialized and ready to use */
+};
+
+struct dlobj_initfn_rec {
+    struct dlobj_initfn_rec *next; /* chain */
+    const char *name;           /* name of initfn (always w/ leading '_') */
+    ScmDynLoadInitFn fn;        /* function ptr */
+    int initialized;            /* TRUE once fn returns */
+};
+
+struct dlobj_rec {
+    dlobj *next;                /* chain */
+    const char *path;           /* pathname for DSO, including suffix */
+    int loaded;                 /* TRUE if this DSO is already loaded.
+                                   It may need to be initialized, though.
+                                   Check initfns.  */
+    void *handle;               /* whatever dl_open returned */
+    ScmVM *loader;              /* The VM that's holding the lock to operate
+                                   on this DLO. */
+    dlobj_initfn *initfns;      /* list of initializer functions */
+    ScmInternalMutex mutex;
+    ScmInternalCond  cv;
+};
+
 /* NB: we rely on dlcompat library for dlopen instead of using dl_darwin.c
    for now; Boehm GC requires dlopen when compiled with pthread, so there's
    not much point to avoid dlopen here. */
@@ -620,13 +655,56 @@ typedef void (*ScmDynLoadInitFn)(void);
 #include "dl_dummy.c"
 #endif
 
+/* Find dlobj with path, creating one if there aren't, and returns it. */
+static dlobj *find_dlobj(const char *path)
+{
+    dlobj *z = NULL;
+    (void)SCM_INTERNAL_MUTEX_LOCK(ldinfo.dso_mutex);
+    for (z = ldinfo.dso_list; z; z = z->next) {
+        if (strcmp(z->path, path) == 0) break;
+    }
+    if (z == NULL) {
+        z = SCM_NEW(dlobj);
+        z->path = path;
+        z->loader = NULL;
+        z->loaded = FALSE;
+        z->initfns = NULL;
+        (void)SCM_INTERNAL_MUTEX_INIT(z->mutex);
+        (void)SCM_INTERNAL_COND_INIT(z->cv);
+        z->next = ldinfo.dso_list;
+        ldinfo.dso_list = z;
+    }
+    (void)SCM_INTERNAL_MUTEX_UNLOCK(ldinfo.dso_mutex);
+    return z;
+}
+
+static void lock_dlobj(dlobj *dlo)
+{
+    ScmVM *vm = Scm_VM();
+    (void)SCM_INTERNAL_MUTEX_LOCK(dlo->mutex);
+    while (dlo->loader != vm) {
+        if (dlo->loader == NULL) break;
+        (void)SCM_INTERNAL_COND_WAIT(dlo->cv, dlo->mutex);
+    }
+    dlo->loader = vm;
+    (void)SCM_INTERNAL_MUTEX_UNLOCK(dlo->mutex);
+}
+
+static void unlock_dlobj(dlobj *dlo)
+{
+    (void)SCM_INTERNAL_MUTEX_LOCK(dlo->mutex);
+    dlo->loader = NULL;
+    (void)SCM_INTERNAL_COND_BROADCAST(dlo->cv);
+    (void)SCM_INTERNAL_MUTEX_UNLOCK(dlo->mutex);
+}
+
 /* Derives initialization function name from the module file name.
    This function _always_ appends underscore before the symbol.
    The dynamic loader first tries the symbol without underscore,
    then tries with underscore. */
 #define DYNLOAD_PREFIX   "_Scm_Init_"
 
-static const char *get_dynload_initfn(const char *filename)
+static const char *derive_dynload_initfn(const char *filename)
 {
     const char *head, *tail, *s;
     char *name, *d;
@@ -647,180 +725,137 @@ static const char *get_dynload_initfn(const char *filename)
     return name;
 }
 
-#if 0
-/* Aux fn to get a parameter value from *.la file line */
-static const char *get_la_val(const char *start)
+/* find dlobj_initfn from the given dlobj with name. 
+   Assuming the caller holding the lock of OBJ. */
+static dlobj_initfn *find_initfn(dlobj *dlo, const char *name)
 {
-    const char *end;
-    if (start[0] == '\'') start++;
-    end = strrchr(start, '\'');
-    if (end) {
-        char *p = SCM_NEW_ATOMIC2(char*, (end-start+1));
-        memcpy(p, start, (end-start));
-        p[end-start] = '\0';
-        return (const char*)p;
+    dlobj_initfn *fns = dlo->initfns;
+    for (; fns != NULL; fns = fns->next) {
+        if (strcmp(name, fns->name) == 0) return fns;
+    }
+    fns = SCM_NEW(dlobj_initfn);
+    fns->name = name;
+    fns->fn = NULL;
+    fns->initialized = FALSE;
+    fns->next = dlo->initfns;
+    dlo->initfns = fns;
+    return fns;    
+}
+
+/* From the given DSO name, find out the path of the actual DSO */
+static const char *find_dso_path(ScmString *dsoname)
+{
+    ScmObj lpaths = Scm_GetDynLoadPath();
+    ScmObj spath = Scm_FindFile(dsoname, &lpaths, ldinfo.dso_suffixes, TRUE);
+    if (SCM_FALSEP(spath)) {
+        Scm_Error("can't find dlopen-able module %S", dsoname);
+    }
+    return Scm_GetStringConst(SCM_STRING(spath));
+}
+
+/* Obtain the initializer function name (with '_' prepended) */
+const char *get_initfn_name(ScmObj initfn, const char *dsopath)
+{
+    if (SCM_STRINGP(initfn)) {
+        ScmObj _initfn =
+            Scm_StringAppend2(SCM_STRING(Scm_MakeString("_", 1, 1, 0)),
+                              SCM_STRING(initfn));
+        return Scm_GetStringConst(SCM_STRING(_initfn));
     } else {
-        return start;
+        return derive_dynload_initfn(dsopath);
     }
 }
 
-/* We found libtool *.la file.  Retrieve DSO path from it.
-   This routine make some assumption on .la file.  I couldn't
-   find a formal specification of .la file format. */
-static ScmObj find_so_from_la(ScmString *lafile)
+/* Load the DSO.  The caller holds the lock of dlobj.  May throw an error;
+   the caller makes sure it releases the lock even in that case. */
+static void load_dlo(dlobj *dlo)
 {
-    ScmObj f = Scm_OpenFilePort(Scm_GetStringConst(lafile),
-                                O_RDONLY, SCM_PORT_BUFFER_FULL, 0);
-    const char *dlname = NULL, *libdir = NULL;
-    int installed = FALSE;
-    
-    for (;;) {
-        const char *cline;
-        ScmObj line = Scm_ReadLineUnsafe(SCM_PORT(f));
-        if (SCM_EOFP(line)) break;
-        cline = Scm_GetStringConst(SCM_STRING(line));
-        if (strncmp(cline, "dlname=", sizeof("dlname=")-1) == 0) {
-            dlname = get_la_val(cline+sizeof("dlname=")-1);
-            continue;
-        }
-        if (strncmp(cline, "libdir=", sizeof("libdir=")-1) == 0) {
-            libdir = get_la_val(cline+sizeof("libdir=")-1);
-            continue;
-        }
-        if (strncmp(cline, "installed=yes", sizeof("installed=yes")-1) == 0) {
-            installed = TRUE;
-            continue;
-        }
+    ScmVM *vm = Scm_VM();
+    if (SCM_VM_RUNTIME_FLAG_IS_SET(vm, SCM_LOAD_VERBOSE)) {
+        int len = Scm_Length(vm->load_history);
+        SCM_PUTZ(";;", 2, SCM_CURERR);
+        while (len-- > 0) SCM_PUTC(' ', SCM_CURERR);
+        Scm_Printf(SCM_CURERR, "Dynamically Loading %s...\n", dlo->path);
     }
-    Scm_ClosePort(SCM_PORT(f));
-    if (!dlname) return SCM_FALSE;
-    if (installed && libdir) {
-        ScmObj path = Scm_StringAppendC(SCM_STRING(SCM_MAKE_STR(libdir)),
-                                        "/", 1, 1);
-        path = Scm_StringAppend2(SCM_STRING(path),
-                                 SCM_STRING(SCM_MAKE_STR(dlname)));
-        /*Scm_Printf(SCM_CURERR, "Z=%S\n", path);*/
-        if (regfilep(path)) return path;
-    } else {
-        ScmObj dir = Scm_DirName(lafile);
-        ScmObj path = Scm_StringAppendC(SCM_STRING(dir),
-                                        "/" SCM_LIBTOOL_OBJDIR "/",
-                                        sizeof("/" SCM_LIBTOOL_OBJDIR "/")-1,
-                                        sizeof("/" SCM_LIBTOOL_OBJDIR "/")-1);
-        path = Scm_StringAppend2(SCM_STRING(path),
-                                 SCM_STRING(SCM_MAKE_STR(dlname)));
-        /*Scm_Printf(SCM_CURERR, "T=%S\n", path);*/
-        if (regfilep(path)) return path;
+    dlo->handle = dl_open(dlo->path);
+    if (dlo->handle == NULL) {
+        const char *err = dl_error();
+        if (err == NULL) {
+            Scm_Error("failed to link %s dynamically", dlo->path);
+        } else {
+            Scm_Error("failed to link %s dynamically: %s", dlo->path, err);
+        }
+        /*NOTREACHED*/
     }
-    return SCM_FALSE;
+    dlo->loaded = TRUE;
+}
+
+/* Call the DSO's initfn.  The caller holds the lock of dlobj, and responsible
+   to release the lock even when this fn throws an error. */
+static void call_initfn(dlobj *dlo, const char *name)
+{
+    dlobj_initfn *ifn = find_initfn(dlo, name);
+
+    if (ifn->initialized) return;
+
+    if (!ifn->fn) {
+        /* locate initfn.  Name always has '_'.  Whether the actual
+           symbol dl_sym returns has '_' or not depends on the platform,
+           so we first try without '_', then '_'. */
+        ifn->fn = dl_sym(dlo->handle, name+1);
+        if (ifn->fn == NULL) {
+            ifn->fn = (void(*)(void))dl_sym(dlo->handle, name);
+            if (ifn->fn == NULL) {
+                dl_close(dlo->handle);
+                dlo->handle = NULL;
+                Scm_Error("dynamic linking of %s failed: "
+                          "couldn't find initialization function %s",
+                          dlo->path, name);
+                /*NOTREACHED*/
+            }
+        }
+    }
+    
+    /* Call initialization function.  note that there can be arbitrary
+       complex stuff done within func(), including evaluation of
+       Scheme procedures and/or calling dynamic-load for other
+       object.  There's a chance that, with some contrived case,
+       func() can trigger the dynamic loading of the same file we're
+       loading right now.  However, if the code follows the Gauche's
+       standard module structure, such circular dependency is detected
+       by Scm_Load, so we don't worry about it here. */
+    ifn->fn();
+    ifn->initialized = TRUE;
 }
-#endif
 
 /* Dynamically load the specified object by FILENAME.
    FILENAME must not contain the system's suffix (.so, for example).
+   The same name of DSO can be only loaded once.
+   A DSO may contain multiple initialization functions (initfns), in
+   which case each initfn is called at most once.
 */
-ScmObj Scm_DynLoad(ScmString *filename, ScmObj initfn, int export_)
+ScmObj Scm_DynLoad(ScmString *filename, ScmObj initfn, u_long flags/*reserved*/)
 {
-    ScmObj reqname, truename, load_paths = Scm_GetDynLoadPath();
-    void *handle;
-    ScmDynLoadInitFn func;
-    const char *cpath, *initname, *err = NULL;
-    enum  {
-        DLERR_NONE,             /* no error */
-        DLERR_DLOPEN,           /* failure in dlopen */
-        DLERR_NOINITFN,         /* failure in finding initfn */
-    } errtype = DLERR_NONE;
+    const char *dsopath = find_dso_path(filename);
+    const char *initname = get_initfn_name(initfn, dsopath);
+    dlobj *dlo = find_dlobj(dsopath);
 
-    truename = Scm_FindFile(filename, &load_paths, ldinfo.dso_suffixes, TRUE);
-    if (SCM_FALSEP(truename)) {
-        Scm_Error("can't find dlopen-able module %S", filename);
-    }
-    reqname = truename;         /* save requested name */
-    cpath = Scm_GetStringConst(SCM_STRING(truename));
-
-#if 0
-    if ((suff = strrchr(cpath, '.')) && strcmp(suff, ".la") == 0) {
-        truename = find_so_from_la(SCM_STRING(truename));
-        if (SCM_FALSEP(truename)) {
-            Scm_Error("couldn't find dlopen-able module from libtool archive file %s", cpath);
-        }
-        cpath = Scm_GetStringConst(SCM_STRING(truename));
-    }
-#endif
-
-    if (SCM_STRINGP(initfn)) {
-        ScmObj _initfn = Scm_StringAppend2(SCM_STRING(Scm_MakeString("_", 1, 1, 0)),
-                                           SCM_STRING(initfn));
-        initname = Scm_GetStringConst(SCM_STRING(_initfn));
-    } else {
-        /* NB: we use requested name to derive initfn name, instead of
-           the one given in libtool .la file.  For example, on cygwin,
-           the actual DLL that libtool library libfoo.la points to is
-           named cygfoo.dll; we still want Scm_Init_libfoo in that case,
-           not Scm_Init_cygfoo. */
-        initname = get_dynload_initfn(Scm_GetStringConst(SCM_STRING(reqname)));
+    /* Load the dlobj if necessary. */
+    lock_dlobj(dlo);
+    if (!dlo->loaded) {
+        SCM_UNWIND_PROTECT { load_dlo(dlo); }
+        SCM_WHEN_ERROR { unlock_dlobj(dlo); SCM_NEXT_HANDLER; }
+        SCM_END_PROTECT;
     }
 
-    (void)SCM_INTERNAL_MUTEX_LOCK(ldinfo.dso_mutex);
-    if (!SCM_FALSEP(Scm_Member(truename, ldinfo.dso_list, SCM_CMP_EQUAL))) {
-        /* already loaded */
-        goto cleanup;
-    }
-    SCM_UNWIND_PROTECT {
-        ScmVM *vm = Scm_VM();
-        if (SCM_VM_RUNTIME_FLAG_IS_SET(vm, SCM_LOAD_VERBOSE)) {
-            int len = Scm_Length(vm->load_history);
-            SCM_PUTZ(";;", 2, SCM_CURERR);
-            while (len-- > 0) SCM_PUTC(' ', SCM_CURERR);
-            Scm_Printf(SCM_CURERR, "Dynamically Loading %s...\n", cpath);
-        }
-    } SCM_WHEN_ERROR {
-        (void)SCM_INTERNAL_MUTEX_UNLOCK(ldinfo.dso_mutex);
-        SCM_NEXT_HANDLER;
-    } SCM_END_PROTECT;
-    handle = dl_open(cpath);
-    if (handle == NULL) {
-        err = dl_error();
-        errtype = DLERR_DLOPEN;
-        goto cleanup;
-    }
-    /* initname always has '_'.  We first try without '_' */
-    func = dl_sym(handle, initname+1);
-    if (func == NULL) {
-        func = (void(*)(void))dl_sym(handle, initname);
-        if (func == NULL) {
-            dl_close(handle);
-            errtype = DLERR_NOINITFN;
-            goto cleanup;
-        }
-    }
-    /* TODO: if the module initialization function fails,
-       there's no safe way to unload the module, and we
-       can't load the same module again.  We're stuck to
-       the broken module.  This has to be addressed. */
-    SCM_UNWIND_PROTECT {
-        func();
-    } SCM_WHEN_ERROR {
-        (void)SCM_INTERNAL_MUTEX_UNLOCK(ldinfo.dso_mutex);
-        SCM_NEXT_HANDLER;
-    } SCM_END_PROTECT;
-    ldinfo.dso_list = Scm_Cons(truename, ldinfo.dso_list);
-  cleanup:
-    (void)SCM_INTERNAL_MUTEX_UNLOCK(ldinfo.dso_mutex);
-    switch (errtype) {
-    case DLERR_DLOPEN:
-        if (err == NULL) {
-            Scm_Error("failed to link %S dynamically", filename);
-        } else {
-            Scm_Error("failed to link %S dynamically: %s", filename, err);
-        }
-        /*NOTREACHED*/
-    case DLERR_NOINITFN:
-        Scm_Error("dynamic linking of %S failed: couldn't find initialization function %s", filename, initname);
-        /*NOTREACHED*/
-    case DLERR_NONE:
-        break;
-    }
+    /* Now the dlo is loaded.  We need to call initializer. */
+    SCM_ASSERT(dlo->loaded);
+
+    SCM_UNWIND_PROTECT { call_initfn(dlo, initname); }
+    SCM_WHEN_ERROR { unlock_dlobj(dlo);  SCM_NEXT_HANDLER; }
+    SCM_END_PROTECT;
+
+    unlock_dlobj(dlo);
     return SCM_TRUE;
 }
 
@@ -837,10 +872,12 @@ ScmObj Scm_DynLoad(ScmString *filename, ScmObj initfn, int export_)
  *   Besides the list of provided features (ldinfo.provided), the
  *   system keeps two kind of global assoc list for transient information.
  *
- *   ldinfo.providing keeps a list of (<feature> . <thread>), where
- *   <thread> is currently loading a file for <feature>.
+ *   ldinfo.providing keeps a list of (<feature> <thread> <provided> ...),
+ *   where <thread> is currently loading a file for <feature>.
  *   ldinfo.waiting keeps a list of (<thread> . <feature>), where
  *   <thread> is waiting for <feature> to be provided.
+ *   (The <provided> list is pushed by 'provide' while loading <feature>.
+ *   It is used for autprovide feature.  See below).
  *
  *   Scm_Require first checks ldinfo.provided list; if the feature is
  *   already provided, no problem; just return.
@@ -849,22 +886,45 @@ ScmObj Scm_DynLoad(ScmString *filename, ScmObj initfn, int export_)
  *   ldinfo.waiting list and waits for the feature to be provided.
  *
  *   There may be a case that the feature dependency forms a loop because
- *   of bug.  An error should be signaled in such a case, rather than going
+ *   of a bug.  An error should be signaled in such a case, rather than going
  *   to deadlock.   So, when the calling thread finds the required feature
  *   is in the ldinfo.providing alist, it checks the waiting chain of
  *   features, and no threads are waiting for a feature being provided by
  *   the calling thread.
+ *
+ *   When the above checks are all false, the calling thread is responsible
+ *   to load the required feature.  It pushes the feature and itself
+ *   onto the providing list and start loading the file.
+ *
+ * [Autoprovide Feature]
+ *
+ *   When a file is loaded via 'require', it almost always provides the
+ *   required feature.  Thus we allow the file to omit the 'provide' form.
+ *   That is, if a file X.scm is loaded because of (require "X"), and
+ *   there's no 'provide' form in X.scm, the feature "X" is automatically
+ *   provided upon a successful loading of X.scm.
+ *
+ *   If a 'provide' form appears in X.scm, the autoprovide feature is 
+ *   turned off.  It is allowed that X.scm provides features other than
+ *   "X".   As a special case, (provide #f) causes the autoprovide feature
+ *   to be turned of without providing any feature.
+ *
+ *   To track what is provided, the 'provide' form pushes its argument
+ *   to the entry of 'providing' list whose thread matches the calling
+ *   thread.  (It is possible that there's more than one entry in the
+ *   'providing' list, for a required file may call another require form.
+ *   The entry is always pushed at the beginning of the providing list,
+ *   we know that the first matching entry is the current one.)
  */
 
 int Scm_Require(ScmObj feature, int flags, ScmLoadPacket *packet)
 {
-    ScmObj filename;
     ScmVM *vm = Scm_VM();
     ScmObj provided, providing, p, q;
     int loop = FALSE, r;
     ScmLoadPacket xresult;
 
-    load_packet_init(packet);
+    load_packet_prepare(packet);
     if (!SCM_STRINGP(feature)) {
         ScmObj e = Scm_MakeError(Scm_Sprintf("require: string expected, but got %S\n", feature));
         if (flags&SCM_LOAD_PROPAGATE_ERROR) Scm_Raise(e);
@@ -874,6 +934,7 @@ int Scm_Require(ScmObj feature, int flags, ScmLoadPacket *packet)
         }
     }
 
+    /* Check provided, providing and waiting list.  See the comment above. */
     (void)SCM_INTERNAL_MUTEX_LOCK(ldinfo.prov_mutex);
     do {
         provided = Scm_Member(feature, ldinfo.provided, SCM_CMP_EQUAL);
@@ -881,24 +942,18 @@ int Scm_Require(ScmObj feature, int flags, ScmLoadPacket *packet)
         providing = Scm_Assoc(feature, ldinfo.providing, SCM_CMP_EQUAL);
         if (SCM_FALSEP(providing)) break;
 
-        /* Checks for dependency loop */
+        /* Checks for dependencies */
         p = providing;
-        SCM_ASSERT(SCM_PAIRP(p));
-        if (SCM_CDR(p) == SCM_OBJ(vm)) {
-            loop = TRUE;
-            break;
-        }
+        SCM_ASSERT(SCM_PAIRP(p) && SCM_PAIRP(SCM_CDR(p)));
+        if (SCM_CADR(p) == SCM_OBJ(vm)) { loop = TRUE; break; }
         
         for (;;) {
-            q = Scm_Assoc(SCM_CDR(p), ldinfo.waiting, SCM_CMP_EQ);
+            q = Scm_Assq(SCM_CDR(p), ldinfo.waiting);
             if (SCM_FALSEP(q)) break;
             SCM_ASSERT(SCM_PAIRP(q));
             p = Scm_Assoc(SCM_CDR(q), ldinfo.providing, SCM_CMP_EQUAL);
-            SCM_ASSERT(SCM_PAIRP(p));
-            if (SCM_CDR(p) == SCM_OBJ(vm)) {
-                loop = TRUE;
-                break;
-            }
+            SCM_ASSERT(SCM_PAIRP(p) && SCM_PAIRP(SCM_CDR(p)));
+            if (SCM_CADR(p) == SCM_OBJ(vm)) { loop = TRUE; break; }
         }
         if (loop) break;
         ldinfo.waiting = Scm_Acons(SCM_OBJ(vm), feature, ldinfo.waiting);
@@ -907,7 +962,8 @@ int Scm_Require(ScmObj feature, int flags, ScmLoadPacket *packet)
         continue;
     } while (0);
     if (!loop && SCM_FALSEP(provided)) {
-        ldinfo.providing = Scm_Acons(feature, SCM_OBJ(vm), ldinfo.providing);
+        ldinfo.providing =
+            Scm_Acons(feature, SCM_LIST1(SCM_OBJ(vm)), ldinfo.providing);
     }
     (void)SCM_INTERNAL_MUTEX_UNLOCK(ldinfo.prov_mutex);
 
@@ -920,23 +976,30 @@ int Scm_Require(ScmObj feature, int flags, ScmLoadPacket *packet)
         }
     }
         
-    if (!SCM_FALSEP(provided)) return 0;
-    filename = Scm_StringAppendC(SCM_STRING(feature), ".scm", 4, 4);
-    r = Scm_Load(Scm_GetStringConst(SCM_STRING(filename)), 0, &xresult);
+    if (!SCM_FALSEP(provided)) return 0; /* no work to do */
+    r = Scm_Load(Scm_GetStringConst(SCM_STRING(feature)), 0, &xresult);
     if (packet) packet->exception = xresult.exception;
 
     if (r < 0) {
+        /* Load failed */
         (void)SCM_INTERNAL_MUTEX_LOCK(ldinfo.prov_mutex);
         ldinfo.providing = Scm_AssocDeleteX(feature, ldinfo.providing, SCM_CMP_EQUAL);
-        (void)SCM_INTERNAL_COND_SIGNAL(ldinfo.prov_cv);
+        (void)SCM_INTERNAL_COND_BROADCAST(ldinfo.prov_cv);
         (void)SCM_INTERNAL_MUTEX_UNLOCK(ldinfo.prov_mutex);
         if (flags&SCM_LOAD_PROPAGATE_ERROR) Scm_Raise(xresult.exception);
         else return -1;
     }
 
+    /* Success */
     (void)SCM_INTERNAL_MUTEX_LOCK(ldinfo.prov_mutex);
+    p = Scm_Assoc(feature, ldinfo.providing, SCM_CMP_EQUAL);
     ldinfo.providing = Scm_AssocDeleteX(feature, ldinfo.providing, SCM_CMP_EQUAL);
-    (void)SCM_INTERNAL_COND_SIGNAL(ldinfo.prov_cv);
+    /* `Autoprovide' feature */
+    if (SCM_NULLP(SCM_CDDR(p)) 
+        && SCM_FALSEP(Scm_Member(feature, ldinfo.provided, SCM_CMP_EQUAL))) {
+        ldinfo.provided = Scm_Cons(feature, ldinfo.provided);
+    }
+    (void)SCM_INTERNAL_COND_BROADCAST(ldinfo.prov_cv);
     (void)SCM_INTERNAL_MUTEX_UNLOCK(ldinfo.prov_mutex);
     if (packet) packet->loaded = TRUE;
     return 0;
@@ -944,14 +1007,22 @@ int Scm_Require(ScmObj feature, int flags, ScmLoadPacket *packet)
 
 ScmObj Scm_Provide(ScmObj feature)
 {
-    if (!SCM_STRINGP(feature))
-        Scm_Error("provide: string expected, but got %S\n", feature);
+    ScmObj cp;
+    ScmVM *self = Scm_VM();
+    
+    if (!SCM_STRINGP(feature)&&!SCM_FALSEP(feature)) {
+        SCM_TYPE_ERROR(feature, "string");
+    }
     (void)SCM_INTERNAL_MUTEX_LOCK(ldinfo.prov_mutex);
-    if (SCM_FALSEP(Scm_Member(feature, ldinfo.provided, SCM_CMP_EQUAL))) {
+    if (SCM_STRINGP(feature)
+        && SCM_FALSEP(Scm_Member(feature, ldinfo.provided, SCM_CMP_EQUAL))) {
         ldinfo.provided = Scm_Cons(feature, ldinfo.provided);
     }
-    if (!SCM_FALSEP(Scm_Member(feature, ldinfo.providing, SCM_CMP_EQUAL))) {
-        ldinfo.providing = Scm_DeleteX(feature, ldinfo.providing, SCM_CMP_EQUAL);
+    SCM_FOR_EACH(cp, ldinfo.providing) {
+        if (SCM_CADR(SCM_CAR(cp)) == SCM_OBJ(self)) {
+            SCM_SET_CDR(SCM_CDR(SCM_CAR(cp)), SCM_LIST1(feature));
+            break;
+        }
     }
     (void)SCM_INTERNAL_COND_SIGNAL(ldinfo.prov_cv);
     (void)SCM_INTERNAL_MUTEX_UNLOCK(ldinfo.prov_mutex);
@@ -1116,9 +1187,6 @@ ScmObj Scm_ResolveAutoload(ScmAutoload *adata, int flags)
             }
             f = Scm_FindBinding(SCM_MODULE(m), adata->name, 0);
             g = Scm_FindBinding(adata->module, adata->name, 0);
-            if (f == NULL) {
-                Scm_Error("ZIMBER: %S %S\n", m, adata->name);
-            }
             SCM_ASSERT(f != NULL);
             SCM_ASSERT(g != NULL);
             adata->value = SCM_GLOC_GET(f);
@@ -1192,22 +1260,18 @@ void Scm__InitLoad(void)
     SCM_APPEND1(init_dynload_path, t, Scm_ArchitectureDirectory());
 
     init_load_suffixes = t = SCM_NIL;
-    SCM_APPEND1(init_load_suffixes, t, SCM_MAKE_STR(LOAD_SUFFIX));
+    SCM_APPEND1(init_load_suffixes, t, SCM_MAKE_STR(".sci"));
+    SCM_APPEND1(init_load_suffixes, t, SCM_MAKE_STR(".scm"));
 
     (void)SCM_INTERNAL_MUTEX_INIT(ldinfo.path_mutex);
     (void)SCM_INTERNAL_MUTEX_INIT(ldinfo.prov_mutex);
     (void)SCM_INTERNAL_COND_INIT(ldinfo.prov_cv);
     (void)SCM_INTERNAL_MUTEX_INIT(ldinfo.dso_mutex);
 
-    key_paths = SCM_MAKE_KEYWORD("paths");
     key_error_if_not_found = SCM_MAKE_KEYWORD("error-if-not-found");
-    key_environment = SCM_MAKE_KEYWORD("environment");
     key_macro = SCM_MAKE_KEYWORD("macro");
     key_ignore_coding = SCM_MAKE_KEYWORD("ignore-coding");
     
-    SCM_DEFINE(m, "load-from-port", SCM_OBJ(&load_from_port_STUB));
-    SCM_DEFINE(m, "load", SCM_OBJ(&load_STUB));
-
 #define DEF(rec, sym, val) \
     rec = SCM_GLOC(Scm_Define(m, SCM_SYMBOL(sym), val))
 
@@ -1226,5 +1290,5 @@ void Scm__InitLoad(void)
     ldinfo.waiting = SCM_NIL;
     ldinfo.dso_suffixes = SCM_LIST2(SCM_MAKE_STR(".la"),
                                     SCM_MAKE_STR("." SHLIB_SO_SUFFIX));
-    ldinfo.dso_list = SCM_NIL;
+    ldinfo.dso_list = NULL;
 }
