@@ -1,7 +1,7 @@
 /*
  * macro.c - macro implementation
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: macro.c,v 1.68 2007/08/24 23:55:43 shirok Exp $
  */
 
 #define LIBGAUCHE_BODY
@@ -915,9 +913,6 @@ ScmObj Scm_CompileSyntaxRules(ScmObj name, ScmObj literals, ScmObj rules,
     ScmSyntaxRules *sr;
 
     if (SCM_IDENTIFIERP(name)) name = SCM_OBJ(SCM_IDENTIFIER(name)->name);
-    else if (!SCM_SYMBOLP(name)) {
-        Scm_Error("symbol required, but got %S", name);
-    }
     if (!SCM_MODULEP(mod)) Scm_Error("module required, but got %S", mod);
     sr = compile_rules(name, literals, rules, SCM_MODULE(mod), env);
     return Scm_MakeMacro(SCM_SYMBOL(name), synrule_transform, (void*)sr);
@@ -951,16 +946,17 @@ ScmObj Scm_VMMacroExpand(ScmObj expr, ScmObj env, int oncep)
             /* local syntactic binding */
             mac = SCM_MACRO(sym);
         } else {
+            ScmGloc *g = NULL;
             if (SCM_IDENTIFIERP(sym)) {
-                sym = SCM_OBJ(SCM_IDENTIFIER(sym)->name);
+                g = Scm_FindBinding(SCM_IDENTIFIER(sym)->module,
+                                    SCM_IDENTIFIER(sym)->name, 0);
+            } else if (SCM_SYMBOLP(sym)) {
+                g = Scm_FindBinding(Scm_VM()->module,
+                                    SCM_SYMBOL(sym), 0);
             }
-            if (SCM_SYMBOLP(sym)) {
-                ScmGloc *g = Scm_FindBinding(Scm_VM()->module,
-                                             SCM_SYMBOL(sym), 0);
-                if (g) {
-                    ScmObj gv = SCM_GLOC_GET(g);
-                    if (SCM_MACROP(gv)) mac = SCM_MACRO(gv);
-                }
+            if (g) {
+                ScmObj gv = SCM_GLOC_GET(g);
+                if (SCM_MACROP(gv)) mac = SCM_MACRO(gv);
             }
         }
     }
