@@ -1,7 +1,7 @@
 /*
  * paths.c - get 'known' pathnames, such as the system's library directory.
  *
- *   Copyright (c) 2005-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2005-2010  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,9 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: paths.c,v 1.4 2007/09/15 12:30:50 shirok Exp $
- *
  */
 
 /* This file is used by both libgauche and gauche-config.  The latter
@@ -95,3 +92,13 @@ void Scm_GetSiteArchitectureDirectory(char *buf, int buflen,
     maybe_prepend_install_dir(GAUCHE_SITE_ARCH_DIR, buf, buflen, errfn);
 }
 
+/* On windows and darwin, this returns the runtime's prefix directory
+   that can be replaced with '@'.  Note that it calls errfn on other
+   platforms.  Eventually we want to cover more platforms, for getting
+   directory of the running binary is sometimes useful in general. */
+void Scm_GetRuntimeDirectory(char *buf, int buflen,
+                             void (*errfn)(const char *, ...))
+{
+    get_install_dir(buf, buflen, errfn);
+}
+
