@@ -1,7 +1,7 @@
 /*
  * port.c - port implementation
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: port.c,v 1.147 2007/11/30 11:16:04 shirok Exp $
  */
 
 #define LIBGAUCHE_BODY
@@ -83,7 +81,7 @@ SCM_DEFINE_BASE_CLASS(Scm_LimitedLengthPortClass,
 
 /* Cleaning up:
  *   The underlying file descriptor/stream may be closed when the port
- *   is explicitly closed by close-port, or implicitly destroyed by
+ *   is explicitly closed by close-port, or implicitly destroyed by the
  *   garbage collector.  To keep consistency, Scheme ports should never
  *   share the same file descriptor.  However, C code and Scheme port
  *   may share the same file descriptor for efficiency (e.g. stdios).
@@ -137,7 +135,10 @@ static ScmPort *make_port(ScmClass *klass, int dir, int type)
     port->closed = FALSE;
     port->error = FALSE;
     port->ownerp = FALSE;
-    port->flags = 0;
+    port->flags =
+        SCM_VM_RUNTIME_FLAG_IS_SET(Scm_VM(), SCM_CASE_FOLD)
+        ? SCM_PORT_CASE_FOLD
+        : 0;
     port->name = SCM_FALSE;
     (void)SCM_INTERNAL_FASTLOCK_INIT(port->lock);
     port->lockOwner = NULL;
@@ -241,7 +242,6 @@ int Scm_PortFileNo(ScmPort *port)
 /* Duplicates the file descriptor of the source port, and set it to
    the destination port.  Both source and destination port must be
    file ports. */
-#if !defined(GAUCHE_WINDOWS)
 void Scm_PortFdDup(ScmPort *dst, ScmPort *src)
 {
     int r, srcfd, dstfd;
@@ -268,11 +268,14 @@ void Scm_PortFdDup(ScmPort *dst, ScmPort *src)
         /* flush the current buffer */
         Scm_Flush(dst);
     }
+#if defined(GAUCHE_WINDOWS)
+    SCM_SYSCALL(r, _dup2(srcfd, dstfd));
+#else  /*!GAUCHE_WINDOWS*/
     SCM_SYSCALL(r, dup2(srcfd, dstfd));
+#endif /*!GAUCHE_WINDOWS*/
     if (r < 0) Scm_SysError("dup2 failed");
     dst->src.buf.data = (void*)(intptr_t)r;
 }
-#endif /*!defined(GAUCHE_WINDOWS)*/
 
 /* Low-level function to find if the file descriptor is ready or not.
    DIR specifies SCM_PORT_INPUT or SCM_PORT_OUTPUT.
@@ -299,9 +302,24 @@ int Scm_FdReady(int fd, int dir)
     if (r < 0) Scm_SysError("select failed");
     if (r > 0) return SCM_FD_READY;
     else       return SCM_FD_WOULDBLOCK;
-#else  /*!HAVE_SELECT*/
+#elif  defined(GAUCHE_WINDOWS)
+    if (dir == SCM_PORT_OUTPUT) {
+        /* We assume it is always ok */
+        return SCM_FD_READY;
+    } else {
+        HANDLE h = (HANDLE)_get_osfhandle(fd);
+        DWORD avail;
+        if (h == INVALID_HANDLE_VALUE) return SCM_FD_READY;
+        if (PeekNamedPipe(h, NULL, 0, NULL, &avail, NULL) == 0) {
+            /* We assume the port isn't an end of a pipe. */
+            return SCM_FD_UNKNOWN;
+        }
+        if (avail == 0) return SCM_FD_WOULDBLOCK;
+        else return SCM_FD_READY;
+    }
+#else  /*!HAVE_SELECT && !GAUCHE_WINDOWS */
     return SCM_FD_UNKNOWN;
-#endif /*!HAVE_SELECT*/
+#endif /*!HAVE_SELECT && !GAUCHE_WINDOWS */
 }
 
 /*===============================================================
@@ -689,13 +707,16 @@ static void register_buffered_port(ScmPort *port)
   retry:
     h = i = (int)PORT_HASH(port);
     c = 0;
-    /* search an available entry by quadratic hash */
+    /* search an available entry by quadratic hash
+       used entry may have #<port> or #t.  #t is for transient state
+       during Scm_FlushAllPorts()---see below. */
     (void)SCM_INTERNAL_MUTEX_LOCK(active_buffered_ports.mutex);
-    while (SCM_PORTP(Scm_WeakVectorRef(active_buffered_ports.ports, i, SCM_FALSE))) {
+    while (!SCM_FALSEP(Scm_WeakVectorRef(active_buffered_ports.ports,
+                                         i, SCM_FALSE))) {
         i -= ++c; while (i<0) i+=PORT_VECTOR_SIZE;
         if (i == h) {
-            // Vector entry is full.  We run global GC to try to collect
-            // unused entry.
+            /* Vector entry is full.  We run global GC to try to collect 
+               unused entry. */
             need_gc = TRUE;
             break;
         }
@@ -707,8 +728,8 @@ static void register_buffered_port(ScmPort *port)
 
     if (need_gc) {
         if (tried_gc) {
-            // We should probably try to extend the weak vector.
-            // But for the time being...
+            /* We should probably try to extend the weak vector.
+               But for the time being... */
             Scm_Panic("active buffered port table overflow");
         } else {
             GC_gcollect();
@@ -970,8 +991,14 @@ ScmObj Scm_MakePortWithFd(ScmObj name, int direction,
     bufrec.closer = file_closer;
     bufrec.ready = file_ready;
     bufrec.filenum = file_filenum;
-    bufrec.seeker = NULL;
     bufrec.data = (void*)(intptr_t)fd;
+
+    /* Check if the given fd is seekable, and set seeker if so. */
+    if (lseek(fd, 0, SEEK_CUR) < 0) {
+        bufrec.seeker = NULL;
+    } else {
+        bufrec.seeker = file_seeker;
+    }
     
     p = Scm_MakeBufferedPort(SCM_CLASS_PORT, name, direction, ownerp, &bufrec);
     return p;
@@ -1107,7 +1134,7 @@ ScmObj Scm__GetRemainingInputStringCompat(ScmPort *port)
  */
 
 /* To create a procedural port, fill in the ScmPortVTable function
-   pointers and pass it to Scm_MakeVirutalPort.  You don't need to
+   pointers and pass it to Scm_MakeVirtualPort.  You don't need to
    provide all the functions; put NULL if you think you don't
    provide the functionality.
 */
@@ -1556,20 +1583,21 @@ static ScmObj scm_stdin  = SCM_UNBOUND;
 static ScmObj scm_stdout = SCM_UNBOUND;
 static ScmObj scm_stderr = SCM_UNBOUND;
 
-ScmObj Scm_Stdin(void)
-{
-    return scm_stdin;
-}
-
-ScmObj Scm_Stdout(void)
-{
-    return scm_stdout;
-}
-
-ScmObj Scm_Stderr(void)
-{
-    return scm_stderr;
-}
+#define DEFSTDPORT(Name, var)                           \
+    ScmObj SCM_CPP_CAT(Scm_, Name)(void)                \
+    {                                                   \
+        return var;                                     \
+    }                                                   \
+    ScmObj SCM_CPP_CAT(Scm_Set, Name)(ScmPort *port)    \
+    {                                                   \
+        ScmObj oldp = var;                              \
+        var = SCM_OBJ(port);                            \
+        return oldp;                                    \
+    }
+ 
+DEFSTDPORT(Stdin, scm_stdin)
+DEFSTDPORT(Stdout, scm_stdout)
+DEFSTDPORT(Stderr, scm_stderr)
 
 ScmObj Scm_SetCurrentInputPort(ScmPort *port)
 {
@@ -1595,7 +1623,6 @@ ScmObj Scm_SetCurrentErrorPort(ScmPort *port)
     return oldp;
 }
 
-
 /*===============================================================
  * Initialization
  */
@@ -1622,8 +1649,97 @@ void Scm__InitPort(void)
     scm_stderr = Scm_MakePortWithFd(SCM_MAKE_STR("(stderr)"),
                                     SCM_PORT_OUTPUT, 2,
                                     SCM_PORT_BUFFER_NONE, TRUE);
+
+    /* The root VM is initialized with bogus standard ports; we need to
+       reset them. */
+    Scm_VM()->curin  = SCM_PORT(scm_stdin);
+    Scm_VM()->curout = SCM_PORT(scm_stdout);
+    Scm_VM()->curerr = SCM_PORT(scm_stderr);
+    
     key_full   = Scm_MakeKeyword(SCM_STRING(SCM_MAKE_STR("full")));
     key_modest = Scm_MakeKeyword(SCM_STRING(SCM_MAKE_STR("modest")));
     key_line   = Scm_MakeKeyword(SCM_STRING(SCM_MAKE_STR("line")));
     key_none   = Scm_MakeKeyword(SCM_STRING(SCM_MAKE_STR("none")));
 }
+
+/* Windows specific:
+
+   When we run Windows no-console mode, stdios are bogus (gosh-noconsole
+   wires them to NUL device, but when libgauche is called from other
+   applications, we can't assume that.)   It is too painful to be so,
+   since when Scheme program tries to write to stdout or stderr it just
+   crashes without any information at all.
+
+   So, this function re-wires Scheme standard output and error output
+   to a special port; when an output is made for the first time
+   the port opens up a console by AllocConsole(), and redirects further
+   output to it.
+
+   Unfortunately we can't do this at initialization time, since
+   Scm_Init() doesn't have a way to know whether we're in console mode
+   or not.  Only the application knows, thus it needs to call this API
+   after Scheme system is initialized.
+
+   NB: AllocConsole and port swapping should be mutexed, but currently
+   we don't support threads on native Windows, so no need to worry.
+ */
+#if defined(GAUCHE_WINDOWS)
+static int trapper_flusher(ScmPort *p, int cnt, int forcep)
+{
+    static int consoleCreated = FALSE;
+    size_t nwrote = 0;
+    int size = SCM_PORT_BUFFER_AVAIL(p);
+    char *buf = p->src.buf.buffer;
+    
+    if (!consoleCreated) {
+        AllocConsole();
+        freopen("CONOUT$", "w", stdout);
+        consoleCreated = TRUE;
+    }
+
+    while ((!forcep && nwrote == 0) || (forcep && nwrote < cnt)) {
+        size_t r = fwrite(buf, 1, size, stdout);
+        if (ferror(stdout)) {
+            /* NB: Double fault will be caught in the error handling
+               mechanism, so we don't need to worry it here. */
+            Scm_Error("output to CONOUT$ failed");
+        }
+        nwrote += r;
+        buf += r;
+    }
+    fflush(stdout);
+    return nwrote;
+}
+
+static ScmObj make_trapper_port()
+{
+    ScmPortBuffer bufrec;
+
+    bufrec.mode = SCM_PORT_BUFFER_LINE;
+    bufrec.buffer = NULL;
+    bufrec.size = 0;
+    bufrec.filler = NULL;
+    bufrec.flusher = trapper_flusher;
+    bufrec.closer = NULL;
+    bufrec.ready = NULL;
+    bufrec.filenum = NULL;
+    bufrec.seeker = NULL;
+    bufrec.data = NULL;
+    return Scm_MakeBufferedPort(SCM_CLASS_PORT,
+                                SCM_MAKE_STR("(console output)"),
+                                SCM_PORT_OUTPUT, TRUE, &bufrec);
+}
+
+void Scm__SetupPortsForWindows(int has_console)
+{
+    if (!has_console) {
+        ScmObj trapperPort = make_trapper_port();
+        scm_stdout = trapperPort;
+        scm_stderr = trapperPort;
+        Scm_VM()->curout = SCM_PORT(scm_stdout);
+        Scm_VM()->curerr = SCM_PORT(scm_stderr);
+    }
+}
+#endif /*defined(GAUCHE_WINDOWS)*/
+
+
