@@ -1,7 +1,7 @@
 /*
  * portapi.c - port common API
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: portapi.c,v 1.32 2007/08/24 23:55:43 shirok Exp $
  */
 
 /* This file is included _twice_ by port.c to define safe- and unsafe-
@@ -483,6 +481,7 @@ int Scm_GetbUnsafe(ScmPort *p)
             Scm_PortError(p, SCM_PORT_ERROR_INPUT,
                           "bad port type for input: %S", p);
         }
+        p->bytes++;
     }
     UNLOCK(p);
     return b;
@@ -555,6 +554,7 @@ int Scm_GetcUnsafe(ScmPort *p)
         }
         first = (unsigned char)*p->src.buf.current++;
         nb = SCM_CHAR_NFOLLOWS(first);
+        p->bytes++;
         if (nb > 0) {
             if (p->src.buf.current + nb > p->src.buf.end) {
                 /* The buffer doesn't have enough bytes to consist a char.
@@ -591,6 +591,7 @@ int Scm_GetcUnsafe(ScmPort *p)
                 SCM_CHAR_GET(p->src.buf.current-1, c);
                 p->src.buf.current += nb;
             }
+            p->bytes += nb;
         } else {
             c = first;
             if (c == '\n') p->line++;
@@ -604,6 +605,7 @@ int Scm_GetcUnsafe(ScmPort *p)
         }
         first = (unsigned char)*p->src.istr.current++;
         nb = SCM_CHAR_NFOLLOWS(first);
+        p->bytes++;
         if (nb > 0) {
             if (p->src.istr.current + nb > p->src.istr.end) {
                 /* TODO: make this behavior customizable */
@@ -613,6 +615,7 @@ int Scm_GetcUnsafe(ScmPort *p)
             }
             SCM_CHAR_GET(p->src.istr.current-1, c);
             p->src.istr.current += nb;
+            p->bytes += nb;
         } else {
             c = first;
             if (c == '\n') p->line++;
@@ -626,8 +629,7 @@ int Scm_GetcUnsafe(ScmPort *p)
         return c;
     default:
         UNLOCK(p);
-        Scm_PortError(p, SCM_PORT_ERROR_INPUT,
-                      "bad port type for input: %S", p);
+        Scm_PortError(p, SCM_PORT_ERROR_INPUT, "bad port type for input: %S", p);
     }
     return 0;/*dummy*/
 }
@@ -712,21 +714,23 @@ int Scm_GetzUnsafe(char *buf, int buflen, ScmPort *p)
     switch (SCM_PORT_TYPE(p)) {
     case SCM_PORT_FILE:
         SAFE_CALL(p, siz = bufport_read(p, buf, buflen));
+        p->bytes += siz;
         UNLOCK(p);
         if (siz == 0) return EOF;
         else return siz;
     case SCM_PORT_ISTR:
         r = GETZ_ISTR(p, buf, buflen);
+        p->bytes += r;
         UNLOCK(p);
         return r;
     case SCM_PORT_PROC:
         SAFE_CALL(p, r = p->src.vt.Getz(buf, buflen, p));
+        p->bytes += r;
         UNLOCK(p);
         return r;
     default:
         UNLOCK(p);
-        Scm_PortError(p, SCM_PORT_ERROR_INPUT,
-                      "bad port type for input: %S", p);
+        Scm_PortError(p, SCM_PORT_ERROR_INPUT, "bad port type for input: %S", p);
     }
     return -1;                  /* dummy */
 }
@@ -934,11 +938,16 @@ ScmObj Scm_PortSeekUnsafe(ScmPort *p, ScmObj off, int whence)
             /* NB: possible optimization: the specified position is within
                the current buffer, we can avoid calling seeker. */
             if (SCM_PORT_DIR(p)&SCM_PORT_INPUT) {
+                char *c = p->src.buf.current; /* save current ptr */
                 if (whence == SEEK_CUR) {
-                    o -= (off_t)(p->src.buf.end - p->src.buf.current);
+                    o -= (off_t)(p->src.buf.end - c);
                 }
                 p->src.buf.current = p->src.buf.end; /* invalidate buffer */
                 SAFE_CALL(p, r = p->src.buf.seeker(p, o, whence));
+                if (r == (off_t)-1) {
+                    /* This may happend if seeker somehow gave up */
+                    p->src.buf.current = c;
+                }
             } else {
                 SAFE_CALL(p, bufport_flush(p, 0, TRUE));
                 SAFE_CALL(p, r = p->src.buf.seeker(p, o, whence));
