@@ -1,7 +1,7 @@
 /*
  * prof.c - profiler
  *
- *   Copyright (c) 2005-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2005-2010  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: prof.c,v 1.10 2007/08/24 23:55:43 shirok Exp $
  */
 
 #define LIBGAUCHE_BODY
@@ -144,16 +142,16 @@ void collect_samples(ScmVMProfiler *prof)
 {
     int i, cnt;
     for (i=0; i<prof->currentSample; i++) {
-        ScmHashEntry *e = Scm_HashTableGet(prof->statHash,
-                                           prof->samples[i].func);
-        if (e == NULL) {
+        ScmObj e = Scm_HashTableRef(prof->statHash,
+                                    prof->samples[i].func, SCM_UNBOUND);
+        if (SCM_UNBOUNDP(e)) {
             /* NB: just for now */
             Scm_Warn("profiler: uncounted object appeared in a sample: %p (%S)\n",
                      prof->samples[i].func, prof->samples[i].func);
         } else {
-            SCM_ASSERT(SCM_PAIRP(e->value));
-            cnt = SCM_INT_VALUE(SCM_CDR(e->value)) + 1;
-            SCM_SET_CDR(e->value, SCM_MAKE_INT(cnt));
+            SCM_ASSERT(SCM_PAIRP(e));
+            cnt = SCM_INT_VALUE(SCM_CDR(e)) + 1;
+            SCM_SET_CDR(e, SCM_MAKE_INT(cnt));
         }
     }
 }
@@ -181,7 +179,7 @@ void Scm_ProfilerCountBufferFlush(ScmVM *vm)
 
     ncounts = vm->prof->currentCount;
     for (i=0; i<ncounts; i++) {
-        ScmHashEntry *e;
+        ScmObj e;
         int cnt;
         
         func = vm->prof->counts[i].func;
@@ -192,16 +190,20 @@ void Scm_ProfilerCountBufferFlush(ScmVM *vm)
             func = SCM_OBJ(SCM_METHOD(func)->data);
         }
         
-        e = Scm_HashTableAdd(vm->prof->statHash,
+        e = Scm_HashTableSet(vm->prof->statHash,
                              vm->prof->counts[i].func,
-                             SCM_FALSE);
-        if (SCM_FALSEP(e->value)) {
-            e->value = Scm_Cons(SCM_MAKE_INT(0), SCM_MAKE_INT(0));
+                             SCM_FALSE,
+                             SCM_DICT_NO_OVERWRITE);
+        if (SCM_FALSEP(e)) {
+            e = Scm_HashTableSet(vm->prof->statHash,
+                                 vm->prof->counts[i].func,
+                                 Scm_Cons(SCM_MAKE_INT(0), SCM_MAKE_INT(0)),
+                                 0);
         }
 
-        SCM_ASSERT(SCM_PAIRP(e->value));
-        cnt = SCM_INT_VALUE(SCM_CAR(e->value)) + 1;
-        SCM_SET_CAR(e->value, SCM_MAKE_INT(cnt));
+        SCM_ASSERT(SCM_PAIRP(e));
+        cnt = SCM_INT_VALUE(SCM_CAR(e)) + 1;
+        SCM_SET_CAR(e, SCM_MAKE_INT(cnt));
     }
     vm->prof->currentCount = 0;
 
