@@ -1,7 +1,7 @@
 /*
  * read.c - reader
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: read.c,v 1.98 2008/02/02 05:14:42 shirok Exp $
  */
 
 #include <stdio.h>
@@ -60,13 +58,14 @@ static ScmObj read_word(ScmPort *port, ScmChar initial, ScmReadContext *ctx,
 static ScmObj read_symbol(ScmPort *port, ScmChar initial, ScmReadContext *ctx);
 static ScmObj read_number(ScmPort *port, ScmChar initial, ScmReadContext *ctx);
 static ScmObj read_symbol_or_number(ScmPort *port, ScmChar initial, ScmReadContext *ctx);
-static ScmObj read_escaped_symbol(ScmPort *port, ScmChar delim);
+static ScmObj read_escaped_symbol(ScmPort *port, ScmChar delim, int interned);
 static ScmObj read_keyword(ScmPort *port, ScmReadContext *ctx);
 static ScmObj read_regexp(ScmPort *port);
 static ScmObj read_charset(ScmPort *port);
 static ScmObj read_sharp_comma(ScmPort *port, ScmReadContext *ctx);
 static ScmObj process_sharp_comma(ScmPort *port, ScmObj key, ScmObj args,
                                   ScmReadContext *ctx, int has_ref);
+static ScmObj read_shebang(ScmPort *port, ScmReadContext *ctx);
 static ScmObj read_reference(ScmPort *port, ScmChar ch, ScmReadContext *ctx);
 static ScmObj maybe_uvector(ScmPort *port, char c, ScmReadContext *ctx);
 
@@ -79,6 +78,12 @@ static struct {
     ScmInternalMutex mutex;
 } readCtorData = { NULL };
 
+/* Table of hash-bang directive */
+static struct {
+    ScmHashTable *table;
+    ScmInternalMutex mutex;
+} hashBangData = { NULL };
+
 /*----------------------------------------------------------------
  * Entry points
  *   Note: Entire read operation are done while locking the input port.
@@ -112,19 +117,16 @@ ScmObj Scm_ReadWithContext(ScmObj port, ScmReadContext *ctx)
 
 ScmObj Scm_Read(ScmObj port)
 {
-    ScmReadContext ctx;
-    read_context_init(Scm_VM(), &ctx);
-    return Scm_ReadWithContext(port, &ctx);
+    return Scm_ReadWithContext(port, Scm_MakeReadContext(NULL));
 }
 
 /* Convenience functions */
 ScmObj Scm_ReadFromString(ScmString *str)
 {
     ScmObj inp = Scm_MakeInputStringPort(str, TRUE), r;
-    ScmReadContext ctx;
-    read_context_init(Scm_VM(), &ctx);
-    r = read_item(SCM_PORT(inp), &ctx);
-    read_context_flush(&ctx);
+    ScmReadContext *ctx = Scm_MakeReadContext(NULL);
+    r = read_item(SCM_PORT(inp), ctx);
+    read_context_flush(ctx);
     return r;
 }
 
@@ -133,10 +135,9 @@ ScmObj Scm_ReadFromCString(const char *cstr)
     ScmObj s = SCM_MAKE_STR_IMMUTABLE(cstr);
     ScmObj inp = Scm_MakeInputStringPort(SCM_STRING(s), TRUE);
     ScmObj r;
-    ScmReadContext ctx;
-    read_context_init(Scm_VM(), &ctx);
-    r = read_item(SCM_PORT(inp), &ctx);
-    read_context_flush(&ctx);
+    ScmReadContext *ctx = Scm_MakeReadContext(NULL);
+    r = read_item(SCM_PORT(inp), ctx);
+    read_context_flush(ctx);
     return r;
 }
 
@@ -166,21 +167,41 @@ ScmObj Scm_ReadListWithContext(ScmObj port, ScmChar closer, ScmReadContext *ctx)
 
 ScmObj Scm_ReadList(ScmObj port, ScmChar closer)
 {
-    ScmReadContext ctx;
-    read_context_init(Scm_VM(), &ctx);
-    return Scm_ReadListWithContext(port, closer, &ctx);
+    ScmReadContext *ctx = Scm_MakeReadContext(NULL);
+    return Scm_ReadListWithContext(port, closer, ctx);
 }
 
-static void read_context_init(ScmVM *vm, ScmReadContext *ctx)
+/*----------------------------------------------------------------
+ * Read context
+ */
+
+ScmReadContext *Scm_MakeReadContext(ScmReadContext *proto)
 {
-    ctx->flags = SCM_READ_SOURCE_INFO;
-    if (SCM_VM_RUNTIME_FLAG_IS_SET(vm, SCM_CASE_FOLD)) {
-        ctx->flags |= SCM_READ_CASE_FOLD;
+    ScmReadContext *ctx = SCM_NEW(ScmReadContext);
+    SCM_SET_CLASS(ctx, SCM_CLASS_READ_CONTEXT);
+    if (proto == NULL) {
+        ctx->flags = SCM_READ_SOURCE_INFO;
+        ctx->table = NULL;
+        ctx->pending = SCM_NIL;
+    } else {
+        /* TODO: we haven't thought well of the case when prototype
+           is given.  Should we copy the #n= table as well?  How should
+           we handle pending read references? */
+        ctx->flags = proto->flags;
+        ctx->table = NULL;
+        ctx->pending = SCM_NIL;
     }
-    ctx->table = NULL;
-    ctx->pending = SCM_NIL;
+    return ctx;
 }
 
+static void read_context_print(ScmObj obj, ScmPort *port,
+                               ScmWriteContext *ctx)
+{
+    Scm_Printf(port, "#<read-context %p>", obj);
+}
+
+SCM_DEFINE_BUILTIN_CLASS_SIMPLE(Scm_ReadContextClass, read_context_print);
+
 /*----------------------------------------------------------------
  * Error
  */
@@ -249,7 +270,7 @@ static ScmObj ref_val(ScmObj ref)
 static ScmObj ref_register(ScmReadContext *ctx, ScmObj obj, int refnum)
 {
     SCM_ASSERT(ctx->table);
-    Scm_HashTablePut(ctx->table, SCM_MAKE_INT(refnum), obj);
+    Scm_HashTableSet(ctx->table, SCM_MAKE_INT(refnum), obj, 0);
     return obj;
 }
 
@@ -371,27 +392,37 @@ static void read_nested_comment(ScmPort *port, ScmReadContext *ctx)
     }
 }
 
+static void read_comment(ScmPort *port) /* leading semicolon is already read */
+{
+    for (;;) {
+        /* NB: comment may contain unexpected character code.
+           for the safety, we read bytes here. */
+        int c = Scm_GetbUnsafe(port);
+        if (c == '\n') {
+            /* oops.  ugly. */
+            port->line++;
+            break;
+        }
+        if (c == EOF) break;
+    }
+}
+
 static int skipws(ScmPort *port, ScmReadContext *ctx)
 {
+    ScmChar c0;
+    
     for (;;) {
         int c = Scm_GetcUnsafe(port);
         if (c == EOF) return c;
-        if (c <= 256 && isspace(c)) continue;
-        if (c == ';') {
-            for (;;) {
-                /* NB: comment may contain unexpected character code.
-                   for the safety, we read bytes here. */
-                c = Scm_GetbUnsafe(port);
-                if (c == '\n') {
-                    /* oops.  ugly. */
-                    port->line++;
-                    break;
-                }
-                if (c == EOF) return EOF;
+        if (c <= 127) {
+            if (isspace(c)) continue;
+            if (c == ';') {
+                read_comment(port);
+                continue;
             }
-            continue;
+            return c;
         }
-        return c;
+        else if (!SCM_CHAR_EXTRA_WHITESPACE(c)) return c;
     }
 }
 
@@ -423,12 +454,8 @@ static ScmObj read_internal(ScmPort *port, ScmReadContext *ctx)
                 Scm_UngetcUnsafe(c1, port);
                 return read_number(port, c, ctx);
             case '!':
-                /* allow `#!' magic of executable */
-                for (;;) {
-                    c = Scm_GetcUnsafe(port);
-                    if (c == '\n') return SCM_UNDEFINED;
-                    if (c == EOF) return SCM_EOF;
-                }
+                /* #! is either a script shebang or a reader directive */
+                return read_shebang(port, ctx);
             case '/':
                 /* #/.../ literal regexp */
                 return read_regexp(port);
@@ -482,6 +509,17 @@ static ScmObj read_internal(ScmPort *port, ScmReadContext *ctx)
                     if (c2 == '"') return read_string(port, TRUE, ctx);
                     Scm_ReadError(port, "unsupported #*-syntax: #*%C", c2);
                 }
+            case ':':
+                /* #:name - uninterned symbol */
+                {
+                    int c2 = Scm_GetcUnsafe(port);
+                    if (c2 == '|') {
+                        return read_escaped_symbol(port, c2, FALSE);
+                    } else {
+                        ScmObj name = read_word(port, c2, ctx, FALSE);
+                        return Scm_MakeSymbol(SCM_STRING(name), FALSE);
+                    }
+                }
             case ';':
                 /* #;expr - comment out sexpr */
                 {
@@ -512,7 +550,7 @@ static ScmObj read_internal(ScmPort *port, ScmReadContext *ctx)
             }
         }
     case '|':
-        return read_escaped_symbol(port, '|');
+        return read_escaped_symbol(port, '|', TRUE);
     case '[':
         /* TODO: make it customizable */
         return read_list(port, ']', ctx);
@@ -717,6 +755,8 @@ static ScmObj read_string(ScmPort *port, int incompletep,
 #define ACCUMULATE(var)                                 \
     if (incompletep) { SCM_DSTRING_PUTB(&ds, var); }    \
     else             { SCM_DSTRING_PUTC(&ds, var); }
+#define INTRALINE_WS(var)                               \
+    ((var)==' ' || (var)=='\t' || SCM_CHAR_EXTRA_WHITESPACE_INTRALINE(var))
 
     for (;;) {
         FETCH(c);
@@ -754,6 +794,7 @@ static ScmObj read_string(ScmPort *port, int incompletep,
                 break;
             }
                 /* R6RS-style line continuation handling*/
+            line_continuation:
             case ' ':
             case '\t': {
                 for (;;) {
@@ -761,7 +802,7 @@ static ScmObj read_string(ScmPort *port, int incompletep,
                     if (c == EOF) goto eof_exit;
                     if (c == '\r') goto cont_cr;
                     if (c == '\n') goto cont_lf;
-                    if (c != ' ' && c != '\t') goto cont_err;
+                    if (!INTRALINE_WS(c)) goto cont_err;
                 }
             }
                 /*FALLTHROUGH*/
@@ -770,7 +811,7 @@ static ScmObj read_string(ScmPort *port, int incompletep,
                 FETCH(c);
                 if (c == EOF)  goto eof_exit;
                 if (c == '\\') goto backslash;
-                if (c != '\n' && c != ' ' && c != '\t') {
+                if (c != '\n' && !INTRALINE_WS(c)) {
                     ACCUMULATE(c);
                     break;
                 }
@@ -782,7 +823,7 @@ static ScmObj read_string(ScmPort *port, int incompletep,
                     FETCH(c);
                     if (c == EOF) goto eof_exit;
                     if (c == '\\') goto backslash;
-                    if (c != ' ' && c != '\t') {
+                    if (!INTRALINE_WS(c)) {
                         ACCUMULATE(c);
                         break;
                     }
@@ -790,10 +831,11 @@ static ScmObj read_string(ScmPort *port, int incompletep,
                 break;
             }
             default:
-                ACCUMULATE(c); break;
+                if (SCM_CHAR_EXTRA_WHITESPACE_INTRALINE(c)) goto line_continuation;
+                else ACCUMULATE(c);
             }
             break;
-        }       
+        } 
         default: ACCUMULATE(c); break;
         }
     }
@@ -813,23 +855,26 @@ static ScmObj read_string(ScmPort *port, int incompletep,
 
 static struct char_name {
     const char *name;
+    int size;
     ScmObj ch;
 } char_names[] = {
-    { "space",        SCM_MAKE_CHAR(' ')  },
-    { "newline",      SCM_MAKE_CHAR('\n') },
-    { "nl",           SCM_MAKE_CHAR('\n') },
-    { "lf",           SCM_MAKE_CHAR('\n') },
-    { "return",       SCM_MAKE_CHAR('\r') },
-    { "cr",           SCM_MAKE_CHAR('\r') },
-    { "tab",          SCM_MAKE_CHAR('\t') },
-    { "ht",           SCM_MAKE_CHAR('\t') },
-    { "page",         SCM_MAKE_CHAR('\f') },
-    { "escape",       SCM_MAKE_CHAR(0x1b) },
-    { "esc",          SCM_MAKE_CHAR(0x1b) },
-    { "delete",       SCM_MAKE_CHAR(0x7f) },
-    { "del",          SCM_MAKE_CHAR(0x7f) },
-    { "null",         SCM_MAKE_CHAR(0)    },
-    { NULL, 0 }
+#define DEFCHAR(name, char) \
+    { #name, sizeof(#name)-1, SCM_MAKE_CHAR(char) }
+    DEFCHAR(space,   ' '),
+    DEFCHAR(newline, '\n'),
+    DEFCHAR(nl,      '\n'),
+    DEFCHAR(lf,      '\n'),
+    DEFCHAR(return,  '\r'),
+    DEFCHAR(cr,      '\r'),
+    DEFCHAR(tab,     '\t'),
+    DEFCHAR(ht,      '\t'),
+    DEFCHAR(page,    '\f'),
+    DEFCHAR(escape,  0x1b),
+    DEFCHAR(esc,     0x1b),
+    DEFCHAR(delete,  0x7f),
+    DEFCHAR(del,     0x7f),
+    DEFCHAR(null,    0),
+    { NULL, 0, 0 }
 };
 
 static ScmObj read_char(ScmPort *port, ScmReadContext *ctx)
@@ -877,7 +922,10 @@ static ScmObj read_char(ScmPort *port, ScmReadContext *ctx)
         }
 
         while (cntab->name) {
-            if (strncmp(cntab->name, cname, namesize) == 0) return cntab->ch;
+            if (cntab->size == namesize
+                && strncmp(cntab->name, cname, namesize) == 0) {
+                return cntab->ch;
+            }
             cntab++;
         }
       unknown:
@@ -899,7 +947,7 @@ static ScmObj read_word(ScmPort *port, ScmChar initial, ScmReadContext *ctx,
                         int temp_case_fold)
 {
     int c = 0;
-    int case_fold = temp_case_fold || (ctx->flags & SCM_READ_CASE_FOLD);
+    int case_fold = temp_case_fold || SCM_PORT_CASE_FOLD(port);
     ScmDString ds;
     Scm_DStringInit(&ds);
     if (initial != SCM_CHAR_INVALID) {
@@ -945,11 +993,20 @@ static ScmObj read_symbol_or_number(ScmPort *port, ScmChar initial, ScmReadConte
 
 static ScmObj read_keyword(ScmPort *port, ScmReadContext *ctx)
 {
-    ScmString *s = SCM_STRING(read_word(port, SCM_CHAR_INVALID, ctx, FALSE));
-    return Scm_MakeKeyword(s);
+    int c2 = Scm_GetcUnsafe(port);
+    ScmObj name;
+    
+    if (c2 == '|') {
+        name = read_escaped_symbol(port, c2, FALSE); /* read as uninterned */
+        return Scm_MakeKeyword(SCM_SYMBOL_NAME(name));
+    } else {
+        Scm_UngetcUnsafe(c2, port);
+        name = read_word(port, SCM_CHAR_INVALID, ctx, FALSE);
+        return Scm_MakeKeyword(SCM_STRING(name));
+    }
 }
 
-static ScmObj read_escaped_symbol(ScmPort *port, ScmChar delim)
+static ScmObj read_escaped_symbol(ScmPort *port, ScmChar delim, int interned)
 {
     int c = 0;
     ScmDString ds;
@@ -961,7 +1018,7 @@ static ScmObj read_escaped_symbol(ScmPort *port, ScmChar delim)
             goto err;
         } else if (c == delim) {
             ScmString *s = SCM_STRING(Scm_DStringGet(&ds, 0));
-            return Scm_Intern(s);
+            return Scm_MakeSymbol(s, interned);
         } else if (c == '\\') {
             /* CL-style single escape */
             c = Scm_GetcUnsafe(port);
@@ -1028,7 +1085,7 @@ static ScmObj read_charset(ScmPort *port)
 
 static ScmObj read_reference(ScmPort *port, ScmChar ch, ScmReadContext *ctx)
 {
-    ScmHashEntry *e = NULL;
+    ScmObj e = SCM_UNBOUND;
     int refnum = Scm_DigitToInt(ch, 10);
 
     for (;;) {
@@ -1049,14 +1106,15 @@ static ScmObj read_reference(ScmPort *port, ScmChar ch, ScmReadContext *ctx)
     if (ch == '#') {
         /* #digit# - back reference */
         if (ctx->table == NULL
-            || (e = Scm_HashTableGet(ctx->table, Scm_MakeInteger(refnum))) == NULL) {
+            || SCM_UNBOUNDP(e = Scm_HashTableRef(ctx->table,
+                                                 Scm_MakeInteger(refnum),
+                                                 SCM_UNBOUND))) {
             Scm_ReadError(port, "invalid reference number in #%d#", refnum);
         }
-        if (SCM_READ_REFERENCE_P(e->value)
-            && SCM_READ_REFERENCE_REALIZED(e->value)) {
-            return SCM_READ_REFERENCE(e->value)->value;
+        if (SCM_READ_REFERENCE_P(e) && SCM_READ_REFERENCE_REALIZED(e)) {
+            return SCM_READ_REFERENCE(e)->value;
         } else {
-            return e->value;
+            return e;
         }
     } else {
         /* #digit= - register */
@@ -1067,8 +1125,11 @@ static ScmObj read_reference(ScmPort *port, ScmChar ch, ScmReadContext *ctx)
             ctx->table =
                 SCM_HASH_TABLE(Scm_MakeHashTableSimple(SCM_HASH_EQV, 0));
         }
-        if (Scm_HashTableGet(ctx->table, Scm_MakeInteger(refnum)) != NULL) {
-            Scm_ReadError(port, "duplicate back-reference number in #%d=", refnum);
+        if (!SCM_UNBOUNDP(Scm_HashTableRef(ctx->table,
+                                           Scm_MakeInteger(refnum),
+                                           SCM_UNBOUND))) {
+            Scm_ReadError(port, "duplicate back-reference number in #%d=",
+                          refnum);
         }
         ref_register(ctx, ref, refnum);
         val = read_item(port, ctx);
@@ -1081,7 +1142,12 @@ static ScmObj read_reference(ScmPort *port, ScmChar ch, ScmReadContext *ctx)
  * SRFI-10 support
  */
 
-ScmObj Scm_DefineReaderCtor(ScmObj symbol, ScmObj proc, ScmObj finisher)
+/* NB: The 'module' argument of DefineReaderCtor and GetReaderCtor may
+   be used in future to make reader-ctor binding associated with modules.
+   For now, it is not used and the caller should pass SCM_FALSE. */
+
+ScmObj Scm_DefineReaderCtor(ScmObj symbol, ScmObj proc, ScmObj finisher,
+                            ScmObj module /*reserved*/)
 {
     ScmObj pair;
     if (!SCM_PROCEDUREP(proc)) {
@@ -1089,11 +1155,20 @@ ScmObj Scm_DefineReaderCtor(ScmObj symbol, ScmObj proc, ScmObj finisher)
     }
     pair = Scm_Cons(proc, finisher);
     (void)SCM_INTERNAL_MUTEX_LOCK(readCtorData.mutex);
-    Scm_HashTablePut(readCtorData.table, symbol, pair);
+    Scm_HashTableSet(readCtorData.table, symbol, pair, 0);
     (void)SCM_INTERNAL_MUTEX_UNLOCK(readCtorData.mutex);
     return SCM_UNDEFINED;
 }
 
+ScmObj Scm_GetReaderCtor(ScmObj symbol, ScmObj module /*reserved*/)
+{
+    ScmObj r;
+    (void)SCM_INTERNAL_MUTEX_LOCK(readCtorData.mutex);
+    r = Scm_HashTableRef(readCtorData.table, symbol, SCM_FALSE);
+    (void)SCM_INTERNAL_MUTEX_UNLOCK(readCtorData.mutex);
+    return r;
+}
+
 static ScmObj read_sharp_comma(ScmPort *port, ScmReadContext *ctx)
 {
     int len, has_ref, line=-1;
@@ -1120,26 +1195,78 @@ static ScmObj read_sharp_comma(ScmPort *port, ScmReadContext *ctx)
 static ScmObj process_sharp_comma(ScmPort *port, ScmObj key, ScmObj args,
                                   ScmReadContext *ctx, int has_ref)
 {
-    ScmHashEntry *e;
-    ScmObj r;
+    ScmObj r, e;
 
     if (ctx->flags & SCM_READ_DISABLE_CTOR) return SCM_FALSE;
 
     (void)SCM_INTERNAL_MUTEX_LOCK(readCtorData.mutex);
-    e = Scm_HashTableGet(readCtorData.table, key);
+    e = Scm_HashTableRef(readCtorData.table, key, SCM_FALSE);
     (void)SCM_INTERNAL_MUTEX_UNLOCK(readCtorData.mutex);
 
-    if (e == NULL) Scm_ReadError(port, "unknown #,-key: %S", key);
-    SCM_ASSERT(SCM_PAIRP(e->value));
-    r = Scm_ApplyRec(SCM_CAR(e->value), args);
-    if (has_ref) ref_push(ctx, r, SCM_CDR(e->value));
+    if (!SCM_PAIRP(e)) Scm_ReadError(port, "unknown #,-key: %S", key);
+    r = Scm_ApplyRec(SCM_CAR(e), args);
+    if (has_ref) ref_push(ctx, r, SCM_CDR(e));
     return r;
 }
 
 static ScmObj reader_ctor(ScmObj *args, int nargs, void *data)
 {
     ScmObj optarg = (nargs > 2? args[2] : SCM_FALSE);
-    return Scm_DefineReaderCtor(args[0], args[1], optarg);
+    return Scm_DefineReaderCtor(args[0], args[1], optarg, SCM_FALSE);
+}
+
+/*----------------------------------------------------------------
+ * #!-support
+ */
+
+ScmObj Scm_DefineReaderDirective(ScmObj symbol, ScmObj proc)
+{
+    (void)SCM_INTERNAL_MUTEX_LOCK(hashBangData.mutex);
+    Scm_HashTableSet(hashBangData.table, symbol, proc, 0);
+    (void)SCM_INTERNAL_MUTEX_UNLOCK(hashBangData.mutex);
+    return SCM_UNDEFINED;
+}
+
+static ScmObj read_shebang(ScmPort *port, ScmReadContext *ctx)
+{
+    /* If '#!' appears in the beginning of the input port, and it is
+       followed by '/' or a space, then we consider it as the
+       beginning of shebang and discards entire line.  Otherwise, we
+       take this as #!<identifier> directive as specified in R6RS, and
+       calls appropriate handler.
+
+       R6RS is actually not very clean at this corner.  It requires
+       distinct modes to parse a script (which always begins with
+       shebang) and to parse R6RS program (in which '#!'  always
+       introduces #!<identifier> token).  There's no way for just one
+       parser that strictly covers both situation.
+    */
+    int c2 = Scm_GetcUnsafe(port);
+    if (port->bytes == 3 && (c2 == '/' || c2 == ' ')) {
+        /* shebang */
+        for (;;) {
+            c2 = Scm_GetcUnsafe(port);
+            if (c2 == '\n') return SCM_UNDEFINED;
+            if (c2 == EOF) return SCM_EOF;
+        }
+        /*NOTREACHED*/
+    } else {
+        ScmObj id = read_symbol(port, c2, ctx), e, r;
+        SCM_ASSERT(SCM_SYMBOLP(id));
+        (void)SCM_INTERNAL_MUTEX_LOCK(hashBangData.mutex);
+        e = Scm_HashTableRef(hashBangData.table, id, SCM_FALSE);
+        (void)SCM_INTERNAL_MUTEX_UNLOCK(hashBangData.mutex);
+        if (SCM_FALSEP(e)) {
+            Scm_Warn("Ignoring unrecognized hash-bang directive: #!%S", id);
+            return SCM_UNDEFINED;
+        }
+        /* Reader directive may return zero or one value.  When it returns
+           no values, we call Scm_VMSetResult to adjust the number of values.
+         */
+        r = Scm_ApplyRec3(e, id, SCM_OBJ(port), SCM_OBJ(ctx));
+        if (Scm_VMGetNumResults(Scm_VM()) == 1) return r;
+        else { Scm_VMSetResult(SCM_UNDEFINED); return SCM_UNDEFINED; }
+    }
 }
 
 /*----------------------------------------------------------------
@@ -1221,6 +1348,10 @@ void Scm__InitRead(void)
     Scm_DefineReaderCtor(SCM_SYM_DEFINE_READER_CTOR,
                          Scm_MakeSubr(reader_ctor, NULL, 2, 1,
                                       SCM_SYM_DEFINE_READER_CTOR),
-                         SCM_FALSE);
+                         SCM_FALSE, SCM_FALSE);
+
+    hashBangData.table =
+        SCM_HASH_TABLE(Scm_MakeHashTableSimple(SCM_HASH_EQ, 0));
+    (void)SCM_INTERNAL_MUTEX_INIT(hashBangData.mutex);
 }
 
