@@ -1,7 +1,7 @@
 /*
  * signal.c - signal handling
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: signal.c,v 1.54 2007/09/16 04:15:59 shirok Exp $
  */
 
 #define LIBGAUCHE_BODY
@@ -77,7 +75,7 @@ static int sigprocmask_win(int how, const sigset_t *set, sigset_t *oldset);
 
 /* Master signal handler vector. */
 static struct sigHandlersRec {
-    ScmObj handlers[NSIG];      /* Scheme signal handlers.  This is #f on
+    ScmObj handlers[NSIG];      /* Scheme signal handlers.  This is #<undef> on
                                    signals to which Gauche does not install
                                    C-level signal handler (sig_handle). */
     ScmSysSigset *masks[NSIG];  /* Signal masks during executing Scheme
@@ -352,7 +350,8 @@ static void sig_handle(int signum)
     } else if (++vm->sigq.sigcounts[signum] >= signalPendingLimit) {
         Scm_Abort("Received too many signals before processing them.  Exitting for the emergency...\n");
     }
-    vm->queueNotEmpty |= SCM_VM_SIGQ_MASK;
+    vm->signalPending = TRUE;
+    vm->attentionRequest = TRUE;
 }
 
 /*-------------------------------------------------------------------
@@ -410,7 +409,7 @@ void Scm_SigCheck(ScmVM *vm)
     SIGPROCMASK(SIG_BLOCK, &sigHandlers.masterSigset, &omask);
     memcpy(sigcounts, vm->sigq.sigcounts, NSIG * sizeof(unsigned char));
     Scm_SignalQueueClear(&vm->sigq);
-    vm->queueNotEmpty &= ~SCM_VM_SIGQ_MASK;
+    vm->signalPending = FALSE;
     SIGPROCMASK(SIG_SETMASK, &omask, NULL);
 
     /* Now, prepare queued signal handlers
@@ -581,7 +580,7 @@ ScmObj Scm_SetSignalHandler(ScmObj sigs, ScmObj handler, ScmSysSigset *mask)
     struct sigaction act;
     struct sigdesc *desc;
     sigset_t sigset;
-    int badproc = FALSE, sigactionfailed = FALSE;
+    int badproc = FALSE, sigactionfailed = FALSE, passthrough = FALSE;
 
     if (SCM_INTP(sigs)) {
         int signum = SCM_INT_VALUE(sigs);
@@ -596,13 +595,8 @@ ScmObj Scm_SetSignalHandler(ScmObj sigs, ScmObj handler, ScmSysSigset *mask)
         Scm_Error("bad signal number: must be an integer signal number or a <sys-sigset> object, but got %S", sigs);
     }
 
-    if (mask == NULL) {
-        /* If no mask is specified, block singals in SIGS. */
-        mask = make_sigset();
-        mask->set = sigset;
-    }
-    
-    (void)SCM_INTERNAL_MUTEX_LOCK(sigHandlers.mutex);
+    if (SCM_UNDEFINEDP(handler)) return SCM_UNDEFINED;
+
     if (SCM_TRUEP(handler)) {
         act.sa_handler = SIG_DFL;
     } else if (SCM_FALSEP(handler)) {
@@ -613,13 +607,21 @@ ScmObj Scm_SetSignalHandler(ScmObj sigs, ScmObj handler, ScmSysSigset *mask)
     } else {
         badproc = TRUE;
     }
+
+    if (mask == NULL) {
+        /* If no mask is specified, block singals in SIGS. */
+        mask = make_sigset();
+        mask->set = sigset;
+    }
+
+    (void)SCM_INTERNAL_MUTEX_LOCK(sigHandlers.mutex);
     if (!badproc) {
         sigfillset(&act.sa_mask); /* we should block all the signals */
         act.sa_flags = 0;
         for (desc=sigDesc; desc->name; desc++) {
             if (!sigismember(&sigset, desc->num)) continue;
             if (!sigismember(&sigHandlers.masterSigset, desc->num)) continue;
-            if (sigaction(desc->num, &act, NULL) != 0) {
+            else if (sigaction(desc->num, &act, NULL) != 0) {
                 sigactionfailed = desc->num;
             } else {
                 sigHandlers.handlers[desc->num] = handler;
@@ -747,6 +749,24 @@ void Scm_SetMasterSigmask(sigset_t *set)
  * Other signal-related operations
  */
 
+/*
+ * Convenience routines hiding platform-dependent stuff
+ */
+void Scm_GetSigmask(sigset_t *mask)
+{
+    if (SIGPROCMASK(SIG_SETMASK, NULL, mask) != 0) {
+        Scm_SysError("sigprocmask failed");
+    }
+}
+
+void Scm_SetSigmask(sigset_t *mask)
+{
+    if (SIGPROCMASK(SIG_SETMASK, mask, NULL) != 0) {
+        Scm_SysError("sigprocmask failed");
+    }
+}
+
+
 /*
  * set signal mask
  */
@@ -763,7 +783,7 @@ ScmObj Scm_SysSigmask(int how, ScmSysSigset *newmask)
         }
     }
     if (SIGPROCMASK(how, newset, &(oldmask->set)) != 0) {
-        Scm_Error("sigprocmask failed");
+        Scm_SysError("sigprocmask failed");
     }
     return SCM_OBJ(oldmask);
 }
@@ -800,7 +820,7 @@ static void scm_sigsuspend(sigset_t *mask)
     ScmVM *vm = Scm_VM();
     for (;;) {
         SIGPROCMASK(SIG_BLOCK, &sigHandlers.masterSigset, &omask);
-        if (vm->queueNotEmpty & SCM_VM_SIGQ_MASK) {
+        if (vm->signalPending) {
             SIGPROCMASK(SIG_SETMASK, &omask, NULL);
             Scm_SigCheck(vm);
             continue;
@@ -938,7 +958,7 @@ void Scm__InitSignal(void)
 
     (void)SCM_INTERNAL_MUTEX_INIT(sigHandlers.mutex);
     sigemptyset(&sigHandlers.masterSigset);
-    for (i=0; i<NSIG; i++) sigHandlers.handlers[i] = SCM_FALSE;
+    for (i=0; i<NSIG; i++) sigHandlers.handlers[i] = SCM_UNDEFINED;
     
     Scm_InitStaticClass(&Scm_SysSigsetClass, "<sys-sigset>",
                         mod, NULL, 0);
