@@ -1,6 +1,5 @@
 /*
  * Test the lowest-level numeric routines.
- * $Id: test-arith.c,v 1.9 2007/08/24 23:55:44 shirok Exp $
  */
 
 #include "gauche.h"
@@ -80,7 +79,7 @@ void message(FILE *out, const char *m, int filler)
  * UADD
  */
 #define TEST_UADD(x_, y_, c_, rexp, cexp)               \
-  TEST5(x_, y_, c_, rexp, cexp, "%u", "+", UADD)
+  TEST5(x_, y_, c_, rexp, cexp, "%lu", "+", UADD)
 
 void test_uadd(void)
 {
@@ -88,36 +87,36 @@ void test_uadd(void)
     TEST_SECTION("UADD");
 
     /* MAX + 0 + 0 => [MAX, 0] */
-    TEST_UADD(UMAX, 0, 0, UMAX, 0);
+    TEST_UADD(UMAX, 0UL, 0UL, UMAX, 0UL);
     /* MAX + 1 + 0 => [0, 1]   */
-    TEST_UADD(UMAX, 1, 0, 0, 1);
+    TEST_UADD(UMAX, 1UL, 0UL, 0UL, 1UL);
     /* MAX + 2 + 0 => [1, 1]   */
-    TEST_UADD(UMAX, 2, 0, 1, 1);
+    TEST_UADD(UMAX, 2UL, 0UL, 1UL, 1UL);
     /* MAX + MAX + 0 => [MAX-1, 1] */
-    TEST_UADD(UMAX, UMAX, 0, UMAX-1, 1);
+    TEST_UADD(UMAX, UMAX, 0UL, UMAX-1, 1UL);
     /* MAX/2 + MAX/2 + 0 => [MAX-1, 0] */
-    TEST_UADD(UMAX/2, UMAX/2, 0, UMAX-1, 0);
+    TEST_UADD(UMAX/2, UMAX/2, 0UL, UMAX-1, 0UL);
     /* MAX/2 + MAX + 0 => [MAX/2-1, 1] */
-    TEST_UADD(UMAX/2, UMAX, 0, UMAX/2-1, 1);
+    TEST_UADD(UMAX/2, UMAX, 0UL, UMAX/2-1, 1UL);
     /* MAX-1 + 0 + 1 => [MAX, 0] */
-    TEST_UADD(UMAX-1, 0, 1, UMAX, 0);
+    TEST_UADD(UMAX-1, 0UL, 1UL, UMAX, 0UL);
     /* MAX + 0 + 1 => [0, 1] */
-    TEST_UADD(UMAX, 0, 1, 0, 1);
+    TEST_UADD(UMAX, 0UL, 1UL, 0UL, 1UL);
     /* MAX + 1 + 1 => [1, 1] */
-    TEST_UADD(UMAX, 1, 1, 1, 1);
+    TEST_UADD(UMAX, 1UL, 1UL, 1UL, 1UL);
     /* MAX + MAX + 1 => [MAX, 1] */
-    TEST_UADD(UMAX, UMAX, 1, UMAX, 1);
+    TEST_UADD(UMAX, UMAX, 1UL, UMAX, 1UL);
     /* MAX/2 + MAX/2 + 1 => [MAX, 0] */
-    TEST_UADD(UMAX/2, UMAX/2, 1, UMAX, 0);
+    TEST_UADD(UMAX/2, UMAX/2, 1UL, UMAX, 0UL);
     /* MAX/2 + MAX + 1 => [MAX/2, 1] */
-    TEST_UADD(UMAX/2, UMAX, 1, UMAX/2, 1);
+    TEST_UADD(UMAX/2, UMAX, 1UL, UMAX/2, 1UL);
 }
 
 /*
  * UADDOV
  */
 #define TEST_UADDOV(x_, y_, rexp, cexp)         \
-    TESTOV(x_, y_, rexp, cexp, "%u", "+", UADDOV)
+    TESTOV(x_, y_, rexp, cexp, "%lu", "+", UADDOV)
 
 void test_uaddov(void)
 {
@@ -125,21 +124,21 @@ void test_uaddov(void)
     TEST_SECTION("UADDOV");
 
     /* MAX + 0 => MAX */
-    TEST_UADDOV(UMAX, 0, UMAX, 0);
+    TEST_UADDOV(UMAX, 0UL, UMAX, 0UL);
     /* MAX + 1 => overflow */
-    TEST_UADDOV(UMAX, 1, 0, 1);
+    TEST_UADDOV(UMAX, 1UL, 0UL, 1UL);
     /* MAX + 2 => overflow */
-    TEST_UADDOV(UMAX, 2, 0, 1);
+    TEST_UADDOV(UMAX, 2UL, 0UL, 1UL);
     /* MAX + MAX => overflow */
-    TEST_UADDOV(UMAX, UMAX, 0, 1);
+    TEST_UADDOV(UMAX, UMAX, 0UL, 1UL);
     /* MAX/2 + MAX/2 => MAX-1, 0 */
-    TEST_UADDOV(UMAX/2, UMAX/2, UMAX-1, 0);
+    TEST_UADDOV(UMAX/2, UMAX/2, UMAX-1, 0UL);
     /* MAX/2 + MAX => overflow */
-    TEST_UADDOV(UMAX/2, UMAX, 0, 1);
+    TEST_UADDOV(UMAX/2, UMAX, 0UL, 1UL);
     /* MAX-1 + 1 => MAX */
-    TEST_UADDOV(UMAX-1, 1, UMAX, 0);
+    TEST_UADDOV(UMAX-1, 1UL, UMAX, 0UL);
     /* MAX-1 + 2 => overflow */
-    TEST_UADDOV(UMAX-1, 2, 0, 1);
+    TEST_UADDOV(UMAX-1, 2UL, 0UL, 1UL);
 }
 
 /*
@@ -154,46 +153,46 @@ void test_saddov(void)
     TEST_SECTION("SADDOV");
 
     /* 1 + 1 => 2 */
-    TEST_SADDOV(1, 1, 2, 0);
+    TEST_SADDOV(1L, 1L, 2L, 0L);
     /* -1 + -1 => -2 */
-    TEST_SADDOV(-1, -1, -2, 0);
+    TEST_SADDOV(-1L, -1L, -2L, 0L);
     /* SMAX + 0 => SMAX */
-    TEST_SADDOV(SMAX, 0, SMAX, 0);
+    TEST_SADDOV(SMAX, 0L, SMAX, 0L);
     /* SMAX + 1 => overflow */
-    TEST_SADDOV(SMAX, 1, 0, 1);
+    TEST_SADDOV(SMAX, 1L, 0L, 1L);
     /* SMAX + 2 => overflow */
-    TEST_SADDOV(SMAX, 2, 0, 1);
+    TEST_SADDOV(SMAX, 2L, 0L, 1L);
     /* SMAX + -1 => SMAX-1 */
-    TEST_SADDOV(SMAX, -1, SMAX-1, 0);
+    TEST_SADDOV(SMAX, -1L, SMAX-1, 0L);
     /* SMAX + -SMAX => 0 */
-    TEST_SADDOV(SMAX, -SMAX, 0, 0);
+    TEST_SADDOV(SMAX, -SMAX, 0L, 0L);
     /* SMAX + -SMAX-1 => -1 */
-    TEST_SADDOV(SMAX, -SMAX-1, -1, 0);
+    TEST_SADDOV(SMAX, -SMAX-1, -1L, 0L);
     /* SMAX/2 + SMAX/2 => SMAX-1 */
-    TEST_SADDOV(SMAX/2, SMAX/2, SMAX-1, 0);
+    TEST_SADDOV(SMAX/2, SMAX/2, SMAX-1, 0L);
     /* SMAX/2 + -SMAX/2 => 0 */
-    TEST_SADDOV(SMAX/2, -(SMAX/2), 0, 0);
+    TEST_SADDOV(SMAX/2, -(SMAX/2), 0L, 0L);
     /* -SMAX/2 + -SMAX/2 => -(SMAX-1) */
-    TEST_SADDOV(-(SMAX/2), -(SMAX/2), -(SMAX-1), 0);
+    TEST_SADDOV(-(SMAX/2), -(SMAX/2), -(SMAX-1), 0L);
     /* 0 + -SMAX => -SMAX */
-    TEST_SADDOV(0, -SMAX, -SMAX, 0);
+    TEST_SADDOV(0L, -SMAX, -SMAX, 0L);
     /* -1 + -SMAX => SMIN */
-    TEST_SADDOV(-1, -SMAX, SMIN, 0);
+    TEST_SADDOV(-1L, -SMAX, SMIN, 0L);
     /* -2 + -SMAX => -overflow */
-    TEST_SADDOV(-2, -SMAX, 0, -1);
+    TEST_SADDOV(-2L, -SMAX, 0L, -1L);
     /* -SMAX + -2 => -overflow */
-    TEST_SADDOV(-SMAX, -2, 0, -1);
+    TEST_SADDOV(-SMAX, -2L, 0L, -1L);
     /* -SMAX + -SMAX => -overflow */
-    TEST_SADDOV(-SMAX, -SMAX, 0, -1);
+    TEST_SADDOV(-SMAX, -SMAX, 0L, -1L);
     /* -SMAX + SMIN => -overflow */
-    TEST_SADDOV(-SMAX, SMIN, 0, -1);
+    TEST_SADDOV(-SMAX, SMIN, 0L, -1L);
 }
 
 /*
  * USUB
  */
 #define TEST_USUB(x_, y_, c_, rexp, cexp)               \
-    TEST5(x_, y_, c_, rexp, cexp, "%u", "-", USUB)
+    TEST5(x_, y_, c_, rexp, cexp, "%lu", "-", USUB)
 
 void test_usub(void)
 {
@@ -201,36 +200,36 @@ void test_usub(void)
     TEST_SECTION("USUB");
 
     /* MAX - 0 - 0 => [MAX, 0] */
-    TEST_USUB(UMAX, 0, 0, UMAX, 0);
+    TEST_USUB(UMAX, 0UL, 0UL, UMAX, 0UL);
     /* MAX - 1 - 0 => [MAX-1, 0] */
-    TEST_USUB(UMAX, 1, 0, UMAX-1, 0);
+    TEST_USUB(UMAX, 1UL, 0UL, UMAX-1, 0UL);
     /* 0 - MAX - 0 => [1, 1] */
-    TEST_USUB(0, UMAX, 0, 1, 1);
+    TEST_USUB(0UL, UMAX, 0UL, 1UL, 1UL);
     /* 1 - MAX - 0 => [2, 1] */
-    TEST_USUB(1, UMAX, 0, 2, 1);
+    TEST_USUB(1UL, UMAX, 0UL, 2UL, 1UL);
     /* MAX - MAX/2 - 0 => [MAX/2+1, 0] */
-    TEST_USUB(UMAX, UMAX/2, 0, UMAX/2+1, 0);
+    TEST_USUB(UMAX, UMAX/2, 0UL, UMAX/2+1, 0UL);
     /* MAX/2 - MAX - 0 => [MAX/2+1, 1] */
-    TEST_USUB(UMAX/2, UMAX, 0, UMAX/2+1, 1);
+    TEST_USUB(UMAX/2, UMAX, 0UL, UMAX/2+1, 1UL);
     /* MAX - 0 - 1 => [MAX-1, 0] */
-    TEST_USUB(UMAX, 0, 1, UMAX-1, 0);
+    TEST_USUB(UMAX, 0UL, 1UL, UMAX-1, 0UL);
     /* MAX - 1 - 1 => [MAX-2, 0] */
-    TEST_USUB(UMAX, 1, 1, UMAX-2, 0);
+    TEST_USUB(UMAX, 1UL, 1UL, UMAX-2, 0UL);
     /* 0 - MAX - 1 => [0, 1] */
-    TEST_USUB(0, UMAX, 1, 0, 1);
+    TEST_USUB(0UL, UMAX, 1UL, 0UL, 1UL);
     /* 1 - MAX - 1 => [1, 1] */
-    TEST_USUB(1, UMAX, 1, 1, 1);
+    TEST_USUB(1UL, UMAX, 1UL, 1UL, 1UL);
     /* MAX - MAX/2 - 1 => [MAX/2, 0] */
-    TEST_USUB(UMAX, UMAX/2, 1, UMAX/2, 0);
+    TEST_USUB(UMAX, UMAX/2, 1UL, UMAX/2, 0UL);
     /* MAX/2 - MAX - 1 => [MAX/2, 1] */
-    TEST_USUB(UMAX/2, UMAX, 1, UMAX/2, 1);
+    TEST_USUB(UMAX/2, UMAX, 1UL, UMAX/2, 1UL);
 }
 
 /*
  * USUBOV
  */
 #define TEST_USUBOV(x_, y_, rexp, cexp)         \
-    TESTOV(x_, y_, rexp, cexp, "%u", "-", USUBOV)
+    TESTOV(x_, y_, rexp, cexp, "%lu", "-", USUBOV)
 
 void test_usubov(void)
 {
@@ -238,19 +237,19 @@ void test_usubov(void)
     TEST_SECTION("USUBOV");
 
     /* MAX - 0  => MAX */
-    TEST_USUBOV(UMAX, 0, UMAX, 0);
+    TEST_USUBOV(UMAX, 0UL, UMAX, 0UL);
     /* MAX - 1  => MAX-1 */
-    TEST_USUBOV(UMAX, 1, UMAX-1, 0);
+    TEST_USUBOV(UMAX, 1UL, UMAX-1, 0UL);
     /* MAX - MAX => 0 */
-    TEST_USUBOV(UMAX, UMAX, 0, 0);
+    TEST_USUBOV(UMAX, UMAX, 0UL, 0UL);
     /* MAX-1 - MAX => overflow */
-    TEST_USUBOV(UMAX-1, UMAX, 0, 1);
+    TEST_USUBOV(UMAX-1, UMAX, 0UL, 1UL);
     /* MAX - MAX-1 => 1 */
-    TEST_USUBOV(UMAX, UMAX-1, 1, 0);
+    TEST_USUBOV(UMAX, UMAX-1, 1UL, 0UL);
     /* 0 - 1 => overflow */
-    TEST_USUBOV(0, 1, 0, 1);
+    TEST_USUBOV(0UL, 1UL, 0UL, 1UL);
     /* 0 - MAX => overflow */
-    TEST_USUBOV(0, UMAX, 0, 1);
+    TEST_USUBOV(0UL, UMAX, 0UL, 1UL);
 }
 
 /*
@@ -265,41 +264,41 @@ void test_ssubov(void)
     TEST_SECTION("SSUBOV");
 
     /* 1 - 1 => 0 */
-    TEST_SSUBOV(1, 1, 0, 0);
+    TEST_SSUBOV(1L, 1L, 0L, 0L);
     /* -1 - -1 => 0 */
-    TEST_SSUBOV(-1, -1, 0, 0);
+    TEST_SSUBOV(-1L, -1L, 0L, 0L);
     /* SMAX - 0 => SMAX */
-    TEST_SSUBOV(SMAX, 0, SMAX, 0);
+    TEST_SSUBOV(SMAX, 0L, SMAX, 0L);
     /* SMAX - 1 => SMAX-1 */
-    TEST_SSUBOV(SMAX, 1, SMAX-1, 0);
+    TEST_SSUBOV(SMAX, 1L, SMAX-1, 0L);
     /* SMAX - -1 => overflow */
-    TEST_SSUBOV(SMAX, -1, 0, 1);
+    TEST_SSUBOV(SMAX, -1L, 0L, 1L);
     /* SMAX - -2 => overflow */
-    TEST_SSUBOV(SMAX, -2, 0, 1);
+    TEST_SSUBOV(SMAX, -2L, 0L, 1L);
     /* SMAX - -SMAX => overflow */
-    TEST_SSUBOV(SMAX, -SMAX, 0, 1);
+    TEST_SSUBOV(SMAX, -SMAX, 0L, 1L);
     /* SMAX - SMAX => 0 */
-    TEST_SSUBOV(SMAX, SMAX, 0, 0);
+    TEST_SSUBOV(SMAX, SMAX, 0L, 0L);
     /* SMAX/2 - -SMAX/2 => SMAX-1 */
-    TEST_SSUBOV(SMAX/2, -(SMAX/2), SMAX-1, 0);
+    TEST_SSUBOV(SMAX/2, -(SMAX/2), SMAX-1, 0L);
     /* SMAX/2 - SMAX/2 => 0 */
-    TEST_SSUBOV(SMAX/2, SMAX/2, 0, 0);
+    TEST_SSUBOV(SMAX/2, SMAX/2, 0L, 0L);
     /* -SMAX/2 - SMAX/2 => 0 */
-    TEST_SSUBOV(-(SMAX/2), SMAX/2, -(SMAX-1), 0);
+    TEST_SSUBOV(-(SMAX/2), SMAX/2, -(SMAX-1), 0L);
     /* 0 - -SMAX => SMAX */
-    TEST_SSUBOV(0, -SMAX, SMAX, 0);
+    TEST_SSUBOV(0L, -SMAX, SMAX, 0L);
     /* 1 - -SMAX => overflow */
-    TEST_SSUBOV(1, -SMAX, 0, 1);
+    TEST_SSUBOV(1L, -SMAX, 0L, 1L);
     /* 0 - -SMAX => SMAX */
-    TEST_SSUBOV(0, -SMAX, SMAX, 0);
+    TEST_SSUBOV(0L, -SMAX, SMAX, 0L);
     /* SMIN - 1 => -overflow */
-    TEST_SSUBOV(SMIN, 1, 0, -1);
+    TEST_SSUBOV(SMIN, 1L, 0L, -1L);
     /* SMIN - -1 => SMIN+1 */
-    TEST_SSUBOV(SMIN, -1, SMIN+1, 0);
+    TEST_SSUBOV(SMIN, -1L, SMIN+1, 0L);
     /* -SMAX - -SMAX => 0 */
-    TEST_SSUBOV(-SMAX, -SMAX, 0, 0);
+    TEST_SSUBOV(-SMAX, -SMAX, 0L, 0L);
     /* -SMAX - SMIN => 1 */
-    TEST_SSUBOV(-SMAX, SMIN, 1, 0);
+    TEST_SSUBOV(-SMAX, SMIN, 1L, 0L);
 }
 
 /*
@@ -307,7 +306,7 @@ void test_ssubov(void)
  */
 #define TEST_UMUL(x_, y_, hiexp, loexp)                                 \
     do {                                                                \
-        printf("testing %u*%u expects hi=%u, lo=%u =>", x_, y_,        \
+        printf("testing %lu*%lu expects hi=%lu, lo=%lu =>", x_, y_,     \
                hiexp, loexp);                                           \
         x = x_;                                                         \
         y = y_;                                                         \
@@ -316,7 +315,7 @@ void test_ssubov(void)
             printf("ok\n");                                             \
         } else {                                                        \
             errcount++;                                                 \
-            printf("ERROR: got hi=%u, lo=%u\n", hi, lo);                \
+            printf("ERROR: got hi=%lu, lo=%lu\n", hi, lo);              \
         }                                                               \
     } while (0)
 
@@ -326,13 +325,13 @@ void test_umul(void)
     TEST_SECTION("UMUL");
 
     /* MAX * MAX => [MAX-1, 1] */
-    TEST_UMUL(UMAX, UMAX, UMAX-1, 1);
+    TEST_UMUL(UMAX, UMAX, UMAX-1, 1UL);
     /* MAX-1 * MAX-1 => [MAX-3, 4] */
-    TEST_UMUL(UMAX-1, UMAX-1, UMAX-3, 4);
+    TEST_UMUL(UMAX-1, UMAX-1, UMAX-3, 4UL);
     /* MAX/2 * 2 => [0, MAX-1] */
-    TEST_UMUL(UMAX/2, 2, 0, UMAX-1);
+    TEST_UMUL(UMAX/2, 2UL, 0UL, UMAX-1);
     /* MAX/2+1 * 2 => [1, 0] */
-    TEST_UMUL(UMAX/2+1, 2, 1, 0);
+    TEST_UMUL(UMAX/2+1, 2UL, 1UL, 0UL);
 }
 
 /*
@@ -347,27 +346,27 @@ void test_umulov(void)
     TEST_SECTION("UMULOV");
 
     /* 0 * 1 => 0 */
-    TEST_UMULOV(0, 1, 0, 0);
+    TEST_UMULOV(0UL, 1UL, 0UL, 0UL);
     /* 1 * 0 => 0 */
-    TEST_UMULOV(1, 0, 0, 0);
+    TEST_UMULOV(1UL, 0UL, 0UL, 0UL);
     /* 1 * 1 => 1 */
-    TEST_UMULOV(1, 1, 1, 0);
+    TEST_UMULOV(1UL, 1UL, 1UL, 0UL);
     /* 1 * UMAX => UMAX */
-    TEST_UMULOV(1, UMAX, UMAX, 0);
+    TEST_UMULOV(1UL, UMAX, UMAX, 0UL);
     /* 2 * UMAX => overflow */
-    TEST_UMULOV(2, UMAX, 0, 1);
+    TEST_UMULOV(2UL, UMAX, 0UL, 1UL);
     /* UMAX/2 * 2 => UMAX-1 */
-    TEST_UMULOV(UMAX/2, 2, UMAX-1, 0);
+    TEST_UMULOV(UMAX/2, 2UL, UMAX-1, 0UL);
     /* UMAX/2 * 3 => overflow */
-    TEST_UMULOV(UMAX/2, 3, 0, 1);
+    TEST_UMULOV(UMAX/2, 3UL, 0UL, 1UL);
     /* UMAX>>4 * UMAX>>4 => overflow */
-    TEST_UMULOV((UMAX>>4), (UMAX>>4), 0, 1);
+    TEST_UMULOV((UMAX>>4), (UMAX>>4), 0UL, 1UL);
     /* UMAX>>8 * UMAX>>8 => overflow */
-    TEST_UMULOV((UMAX>>8), (UMAX>>8), 0, 1);
+    TEST_UMULOV((UMAX>>8), (UMAX>>8), 0UL, 1UL);
     /* UMAX * UMAX => overflow */
-    TEST_UMULOV(UMAX, UMAX, 0, 1);
+    TEST_UMULOV(UMAX, UMAX, 0UL, 1UL);
     /* UMAX-1 * UMAX-1 => overflow */
-    TEST_UMULOV(UMAX-1, UMAX-1, 0, 1);
+    TEST_UMULOV(UMAX-1, UMAX-1, 0UL, 1UL);
 }
 
 /*
@@ -382,77 +381,77 @@ void test_smulov(void)
     TEST_SECTION("SMULOV");
 
     /* 0 * 1 => 0 */
-    TEST_SMULOV(0, 1, 0, 0);
+    TEST_SMULOV(0L, 1L, 0L, 0L);
     /* 1 * 0 => 0 */
-    TEST_SMULOV(1, 0, 0, 0);
+    TEST_SMULOV(1L, 0L, 0L, 0L);
     /* 1 * 1 => 1 */
-    TEST_SMULOV(1, 1, 1, 0);
+    TEST_SMULOV(1L, 1L, 1L, 0L);
     /* 1 * -1 => -1 */
-    TEST_SMULOV(1, -1, -1, 0);
+    TEST_SMULOV(1L, -1L, -1L, 0L);
     /* -1 * 1 => -1 */
-    TEST_SMULOV(-1, 1, -1, 0);
+    TEST_SMULOV(-1L, 1L, -1L, 0L);
     /* -1 * -1 => 1 */
-    TEST_SMULOV(-1, -1, 1, 0);
+    TEST_SMULOV(-1L, -1L, 1L, 0L);
     /* 1 * SMAX => SMAX */
-    TEST_SMULOV(1, SMAX, SMAX, 0);
+    TEST_SMULOV(1L, SMAX, SMAX, 0L);
     /* 1 * -SMAX => -SMAX */
-    TEST_SMULOV(1, -SMAX, -SMAX, 0);
+    TEST_SMULOV(1L, -SMAX, -SMAX, 0L);
     /* -1 * SMAX => -SMAX */
-    TEST_SMULOV(-1, SMAX, -SMAX, 0);
+    TEST_SMULOV(-1L, SMAX, -SMAX, 0L);
     /* -1 * -SMAX => SMAX */
-    TEST_SMULOV(-1, -SMAX, SMAX, 0);
+    TEST_SMULOV(-1L, -SMAX, SMAX, 0L);
     /* 1 * SMIN => SMIN */
-    TEST_SMULOV(1, SMIN, SMIN, 0);
+    TEST_SMULOV(1L, SMIN, SMIN, 0L);
     /* -1 * SMIN => overflow */
-    TEST_SMULOV(-1, SMIN, 0, 1);
+    TEST_SMULOV(-1L, SMIN, 0L, 1L);
     /* 2 * SMAX => overflow */
-    TEST_SMULOV(2, SMAX, 0, 1);
+    TEST_SMULOV(2L, SMAX, 0L, 1L);
     /* 2 * -SMAX => -overflow */
-    TEST_SMULOV(2, -SMAX, 0, -1);
+    TEST_SMULOV(2L, -SMAX, 0L, -1L);
     /* -2 * SMAX => -overflow */
-    TEST_SMULOV(-2, SMAX, 0, -1);
+    TEST_SMULOV(-2L, SMAX, 0L, -1L);
     /* -2 * -SMAX => overflow */
-    TEST_SMULOV(-2, -SMAX, 0, 1);
+    TEST_SMULOV(-2L, -SMAX, 0L, 1L);
     /* SMAX/2 * 2 => SMAX-1 */
-    TEST_SMULOV(SMAX/2, 2, SMAX-1, 0);
+    TEST_SMULOV(SMAX/2, 2L, SMAX-1, 0L);
     /* SMAX/2 * -2 => -(SMAX-1) */
-    TEST_SMULOV(SMAX/2, -2, -(SMAX-1), 0);
+    TEST_SMULOV(SMAX/2, -2L, -(SMAX-1), 0L);
     /* -SMAX/2 * 2 => -(SMAX-1) */
-    TEST_SMULOV(-(SMAX/2), 2, -(SMAX-1), 0);
+    TEST_SMULOV(-(SMAX/2), 2L, -(SMAX-1), 0L);
     /* -SMAX/2 * -2 => SMAX-1 */
-    TEST_SMULOV(-(SMAX/2), -2, SMAX-1, 0);
+    TEST_SMULOV(-(SMAX/2), -2L, SMAX-1, 0L);
     /* SMAX/2+1 * 2 => overflow */
-    TEST_SMULOV(SMAX/2+1, 2, 0, 1);
+    TEST_SMULOV(SMAX/2+1, 2L, 0L, 1L);
     /* SMAX/2+1 * -2 => SMIN */
-    TEST_SMULOV(SMAX/2+1, -2, SMIN, 0);
+    TEST_SMULOV(SMAX/2+1, -2L, SMIN, 0L);
     /* -(SMAX/2+1) * 2 => SMIN */
-    TEST_SMULOV(-(SMAX/2+1), 2, SMIN, 0);
+    TEST_SMULOV(-(SMAX/2+1), 2L, SMIN, 0L);
     /* -(SMAX/2+1) * -2 => overflow */
-    TEST_SMULOV(-(SMAX/2+1), -2, 0, 1);
+    TEST_SMULOV(-(SMAX/2+1), -2L, 0L, 1L);
     /* SMAX>>4 * SMAX>>4 => overflow */
-    TEST_SMULOV((SMAX>>4), (SMAX>>4), 0, 1);
+    TEST_SMULOV((SMAX>>4), (SMAX>>4), 0L, 1L);
     /* SMAX>>4 * -SMAX>>4 => -overflow */
-    TEST_SMULOV((SMAX>>4), -(SMAX>>4), 0, -1);
+    TEST_SMULOV((SMAX>>4), -(SMAX>>4), 0L, -1L);
     /* -SMAX>>4 * SMAX>>4 => -overflow */
-    TEST_SMULOV(-(SMAX>>4), (SMAX>>4), 0, -1);
+    TEST_SMULOV(-(SMAX>>4), (SMAX>>4), 0L, -1L);
     /* -SMAX>>4 * -SMAX>>4 => overflow */
-    TEST_SMULOV(-(SMAX>>4), -(SMAX>>4), 0, 1);
+    TEST_SMULOV(-(SMAX>>4), -(SMAX>>4), 0L, 1L);
     /* SMAX>>8 * SMAX>>8 => overflow */
-    TEST_SMULOV((SMAX>>8), (SMAX>>8), 0, 1);
+    TEST_SMULOV((SMAX>>8), (SMAX>>8), 0L, 1L);
     /* SMAX>>8 * -SMAX>>8 => -overflow */
-    TEST_SMULOV((SMAX>>8), -(SMAX>>8), 0, -1);
+    TEST_SMULOV((SMAX>>8), -(SMAX>>8), 0L, -1L);
     /* -SMAX>>8 * SMAX>>8 => -overflow */
-    TEST_SMULOV(-(SMAX>>8), (SMAX>>8), 0, -1);
+    TEST_SMULOV(-(SMAX>>8), (SMAX>>8), 0L, -1L);
     /* -SMAX>>8 * -SMAX>>8 => overflow */
-    TEST_SMULOV(-(SMAX>>8), -(SMAX>>8), 0, 1);
+    TEST_SMULOV(-(SMAX>>8), -(SMAX>>8), 0L, 1L);
     /* SMAX * SMAX => overflow */
-    TEST_SMULOV(SMAX, SMAX, 0, 1);
+    TEST_SMULOV(SMAX, SMAX, 0L, 1L);
     /* SMAX * -SMAX => -overflow */
-    TEST_SMULOV(SMAX, -SMAX, 0, -1);
+    TEST_SMULOV(SMAX, -SMAX, 0L, -1L);
     /* -SMAX * SMAX => -overflow */
-    TEST_SMULOV(-SMAX, SMAX, 0, -1);
+    TEST_SMULOV(-SMAX, SMAX, 0L, -1L);
     /* -SMAX * -SMAX => overflow */
-    TEST_SMULOV(-SMAX, -SMAX, 0, 1);
+    TEST_SMULOV(-SMAX, -SMAX, 0L, 1L);
 }
 
 /*=============================================================
