@@ -1,7 +1,7 @@
 /*
  * treemap.c - tree map implementation
  *
- *   Copyright (c) 2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2007-2010  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: treemap.c,v 1.8 2007/04/08 01:29:21 shirok Exp $
  */
 
 #define LIBGAUCHE_BODY
@@ -160,7 +158,10 @@ static Node *core_bound(ScmTreeCore *tc, ScmTreeCoreBoundOp op, int pop)
     Node *root = ROOT(tc);
     if (root) {
         Node *n = (op == SCM_TREE_CORE_MIN)? leftmost(root) : rightmost(root);
-        if (pop) n = delete_node(tc, n);
+        if (pop) {
+            n = delete_node(tc, n);
+            tc->num_entries--;
+        }
         return n;
     } else {
         return NULL;
