@@ -2,8 +2,6 @@
  * This code fragment implements the core of procedure and generic function
  * calling sequence in Gauche VM.  It is included by vm.c twice, with slight
  * difference switched by preprocessor macros.
- *
- * $Id: vmcall.c,v 1.2 2008/02/05 03:00:16 shirok Exp $
  */
 
 /* ADJUST_ARGUMENT_FRAME
@@ -11,61 +9,110 @@
  * Checks the argument count is OK for call to PROC.  if PROC takes &rest
  * args, fold those arguments to the list.  Modifies ARGC to hold the
  * adjusted size of the argument frame.
+ *
+ * There's two cases in how the arguments are pushed in the VM stack
+ * (normal call and apply call), and there's two cases in how the arguments
+ * should be prepared (fixed-argument call and variable-argument call), so
+ * there are four cases in total.
+ *
+ * Input:
+ *  In normal call, arguments at the call site are pushed in the stack
+ *  in left-to-right order.  For example, if the call site is (f x y z),
+ *  the top 3 stack entries of the stack are like this ('|' indicates 
+ *  stack top; stack grows to right):
+ *
+ *      ... x  y  z|
+ *
+ *  in this case, input ARGC is 3.
+ *
+ *  In apply call, the stack top is a list of tail of argument list.
+ *  If the call site is (apply f x (list y z)), input ARGC is 2 and the
+ *  stack top has these two entries:
+ *
+ *      ... x  (y z)|
+ *
+ * Output:
+ *  Suppose proc->required == N and proc->optional == M.
+ *  If M == 0, it is a fixed-argument procedure.  The stack must contain
+ *  N values, and ARGC must be N.
+ *
+ *  If M > 0, the stack must contain between N+1 to N+M values, where 
+ *  the last value is the tail of the argument list.  For standard Scheme
+ *  variable argument procedure, M is always 1 and the stack contains
+ *  N required arguments plus one list of 'rest' argument.
  */
 
 #undef ADJUST_ARGUMENT_FRAME
 #if !defined(APPLY_CALL)
-#define ADJUST_ARGUMENT_FRAME(proc, argc)               \
-    do {                                                \
-        int reqargs, restarg;                           \
-        reqargs = SCM_PROCEDURE_REQUIRED(proc);         \
-        restarg = SCM_PROCEDURE_OPTIONAL(proc);         \
-        if (restarg) {                                  \
-            ScmObj p = SCM_NIL, a;                      \
-            if (argc < reqargs) wna(VAL0, argc);        \
-            /* fold &rest args */                       \
-            while (argc > reqargs) {                    \
-                POP_ARG(a);                             \
-                p = Scm_Cons(a, p);                     \
-                argc--;                                 \
-            }                                           \
-            PUSH_ARG(p);                                \
-            argc++;                                     \
-        } else {                                        \
-            if (argc != reqargs) wna(VAL0, argc);       \
-        }                                               \
+#define ADJUST_ARGUMENT_FRAME(proc, argc)                               \
+    do {                                                                \
+        int reqargs, optargs;                                           \
+        reqargs = SCM_PROCEDURE_REQUIRED(proc);                         \
+        optargs = SCM_PROCEDURE_OPTIONAL(proc);                         \
+        if (optargs) {                                                  \
+            ScmObj p = SCM_NIL, a;                                      \
+            if (argc < reqargs) {                                       \
+                wna(vm, VAL0, argc, -1); RETURN_OP(); NEXT;             \
+            }                                                           \
+            /* fold &rest args */                                       \
+            while (argc > reqargs+optargs-1) {                          \
+                POP_ARG(a);                                             \
+                p = Scm_Cons(a, p);                                     \
+                argc--;                                                 \
+            }                                                           \
+            PUSH_ARG(p);                                                \
+            argc++;                                                     \
+        } else {                                                        \
+            if (argc != reqargs) {                                      \
+                wna(vm, VAL0, argc, -1); RETURN_OP(); NEXT;             \
+            }                                                           \
+        }                                                               \
     } while (0)
 #else /*APPLY_CALL*/
 #define ADJUST_ARGUMENT_FRAME(proc, argc)                               \
     do {                                                                \
-        int reqargs, restarg;                                           \
+        int reqargs, optargs;                                           \
         int rargc = Scm_Length(*(SP-1)), c;                             \
         ScmObj p, a;                                                    \
         reqargs = SCM_PROCEDURE_REQUIRED(proc);                         \
-        restarg = SCM_PROCEDURE_OPTIONAL(proc);                         \
-        if ((!restarg && ((rargc+argc-1) != reqargs))                   \
-            || (restarg && ((rargc+argc-1) < reqargs))) {               \
-            wna(VAL0, rargc+argc-1);                                    \
-        }                                                               \
-        if (argc+rargc < reqargs+(restarg?1:0)) {                       \
-            CHECK_STACK(reqargs+(restarg?1:0) - (argc+rargc));          \
-        }                                                               \
-        POP_ARG(p);                                                     \
-        if (argc-1 > reqargs) {                                         \
-            /* fold rest args */                                        \
-            p = Scm_CopyList(p);                                        \
-            for (c=argc-1; c > reqargs; c--) {                          \
-                POP_ARG(a);                                             \
-                p = Scm_Cons(a, p);                                     \
+        optargs = SCM_PROCEDURE_OPTIONAL(proc);                         \
+        if (optargs) {                                                  \
+            if ((rargc+argc-1) < reqargs) {                             \
+                wna(vm, VAL0, rargc+argc-1, rargc); RETURN_OP(); NEXT;  \
+            }                                                           \
+            POP_ARG(p);  /* tail of arglist */                          \
+            if (argc > reqargs+optargs) {                               \
+                /* fold rest args. */                                   \
+                p = Scm_CopyList(p);                                    \
+                for (c=argc; c>reqargs+optargs; c--) {                  \
+                    POP_ARG(a);                                         \
+                    p = Scm_Cons(a, p);                                 \
+                }                                                       \
+                PUSH_ARG(p);                                            \
+            } else {                                                    \
+                /* 'unfold' rest arg */                                 \
+                CHECK_STACK(reqargs + optargs - argc + 1);              \
+                for (c=argc; SCM_PAIRP(p) && c<reqargs+optargs; c++) {  \
+                    PUSH_ARG(SCM_CAR(p));                               \
+                    p = SCM_CDR(p);                                     \
+                }                                                       \
+                p = Scm_CopyList(p);                                    \
+                PUSH_ARG(p);                                            \
             }                                                           \
-            PUSH_ARG(p);                                                \
         } else {                                                        \
-            /* 'unfold' rest arg */                                     \
-            for (c=argc-1; c<reqargs; c++) {                            \
-                PUSH_ARG(SCM_CAR(p));                                   \
-                p = SCM_CDR(p);                                         \
+            /* no optargs */                                            \
+            if ((rargc+argc-1) != reqargs) {                            \
+                wna(vm, VAL0, rargc+argc-1, rargc); RETURN_OP(); NEXT;  \
+            }                                                           \
+            POP_ARG(p);  /* tail of arglist */                          \
+            if (rargc > 0) {                                            \
+                /* 'unfold' rest arg */                                 \
+                CHECK_STACK(rargc);                                     \
+                do {                                                    \
+                    PUSH_ARG(SCM_CAR(p));                               \
+                    p = SCM_CDR(p);                                     \
+                } while (--rargc > 0);                                  \
             }                                                           \
-            if (restarg) PUSH_ARG(p);                                   \
         }                                                               \
         argc = SP-ARGP;                                                 \
     } while (0)
@@ -125,10 +172,19 @@
         ADJUST_ARGUMENT_FRAME(VAL0, argc);
         SP = ARGP;
         PC = PC_TO_RETURN;
+#if GAUCHE_FFX
+        /* We can't pass FLONUM_REGs unless the subr is flagged as
+           IMMEDIATE_ARG. */
+        if (!(SCM_SUBR_FLAGS(VAL0)&SCM_SUBR_IMMEDIATE_ARG)) {
+            ScmObj *ap = SP;
+            int i = 0;
+            for (; i<argc; i++, ap++) SCM_FLONUM_ENSURE_MEM(*ap);
+        }
+#endif
 
         SCM_PROF_COUNT_CALL(vm, VAL0);
         VAL0 = SCM_SUBR(VAL0)->func(ARGP, argc, SCM_SUBR(VAL0)->data);
-        /* the subr may substituted pc, so we need to check
+        /* the subr may have substituted pc, so we need to check
            if we can pop the continuation immediately. */
         if (TAIL_POS()) RETURN_OP();
         NEXT;
@@ -145,6 +201,7 @@
         PC = vm->base->code;
         CHECK_STACK(vm->base->maxstack);
         SCM_PROF_COUNT_CALL(vm, SCM_OBJ(vm->base));
+        VAL0 = SCM_MAKE_INT(argc); /* keep argc to VAL0. */
         NEXT;
     }
 
@@ -210,6 +267,13 @@
                 PUSH_ARG(args);
             }
 #endif /*APPLY_CALL*/
+#if GAUCHE_FFX
+            {
+                ScmObj *ap = ARGP;
+                int i = 0;
+                for (;i<argc; i++, ap++) SCM_FLONUM_ENSURE_MEM(*ap);
+            }
+#endif /*GAUCHE_FFX*/
             mm = Scm_SortMethods(mm, ARGP, argc);
             nm = Scm_MakeNextMethod(SCM_GENERIC(VAL0), SCM_CDR(mm),
                                     ARGP, argc, TRUE, APP);
@@ -250,6 +314,9 @@
             if (!apply_call_p) goto do_method_call;
 #endif /*APPLY_CALL*/
         }
+    } else if (proctype == SCM_PROC_METHOD) {
+        VM_ERR(("Attempt to call a method %S without using a generic function.",
+                VAL0));
     } else {
         Scm_Panic("something's wrong.");
     }
@@ -310,6 +377,7 @@
         PC = vm->base->code;
         CHECK_STACK(vm->base->maxstack);
         SCM_PROF_COUNT_CALL(vm, SCM_OBJ(vm->base));
+        VAL0 = SCM_MAKE_INT(argc); /* keep argc to VAL0. */
     }
     NEXT;
 }
