@@ -1,7 +1,7 @@
 /*
  * vmstat.c - statistics gathering code for vm.c
  *
- *   Copyright (c) 2005-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2005-2010  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: vmstat.c,v 1.2 2007/03/02 07:39:14 shirok Exp $
  */
 
 /* This file is included from vm.c */
@@ -71,11 +69,11 @@ static ScmWord fetch_insn_counting(ScmVM *vm, ScmWord code)
         lref_freq[dep][off]++;
         break;
     }
-    case SCM_VM_LSET0: lset_freq[0][0]++;
-    case SCM_VM_LSET1: lset_freq[0][1]++;
-    case SCM_VM_LSET2: lset_freq[0][2]++;
-    case SCM_VM_LSET3: lset_freq[0][3]++;
-    case SCM_VM_LSET4: lset_freq[0][4]++;
+    case SCM_VM_LSET0: lset_freq[0][0]++; break;
+    case SCM_VM_LSET1: lset_freq[0][1]++; break;
+    case SCM_VM_LSET2: lset_freq[0][2]++; break;
+    case SCM_VM_LSET3: lset_freq[0][3]++; break;
+    case SCM_VM_LSET4: lset_freq[0][4]++; break;
     case SCM_VM_LSET:
     {
         int dep = SCM_VM_INSN_ARG0(code);
