@@ -2,8 +2,6 @@
 ;; Test hash table
 ;;
 
-;; $Id: hash.scm,v 1.12 2007/04/20 20:49:09 shirok Exp $
-
 (use gauche.test)
 (use srfi-1)
 (use srfi-13)
@@ -35,7 +33,7 @@
 (test* "b => non" #t
        (hash-table-get  h-eq 'b #t))
 
-(test* "b => error" *test-error*
+(test* "b => error" (test-error)
        (hash-table-get h-eq 'b))
 
 (test* "b => \"b\"" "b"
@@ -102,7 +100,7 @@
 (test* "b => non" #t
        (hash-table-get  h-eqv 'b #t))
 
-(test* "b => error" *test-error*
+(test* "b => error" (test-error)
        (hash-table-get  h-eqv 'b))
 
 (test* "b => \"b\"" "b"
@@ -150,11 +148,16 @@
 (test* "hash-table-values(2)" #t
        (lset= equal? (hash-table-values h-eqv) '(8 "b" #\c -1 4 5 PI)))
 
+(define h-eqv-copied (hash-table-copy h-eqv))
+
 (test* "delete!" #f
        (begin
          (hash-table-delete! h-eqv 87592876592374659237845692374523694756)
          (hash-table-get h-eqv 87592876592374659237845692374523694756 #f)))
 
+(test* "copy" #t
+       (lset= equal? (hash-table-values h-eqv-copied) '(8 "b" #\c -1 4 5 PI)))
+
 ;;------------------------------------------------------------------
 (test-section "equal?-hash")
 
@@ -174,7 +177,7 @@
 (test* "b => non" #t
        (hash-table-get  h-equal 'b #t))
 
-(test* "b => error" *test-error*
+(test* "b => error" (test-error)
        (hash-table-get  h-equal 'b))
 
 (test* "b => \"b\"" "b"
@@ -253,7 +256,7 @@
 (test* "\"b\" => non" #t
        (hash-table-get  h-string "b" #t))
 
-(test* "\"b\" => non" *test-error*
+(test* "\"b\" => non" (test-error)
        (hash-table-get  h-string "b"))
 
 (test* "\"b\" => \"b\"" "b"
