@@ -1,6 +1,6 @@
 ;;
 ;; test for listener
-;; $Id: listener.scm,v 1.5 2007/08/24 23:56:30 shirok Exp $
+;;
 
 (use gauche.test)
 
@@ -76,7 +76,7 @@
 (sexp-tester #t "#/\\/usr\\/bin/")
 (sexp-tester #f "#/\\/usr\\/bin  ")
 (sexp-tester #t "(#/(/ . a)")
-(sexp-tester *test-error* "(ibanr #<booba> )")
+(sexp-tester (test-error) "(ibanr #<booba> )")
 
 (test-section "listener")
 
