@@ -10,20 +10,21 @@
 
 ;; Some abstraction for win32... The file.util module provides
 ;; higher abstraction, but we're not at the stage of using it yet.
-
-(define *win32*
-  ;; we can't use string-suffix? yet.
-  (let* ((arch (gauche-architecture))
-	 (len  (string-length arch)))
-    (and (> len 7)
-	 (string=? (substring arch (- len 7) len) "mingw32"))))
-
 (define (P path) (sys-normalize-pathname path))
 
+(define (rmrf . files)
+  (dolist [f files]
+    (cond-expand
+     [gauche.os.windows
+      (sys-system #`"rmdir /q /s ,(P f) > NUL 2>&1")
+      (sys-system #`"del /q ,(P f) > NUL 2>&1")]
+     [else
+      (sys-system #`"rm -rf ,f > /dev/null")])))
+
 ;;----------------------------------------------------------------
 (test-section "require and provide")
 
-(sys-system "rm -rf test.o")
+(rmrf "test.o")
 (sys-mkdir "test.o" #o777)
 (with-output-to-file "test.o/a.scm"
   (lambda ()
@@ -38,7 +39,7 @@
          (eval '(require "test.o/a") (interaction-environment))
          #t))
 
-(sys-system "rm -rf test.o")
+(rmrf "test.o")
 (sys-mkdir "test.o" #o777)
 (with-output-to-file "test.o/b.scm"
   (lambda ()
@@ -52,10 +53,10 @@
     (newline)))
 
 (test* "detecting loop of require"
-       *test-error*
+       (test-error)
        (eval '(require "test.o/b") (interaction-environment)))
 
-(sys-system "rm -rf test.o")
+(rmrf "test.o")
 (sys-mkdir "test.o" #o777)
 (with-output-to-file "test.o/d.scm"
   (lambda ()
@@ -103,6 +104,96 @@
          (eval '(load "test.o/d") (find-module 'load.test))
          (with-module load.test foo)))
 
+;; current-load-* --------------------------------------
+(test-section "current-load-* info")
+
+(with-output-to-file "test.o/c1.scm"
+  (lambda ()
+    (write '(print (current-load-history)))
+    (write '(print (current-load-next)))
+    (write '(print (current-load-port)))
+    (write '(print (current-load-path)))
+    (write '(load "./test.o/c2.scm"))
+    (write '(print (current-load-history)))
+    (write '(print (current-load-next)))
+    (write '(print (current-load-port)))
+    (write '(print (current-load-path)))))
+
+(with-output-to-file "test.o/c2.scm"
+  (lambda ()
+    (write '(print (current-load-history)))
+    (write '(print (current-load-next)))
+    (write '(print (current-load-port)))
+    (write '(print (current-load-path)))))
+
+(with-output-to-file "test.o/c.out"
+  (lambda () (load "./test.o/c1.scm")))
+
+(with-input-from-file "test.o/c.out"
+  (lambda ()
+    (test* "current-load-history (1)" #/^\(\(#<iport [.\/]*test\/load\.scm/
+           (read-line) rxmatch)
+    (test* "current-load-next (1)" "()" (read-line))
+    (test* "current-load-port (1)" #/^#<iport \.\/test\.o\/c1\.scm/
+           (read-line) rxmatch)
+    (test* "current-load-path (1)" "./test.o/c1.scm"
+           (read-line) rxmatch)
+
+    (test* "current-load-history (2)"
+           #/^\(\(#<iport [.\/]*test\.o\/c1\.scm [^\)]*\) \(#<iport [.\/]*test\/load\.scm [^\)]*\)/
+           (read-line) rxmatch)
+    (test* "current-load-next (2)" "()" (read-line))
+    (test* "current-load-port (2)" #/^#<iport \.\/test\.o\/c2\.scm/
+           (read-line) rxmatch)
+    (test* "current-load-path (2)" "./test.o/c2.scm"
+           (read-line) rxmatch)
+
+    (test* "current-load-history (3)" #/^\(\(#<iport [.\/]*test\/load\.scm/
+           (read-line) rxmatch)
+    (test* "current-load-next (3)" "()" (read-line))
+    (test* "current-load-port (3)" #/^#<iport \.\/test\.o\/c1\.scm/
+           (read-line) rxmatch)
+    (test* "current-load-path (3)" "./test.o/c1.scm"
+           (read-line) rxmatch)
+    ))
+
+;; include --------------------------------------
+(test-section "include")
+
+(with-output-to-file "test.o/inc0.scm"
+  (lambda ()
+    (write '(define inc-var 4))
+    (write 'inc-var)))
+(with-output-to-file "test.o/inc1.scm"
+  (lambda ()
+    (write '(define inc-var 10))
+    (write '(let ((x inc-var))
+              (include "inc0.scm")
+              (+ x inc-var)))))
+(with-output-to-file "test.o/inc2.scm"
+  (lambda ()
+    (write '(define inc-var2 (let () (include "inc0"))))))
+
+(define (include-into-toplevel filename)
+  (eval `(begin (define inc-var 0)
+                (include ,filename))
+        (current-module)))
+
+(test* "include (relative to current, expands into toplevel)" 4
+       (include-into-toplevel "test.o/inc0"))
+(test* "include (relative to current, expands inside let)" 4
+       (let ((inc-var 2))
+         (include "test.o/inc0")
+         inc-var))
+(test* "include (absolute, expands into toplevel)" 4
+       (include-into-toplevel
+        (sys-normalize-pathname "test.o/inc0" :absolute #t)))
+(test* "include (nested)" 14
+       (include-into-toplevel "test.o/inc1"))
+(test* "include (postcondition)" 10
+       (eval 'inc-var (current-module)))
+(test* "include (within init part)" 4
+       (let () (include "test.o/inc2.scm") inc-var2))
 
 ;; autoloading -----------------------------------------
 (test-section "autoload")
@@ -117,7 +208,7 @@
   (lambda ()
     (write '(define foo 0))))
 (autoload "test.o/l1" foo1)
-(test* "autoload (file/error)" *test-error* foo1)
+(test* "autoload (file/error)" (test-error) foo1)
 
 (with-output-to-file "test.o/l0.scm"
   (lambda ()
@@ -130,7 +221,7 @@
 (test* "autoload environment" #t
        (load "./test.o/l0.scm"))
 
-(sys-system "rm -rf test.o")
+(rmrf "test.o")
 
 ;; library utilities -----------------------------------
 
@@ -302,6 +393,6 @@
 ;; we check module here, since gauche.libutil is autoloaded.
 (test-module 'gauche.libutil)
 
-(sys-system "rm -rf test.o")
+(rmrf "test.o")
 
 (test-end)
