@@ -91,8 +91,55 @@
 ;;----------------------------------------------------------------
 (test-section "application")
 
-(prim-test "apply" '(1 2 3) (lambda ()  (apply list 1 '(2 3))))
-(prim-test "apply" '(1 2 3) (lambda ()  (apply apply (list list 1 2 '(3)))))
+(prim-test "apply" '(1 2 3) (lambda ()  (apply list '(1 2 3))))
+(prim-test "apply" '(2 3 4) (lambda ()  (apply list 2 '(3 4))))
+(prim-test "apply" '(3 4 5) (lambda ()  (apply list 3 4 '(5))))
+(prim-test "apply" '(4 5 6) (lambda ()  (apply list 4 5 6 '())))
+
+(prim-test "apply^2" '() (lambda () (apply apply list '() '())))
+(prim-test "apply^2" '() (lambda () (apply apply list '(()))))
+(prim-test "apply^2" '(1 . 2) (lambda () (apply apply cons '((1 2)))))
+(prim-test "apply^2" '(3 . 4) (lambda () (apply apply cons 3 '((4)))))
+(prim-test "apply^2" '(5 . 6) (lambda () (apply apply (list cons 5 '(6)))))
+                                          
+
+(prim-test "apply" '(6 7 8) (lambda ()  (apply apply (list list 6 7 '(8)))))
+
+
+;; This tests 'unfolding' path in ADJUST_ARGUMENT_FRAME.
+(prim-test "apply, copying args" '(1 2 3)
+           (lambda ()
+             (let ((orig (list 1 2 3)))
+               (let ((new (apply list orig)))
+                 (set-car! (cdr new) '100)
+                 orig))))
+
+;; This tests 'folding' path in ADJUST_ARGUMENT_FRAME
+(prim-test "apply, copying args" '(2 3)
+           (lambda ()
+             (let ((orig (list 2 3)))
+               (let ((new (apply list 1 orig)))
+                 (set-car! (cdr new) '100)
+                 orig))))
+
+;; This test exhibits the optimizer bug reported by Michael Campbell.
+(define bug-optimizer-local-inliner
+  (lambda (flag)
+    (define (a . args)
+      (receive x args
+        (cons x x)
+        (apply values x))
+      (apply format args))
+    (define (b bar)
+      (a "~a" bar))
+    (b 1)
+    (cond
+     (flag (b 1))
+     (else (a "~a" 1)))))
+(prim-test "apply local inliner optimizer" "1"
+           (lambda () (bug-optimizer-local-inliner #f)) equal?)
+(prim-test "apply local inliner optimizer" "1"
+           (lambda () (bug-optimizer-local-inliner #t)) equal?)
 
 (prim-test "map" '()         (lambda ()  (map car '())))
 (prim-test "map" '(1 2 3)    (lambda ()  (map car '((1) (2) (3)))))
