@@ -2,8 +2,6 @@
 ;; testing regexp
 ;;
 
-;; $Id: regexp.scm,v 1.27 2007/12/23 06:58:48 shirok Exp $
-
 (use gauche.test)
 (use srfi-1)
 
@@ -66,6 +64,34 @@
 (test-regexp-parse "(?<name>)(?<name>)\\k<name>"
                    '(0 #f (1 name) (2 name) (alt (backref . 2) (backref . 1))))
 
+;;-------------------------------------------------------------------------
+(test-section "regexp-unparse")
+
+(define (test-regexp-unparse src)
+  (let ((ast (if (string? src) (regexp-optimize (regexp-parse src)) src)))
+    (test* (format "regexp-unparse ~s" src) ast
+           (regexp-optimize (regexp-parse (regexp-unparse ast)))
+           equal?)))
+
+(for-each test-regexp-unparse
+          '(;; simple ones
+            "" "a" "ab" "a*" "a+" "a?" "ab*" "ab+" "ab?" "a*b" "a+b" "a?b"
+            "a{3}" "a{2,4}" "a{3,}" "a|b" "a|b|c" "^ab" "^a|b" "^a|b$"
+            "." "\\." "\\**" "\\(\\)\\{\\}\\[\\]" "\\bfoo\\B"
+            ;; charset
+            "[a-z]" "[^a-z]" "[a-z]+" "[a^ef-]" "[a]" "[.]" "[\\[]" "[\\^]"
+            ;; grouping
+            "(a)(b)" "(a|b)c(d)" "(a(b(c|d)|e))f"
+            "(?:abc)*" "(ab(?:cd)?e{3,4})" "(?i:ab(?-i:cd)ef)"
+            ;; backref
+            "(a)bc\\1" "ab(?<foo>c)(d)\\k<foo>"
+            ;; once, lookahead, lookbehind
+            "(?>abc)" "(?=a*b*c)" "(?!a*b*c)" "(?<=a[bc])" "(?<!a[bc])"
+            "(a)(?(1)b)" "(a)(?(1)b|c)" "(a)(?(1)|c)"
+            "(?(?=ab)cd|ef)" "(?(?!ab)cd|ef)"
+            "(?(?<=ab)cd|ef)" "(?(?<!ab)cd|ef)" 
+            ))
+
 ;;-------------------------------------------------------------------------
 (test-section "regexp-writer")
 
@@ -554,7 +580,13 @@
 (test* "a{1,2}" "baaaabaa" (rxmatch-after (#/a{1,2}/ "abaaaabaa")))
 (test* "a{2,3}" "abaa"     (rxmatch-after (#/a{2,3}/ "abaaaabaa")))
 (test* "a{b,c}" "def"      (rxmatch-after (#/a{b,c}/ "za{b,c}def")))
-(test* "a{,2}*" "def"       (rxmatch-after (#/a{,2}*/  "za{,2}}def")))
+;; The following test assumes Perl compatibility.  (See the comment in
+;; rc1_lex_minmax in regexp.c.
+;(test* "a{,2}*" "def"       (rxmatch-after (#/a{,2}*/  "za{,2}}def")))
+;; The above should be written in Oniguruma compatible mode.
+(test* "a\\{,2}*" "def"       (rxmatch-after (#/a\{,2}*/  "za{,2}}def")))
+;; The following test assumes Oniguruma compability.
+(test* "a{,3}"  "baaaab"   (rxmatch-after (#/a{,3}/ "aabaaaab")))
 
 (test* "(ab){2}"   "abba"      (rxmatch-after (#/(ab){2}/ "babbabababba")))
 (test* "(ab){2,2}" "abba"      (rxmatch-after (#/(ab){2,2}/ "babbabababba")))
@@ -576,11 +608,11 @@
 (test* "(\\d{2}){2}" '("1234" "34")
        (match&list #/(\d{2}){2}/ "a12345b"))
 
-(test* "{2}" *test-error*  (regexp? (string->regexp "{2}")))
+(test* "{2}" (test-error)  (regexp? (string->regexp "{2}")))
 (test* "{z}" #t            (regexp? (string->regexp "{z}")))
 (test* "{-1}" #t           (regexp? (string->regexp "{-1}")))
-(test* "{300}" *test-error* (regexp? (string->regexp "{300}")))
-(test* "{3,1}" *test-error* (regexp? (string->regexp "{3,1}")))
+(test* "{300}" (test-error) (regexp? (string->regexp "{300}")))
+(test* "{3,1}" (test-error) (regexp? (string->regexp "{3,1}")))
 
 ;;-------------------------------------------------------------------------
 (test-section "{n,m} (non-greedy)")
@@ -672,9 +704,9 @@
        (match&list #/(.+)\1/i "AbCaBC"))
 (test* "/(.+)\\1/i" #f
        (match&list #/(.+)\1/ "AbCAb1"))
-(test* "^\\1(.)$" *test-error*
+(test* "^\\1(.)$" (test-error)
        (string->regexp "^\\1(.)"))
-(test* "^(\\1)$" *test-error*
+(test* "^(\\1)$" (test-error)
        (string->regexp "^(\\1)$"))
 
 ;;-------------------------------------------------------------------------
@@ -802,7 +834,7 @@
        (match&list #/(?<=\d{3})(?<!999)foo/ "865foo"))
 (test* "(?<=\d{3})(?<!999)foo" #f
        (match&list #/(?<=\d{3})(?<!999)foo/ "999foo"))
-(test* "(?<=(?>a*))" *test-error*
+(test* "(?<=(?>a*))" (test-error)
        (string->regexp "(?<=(?>a*))"))
 (test* "(abc)...(?<=\\1)" '("abcabc" "abc")
        (match&list #/(abc)...(?<=\1)/ "abcabc"))
@@ -824,7 +856,7 @@
          (list (m 'foo) (m 'bar))))
 (test* "(?<foo>a)(?<bar>.*)" '("abcd" "a" "bcd")
        (match&list #/(?<foo>a)(?<bar>.*)/ "abcd"))
-(test* "(?<foo>a)" *test-error*
+(test* "(?<foo>a)" (test-error)
        (let1 m (#/(?<foo>a)/ "abcd")
          (m 'bar)))
 (test* "(?<foo>^a$)" '("a" "a")
@@ -845,7 +877,7 @@
        (match&list #/(?<foo>.+)\k<foo>/ "abcabc"))
 (test* "(?<foo>.+)\\k<foo>" #f
        (match&list #/(?<foo>.+)\k<foo>/ "abcdef"))
-(test* "\\k<foo>" *test-error*
+(test* "\\k<foo>" (test-error)
        (regexp-parse "\\k<foo>"))
 (test* "rxmatch-before" "abc"
        (rxmatch-before (#/(?<foo>def)/ "abcdefghi") 'foo))
@@ -878,16 +910,16 @@
        (match&list #/(?(?<=a)b)/ "ab"))
 (test* "(?(?<=a)b|c)" #f
        (match&list #/(?(?<=a)b)/ "ac"))
-(test* "(?(?a)b|c)" *test-error*
+(test* "(?(?a)b|c)" (test-error)
        (regexp-parse "(?(?a)b|c)"))
 (test* "()(?(1))" '("" "")
        (match&list #/()(?(1))/ ""))
 
-(test* "()(?(" *test-error*
+(test* "()(?(" (test-error)
        (regexp-parse "()((?("))
-(test* "()(?(1" *test-error*
+(test* "()(?(1" (test-error)
        (regexp-parse "()((?(1"))
-(test* "()(?(1)b|c|d)" *test-error*
+(test* "()(?(1)b|c|d)" (test-error)
        (regexp-parse "()(?(1)b|c|d)"))
 
 ;;-------------------------------------------------------------------------
@@ -964,7 +996,7 @@
 (test* "regexp-replace" "abc|def|ghi"
        (regexp-replace #/(?<match>def|DEF)/ "abcdefghi" "|\\k<match>|"))
 
-(test* "regexp-replace" *test-error*
+(test* "regexp-replace" (test-error)
        (regexp-replace #/(?<match>def|DEF)/ "abcdefghi" "|\\k<matchee>|"))
 
 (test* "regexp-replace" "abraabra**brabra**brabrabracadabrabrabra"
@@ -997,9 +1029,9 @@
                                      (/ (string-length (rxmatch-substring m 1))
                                         3)))))
 
-(test* "regexp-replace-all" *test-error*
+(test* "regexp-replace-all" (test-error)
        (regexp-replace-all #/\d*/ "abcdef" "X"))
-(test* "regexp-replace-all" *test-error*
+(test* "regexp-replace-all" (test-error)
        (regexp-replace-all #/\d*/ "123abcdef" "X"))
 
 (test* "regexp-replace*" "cbazzbc"
