@@ -2,8 +2,6 @@
 ;; Test for SRFIs
 ;;
 
-;; $Id: srfi.scm,v 1.45 2007/04/05 05:56:58 shirok Exp $
-
 (use gauche.test)
 
 (test-start "SRFIs")
@@ -134,7 +132,7 @@
                (code (and-let* ((x (circular-list? foo)))
                        (take foo 3))))
              (make-module #f)))
-(test* "program (requires, no such feature)" *test-error*
+(test* "program (requires, no such feature)" (test-error)
        (eval '(program
                (requires no-such-feature))
              (make-module #f)))
@@ -175,7 +173,7 @@
                 (else (code (define x 2))))
                (code (+ x x)))
              (make-module #f)))
-(test* "program (feature-cond w/o else)" *test-error*
+(test* "program (feature-cond w/o else)" (test-error)
        (eval '(program
                (feature-cond
                 ((not srfi-1) (code (define x 5)))))
@@ -183,57 +181,29 @@
 
 (sys-system "rm -rf test.o")
 
-;;-----------------------------------------------------------------------
-(test-section "srfi-9")
-(use srfi-9)
-(test-module 'srfi-9)
-
-(define-record-type pare
-  (kons x y)
-  pare?
-  (x kar set-kar!)
-  (y kdr))
-
-(test* "pare kons" #t (pare? (kons 1 2)))
-(test* "pare kons" #f (pare? (cons 1 2)))
-(test* "pare kar" 1 (kar (kons 1 2)))
-(test* "pare kdr" 2 (kdr (kons 1 2)))
-(test* "pare set-kar!" 3 (let ((k (kons 1 2))) (set-kar! k 3) (kar k)))
-
-(define-record-type xpare
-  (xkons y x)
-  xpare?
-  (x kar)
-  (y kdr))
-
-(test* "xpare kons" '(1 . 2)
-       (let ((k (xkons 2 1))) (cons (kar k) (kdr k))))
-
-;;-----------------------------------------------------------------------
-(test-section "srfi-10")
-(use srfi-10)
-(test-module 'srfi-10)
-
-(test "read ctor 1a" '(1 2 #f "4 5")
-      (lambda ()
-        (define-reader-ctor 'list list)
-        (with-input-from-string "#,(list 1 2 #f \"4 5\")" read)))
-(test "read ctor 1b" 3
-      (lambda ()
-        (define-reader-ctor '+ +)
-        (with-input-from-string "#,(+ 1 2)" read)))
-(define-reader-ctor 'my-vector
-  (lambda x (apply vector (cons 'my-vector x))))
-(test* "read ctor 2a" '#(my-vector (my-vector 1 2))
-       (with-input-from-string "#,(my-vector (my-vector 1 2))" read))
-(test* "read ctor 2b" '#(my-vector #(my-vector 1 2))
-       (with-input-from-string "#,(my-vector #,(my-vector 1 2))" read))
-
 ;;-----------------------------------------------------------------------
 (test-section "srfi-14")
 (use srfi-14)
 (test-module 'srfi-14)
 
+;; char-set writer
+(let ()
+  (define (char-set-printer-tester p)
+    (test* "char-set-printer" (car p)
+           (write-to-string (list->char-set (cdr p)))))
+  (for-each char-set-printer-tester
+            '(("#[ace]" #\a #\e #\c)
+              ("#[ab]"  #\a #\b)
+              ("#[a-c]" #\a #\b #\c)
+              ("#[a-d]" #\a #\b #\c #\d)
+              ("#[a-ce]" #\a #\b #\c #\e)
+              ("#[acd]" #\a #\c #\d)
+              ("#[ac-e]" #\a #\c #\d #\e)
+              ("#[ac-e]" #\a #\c #\d #\e)
+              ("#[\\-\\[\\]]" #\[ #\] #\-)
+              ("#[\\^a]" #\^ #\a)
+              ("#[!^]" #\^ #\!))))
+
 ;; Test samples taken from Olin Shivers' test suite,
 ;; http://srfi.schemers.org/srfi-14/srfi-14-tests.scm
 ;; TODO: This doesn't test characters beyond ASCII.  See char-set.euc.scm.
@@ -387,6 +357,56 @@
        (char-set-contains? (->char-set "xyz") #\x))
 (test* "char-set-contains?" #f
        (char-set-contains? (->char-set "xyz") #\a))
+(test* "char-set-complement (ascii, nohit)" #f
+       (char-set-contains? (char-set-complement (char-set #\a)) #\a))
+(test* "char-set-complement (ascii, hit)" #t
+       (char-set-contains? (char-set-complement (char-set #\a)) #\b))
+(test* "char-set-complement (~#x80, nohit)" #f
+       (char-set-contains? (char-set-complement (char-set (integer->char #x80)))
+                           (integer->char #x80)))
+(test* "char-set-complement (~#x80, hit)" #t
+       (char-set-contains? (char-set-complement (char-set (integer->char #x80)))
+                           (integer->char #x81)))
+(test* "char-set-complement (~~#x80, hit)" #t
+       (char-set-contains? (char-set-complement
+                            (char-set-complement
+                             (char-set (integer->char #x80))))
+                           (integer->char #x80)))
+(test* "char-set-complement (~~#x80, nohit)" #f
+       (char-set-contains? (char-set-complement
+                            (char-set-complement
+                             (char-set (integer->char #x80))))
+                           (integer->char #x81)))
+(test* "char-set-complement (~#x82, nohit)" #f
+       (char-set-contains? (char-set-complement (char-set (integer->char #x82)))
+                           (integer->char #x82)))
+(test* "char-set-complement (~#x82, hit-upper)" #t
+       (char-set-contains? (char-set-complement (char-set (integer->char #x82)))
+                           (integer->char #x83)))
+(test* "char-set-complement (~#x82, hit-lower)" #t
+       (char-set-contains? (char-set-complement (char-set (integer->char #x82)))
+                           (integer->char #x81)))
+(test* "char-set-complement (~~#x82, hit)" #t
+       (char-set-contains? (char-set-complement
+                            (char-set-complement
+                             (char-set (integer->char #x82))))
+                           (integer->char #x82)))
+(test* "char-set-complement (~~#x82, nohit-upper)" #f
+       (char-set-contains? (char-set-complement
+                            (char-set-complement
+                             (char-set (integer->char #x82))))
+                           (integer->char #x83)))
+(test* "char-set-complement (~~#x82, nohit-lower)" #f
+       (char-set-contains? (char-set-complement
+                            (char-set-complement
+                             (char-set (integer->char #x82))))
+                           (integer->char #x81)))
+(test* "char-set-complement (~empty)" #t
+       (char-set-contains? (char-set-complement (char-set))
+                           (integer->char #x80)))
+(test* "char-set-complement (~~empty)" #f
+       (char-set-contains? (char-set-complement (char-set-complement (char-set)))
+                           (integer->char #x80)))
 (test* "char-set-every" #t
        (char-set-every char-lower-case? (->char-set "abcd")))
 (test* "char-set-every" #f
@@ -484,33 +504,7 @@
                 (char-set= i (->char-set "abcdefABCDEF"))))))
 
 ;;-----------------------------------------------------------------------
-(test-section "srfi-16")
-
-(test* "case-lambda (plus)" '(0 1 3 6 10)
-       (let ()
-         (define plus
-           (case-lambda 
-            (() 0)
-            ((x) x)
-            ((x y) (+ x y))
-            ((x y z) (+ (+ x y) z))
-            (args (apply + args))))
-         (list (plus) (plus 1) (plus 1 2) (plus 1 2 3) (plus 1 2 3 4))))
-
-(test* "case-lambda (mul2)" *test-error*
-       (let ()
-         (define mul2
-           (case-lambda 
-            (() 1)
-            ((x) x)
-            ((x y) (* x y))))
-         (mul2 1 2 3)))
-
-(test* "case-lambda (matching order)" '(1 (2))
-       ((case-lambda
-         ((x . y) (list x y))
-         ((x y) (list x y)))
-        1 2))
+;; srfi-16 case-lambda : moved to procedure.scm (builtin)
 
 ;;-----------------------------------------------------------------------
 (test-section "srfi-17")
