@@ -2,7 +2,10 @@
 ;; test for system related procedures
 ;;
 
+#!no-fold-case
+
 (use gauche.test)
+(use gauche.config)
 (use srfi-1)
 (use srfi-11)                           ;let-values
 (use srfi-13)
@@ -61,11 +64,16 @@
    (gauche.os.windows (get-command-output "cd"))
    (else
     (cond
-     ((sys-access "/bin/pwd" |X_OK|) (get-command-output "/bin/pwd"))
-     ((sys-access "/usr/bin/pwd" |X_OK|) (get-command-output "/usr/bin/pwd"))
-     ((sys-access "/sbin/pwd" |X_OK|) (get-command-output "/sbin/pwd"))
-     (else (get-command-output "pwd"))))))
-
+     [(sys-access "/bin/pwd" X_OK)
+      ;; On MacOSX, /bin/pwd returns _logical_ pathname by default,
+      ;; which is IMHO a bad decision (the behavior is /bin/pwd -L in
+      ;; other BSDs).  Anyway we have to cope with it.
+      (if (string-contains (gauche-config "--arch") "darwin")
+        (get-command-output "/bin/pwd -P")
+        (get-command-output "/bin/pwd"))]
+     [(sys-access "/usr/bin/pwd" X_OK) (get-command-output "/usr/bin/pwd")]
+     [(sys-access "/sbin/pwd" X_OK) (get-command-output "/sbin/pwd")]
+     [else (get-command-output "pwd")]))))
 
 ;;-------------------------------------------------------------------
 (test-section "environment")
@@ -173,13 +181,13 @@
 
 (test* "access" '(#f #f #f #f)
        (map (lambda (flag) (sys-access "test.dir" flag))
-            (list |F_OK| |R_OK| |W_OK| |X_OK|)))
+            (list F_OK R_OK W_OK X_OK)))
 
 (cmd-touch "test.dir")
 
 (test* "unlink" #f
        (begin
-         (sys-unlink "test.dir") (sys-access "test.dir" |F_OK|)))
+         (sys-unlink "test.dir") (sys-access "test.dir" F_OK)))
 
 (cond-expand
  (gauche.os.windows
@@ -216,8 +224,8 @@
 (test* "rename" '(#f #t)
        (begin
          (sys-rename "test.dir/xyzzy" "test.dir/zzZzz")
-         (list (sys-access "test.dir/xyzzy" |F_OK|)
-               (sys-access "test.dir/zzZzz" |F_OK|))))
+         (list (sys-access "test.dir/xyzzy" F_OK)
+               (sys-access "test.dir/zzZzz" F_OK))))
 
 (test* "readdir" '("." ".." "zzZzz")
        (sort (sys-readdir "test.dir")))
@@ -253,7 +261,7 @@
        (begin
          (sys-unlink "test.dir/zzZzz")
          (sys-rmdir "test.dir")
-         (sys-access "test.dir" |F_OK|)))
+         (sys-access "test.dir" F_OK)))
 
 ;;-------------------------------------------------------------------
 (test-section "time")
@@ -397,7 +405,8 @@
    (else (sys-sleep 1))))
 
 (cond-expand
- ((not gauche.os.windows)  ;; win32 doesn't support fork at all.
+ ((and (not gauche.os.windows)  ;; win32 doesn't support fork at all.
+       (not gauche.os.cygwin))	;; cygwin's fork is not reliable.
   (test* "fork & wait" #t
          (let ((pid (sys-fork)))
            (if (= pid 0)
@@ -421,23 +430,23 @@
            (if (= pid 0)
              (begin (sys-pause) (sys-exit 0))
              (begin 
-               (sys-kill pid |SIGKILL|)
+               (sys-kill pid SIGKILL)
                (receive (rpid code) (sys-wait)
                  (and (= rpid pid)
                       (sys-wait-signaled? code)
-                      (= (sys-wait-termsig code) |SIGKILL|)))))))
+                      (= (sys-wait-termsig code) SIGKILL)))))))
 
   (test* "fork, wait, kill & sleep" #t
          (let1 pid (sys-fork)
            (if (= pid 0)
              (begin (nap) (sys-exit 0))
              (begin 
-               (sys-kill pid |SIGSTOP|) 
+               (sys-kill pid SIGSTOP) 
                (receive (rpid code) (sys-waitpid pid :untraced #t)
                  (and (= rpid pid)
                       (sys-wait-stopped? code)
-                      (= (sys-wait-stopsig code) |SIGSTOP|)
-                      (begin (sys-kill pid |SIGCONT|)
+                      (= (sys-wait-stopsig code) SIGSTOP)
+                      (begin (sys-kill pid SIGCONT)
                              (receive (rpid code) (sys-wait)
                                (and (= rpid pid)
                                     (sys-wait-exited? code)
@@ -566,7 +575,7 @@
 (test-section "signal handling")
 
 (cond-expand
- ((not gauche.os.windows)
+ [(not gauche.os.windows)
 
   (test* "sigalrm1" SIGALRM
          (call/cc
@@ -588,7 +597,7 @@
                (sys-alarm 1)
                (sys-pause))))))
 
-  (test* "sigalrm3" *test-error*
+  (test* "sigalrm3" (test-error)
          (call/cc
           (lambda (k)
             (with-signal-handlers
@@ -659,7 +668,8 @@
 
   ;; NB: on cygwin (as of 1.5.25), sigmask doesn't work reliably so
   ;; we skip test for now.
-  (unless (string-contains (gauche-architecture) "-cygwin")
+  (cond-expand
+   [(not gauche.os.cygwin)
     (test* "sigmask" 'hup
            (let ((sig #f)
                  (chld #f)
@@ -727,12 +737,13 @@
                                  (sys-sigset) SIGHUP))
 
       (set-signal-handler! SIGINT #f)
-      )
-    ) ;; !cygwin
+      )]
+   [else]);; !cygwin
 
   ;; sys-sigwait
   (cond-expand
-   (gauche.sys.sigwait
+   [(and gauche.sys.sigwait
+         (not gauche.os.cygwin))
     (let ()
       (define z (lambda (n) (raise 'foo)))
       
@@ -762,10 +773,11 @@
       (test* "sys-sigwait / signal handler restoration" 'foo
              (guard (e (else e))
                (sys-kill (sys-getpid) SIGINT))))
-    )) ;; gauche.sys.sigwait
+    ]
+   [else]) ; (not (and gauche.sys.sigwait (not gauche.os.cygwin)))
 
-  ) ;; (not gauche.os.windows)
- (else))
+  ] 
+ [else]) ; gauche.os.windows
 
 (test-end)
 
