@@ -428,7 +428,7 @@
        (receive r (split-at* '(a b c d) 0) r))
 (test* "split-at* (boundary)" '((a b c d) ())
        (receive r (split-at* '(a b c d) 4) r))
-(test* "split-at* (error)" *test-error*
+(test* "split-at* (error)" (test-error)
        (receive r (split-at* '(a b c d) -1) r))
 (test* "split-at* (shorten)" '((a b c d) ())
        (receive r (split-at* '(a b c d) 5) r))
@@ -440,7 +440,7 @@
 (test* "take* (normal)" '(a b c)      (take* '(a b c d) 3))
 (test* "take* (boundary)" '()         (take* '(a b c d) 0))
 (test* "take* (boundary)" '(a b c d)  (take* '(a b c d) 4))
-(test* "take* (error)" *test-error*   (take* '(a b c d) -1))
+(test* "take* (error)" (test-error)   (take* '(a b c d) -1))
 (test* "take* (shorten)" '(a b c d)   (take* '(a b c d) 5))
 (test* "take* (fill)" '(a b c d #f #f) (take* '(a b c d) 6 #t))
 (test* "take* (fill)" '(a b c d z z)  (take* '(a b c d) 6 #t 'z))
@@ -448,20 +448,20 @@
 (test* "drop* (normal)" '(c d)       (drop* '(a b c d) 2))
 (test* "drop* (boundary)" '(a b c d) (drop* '(a b c d) 0))
 (test* "drop* (boundary)" '()        (drop* '(a b c d) 4))
-(test* "drop* (error)" *test-error*  (drop* '(a b c d) -3))
+(test* "drop* (error)" (test-error)  (drop* '(a b c d) -3))
 (test* "drop* (past)" '()            (drop* '(a b c d) 5))
 
 (test* "take-right* (normal)" '(b c d)  (take-right* '(a b c d) 3))
 (test* "take-right* (boundary)" '()     (take-right* '(a b c d) 0))
 (test* "take-right* (boundary)" '(a b c d) (take-right* '(a b c d) 4))
-(test* "take-right* (error)" *test-error*  (take-right* '(a b c d) -1))
+(test* "take-right* (error)" (test-error)  (take-right* '(a b c d) -1))
 (test* "take-right* (shorten)" '(a b c d)  (take-right* '(a b c d) 6))
 (test* "take-right* (fill)" '(z z a b c d) (take-right* '(a b c d) 6 #t 'z))
 
 (test* "drop-right* (normal)" '(a b c)  (drop-right* '(a b c d) 1))
 (test* "drop-right* (boundary)" '()     (drop-right* '(a b c d) 4))
 (test* "drop-right* (boundary)" '(a b c d) (drop-right* '(a b c d) 0))
-(test* "drop-right* (error)" *test-error*  (drop-right* '(a b c d) -1))
+(test* "drop-right* (error)" (test-error)  (drop-right* '(a b c d) -1))
 (test* "drop-right* (past)" '()         (drop-right* '(a b c d) 7))
 
 (test* "slices (normal)" '((0 1 2 3) (4 5 6 7) (8 9 10 11) (12 13 14 15))
@@ -548,117 +548,6 @@
 (test* "assv-set!" '((9 . c) (3 . a) (5 . b))
        (assv-set! (list (cons 3 'a) (cons 5 'b)) 9 'c))
 
-;;-----------------------------------------------
-(test-section "util.queue")
-(use util.queue)
-(test-module 'util.queue)
-
-(test* "queue?" #f (queue? (cons 'a 'b)))
-(test* "queue?" #f (queue? 3))
-(test* "queue?" #f (queue? '()))
-(define q (make-queue))
-
-(test* "queue?" #t (queue? q))
-(test* "enqueue!" #t (begin (enqueue! q 'a) (queue? q)))
-(test* "enqueue!" #t (begin (enqueue! q 'b) (queue? q)))
-(test* "enqueue!" #t (begin (enqueue! q 'c) (queue? q)))
-
-(test* "queue-front" 'a (queue-front q))
-(test* "queue-rear" 'c (queue-rear q))
-
-(test* "enqueue!" '(a f)
-       (begin
-         (enqueue! q 'd 'e 'f)
-         (list (queue-front q) (queue-rear q))))
-
-(test* "dequeue!" 'a (dequeue! q))
-(test* "dequeue!" 'b (dequeue! q))
-(test* "queue-empty?" #f (queue-empty? q))
-(test* "dequeue!" 'c (dequeue! q))
-(test* "dequeue!" 'd (dequeue! q))
-(test* "dequeue!" 'e (dequeue! q))
-(test* "dequeue!" 'f (dequeue! q))
-(test* "queue-empty?" #t (queue-empty? q))
-
-(test* "queue-push!" '(c a)
-       (begin
-         (queue-push! q 'a) (queue-push! q 'b) (queue-push! q 'c)
-         (list (queue-front q) (queue-rear q))))
-(test* "queue-push!" '(f a)
-       (begin
-         (queue-push! q 'd 'e 'f)
-         (list (queue-front q) (queue-rear q))))
-(test* "queue-pop!" 'f (queue-pop! q))
-(test* "queue-pop!" 'e (queue-pop! q))
-(test* "queue-empty?" #f (queue-empty? q))
-(test* "queue-pop!" 'd (queue-pop! q))
-(test* "queue-pop!" 'c (queue-pop! q))
-(test* "queue-pop!" 'b (queue-pop! q))
-(test* "queue-pop!" 'a (queue-pop! q))
-(test* "queue-empty?" #t (queue-empty? q))
-
-(test* "dequeue-all!" '(a b c d e)
-       (begin (enqueue! q 'a 'b 'c 'd 'e) (dequeue-all! q)))
-(test* "dequeue-all!" '()
-       (dequeue-all! q))
-(test* "dequeue-all!" #t
-       (queue-empty? q))
-
-(test* "find-in-queue" #f
-       (find-in-queue (cut eq? <> 'a) q))
-(test* "find-in-queue" 'a
-       (begin (enqueue! q 'a 'b 'c 'd 'e)
-              (find-in-queue (cut eq? <> 'a) q)))
-(test* "find-in-queue" 'c
-       (find-in-queue (cut eq? <> 'c) q))
-(test* "find-in-queue" 'e
-       (find-in-queue (cut eq? <> 'e) q))
-(test* "find-in-queue" '#f
-       (find-in-queue (cut eq? <> 'f) q))
-
-(test* "remove-from-queue!" #f
-       (remove-from-queue! (cut eq? <> 'f) q))
-(test* "remove-from-queue!" #t
-       (remove-from-queue! (cut eq? <> 'e) q))
-(test* "remove-from-queue!" #f
-       (remove-from-queue! (cut eq? <> 'e) q))
-(test* "remove-from-queue!" #t
-       (remove-from-queue! (cut eq? <> 'a) q))
-(test* "remove-from-queue!" #t
-       (remove-from-queue! (cut memq <> '(b c)) q))
-(test* "remove-from-queue!" #t
-       (remove-from-queue! (cut eq? <> 'd) q))
-(test* "remove-from-queue!" #t
-       (queue-empty? q))
-(test* "remove-from-queue!" #f
-       (remove-from-queue! (cut eq? <> 'd) q))
-
-
-(let ((q (make-queue)))
-  (test* "enqueue-unique!" '("a")
-         (begin (enqueue-unique! q equal? "a")
-                (queue->list q)))
-  (test* "enqueue-unique!" '("a" "b")
-         (begin (enqueue-unique! q equal? "b")
-                (queue->list q)))
-  (test* "enqueue-unique!" '("a" "b")
-         (begin (enqueue-unique! q equal? "a")
-                (queue->list q)))
-  (test* "enqueue-unique!" '("a" "b" "c" "d")
-         (begin (enqueue-unique! q equal? "a" "b" "c" "d")
-                (queue->list q)))
-  (test* "queue-push-unique!" '("e" "a" "b" "c" "d")
-         (begin (queue-push-unique! q equal? "d" "e")
-                (queue->list q)))
-  (set! q (make-queue))
-  (test* "queue-push-unique!" '("e" "d")
-         (begin (queue-push-unique! q equal? "d" "e")
-                (queue->list q)))
-  (test* "queue-push-unique!" '("c" "b" "a" "e" "d")
-         (begin (queue-push-unique! q equal? "a" "b" "c" "d" "e")
-                (queue->list q)))
-  )
-
 ;;-----------------------------------------------
 (test-section "util.rbtree")
 (use util.rbtree)
@@ -673,13 +562,13 @@
   (test* "make-rbtree" #t
          (begin (set! tree1 (make-rbtree = <))
                 (rbtree? tree1)))
-  (test* "rbtree-get" *test-error*
+  (test* "rbtree-get" (test-error)
          (rbtree-get #f 0 'foo))
   (test* "rbtree-get" 'not-found
          (rbtree-get tree1 0 'not-found))
-  (test* "rbtree-get" *test-error*
+  (test* "rbtree-get" (test-error)
          (rbtree-get tree1 0))
-  (test* "rbtree-put!" *test-error*
+  (test* "rbtree-put!" (test-error)
          (rbtree-put! #f 0 'foo))
   (test* "rbtree-put!" "0"
          (begin (rbtree-put! tree1 0 "0")
@@ -720,7 +609,7 @@
          (rbtree-empty? tree1))
   (test* "rbtree-empty?" #t
          (rbtree-empty? (make-rbtree < =)))
-  (test* "rbtree-empty?" *test-error*
+  (test* "rbtree-empty?" (test-error)
          (rbtree-empty? 'wrong-arg))
   (test* "rbtree-exists?" '(#t #f)
          (let1 tree (make-rbtree = <)
@@ -757,8 +646,8 @@
 
   (test* "rbtree-min" '(0 . "0") (rbtree-min tree2))
   (test* "rbtree-max" '(2 . "2") (rbtree-max tree2))
-  (test* "rbtree-min" *test-error* (rbtree-min 'wrong-arg))
-  (test* "rbtree-min" *test-error* (rbtree-min 'wrong-arg))
+  (test* "rbtree-min" (test-error) (rbtree-min 'wrong-arg))
+  (test* "rbtree-min" (test-error) (rbtree-min 'wrong-arg))
   (test* "rbtree-min" 'default (rbtree-min empty 'default))
   (test* "rbtree-max" 'default (rbtree-max empty 'default))
 
@@ -843,10 +732,10 @@
 (test* "make-record-type" #t
        (is-a? record:1 <class>))
 
-(test* "make-record-type (error)" *test-error*
+(test* "make-record-type (error)" (test-error)
        (make-record-type "record:1" '((x 2) y)))
 
-(test* "make-record-type (error)" *test-error*
+(test* "make-record-type (error)" (test-error)
        (make-record-type "record:1" '(a b a)))
 
 (test* "record-constructor" #t
@@ -1046,27 +935,25 @@
 
 
 ;; using famous socks example (Corman)
-(test* "topological-sort"
-       '(socks undershorts watch shirt tie pants belt jacket shoes)
-       (topological-sort '((shirt tie belt)
-                           (tie jacket)
-                           (belt jacket)
-                           (watch)
-                           (pants shoes belt)
-                           (undershorts pants shoes)
-                           (socks shoes))
-                         eq?))
+(let ([sorted '(socks undershorts watch shirt tie pants belt jacket shoes)]
+      [input '((shirt tie belt)
+               (tie jacket)
+               (belt jacket)
+               (watch)
+               (pants shoes belt)
+               (undershorts pants shoes)
+               (socks shoes))])
 
-(test* "topological-sort"
-       '("socks" "undershorts" "watch" "shirt" "tie" "pants" "belt" "jacket" "shoes")
-       (topological-sort '(("shirt" "tie" "belt")
-                           ("tie" "jacket")
-                           ("belt" "jacket")
-                           ("watch")
-                           ("pants" "shoes" "belt")
-                           ("undershorts" "pants" "shoes")
-                           ("socks" "shoes"))
-                         equal?))
+  (test* "topological-sort, default" sorted
+         (topological-sort input))
+
+  (test* "topological-sort, eq?" sorted
+         (topological-sort input eq?))
+
+  (test* "topological-sort, string=?"
+         (map symbol->string sorted)
+         (topological-sort (map (cut map symbol->string <>) input) string=?))
+  )
 
 ;;-----------------------------------------------
 (test-section "util.trie")
@@ -1095,7 +982,7 @@
                   (trie-num-entries t1)))
     (test* "trie: get" 4
            (trie-get t1 "lilo"))
-    (test* "trie: get (error)" *test-error*
+    (test* "trie: get (error)" (test-error)
            (trie-get t1 "LILO"))
     (test* "trie: get (fallback)" 'foo
            (trie-get t1 "LILO" 'foo))
@@ -1165,7 +1052,7 @@
     (test* "trie: trie-update!" 16
            (begin (trie-update! t1 "liliko`i" (cut + <> 8))
                   (trie-get t1 "liliko`i")))
-    (test* "trie: trie-update! (nonexistent)" *test-error*
+    (test* "trie: trie-update! (nonexistent)" (test-error)
            (trie-update! t1 "humuhumu" (cut + <> 8)))
     (test* "trie: trie-update! (nonexistent)" 16
            (begin (trie-update! t1 "humuhumu" (cut + <> 8) 8)
